@@ -1,62 +1,54 @@
 #ifndef MEGAAPPLICATION_H
 #define MEGAAPPLICATION_H
 
-#include <QApplication>
-#include <QSystemTrayIcon>
-#include <QMenu>
+#include "AppState.h"
+#include "BlockingStageProgressController.h"
+#include "DesktopNotifications.h"
+#include "DownloadFromMegaDialog.h"
+#include "DuplicatedNodeInfo.h"
+#include "HTTPServer.h"
+#include "InfoDialog.h"
+#include "LinkProcessor.h"
+#include "megaapi.h"
+#include "MegaDownloader.h"
+#include "MegaMenuItemAction.h"
+#include "MegaSyncLogger.h"
+#include "MegaUploader.h"
+#include "PasteMegaLinksDialog.h"
+#include "Preferences.h"
+#include "QTMegaListener.h"
+#include "ScanStageController.h"
+#include "SetManager.h"
+#include "SettingsDialog.h"
+#include "SyncInfo.h"
+#include "ThreadPool.h"
+#include "TransferManager.h"
+#include "TransferQuota.h"
+#include "UpdateTask.h"
+#include "UpsellPlans.h"
+#include "Utilities.h"
+
 #include <QAction>
+#include <QApplication>
+#include <QDataStream>
 #include <QDir>
+#include <QFutureWatcher>
 #include <QLocalServer>
 #include <QLocalSocket>
-#include <QDataStream>
-#include <QQueue>
+#include <QMenu>
 #include <QNetworkInterface>
-#include <QFutureWatcher>
+#include <QQueue>
+#include <QSystemTrayIcon>
 
 #include <memory>
 
-#include "gui/TransferManager.h"
-#include "gui/InfoDialog.h"
-#include "gui/UpgradeOverStorage.h"
-#include "gui/SettingsDialog.h"
-#include "gui/DownloadFromMegaDialog.h"
-#include "gui/StreamingFromMegaDialog.h"
-#include "gui/ImportMegaLinksDialog.h"
-#include "gui/MultiQFileDialog.h"
-#include "gui/PasteMegaLinksDialog.h"
-#include "gui/ChangeLogDialog.h"
-#include "Preferences.h"
-#include "HTTPServer.h"
-#include "MegaUploader.h"
-#include "MegaDownloader.h"
-#include "UpdateTask.h"
-#include "MegaSyncLogger.h"
-#include "ThreadPool.h"
-#include "Utilities.h"
-#include "SetManager.h"
-#include "syncs/control/SyncInfo.h"
-#include "syncs/control/SyncController.h"
-#include "megaapi.h"
-#include "QTMegaListener.h"
-#include "gui/QFilterAlertsModel.h"
-#include "gui/MegaAlertDelegate.h"
-#include "gui/VerifyLockMessage.h"
-#include "notifications/DesktopNotifications.h"
-#include "ScanStageController.h"
-#include "TransferQuota.h"
-#include "BlockingStageProgressController.h"
-#include "qml/QmlManager.h"
-#include "qml/QmlDialogManager.h"
-
 class IntervalExecutioner;
 class TransfersModel;
 class StalledIssuesModel;
 
 #ifdef __APPLE__
-    #include "gui/MegaSystemTrayIcon.h"
     #include <mach/mach.h>
     #include <sys/sysctl.h>
-    #include <errno.h>
 #endif
 
 Q_DECLARE_METATYPE(QQueue<QString>)
@@ -67,6 +59,8 @@ class DuplicatedNodeDialog;
 class LoginController;
 class AccountStatusController;
 class StatsEventHandler;
+class UserMessageController;
+class SyncReminderNotificationManager;
 
 enum GetUserStatsReason {
     USERSTATS_LOGGEDIN,
@@ -85,7 +79,7 @@ enum GetUserStatsReason {
     USERSTATS_REMOVEVERSIONS,
 };
 
-class MegaApplication : public QApplication, public mega::MegaListener, public StorageDetailsObserved, public BandwidthDetailsObserved, public AccountDetailsObserved
+class MegaApplication : public QApplication, public mega::MegaListener
 {
     Q_OBJECT
 
@@ -116,10 +110,8 @@ public:
     void onTransferUpdate(mega::MegaApi *api, mega::MegaTransfer *transfer) override;
     void onTransferTemporaryError(mega::MegaApi *api, mega::MegaTransfer *transfer, mega::MegaError* e) override;
     void onAccountUpdate(mega::MegaApi *api) override;
-    void onUserAlertsUpdate(mega::MegaApi *api, mega::MegaUserAlertList *list) override;
     void onUsersUpdate(mega::MegaApi* api, mega::MegaUserList *users) override;
-    void onNodesUpdate(mega::MegaApi* api, mega::MegaNodeList *nodes) override;
-    void onReloadNeeded(mega::MegaApi* api) override;
+    void onNodesUpdate(mega::MegaApi* api, mega::MegaNodeList* nodes) override;
     void onGlobalSyncStateChanged(mega::MegaApi *api) override;
 
     void onGlobalSyncStateChangedImpl();
@@ -145,7 +137,6 @@ public:
     void showWarningMessage(QString message, QString title = MegaSyncApp->getMEGAString());
     void showErrorMessage(QString message, QString title = MegaSyncApp->getMEGAString());
     void showNotificationMessage(QString message, QString title = MegaSyncApp->getMEGAString());
-    void setUploadLimit(int limit);
     void setMaxUploadSpeed(int limit);
     void setMaxDownloadSpeed(int limit);
     void setMaxConnections(int direction, int connections);
@@ -153,7 +144,6 @@ public:
     void startUpdateTask();
     void stopUpdateTask();
     void applyProxySettings();
-    void updateUserStats(bool storage, bool transfer, bool pro, bool force, int source);
     void checkForUpdates();
     // Actually show InfoDialog view, not tray menu.
     void showTrayMenu(QPoint *point = NULL);
@@ -165,15 +155,12 @@ public:
     void createInfoDialogMenus();
     void toggleLogging();
 
-    bool notificationsAreFiltered();
-    bool hasNotifications();
-    bool hasNotificationsOfType(int type);
     std::shared_ptr<mega::MegaNode> getRootNode(bool forceReset = false);
     std::shared_ptr<mega::MegaNode> getVaultNode(bool forceReset = false);
     std::shared_ptr<mega::MegaNode> getRubbishNode(bool forceReset = false);
     void resetRootNodes();
     void initLocalServer();
-    void onboardingFinished(bool fastLogin);
+    void onboardingFinished(bool fastLogin, bool comesFromOnboarding = false);
     void onLoginFinished();
     void onFetchNodesFinished();
     void onLogout();
@@ -185,6 +172,7 @@ public:
 
     TransfersModel* getTransfersModel(){return mTransfersModel;}
     StalledIssuesModel* getStalledIssuesModel(){return mStalledIssuesModel;}
+    UserMessageController* getNotificationController() { return mUserMessageController.get(); }
 
     /**
      * @brief migrates sync configuration and fetches nodes
@@ -199,9 +187,6 @@ public:
 
     void updateTrayIconMenu();
 
-    std::shared_ptr<mega::MegaPricing> getPricing() const;
-
-    QuotaState getTransferQuotaState() const;
     std::shared_ptr<TransferQuota> getTransferQuota() const;
 
     int getAppliedStorageState() const;
@@ -215,6 +200,9 @@ public:
     LoginController* getLoginController();
     AccountStatusController* getAccountStatusController();
 
+    void updateUsedStorage(const bool sendEvent = false);
+    void showUpsellDialog(UpsellPlans::ViewMode viewMode);
+
 signals:
     void startUpdaterThread();
     void tryUpdate();
@@ -229,6 +217,10 @@ signals:
     void pauseStateChanged();
     void addBackup();
     void shellNotificationsProcessed();
+    void updateUserInterface();
+    void requestAppState(AppState::AppStates newAppState);
+    void syncsDialogClosed();
+    void languageChanged();
 
 public slots:
     void updateTrayIcon();
@@ -242,11 +234,15 @@ public slots:
     void importLinks();
     void officialWeb();
     void goToMyCloud();
+    void goToFiles();
+    void openDeviceCentre();
     void pauseTransfers();
     void showChangeLog();
     void uploadActionClicked();
-    void uploadActionClickedFromWindowAfterOverQuotaCheck();
+    void uploadActionFromWindowAfterOverQuotaCheck();
+    void runUploadActionWithTargetHandle(const mega::MegaHandle &targetFolder, QWidget *parent);
     void downloadActionClicked();
+    void downloadACtionClickedWithHandles(const QList<mega::MegaHandle>& handles);
     void streamActionClicked();
     void transferManagerActionClicked(int tab = 0);
     void logoutActionClicked();
@@ -255,17 +251,17 @@ public slots:
     void processUploads();
     void shellUpload(QQueue<QString> newUploadQueue);
     void shellExport(QQueue<QString> newExportQueue);
-    void shellViewOnMega(QByteArray localPath, bool versions);
-    void shellViewOnMega(mega::MegaHandle handle, bool versions);
+    void shellViewOnMega(const QString& localPath, bool versions);
+    void shellViewOnMegaByHandle(mega::MegaHandle handle, bool versions);
     void exportNodes(QList<mega::MegaHandle> exportList, QStringList extraLinks = QStringList());
-    void externalDownload(QQueue<WrappedNode *> newDownloadQueue);
+    void uploadFilesToNode(const QList<QUrl>& files, mega::MegaHandle targetNode, QWidget* caller);
+    void externalDownload(QQueue<WrappedNode> newDownloadQueue);
     void externalLinkDownload(QString megaLink, QString auth);
-    void externalFileUpload(qlonglong targetFolder);
-    void externalFolderUpload(qlonglong targetFolder);
-    void externalFolderSync(qlonglong targetFolder);
+    void externalFileUpload(mega::MegaHandle targetFolder);
+    void externalFolderUpload(mega::MegaHandle targetFolder);
+    void externalFolderSync(mega::MegaHandle targetFolder);
     void externalAddBackup();
     void externalOpenTransferManager(int tab);
-    void internalDownload(long long handle);
     void onRequestLinksFinished();
     void onUpdateCompleted();
     void onUpdateAvailable(bool requested);
@@ -274,11 +270,10 @@ public slots:
     void onUpdateError();
     void rebootApplication(bool update = true);
     void tryExitApplication(bool force = false);
-    void highLightMenuEntry(QAction* action);
     void pauseTransfers(bool pause);
     void checkNetworkInterfaces();
     void checkMemoryUsage();
-    void checkOverStorageStates();
+    void checkOverStorageStates(bool isOnboardingAboutClosing = false);
     void checkOverQuotaStates();
     void periodicTasks();
     void cleanAll();
@@ -289,8 +284,6 @@ public slots:
     void triggerInstallUpdate();
     void scanningAnimationStep();
     void clearDownloadAndPendingLinks();
-    void proExpirityTimedOut();
-    void applyNotificationFilter(int opt);
     void changeState();
 
 #ifdef _WIN32
@@ -305,16 +298,13 @@ public slots:
     int getPrevVersion();
     void onDismissStorageOverquota(bool overStorage);
     void showNotificationFinishedTransfers(unsigned long long appDataId);
-    void setDownloadFinished(const QString& setName,
-                             const QStringList& succeededDownloadedElements,
-                             const QStringList& failedDownloadedElements,
-                             const QString& destinationPath);
     void transferBatchFinished(unsigned long long appDataId, bool fromCancellation);
     void updateStatesAfterTransferOverQuotaTimeHasExpired();
 #ifdef __APPLE__
     void enableFinderExt();
 #endif
     void requestFetchSetFromLink(const QString& link);
+    void onAppStateChanged(AppState::AppStates, AppState::AppStates);
 
 private slots:
     void openFolderPath(QString path);
@@ -329,7 +319,7 @@ private slots:
     void cancelScanningStage();
 
 protected slots:
-    void onUploadsCheckedAndReady(QPointer<DuplicatedNodeDialog> checkDialog);
+    void onUploadsCheckedAndReady(std::shared_ptr<ConflictTypes> conflicts);
     void onPasteMegaLinksDialogFinish(QPointer<PasteMegaLinksDialog>);
     void onDownloadFromMegaFinished(QPointer<DownloadFromMegaDialog> dialog);
     void onDownloadSetFolderDialogFinished(QPointer<DownloadFromMegaDialog> dialog);
@@ -339,17 +329,17 @@ protected:
     void createGuestMenu();
     bool showTrayIconAlwaysNEW();
     void applyStorageState(int state, bool doNotAskForUserStats = false);
-    void processUploadQueue(mega::MegaHandle nodeHandle);
+    void processUploadQueue(mega::MegaHandle nodeHandle, QWidget* caller = nullptr);
     void processDownloadQueue(QString path);
     void disableSyncs();
     void restoreSyncs();
-    void createTransferManagerDialog(TransfersWidget::TM_TAB tab);
-    void deleteMenu(QMenu *menu);
+    void createTransferManagerDialog();
+    void deleteMenu(QMenu* menu);
+    void clearMenu(QMenu* menu, bool deleteAction = false);
     void startHttpServer();
     void startHttpsServer();
     void refreshStorageUIs();
     void manageBusinessStatus(int64_t event);
-    void populateUserAlerts(mega::MegaUserAlertList *list, bool copyRequired);
 
     bool eventFilter(QObject *obj, QEvent *e) override;
     void createInfoDialog();
@@ -366,6 +356,7 @@ protected:
     QAction *windowsUpdateAction;
     QAction *windowsAboutAction;
     QAction *windowsImportLinksAction;
+    QAction *windowsFilesAction;
     QAction *windowsUploadAction;
     QAction *windowsDownloadAction;
     QAction *windowsStreamAction;
@@ -377,15 +368,17 @@ protected:
     QPointer<QMenu> guestMenu;
     QMenu emptyMenu;
 
-    MenuItemAction *exitAction;
-    MenuItemAction *settingsAction;
-    MenuItemAction *importLinksAction;
-    MenuItemAction *uploadAction;
-    MenuItemAction *downloadAction;
-    MenuItemAction *streamAction;
-    MenuItemAction *myCloudAction;
-    MenuItemAction *updateAction;
-    MenuItemAction *aboutAction;
+    MegaMenuItemAction* exitAction;
+    MegaMenuItemAction* settingsAction;
+    MegaMenuItemAction* importLinksAction;
+    MegaMenuItemAction* uploadAction;
+    MegaMenuItemAction* downloadAction;
+    MegaMenuItemAction* streamAction;
+    MegaMenuItemAction* filesAction;
+    MegaMenuItemAction* MEGAWebAction;
+    MenuItemAction* deviceCentreAction;
+    MegaMenuItemAction* updateAction = nullptr;
+    MegaMenuItemAction* aboutAction = nullptr;
     QAction *showStatusAction;
     QPointer<SyncsMenu> mSyncs2waysMenu;
     QPointer<SyncsMenu> mBackupsMenu;
@@ -400,36 +393,26 @@ protected:
 #endif
 
     std::unique_ptr<QTimer> onGlobalSyncStateChangedTimer;
-    QTimer proExpirityTimer;
     int scanningAnimationIndex;
     QPointer<SettingsDialog> mSettingsDialog;
     QPointer<InfoDialog> infoDialog;
     std::shared_ptr<Preferences> preferences;
     SyncInfo *model;
     mega::MegaApi *megaApi;
-    mega::MegaApi *megaApiFolders;
-    QFilterAlertsModel *notificationsProxyModel;
-    QAlertsModel *notificationsModel;
-    MegaAlertDelegate *notificationsDelegate;
-    QObject *context;
-    QString crashReportFilePath;
+    mega::MegaApi* megaApiFolders;
 
     HTTPServer *httpServer;
     mega::MegaHandle fileUploadTarget;
     mega::MegaHandle folderUploadTarget;
 
     QQueue<QString> uploadQueue;
-    QQueue<WrappedNode *> downloadQueue;
+    QQueue<WrappedNode> downloadQueue;
     BlockingBatch mBlockingBatch;
 
     ThreadPool* mThreadPool;
     std::shared_ptr<mega::MegaNode> mRootNode;
     std::shared_ptr<mega::MegaNode> mVaultNode;
     std::shared_ptr<mega::MegaNode> mRubbishNode;
-    bool queuedUserStats[3];
-    int queuedStorageUserStatsReason;
-    long long userStatsLastRequest[3];
-    bool inflightUserStats[3];
     long long cleaningSchedulerExecution;
     long long lastUserActivityExecution;
     long long lastTsBusinessWarning;
@@ -438,11 +421,8 @@ protected:
     int appliedStorageState;
     bool getUserDataRequestReady;
     long long receivedStorageSum;
-    long long maxMemoryUsage;
+    unsigned long long mMaxMemoryUsage;
     int exportOps;
-    std::shared_ptr<mega::MegaPricing> mPricing;
-    std::shared_ptr<mega::MegaCurrency> mCurrency;
-    QPointer<UpgradeOverStorage> mStorageOverquotaDialog;
     mega::QTMegaListener *delegateListener;
     MegaUploader *uploader;
     MegaDownloader *downloader;
@@ -482,7 +462,6 @@ protected:
     long long lastExit;
     bool appfinished;
     bool updateAvailable;
-    bool isLinux;
     bool mIsFirstFileTwoWaySynced;
     bool mIsFirstFileBackedUp;
     bool networkConnectivity;
@@ -498,10 +477,8 @@ protected:
     std::shared_ptr<DesktopNotifications> mOsNotifications;
     AccountStatusController* mStatusController;
     QMutex mMutexOpenUrls;
-    QMap<QString, std::chrono::system_clock::time_point> mOpenUrlsClusterTs;
+    QMap<QUrl, std::chrono::system_clock::time_point> mOpenUrlsClusterTs;
 
-    // Note: mSyncController is used only to add the syncs set up in the onboarding wizard
-    std::unique_ptr<SyncController> mSyncController;
     LogoutController* mLogoutController;
 
     QPointer<TransfersModel> mTransfersModel;
@@ -509,14 +486,24 @@ protected:
     ScanStageController scanStageController;
     std::shared_ptr<FolderTransferListener> mFolderTransferListener;
 
-    bool mDisableGfx;
     StalledIssuesModel* mStalledIssuesModel;
-    StatsEventHandler* mStatsEventHandler;
+    std::unique_ptr<StatsEventHandler> mStatsEventHandler;
 
     SetManager* mSetManager;
+    LinkProcessor* mLinkProcessor;
+
     QString mLinkToPublicSet;
     QList<mega::MegaHandle> mElementHandleList;
     std::unique_ptr<IntervalExecutioner> mIntervalExecutioner;
+    bool mDisableGfx;
+
+    std::unique_ptr<UserMessageController> mUserMessageController;
+
+    std::unique_ptr<mega::MegaGfxProvider> mGfxProvider;
+
+    QPointer<SyncReminderNotificationManager> mSyncReminderNotificationManager;
+
+    bool misSyncingStateWrongLogged;
 
 private:
     void loadSyncExclusionRules(QString email = QString());
@@ -538,6 +525,8 @@ private:
     static QString obfuscateIpv6Address(const QHostAddress& ipAddress);
     static QStringList explodeIpv6(const QHostAddress &ipAddress);
 
+    static bool mightBeCaseSensitivityIssue(const QString& folderPath);
+
     void reconnectIfNecessary(const bool disconnected, const QList<QNetworkInterface>& newNetworkInterfaces);
     bool isIdleForTooLong() const;
 
@@ -556,6 +545,7 @@ private:
 
     bool dontAskForExitConfirmation(bool force);
     void exitApplication();
+    void openFirstActiveSync();
 
     QString getDefaultUploadPath();
 
@@ -575,9 +565,12 @@ private:
 
     void updateMetadata(TransferMetaData* data, const QString& filePath);
 
-    template <class Func>
-    void recreateMenuAction(MenuItemAction** action, QMenu* menu, const QString& actionName,
-                            const char* iconPath, Func slotFunc)
+    template<class Func>
+    void recreateMegaMenuAction(MegaMenuItemAction** action,
+                                QMenu* menu,
+                                const QString& actionName,
+                                const char* iconPath,
+                                Func slotFunc)
     {
         bool previousEnabledState = true;
         if (*action)
@@ -586,15 +579,16 @@ private:
             (*action)->deleteLater();
             *action = nullptr;
         }
-
-        *action = new MenuItemAction(actionName, QLatin1String(iconPath), menu);
-        (*action)->setManagesHoverStates(true);
+        *action = new MegaMenuItemAction(actionName, QLatin1String(iconPath), 0);
         connect(*action, &QAction::triggered, this, slotFunc, Qt::QueuedConnection);
         (*action)->setEnabled(previousEnabledState);
     }
-
-    template <class Func>
-    void recreateAction(QAction** action, QMenu* menu, const QString& actionName, Func slotFunc)
+    template<class Func>
+    void recreateAction(QAction** action,
+                        QMenu* menu,
+                        const QString& actionName,
+                        Func slotFunc,
+                        const QString& iconPath = QString())
     {
         bool previousEnabledState = true;
         if (*action)
@@ -607,6 +601,11 @@ private:
         *action = new QAction(actionName, menu);
         connect(*action, &QAction::triggered, this, slotFunc);
         (*action)->setEnabled(previousEnabledState);
+
+        if (!iconPath.isEmpty())
+        {
+            (*action)->setIcon(QIcon(iconPath));
+        }
     }
 
     void processUpgradeSecurityEvent();
@@ -617,14 +616,22 @@ private:
 
     void sendPeriodicStats() const;
 
+    void createUserMessageController();
+    void closeUpsellStorageDialog();
+
+    void createGfxProvider(const QString& basePath);
+    void startCrashReportingDialog();
+
+    void removeSyncsAndBackupsMenus();
+
+    void initStyleAndResources();
+
 private slots:
     void onFolderTransferUpdate(FolderTransferUpdateEvent event);
     void onNotificationProcessed();
     void onScheduledExecution();
-
-private:
-    QFutureWatcher<NodeCount> mWatcher;
-
+    void onCopyLinkError(const QString& nodeName, const int errorCode);
+    void onOperatingSystemThemeChanged(const Preferences::SystemColorScheme& theme);
 };
 
 #endif // MEGAAPPLICATION_H
