@@ -1,13 +1,14 @@
 #ifndef EMAILREQUESTER_H
 #define EMAILREQUESTER_H
 
-#include <mega/bindings/qt/QTMegaGlobalListener.h>
-#include <mega/bindings/qt/QTMegaRequestListener.h>
+#include "mega/bindings/qt/QTMegaGlobalListener.h"
 
 #include <QMap>
 #include <QObject>
 #include <QRecursiveMutex>
 
+#include <memory>
+
 class RequestInfo: public QObject
 {
     Q_OBJECT
@@ -52,7 +53,7 @@ private:
     QRecursiveMutex mRequestsDataLock;
     QMap<mega::MegaHandle, RequestInfo*> mRequestsData;
     std::unique_ptr<mega::QTMegaGlobalListener> mGlobalListener;
-    static EmailRequester* mInstance;
+    static std::unique_ptr<EmailRequester> mInstance;
 };
 
 #endif
