@@ -1,20 +1,18 @@
 #include "HTTPServer.h"
-#include "Preferences.h"
-#include "Utilities.h"
+
 #include "MegaApplication.h"
-#include "mega/setandelement.h"
+#include "Preferences.h"
+#include "ServiceUrls.h"
 #include "StatsEventHandler.h"
+#include "Utilities.h"
 
 #include <QtConcurrent/QtConcurrent>
 
-
 #include <algorithm>
 
-
 using namespace mega;
 
 const unsigned int HTTPServer::MAX_REQUEST_TIME_SECS = 1800;
-const QString PUBLIC_LINK_START = QString::fromUtf8("https://mega.nz/collection/");
 
 bool ts_comparator(RequestData* i, RequestData *j)
 {
@@ -211,7 +209,7 @@ void HTTPServer::readClient()
             return;
         }
 
-        if (Preferences::HTTPS_ORIGIN_CHECK_ENABLED && !Preferences::HTTPS_ALLOWED_ORIGINS.isEmpty())
+        if (Preferences::HTTPS_ORIGIN_CHECK_ENABLED)
         {
             QString foundOrigin = findCorrespondingAllowedOrigin(headers);
             if (!foundOrigin.isEmpty())
@@ -407,9 +405,9 @@ void HTTPServer::openLinkRequest(QString &response, const HTTPRequest& request)
 
     if (handle.size() == 8 && key.size() == 43)
     {
-        QString link = Preferences::BASE_URL + QString::fromUtf8("/#!%1!%2").arg(handle, key);
+        QString link = ServiceUrls::instance()->getRemoteNodeLinkUrl(handle, key).toString();
         emit onLinkReceived(link, auth);
-        response = QString::fromUtf8("0");
+        response = QLatin1String("0");
 
         auto preferences = Preferences::instance();
         QString defaultPath = preferences->downloadFolder();
@@ -466,7 +464,7 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
 
         if (privateAuth.size() || publicAuth.size())
         {
-            QQueue<WrappedNode *> downloadQueue;
+            QQueue<WrappedNode> downloadQueue;
 
             int end;
             bool firstnode = true;
@@ -477,7 +475,6 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                 if (end < 0)
                 {
                     MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Error parsing webclient request");
-                    qDeleteAll(downloadQueue);
                     downloadQueue.clear();
                     break;
                 }
@@ -489,8 +486,8 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                 long long type = Utilities::extractJSONNumber(file, QString::fromUtf8("t"));
                 if (type < 0)
                 {
-                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Node without type in webclient request");
-                    qDeleteAll(downloadQueue);
+                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR,
+                                 "Node without type in webclient request");
                     downloadQueue.clear();
                     break;
                 }
@@ -498,8 +495,8 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                 QString handle = Utilities::extractJSONString(file, QString::fromUtf8("h"));
                 if (handle.isEmpty())
                 {
-                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Node without handle in webclient request");
-                    qDeleteAll(downloadQueue);
+                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR,
+                                 "Node without handle in webclient request");
                     downloadQueue.clear();
                     break;
                 }
@@ -510,8 +507,8 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                 name = QString::fromUtf8(QByteArray::fromBase64(name.toUtf8().constData()).constData());
                 if (name.isEmpty())
                 {
-                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Node without name in webclient request");
-                    qDeleteAll(downloadQueue);
+                    MegaApi::log(MegaApi::LOG_LEVEL_ERROR,
+                                 "Node without name in webclient request");
                     downloadQueue.clear();
                     break;
                 }
@@ -543,7 +540,8 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                     MegaNode *node = megaApi->createForeignFolderNode(h, nameArray.constData(), p,
                                                                      privateAuthArray.constData(),
                                                                      publicAuthArray.constData());
-                    downloadQueue.append(new WrappedNode(WrappedNode::TransferOrigin::FROM_WEBSERVER, node, undelete));
+                    downloadQueue.append(
+                        WrappedNode(WrappedNode::TransferOrigin::FROM_WEBSERVER, node, undelete));
                 }
                 else
                 {
@@ -564,7 +562,10 @@ void HTTPServer::externalDownloadRequest(QString &response, const HTTPRequest& r
                                                          p, privateAuthArray.constData(),
                                                          publicAuthArray.constData(),
                                                          chatAuth.isEmpty() ? nullptr :  chatAuthArray.constData());
-                        downloadQueue.append(new WrappedNode(WrappedNode::TransferOrigin::FROM_WEBSERVER, node, undelete));
+                        downloadQueue.append(
+                            WrappedNode(WrappedNode::TransferOrigin::FROM_WEBSERVER,
+                                        node,
+                                        undelete));
                         QMap<MegaHandle, RequestTransferData*>::iterator it = webTransferStateRequests.find(h);
                         if (it != webTransferStateRequests.end())
                         {
@@ -593,37 +594,34 @@ void HTTPServer::externalDownloadSetRequest(QString &response, const HTTPRequest
 {
     MegaApi::log(MegaApi::LOG_LEVEL_DEBUG, "ExternalDownloadSet command received from the webclient");
 
-    //! Example command: 'megasync.megaSyncRequest({ a: 'ds', auth: "Y7VXFI6b", k: "2e4l7O_oI4qGxDY5eJCojg", e:["4lIlbGvSCBE", "wgILwZTk1T8", "omKB2C-vuEo", "Hj2JPpIgu1I", "QM8IK0MWN9s", "Lutj3MIgxuI"]})'
+    //! Example command: 'megasync.megaSyncRequest({ a: 'ds', auth: "Y7VXFI6b", k:
+    //! "2e4l7O_oI4qGxDY5eJCojg", e:["4lIlbGvSCBE", "wgILwZTk1T8", "omKB2C-vuEo", "Hj2JPpIgu1I",
+    //! "QM8IK0MWN9s", "Lutj3MIgxuI"]})'
     //!     'auth' contains set_ph (set public handle)
     //!     'k' contains public key
-    //! => Reconstruct link by concatenating:
-    //!     "https://mega.nz/collection/" + auth" + "#" + k
-    //! eg:
-    //!     "https://mega.nz/collection/" + "Y7VXFI6b" + "#" + "2e4l7O_oI4qGxDY5eJCojg"
-    QString auth = Utilities::extractJSONString(request.data, QString::fromUtf8("auth"));
-    QString k = Utilities::extractJSONString(request.data, QString::fromUtf8("k"));
+
+    QString auth = Utilities::extractJSONString(request.data, QLatin1String("auth"));
+    QString k = Utilities::extractJSONString(request.data, QLatin1String("k"));
     if (auth.isEmpty() || k.isEmpty())
     {
         response = QString::number(MegaError::API_EARGS);
         return;
     }
 
-    QString publicLink = PUBLIC_LINK_START + auth + QString::fromUtf8("#") + k;
+    auto publicLink = ServiceUrls::instance()->getRemoteSetLinkUrl(auth, k).toString();
 
     // Get Element IDs
-    QStringList e = Utilities::extractJSONStringList(request.data, QString::fromUtf8("e"));
+    const auto e = Utilities::extractJSONStringList(request.data, QLatin1String("e"));
 
     QList<mega::MegaHandle> handleList;
-    for(const QString& eId : qAsConst(e))
+    for (const auto& eId: e)
     {
-        QByteArray ba = eId.toLocal8Bit();
-        const char *c_str2 = ba.data();
-        size_t size = mega::SetElement::HANDLESIZE;
-        unsigned char* result;
-        megaApi->base64ToBinary(c_str2, &result, &size);
-        const mega::MegaHandle* myHandlePtr = reinterpret_cast<mega::MegaHandle*>(result);
+        auto size = static_cast<size_t>(mega::MegaApi::getSetElementHandleSize());
+        unsigned char* result = nullptr;
+        mega::MegaApi::base64ToBinary(eId.toUtf8().constData(), &result, &size);
+        const auto* myHandlePtr = reinterpret_cast<mega::MegaHandle*>(result);
         handleList.append(*myHandlePtr);
-        delete result;
+        delete[] result;
     }
 
     emit onExternalDownloadSetRequested(publicLink, handleList);
@@ -695,26 +693,40 @@ void HTTPServer::externalFolderUploadRequest(QString &response, const HTTPReques
     }
 }
 
-void HTTPServer::externalFolderSyncRequest(QString &response, const HTTPRequest& request)
+void HTTPServer::externalFolderSyncRequest(QString& response, const HTTPRequest& request)
 {
     MegaApi::log(MegaApi::LOG_LEVEL_DEBUG, "Sync command received from the webclient");
+    const QString userString = Utilities::extractJSONString(request.data, QLatin1String("u"));
+    MegaHandle userHandle = userString.size() ?
+                                MegaApi::base64ToUserHandle(userString.toLatin1().constData()) :
+                                INVALID_HANDLE;
+
     QString targetHandle = Utilities::extractJSONString(request.data, QString::fromUtf8("h"));
     MegaHandle handle = ::mega::INVALID_HANDLE;
     if (targetHandle.size())
     {
         handle = MegaApi::base64ToHandle(targetHandle.toUtf8().constData());
     }
-
-    MegaNode *targetNode = megaApi->getNodeByHandle(handle);
-    if (!targetNode)
+    if (userHandle == ::mega::INVALID_HANDLE)
     {
-        response = QString::number(MegaError::API_ENOENT);
+        response = QString::number(MegaError::API_EARGS);
+        MegaApi::log(MegaApi::LOG_LEVEL_DEBUG,
+                     "Add sync command received from the webclient: invalid user handle");
     }
     else
     {
-        delete targetNode;
-        emit onExternalFolderSyncRequested(handle);
-        response = QString::number(MegaError::API_OK);
+        MegaHandle appUser(megaApi->getMyUserHandleBinary());
+        if (userHandle != appUser && appUser != mega::INVALID_HANDLE)
+        {
+            response = QString::number(MegaError::API_EACCESS);
+            MegaApi::log(MegaApi::LOG_LEVEL_DEBUG,
+                         "Add sync command received from the webclient: user mismatch");
+        }
+        else
+        {
+            emit onExternalFolderSyncRequested(handle);
+            response = QString::number(MegaError::API_OK);
+        }
     }
 }
 
@@ -1002,7 +1014,8 @@ HTTPServer::RequestType HTTPServer::GetRequestType(const HTTPRequest &request)
 
 QString HTTPServer::findCorrespondingAllowedOrigin(const QStringList& headers)
 {
-    for (const QString& allowedOrigin : qAsConst(Preferences::HTTPS_ALLOWED_ORIGINS))
+    const auto allowedOrigins = ServiceUrls::instance()->getHttpAllowedOrigins();
+    for (const QString& allowedOrigin: allowedOrigins)
     {
         QRegExp check = QRegExp(QString::fromUtf8("Origin: %1").arg(allowedOrigin),
                                 Qt::CaseSensitive, QRegExp::Wildcard);
