@@ -1,32 +1,52 @@
 #include "LinkProcessor.h"
-#include "Preferences.h"
-#include "MegaApplication.h"
+
 #include "CommonMessages.h"
+#include "MegaApplication.h"
+#include "Preferences.h"
+#include "RequestListenerManager.h"
+#include "ServiceUrls.h"
+
 #include <QDir>
 
 using namespace mega;
 
-LinkProcessor::LinkProcessor(const QStringList& linkList, MegaApi* megaApi, MegaApi* megaApiFolders)
-    : mMegaApi(megaApi)
-    , mMegaApiFolders(megaApiFolders)
-    , mLinkList(linkList)
-    , mImportParentFolder(mega::INVALID_HANDLE)
-    , mDelegateListener(std::make_shared<QTMegaRequestListener>(megaApi, this))
-    , mDelegateTransferListener(std::make_shared<QTMegaTransferListener>(megaApi, this))
-    , mParentHandler(nullptr)
-    , mRequestCounter(0)
-    , mCurrentIndex(0)
+LinkProcessor::LinkProcessor(MegaApi* megaApi, MegaApi* megaApiFolders)
+    : LinkProcessor(QStringList(), megaApi, megaApiFolders) // Delegate to the second constructor with an empty QStringList
 {
-    for (int i = 0; i < linkList.size(); i++)
-    {
-        mLinkObjects.append(std::make_shared<LinkInvalid>());
-    }
+}
+
+LinkProcessor::LinkProcessor(const QStringList& linkList,
+                             MegaApi* megaApi,
+                             MegaApi* megaApiFolders):
+    mMegaApi(megaApi),
+    mMegaApiFolders(megaApiFolders),
+    mLinkList(linkList),
+    mImportParentFolder(mega::INVALID_HANDLE),
+    mDownloader(std::make_shared<MegaDownloader>(megaApi, this)),
+    mCurrentIndex(0)
+{
+    resetAndSetLinkList(linkList);
+
+    // Register for SDK Request callbacks
+    mDelegateListener = RequestListenerManager::instance().registerAndGetFinishListener(this);
 }
 
 LinkProcessor::~LinkProcessor()
 {
 }
 
+void LinkProcessor::resetAndSetLinkList(const QStringList& linkList)
+{
+    mLinkObjects.clear();
+    mCurrentIndex = 0;
+    mLinkList = linkList;
+
+    for (int i = 0; i < linkList.size(); i++)
+    {
+        mLinkObjects.append(std::make_shared<LinkInvalid>());
+    }
+}
+
 QString LinkProcessor::getLink(int index) const
 {
     if (!(isValidIndex(mLinkList, index))) { return QString::fromUtf8(""); }
@@ -78,22 +98,89 @@ void LinkProcessor::continueOrFinishLinkInfoReq()
     }
 }
 
-void LinkProcessor::createInvalidLinkObject(int index, int error)
+void LinkProcessor::createInvalidLinkObject(int index, int error, const QString& name)
 {
     if (!isValidIndex(mLinkObjects, index)) { return; }
 
     mLinkObjects[index] = std::make_shared<LinkInvalid>();
 
-    if (error == MegaError::API_ETOOMANY) { return; }
+    QString newName = (name.isEmpty()) ?
+                          (QCoreApplication::translate(
+                              "MegaError",
+                              MegaError::getErrorString(error, MegaError::API_EC_IMPORT))) :
+                          name;
 
-    mLinkObjects[index]->setName(QCoreApplication::translate("MegaError",
-                                                             MegaError::getErrorString(error, MegaError::API_EC_IMPORT)));
+    mLinkObjects[index]->setName(newName);
     mLinkObjects[index]->setLinkStatus((error == MegaError::API_ETEMPUNAVAIL) ?
                                            linkStatus::WARNING :
                                            linkStatus::FAILED);
 }
 
-void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e)
+QString LinkProcessor::getReasonForExpiredLink(MegaRequest* request, MegaError* e)
+{
+    int error = e->getErrorCode();
+    long long userStatus = e->getUserStatus();
+    long long linkStatus = e->getLinkStatus();
+
+    // First query the User Status for anomalies
+    switch (userStatus)
+    {
+        case MegaError::UserErrorCode::USER_COPYRIGHT_SUSPENSION:
+            return tr("Terms of Service breach");
+
+        case MegaError::UserErrorCode::USER_ETD_SUSPENSION:
+            return tr("Link owner terminated");
+
+        case MegaError::UserErrorCode::USER_ETD_UNKNOWN:
+        default:
+            break;
+    }
+
+    // Then query the Link Status next
+    switch (linkStatus)
+    {
+        case MegaError::LinkErrorCode::LINK_DELETED_DOWN:
+            return tr("This link has been deleted");
+
+        case MegaError::LinkErrorCode::LINK_DOWN_ETD:
+            return tr("Link owner terminated");
+
+        case MegaError::LinkErrorCode::LINK_UNKNOWN:
+        case MegaError::LinkErrorCode::LINK_UNDELETED:
+        default:
+            break;
+    }
+
+    // Finally, query the Error Code
+    switch (error)
+    {
+        case MegaError::API_EEXPIRED:
+        {
+            return tr("This link has expired");
+        }
+
+        case MegaError::API_ENOENT:
+        {
+            return tr("This link has been deleted");
+        }
+
+        case MegaError::API_EBLOCKED:
+        {
+            return tr("Copyright violation");
+        }
+
+        default:
+            return tr("This link is invalid");
+    }
+}
+
+unsigned long long LinkProcessor::getAppDataId()
+{
+    auto data = TransferMetaDataContainer::createImportedLinkTransferMetaData(mDownloadPath);
+    return data->getAppId();
+}
+
+void LinkProcessor::onRequestFinish(MegaRequest* request, MegaError* e)
 {
     const int error = e->getErrorCode();
 
@@ -106,7 +193,7 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
         if (error != MegaError::API_OK)
         {
             // Invalid Link
-            createInvalidLinkObject(mCurrentIndex, error);
+            createInvalidLinkObject(mCurrentIndex, error, getReasonForExpiredLink(request, e));
         }
         else    // API_OK
         {
@@ -115,7 +202,7 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
             if (!node)
             {
                 // Invalid Link
-                createInvalidLinkObject(mCurrentIndex, error);
+                createInvalidLinkObject(mCurrentIndex, error, getReasonForExpiredLink(request, e));
             }
             else    // Valid Link
             {
@@ -147,8 +234,17 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
     }
 
     case MegaRequest::TYPE_COPY:
+    {
+        if (error != MegaError::API_OK)
+        {
+            MegaNode* node = request->getPublicMegaNode();
+            QString nodeName =
+                (node) ? QString::fromUtf8(node->getName()) : QString::fromUtf8("Invalid Node");
+            emit linkCopyErrorDetected(nodeName, error);
+        }
         processNextTransfer();
         break;
+    }
 
     case MegaRequest::TYPE_LOGIN:
     {
@@ -156,12 +252,11 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
 
         if (error == MegaError::API_OK)
         {
-            mRequestCounter++;
-            mMegaApiFolders->fetchNodes(this);
+            mMegaApiFolders->fetchNodes(mDelegateListener.get());
         }
         else
         {
-            createInvalidLinkObject(mCurrentIndex, error);
+            createInvalidLinkObject(mCurrentIndex, error, getReasonForExpiredLink(request, e));
             mCurrentIndex++;
             sendLinkInfoAvailableSignal(mCurrentIndex - 1);
             continueOrFinishLinkInfoReq();
@@ -171,7 +266,10 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
 
     case MegaRequest::TYPE_FETCH_NODES:
     {
-        if (!isValidIndex(mLinkObjects, mCurrentIndex)) { return; }
+        if (!isValidIndex(mLinkObjects, mCurrentIndex))
+        {
+            break;
+        }
 
         if (error == MegaError::API_OK)
         {
@@ -209,6 +307,11 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
                 rootNode.reset(mMegaApiFolders->getNodeByHandle(handle));
             }
 
+            if (!rootNode)
+            {
+                break;
+            }
+
             Preferences::instance()->setLastPublicHandle(request->getNodeHandle(), MegaApi::AFFILIATE_TYPE_FILE_FOLDER);
             mega::MegaNode* node = mMegaApiFolders->authorizeNode(rootNode.get());
             mLinkObjects[mCurrentIndex] = std::make_shared<LinkNode>(mMegaApi,
@@ -218,7 +321,7 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
         else
         {
             // Invalid Link
-            createInvalidLinkObject(mCurrentIndex, error);
+            createInvalidLinkObject(mCurrentIndex, error, getReasonForExpiredLink(request, e));
         }
 
         mCurrentIndex++;
@@ -231,38 +334,18 @@ void LinkProcessor::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* e
     default:
         break;
     }
-
-    mRequestCounter--;
-    markForDeletionIfNoMoreRequests();
 }
 
 void LinkProcessor::addTransfersAndStartIfNotStartedYet(LinkTransferType transferType)
 {
-    bool noTransferInProgress = mTransferQueue.isEmpty();
-
     for (int i = 0; i < mLinkObjects.size(); i++)
     {
         if (isSelected(i))
         {
             mTransferQueue.push_back({mLinkObjects[i], transferType});
+            processNextTransfer();
         }
     }
-
-    if (noTransferInProgress)
-    {
-        // Start transfers
-        processNextTransfer();
-    }
-}
-
-void LinkProcessor::markForDeletionIfNoMoreRequests()
-{
-    // If this instance of LinkProcessor does not need to wait for
-    // responses to outstanding requests, then it can be destroyed
-    if (!mParentHandler && mRequestCounter == 0)
-    {
-        deleteLater();
-    }
 }
 
 void LinkProcessor::requestLinkInfo()
@@ -270,8 +353,7 @@ void LinkProcessor::requestLinkInfo()
     if (!isValidIndex(mLinkList, mCurrentIndex)) { return; }
 
     const QString link = mLinkList[mCurrentIndex];
-    if (link.startsWith(Preferences::BASE_URL + QString::fromUtf8("/#F!")) ||
-        link.startsWith(Preferences::BASE_URL + QString::fromUtf8("/folder/")))
+    if (ServiceUrls::instance()->isFolderLink(link))
     {
         std::unique_ptr<char []> authToken(mMegaApi->getAccountAuth());
         if (authToken)
@@ -279,17 +361,14 @@ void LinkProcessor::requestLinkInfo()
             mMegaApiFolders->setAccountAuth(authToken.get());
         }
 
-        mRequestCounter++;
         mMegaApiFolders->loginToFolder(link.toUtf8().constData(), mDelegateListener.get());
     }
-    else if (link.startsWith(Preferences::BASE_URL + QString::fromUtf8("/collection/")))
+    else if (ServiceUrls::instance()->isSetLink(link))
     {
-        mRequestCounter++;
         emit requestFetchSetFromLink(link);
     }
     else
     {
-        mRequestCounter++;
         mMegaApi->getPublicNode(link.toUtf8().constData(), mDelegateListener.get());
     }
 }
@@ -301,9 +380,6 @@ void LinkProcessor::requestLinkInfo()
 // ----------------------------------------------------------------------------
 void LinkProcessor::onFetchSetFromLink(const AlbumCollection& collection)
 {
-    // We received a response to a request
-    mRequestCounter--;
-
     if (mCurrentIndex >= mLinkObjects.size()) { return; }
 
     mLinkObjects[mCurrentIndex] = std::make_shared<LinkSet>(mMegaApi, collection);
@@ -311,45 +387,6 @@ void LinkProcessor::onFetchSetFromLink(const AlbumCollection& collection)
 
     sendLinkInfoAvailableSignal(mCurrentIndex - 1);
     continueOrFinishLinkInfoReq();
-    markForDeletionIfNoMoreRequests();
-}
-
-void LinkProcessor::onSetDownloadFinished(const QString& setName,
-                                          const QStringList& succeededDownloadedElements,
-                                          const QStringList& failedDownloadedElements,
-                                          const QString& destinationPath)
-{
-    (void) setName;
-    (void) succeededDownloadedElements;
-    (void) failedDownloadedElements;
-    (void) destinationPath;
-
-    // We received a response to a request
-    mRequestCounter--;
-
-    // A public link (to a Set) has been downloaded, proceed to the next one
-    processNextTransfer();
-    markForDeletionIfNoMoreRequests();
-}
-
-void LinkProcessor::onSetImportFinished(const QString& setName,
-                                        const QStringList& succeededImportElements,
-                                        const QStringList& failedImportElements,
-                                        const QStringList& alreadyExistingImportElements,
-                                        const SetImportParams& sip)
-{
-    (void) setName;
-    (void) succeededImportElements;
-    (void) failedImportElements;
-    (void) alreadyExistingImportElements;
-    (void) sip;
-
-    // We received a response to a request
-    mRequestCounter--;
-
-    // A public link (to a Set) has been imported, proceed to the next one
-    processNextTransfer();
-    markForDeletionIfNoMoreRequests();
 }
 
 // ----------------------------------------------------------------------------
@@ -387,7 +424,6 @@ void LinkProcessor::importLinks(const QString& megaPath)
             return;
         }
 
-        mRequestCounter++;
         mMegaApi->createFolder(CommonMessages::getDefaultImportFolderName().toUtf8().constData(),
                                rootNode.get(), mDelegateListener.get());
     }
@@ -402,7 +438,7 @@ void LinkProcessor::setImportParentNode(MegaNodeSPtr importParentNode)
 {
     if (!importParentNode) { return; }
 
-    for (const auto& linkObjectPtr : mLinkObjects)
+    for (const auto& linkObjectPtr: qAsConst(mLinkObjects))
     {
         // Verify that linkObjectPtr is selected and without errors
         if (!linkObjectPtr ||
@@ -449,7 +485,6 @@ bool LinkProcessor::copyNode(MegaNodeSPtr linkNode, MegaNodeSPtr importParentNod
 
     if (!alreadyExists(linkNode, importParentNode))
     {
-        mRequestCounter++;
         mMegaApi->copyNode(linkNode.get(), importParentNode.get(), mDelegateListener.get());
         return true;
     }
@@ -474,52 +509,14 @@ MegaHandle LinkProcessor::getImportParentFolder()
 //!
 void LinkProcessor::downloadLinks(const QString& localPath)
 {
-    setDownloadPaths(localPath);
+    mDownloadPath = localPath + QDir::separator();
     addTransfersAndStartIfNotStartedYet(LinkTransferType::DOWNLOAD);
 }
 
-//!
-//! \brief LinkProcessor::setDownloadPaths
-//! \param downloadPath: download destination folder on local pc
-//! \Iterates over all LinkObjects and sets the download path for every NODE or SET:
-//! \In case of a SET, a directory with the name of the set is created.
-//!
-void LinkProcessor::setDownloadPaths(const QString& downloadPath)
-{
-    for (int i = 0; i < mLinkObjects.size(); i++)
-    {
-        if (!isSelected(i)) { continue; }
-
-        auto linkObject = mLinkObjects[i];
-
-        switch (linkObject->getLinkType())
-        {
-        case linkType::NODE:
-            linkObject->setDownloadPath(downloadPath);
-            break;
-
-        case linkType::SET:
-        {
-            auto set = std::dynamic_pointer_cast<LinkSet>(linkObject);
-            if (set)
-            {
-                QString setDownloadPath = downloadPath + QDir::separator() + set->getName();
-                set->setDownloadPath(setDownloadPath);
-            }
-            break;
-        }
-
-        default:
-            break;
-        }
-    }
-}
-
 void LinkProcessor::processNextTransfer()
 {
     if (mTransferQueue.isEmpty())
     {
-        markForDeletionIfNoMoreRequests();
         return;
     }
 
@@ -535,7 +532,15 @@ void LinkProcessor::processNextTransfer()
         {
             if (firstItem.transferType == LinkTransferType::DOWNLOAD)
             {
-                startDownload(linkNodePtr->getMegaNode(), linkNodePtr->getDownloadPath());
+                mNodesToDownload.append(WrappedNode(WrappedNode::TransferOrigin::FROM_LINK,
+                                                    linkNodePtr->getMegaNode(),
+                                                    false));
+
+                if (mTransferQueue.isEmpty())
+                {
+                    startDownload(mNodesToDownload);
+                    mNodesToDownload.clear();
+                }
                 return;
             }
             else if ((firstItem.transferType == LinkTransferType::IMPORT) &&
@@ -555,16 +560,15 @@ void LinkProcessor::processNextTransfer()
             if (firstItem.transferType == LinkTransferType::DOWNLOAD)
             {
                 // Request to put this set in preview and download all its elements
-                mRequestCounter++;
-                emit requestDownloadSet(set->getSet(),
-                                        set->getDownloadPath(),
-                                        QList<mega::MegaHandle>()); // Empty list, request all Elements
+                emit requestDownloadSet(
+                    set->getSet(),
+                    mDownloadPath,
+                    QList<mega::MegaHandle>()); // Empty list, request all Elements
                 return;
             }
             else if (firstItem.transferType == LinkTransferType::IMPORT)
             {
                 // Request to put this set in preview and import all its elements
-                mRequestCounter++;
                 emit requestImportSet(set->getSet(),
                                       {set->getImportNode()},
                                       QList<mega::MegaHandle>());
@@ -588,40 +592,25 @@ void LinkProcessor::processNextTransfer()
 //! \param localPath: download destination folder on local pc
 //! \Requests the SDK to download @linkNode to @localPath
 //!
-void LinkProcessor::startDownload(MegaNodeSPtr linkNode, const QString &localPath)
+void LinkProcessor::startDownload(const QQueue<WrappedNode>& nodes)
 {
-    if (!linkNode || localPath.isEmpty()) { return; }
+    if (nodes.isEmpty())
+    {
+        return;
+    }
 
-    const bool startFirst = false;
-    QByteArray path = (localPath + QDir::separator()).toUtf8();
-    const char* name = nullptr;
-    const char* appData = nullptr;
-    MegaCancelToken* cancelToken = nullptr; // No cancellation possible
-    const bool undelete = false;
+    MegaDownloader::DownloadInfo info;
+    info.appId = getAppDataId();
+    info.checkLocalSpace = false;
+    info.downloadQueue = nodes;
+    info.path = mDownloadPath;
+    mDownloader->processDownloadQueue(info);
 
-    mRequestCounter++;
-    mMegaApi->startDownload(linkNode.get(), path.constData(), name, appData, startFirst, cancelToken,
-                            MegaTransfer::COLLISION_CHECK_FINGERPRINT,
-                            MegaTransfer::COLLISION_RESOLUTION_NEW_WITH_N,
-                            undelete,
-                            mDelegateTransferListener.get());
-}
-
-//!
-//! \brief LinkProcessor::onTransferFinish
-//! \Callback after downloading a folder, file or set element
-//!
-void LinkProcessor::onTransferFinish(MegaApi* api, MegaTransfer *transfer, MegaError* error)
-{
-    (void) api;
-    (void) transfer;
-    (void) error;
-
-    // We received a response to a request
-    mRequestCounter--;
-
-    processNextTransfer();
-    markForDeletionIfNoMoreRequests();
+    auto appData = TransferMetaDataContainer::getAppDataById<DownloadTransferMetaData>(info.appId);
+    if (appData)
+    {
+        appData->setIsImportedLink();
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -641,18 +630,6 @@ void LinkProcessor::onLinkSelected(int index, bool selected)
     }
 }
 
-void LinkProcessor::setParentHandler(QObject *parent)
-{
-    connect(parent, &QObject::destroyed, this, [this](){
-        if(mRequestCounter == 0)
-        {
-            deleteLater();
-        }
-    });
-
-    mParentHandler = parent;
-}
-
 //!
 //! \brief LinkProcessor::refreshLinkInfo
 //! \I'm not sure why this is required, but copied this
@@ -665,6 +642,3 @@ void LinkProcessor::refreshLinkInfo()
         sendLinkInfoAvailableSignal(i);
     }
 }
-
-
-
