@@ -1,30 +1,42 @@
 #include "LoginController.h"
 
-#include "MegaApplication.h"
 #include "ConnectivityChecker.h"
+#include "FatalEventHandler.h"
+#include "MegaApplication.h"
+#include "MessageDialogOpener.h"
 #include "Platform.h"
-#include "QMegaMessageBox.h"
-
-#include "TextDecorator.h"
-#include "StatsEventHandler.h"
-
 #include "Preferences.h"
-#include "qml/QmlDialogManager.h"
+#include "QmlDialogManager.h"
+#include "RequestListenerManager.h"
+#include "ServiceUrls.h"
+#include "StatsEventHandler.h"
+#include "TextDecorator.h"
 
 #include <QQmlContext>
 
-LoginController::LoginController(QObject* parent)
-    : QObject{parent}
-      , mMegaApi(MegaSyncApp->getMegaApi())
-      , mPreferences(Preferences::instance())
-      , mDelegateListener(std::make_unique<mega::QTMegaRequestListener>(MegaSyncApp->getMegaApi(), this))
-      , mGlobalListener(std::make_unique<mega::QTMegaGlobalListener>(MegaSyncApp->getMegaApi(), this))
-      , mEmailError(false)
-      , mPasswordError(false)
-      , mProgress(0)
-      , mState(LOGGED_OUT)
-      , mNewAccount(false)
+LoginController::LoginController(QObject* parent):
+    QObject{parent},
+    mMegaApi(MegaSyncApp->getMegaApi()),
+    mPreferences(Preferences::instance()),
+    mGlobalListener(std::make_unique<mega::QTMegaGlobalListener>(MegaSyncApp->getMegaApi(), this)),
+    mEmailError(false),
+    mPasswordError(false),
+    mProgress(0),
+    mState(LOGGED_OUT),
+    mNewAccount(false),
+    mTriggerFatalErrorAfterFetchnodes(false),
+    mForceOnboarding(false),
+    mOnboardingShown(false)
 {
+    ListenerCallbacks lcInfo{
+        this,
+        std::bind(&LoginController::onRequestStart, this, std::placeholders::_1),
+        std::bind(&LoginController::onRequestFinish, this, std::placeholders::_1, std::placeholders::_2),
+        std::bind(&LoginController::onRequestUpdate, this, std::placeholders::_1)
+    };
+
+    mDelegateListener = RequestListenerManager::instance().registerAndGetListener(lcInfo);
+
     mMegaApi->addRequestListener(mDelegateListener.get());
     mMegaApi->addGlobalListener(mGlobalListener.get());
     mConnectivityTimer = new QTimer(this);
@@ -37,6 +49,26 @@ LoginController::LoginController(QObject* parent)
     {
         mMegaApi->resumeCreateAccount(credentials.sessionId.toUtf8().constData());
     }
+
+    // Connect to FatalEventHandler signals
+    auto fatalEventHandler = FatalEventHandler::instance();
+    connect(fatalEventHandler.get(),
+            &FatalEventHandler::requestForceOnBoarding,
+            this,
+            [this]()
+            {
+                mForceOnboarding = true;
+            });
+
+    // Plug into AppStates
+    connect(this,
+            &LoginController::requestAppState,
+            AppState::instance().get(),
+            &AppState::setAppState);
+    connect(AppState::instance().get(),
+            &AppState::appStateChanged,
+            this,
+            &LoginController::onAppStateChanged);
 }
 
 void LoginController::login(const QString& email, const QString& password)
@@ -89,10 +121,59 @@ LoginController::State LoginController::getState() const
 
 void LoginController::setState(State state)
 {
-    if(mState != state)
+    if ((getState() == State::FETCHING_NODES || getState() == State::FETCHING_NODES_2FA) &&
+        state == State::FETCH_NODES_FINISHED)
     {
+        mOnboardingShown = false;
+        // Should the onboarding be shown (without taking "force" into account)?
+        auto showOnboarding =
+            !mPreferences->isOneTimeActionUserDone(Preferences::ONE_TIME_ACTION_ONBOARDING_SHOWN) &&
+            !(mPreferences->isFirstBackupDone() || mPreferences->isFirstSyncDone());
+
+        if (mTriggerFatalErrorAfterFetchnodes)
+        {
+            // The user, if mTriggerFatalErrorAfterFetchnodes is set to true, is presented
+            // with a dialog enjoining them to re-create new syncs/backups, and if the user accepts,
+            // The onboarding dialog is shown. Avoid showing the warning dialog in the first
+            // place if the onboarding dialog will be shown anyway.
+            // Put app in NOMINAL state, which will not show the warning dialog,
+            // or in FATAL_ERROR state, which will show the warning dialog.
+            emit requestAppState(showOnboarding ? AppState::NOMINAL : AppState::FATAL_ERROR);
+            // Return here because we are interrupting the nominal workflow, and requesting
+            // app state change will call LoginController::setState again.
+            return;
+        }
+        else if (mForceOnboarding || showOnboarding)
+        {
+            mOnboardingShown = true;
+            QmlDialogManager::instance()->openOnboardingDialog(mForceOnboarding);
+            state = State::FETCH_NODES_FINISHED_ONBOARDING;
+            mPreferences->setOneTimeActionUserDone(Preferences::ONE_TIME_ACTION_ONBOARDING_SHOWN,
+                                                   true);
+        }
+    }
+    if (getState() != state)
+    {
+        mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_DEBUG,
+                           QString::fromUtf8("LoginController switching state from: %1 to: %2")
+                               .arg(QVariant::fromValue<State>(mState).toString(),
+                                    QVariant::fromValue<State>(state).toString())
+                               .toUtf8()
+                               .constData());
+
         mState = state;
         emit stateChanged();
+
+        if (state == State::FETCH_NODES_FINISHED)
+        {
+            onboardingFinished();
+        }
+
+        if (state == State::FETCH_NODES_FINISHED &&
+            AppState::instance()->getAppState() == AppState::RELOADING)
+        {
+            emit requestAppState(AppState::NOMINAL);
+        }
     }
 }
 
@@ -171,13 +252,14 @@ void LoginController::processOnboardingClosed()
     if(getState() == LoginController::State::FETCH_NODES_FINISHED_ONBOARDING)
     {
         if (!Preferences::instance()->isFirstSyncDone() &&
-            !Preferences::instance()->isFirstBackupDone())
+            !Preferences::instance()->isFirstBackupDone() && !mForceOnboarding)
         {
-            MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::ONBOARDING_CLOSED_WITHOUT_SETTING_SYNCS);
+            MegaSyncApp->getStatsEventHandler()->sendEvent(
+                AppStatsEvents::EventType::ONBOARDING_CLOSED_WITHOUT_SETTING_SYNCS);
         }
 
+        mForceOnboarding = false;
         setState(LoginController::State::FETCH_NODES_FINISHED);
-        onboardingFinished();
     }
 }
 
@@ -186,9 +268,8 @@ bool LoginController::isFetchNodesFinished() const
     return getState() >= LoginController::State::FETCH_NODES_FINISHED_ONBOARDING;
 }
 
-void LoginController::onRequestFinish(mega::MegaApi* api, mega::MegaRequest* request, mega::MegaError* e)
+void LoginController::onRequestFinish(mega::MegaRequest* request, mega::MegaError* e)
 {
-    Q_UNUSED(api)
     switch(request->getType())
     {
         case mega::MegaRequest::TYPE_LOGIN:
@@ -252,10 +333,8 @@ void LoginController::onRequestFinish(mega::MegaApi* api, mega::MegaRequest* req
     }
 }
 
-void LoginController::onRequestUpdate(mega::MegaApi* api, mega::MegaRequest* request)
+void LoginController::onRequestUpdate(mega::MegaRequest* request)
 {
-    Q_UNUSED(api)
-
     if (request->getType() == mega::MegaRequest::TYPE_FETCH_NODES)
     {
         if (request->getTotalBytes() > 0)
@@ -270,9 +349,8 @@ void LoginController::onRequestUpdate(mega::MegaApi* api, mega::MegaRequest* req
     }
 }
 
-void LoginController::onRequestStart(mega::MegaApi* api, mega::MegaRequest* request)
+void LoginController::onRequestStart(mega::MegaRequest* request)
 {
-    Q_UNUSED(api)
     switch(request->getType())
     {
     case mega::MegaRequest::TYPE_LOGIN:
@@ -401,9 +479,8 @@ void LoginController::onLogin(mega::MegaRequest* request, mega::MegaError* e)
 
 void LoginController::onboardingFinished()
 {
-    SyncInfo::instance()->rewriteSyncSettings(); //write sync settings into user's preferences
-
-    MegaSyncApp->onboardingFinished(false);
+    SyncInfo::instance()->onboardingFinished(mOnboardingShown);
+    MegaSyncApp->onboardingFinished(false, mOnboardingShown);
 }
 
 void LoginController::onAccountCreation(mega::MegaRequest* request, mega::MegaError* e)
@@ -491,19 +568,7 @@ void LoginController::onFetchNodes(mega::MegaRequest* request, mega::MegaError*
 
     if(e->getErrorCode() == mega::MegaError::API_OK)
     {
-        if(!mPreferences->isOneTimeActionUserDone(Preferences::ONE_TIME_ACTION_ONBOARDING_SHOWN)
-            && !(mPreferences->isFirstBackupDone() || mPreferences->isFirstSyncDone())) //Onboarding don´t has to be shown to users that
-                                                                                        //doesn´t have one_time_action_onboarding_shown
-        {                                                                               //and they have first backup or first sync done
-            QmlDialogManager::instance()->openOnboardingDialog();
-            setState(FETCH_NODES_FINISHED_ONBOARDING);
-            mPreferences->setOneTimeActionUserDone(Preferences::ONE_TIME_ACTION_ONBOARDING_SHOWN, true);
-        }
-        else
-        {
-            setState(FETCH_NODES_FINISHED);
-            onboardingFinished();
-        }
+        setState(FETCH_NODES_FINISHED);
 
         if(eventPendingStorage)
         {
@@ -570,11 +635,14 @@ void LoginController::fetchNodes(const QString& email)
            // that didn't complete a fetch nodes (i.e. does not have mPreferences logged).
            // that can happen for blocked accounts.
            // Fortunately, the SDK can help us get the email of the session
-    bool needFindingOutEmail = !mPreferences->logged() && email.isEmpty();
+    bool needFindingOutEmail = !mPreferences->logged() && email.isEmpty() &&
+                               AppState::instance()->getAppState() != AppState::RELOADING;
 
     auto loadMigrateAndFetchNodes = [this](const QString& email)
     {
-        if (!mPreferences->logged() && email.isEmpty()) // I still couldn't get the the email: won't be able to access user settings
+        if ((!mPreferences->logged() && email.isEmpty()) // I still couldn't get the the email:
+                                                         // won't be able to access user settings
+            || AppState::instance()->getAppState() == AppState::RELOADING)
         {
             mMegaApi->fetchNodes();
         }
@@ -591,26 +659,25 @@ void LoginController::fetchNodes(const QString& email)
     }
     else // we will ask the SDK the email
     {
-        mMegaApi->getUserEmail(mMegaApi->getMyUserHandleBinary(), new mega::OnFinishOneShot(mMegaApi,  this, [loadMigrateAndFetchNodes]
-                                (bool isContextValid, const mega::MegaRequest& request, const mega::MegaError& e) {
-                                    QString email;
+        auto listener = RequestListenerManager::instance().registerAndGetCustomFinishListener(
+            this,
+            [loadMigrateAndFetchNodes](mega::MegaRequest* request, mega::MegaError* e) {
+                    QString email;
 
-                                    if (e.getErrorCode() == mega::MegaError::API_OK)
-                                    {
-                                        auto emailFromRequest = request.getEmail();
-                                        if (emailFromRequest)
-                                        {
-                                            email = QString::fromUtf8(emailFromRequest);
-                                        }
-                                    }
+                    if (e->getErrorCode() == mega::MegaError::API_OK)
+                    {
+                        auto emailFromRequest = request->getEmail();
+                        if (emailFromRequest)
+                        {
+                            email = QString::fromUtf8(emailFromRequest);
+                        }
+                    }
 
-                                    // in any case, proceed:
-                                    if(isContextValid)
-                                    {
-                                        loadMigrateAndFetchNodes(email);
-                                    }
-                                  }));
+                    // in any case, proceed:
+                    loadMigrateAndFetchNodes(email);
+                });
 
+        mMegaApi->getUserEmail(mMegaApi->getMyUserHandleBinary(), listener.get());
     }
 }
 
@@ -646,31 +713,36 @@ void LoginController::migrateSyncConfToSdk(const QString& email)
 
     foreach(SyncData osd, oldCachedSyncs)
     {
-        mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_DEBUG, QString::fromUtf8("Copying sync data to SDK cache: %1. Name: %2")
-                                                                .arg(osd.mLocalFolder).arg(osd.mName).toUtf8().constData());
+        QString msg1 = QString::fromUtf8("Copying sync data to SDK cache: ") + osd.mLocalFolder
+                           + QString::fromUtf8(". Name: ") + osd.mName;
+        mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_DEBUG, msg1.toUtf8().constData());
+
+        auto listener = RequestListenerManager::instance().registerAndGetCustomFinishListener(
+            this,
+            [this, osd, &oldCacheSyncsCount, needsMigratingFromOldSession, email](mega::MegaRequest* request, mega::MegaError* e) {
+                if (e->getErrorCode() == mega::MegaError::API_OK)
+                {
+                    //preload the model with the restored configuration: that includes info that the SDK does not handle (e.g: syncID)
+                    SyncInfo::instance()->pickInfoFromOldSync(osd, request->getParentHandle(), needsMigratingFromOldSession);
+                    mPreferences->removeOldCachedSync(osd.mPos, email);
+                }
+                else
+                {
+                    QString msg2 = QString::fromUtf8("Failed to copy sync ") + osd.mLocalFolder
+                                       + QString::fromUtf8(": ") + QString::fromUtf8(e->getErrorString());
+                    mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_ERROR, msg2.toUtf8().constData());
+                }
+
+                --oldCacheSyncsCount;
+                if (oldCacheSyncsCount == 0)//All syncs copied to sdk, proceed with fetchnodes
+                {
+                    mMegaApi->fetchNodes();
+                }
+            });
 
         mMegaApi->copySyncDataToCache(osd.mLocalFolder.toUtf8().constData(), osd.mName.toUtf8().constData(),
-                                         osd.mMegaHandle, osd.mMegaFolder.toUtf8().constData(),
-                                         osd.mLocalfp, osd.mEnabled, osd.mTemporarilyDisabled,
-                                         new mega::OnFinishOneShot(mMegaApi, this, [this, osd, &oldCacheSyncsCount, needsMigratingFromOldSession, email](bool, const mega::MegaRequest& request, const mega::MegaError& e)
-                                                                       {
-                                                                           if (e.getErrorCode() == mega::MegaError::API_OK)
-                                                                           {
-                                                                               //preload the model with the restored configuration: that includes info that the SDK does not handle (e.g: syncID)
-                                                                               SyncInfo::instance()->pickInfoFromOldSync(osd, request.getParentHandle(), needsMigratingFromOldSession);
-                                                                               mPreferences->removeOldCachedSync(osd.mPos, email);
-                                                                           }
-                                                                           else
-                                                                           {
-                                                                               mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_ERROR, QString::fromUtf8("Failed to copy sync %1: %2").arg(osd.mLocalFolder).arg(QString::fromUtf8(e.getErrorString())).toUtf8().constData());
-                                                                           }
-
-                                                                           --oldCacheSyncsCount;
-                                                                           if (oldCacheSyncsCount == 0)//All syncs copied to sdk, proceed with fetchnodes
-                                                                           {
-                                                                               mMegaApi->fetchNodes();
-                                                                           }
-                                                                       }));
+                                      osd.mMegaHandle, osd.mMegaFolder.toUtf8().constData(),
+                                      osd.mLocalfp, osd.mEnabled, osd.mTemporarilyDisabled, listener.get());
     }
 
     if (oldCacheSyncsCount == 0)//No syncs to be copied to sdk, proceed with fetchnodes
@@ -708,7 +780,7 @@ void LoginController::loadSyncExclusionRules(const QString& email)
         std::vector<std::string> vExclusions;
         for (const QString& exclusion : exclusions)
         {
-            vExclusions.push_back(exclusion.toStdString());
+            vExclusions.push_back(exclusion.toUtf8().constData());
         }
         mMegaApi->setLegacyExcludedNames(&vExclusions);
     }
@@ -718,7 +790,7 @@ void LoginController::loadSyncExclusionRules(const QString& email)
         std::vector<std::string> vExclusionPaths;
         for (const QString& exclusionPath : exclusionPaths)
         {
-            vExclusionPaths.push_back(exclusionPath.toStdString());
+            vExclusionPaths.push_back(exclusionPath.toUtf8().constData());
         }
         mMegaApi->setLegacyExcludedPaths(&vExclusionPaths);
     }
@@ -761,11 +833,11 @@ void LoginController::setEmail(const QString& email)
     }
 }
 
-long long LoginController::computeExclusionSizeLimit(const long long sizeLimitValue, const int unit)
+unsigned long long LoginController::computeExclusionSizeLimit(const unsigned long long& sizeLimitValue, const int unit)
 {
     const double bytesPerKb = 1024;
     const double sizeLimitPower = pow(bytesPerKb, static_cast<double>(unit));
-    return sizeLimitValue * static_cast<long long>(sizeLimitPower);
+    return sizeLimitValue * static_cast<unsigned long long>(trunc(sizeLimitPower));
 }
 
 void LoginController::runConnectivityCheck()
@@ -819,7 +891,8 @@ void LoginController::runConnectivityCheck()
         }
     }
 
-    ConnectivityChecker *connectivityChecker = new ConnectivityChecker(Preferences::PROXY_TEST_URL);
+    ConnectivityChecker* connectivityChecker =
+        new ConnectivityChecker(ServiceUrls::getProxyTestUrl());
     connectivityChecker->setProxy(proxy);
     connectivityChecker->setTestString(Preferences::PROXY_TEST_SUBSTRING);
     connectivityChecker->setTimeout(Preferences::PROXY_TEST_TIMEOUT_MS);
@@ -847,6 +920,46 @@ void LoginController::onConnectivityCheckFinished(bool success)
     }
 }
 
+void LoginController::onAppStateChanged(AppState::AppStates oldAppState,
+                                        AppState::AppStates newAppState)
+{
+    // We are here only interrested by the RELOADING state
+    if (newAppState == AppState::RELOADING)
+    {
+        // If previous state was a FATAL ERROR, we force a fetchNodes()
+        if (oldAppState == AppState::FATAL_ERROR)
+        {
+            fetchNodes();
+        }
+        // Otherwise, the fetchnodes has been started by the SDK, and we just need to monitor it
+        else
+        {
+            setState(FETCHING_NODES);
+        }
+    }
+    // Transition to FATAL_ERROR_PENDING_FETCHNODES, which can happen when jsacd attribute has been
+    // re-generated and a warning message can be shown to the user.
+    // However the message has to be shown after fetchnodes, so we need to defer processing.
+    else if (newAppState == AppState::FATAL_ERROR_PENDING_FETCHNODES)
+    {
+        mTriggerFatalErrorAfterFetchnodes = true;
+    }
+    // Transition from FATAL_ERROR or FATAL_ERROR_PENDING_FETCHNODES to nominal:
+    // Only act when the app went through FATAL_ERROR_PENDING_FETCHNODES state (this is known by
+    // checking the value of  mTriggerFatalErrorAfterFetchnodes).
+    else if ((oldAppState == AppState::FATAL_ERROR ||
+              oldAppState == AppState::FATAL_ERROR_PENDING_FETCHNODES) &&
+             newAppState == AppState::NOMINAL)
+    {
+        if (mTriggerFatalErrorAfterFetchnodes)
+        {
+            mTriggerFatalErrorAfterFetchnodes = false;
+
+            setState(FETCH_NODES_FINISHED);
+        }
+    }
+}
+
 FastLoginController::FastLoginController(QObject *parent)
     : LoginController(parent)
 {
@@ -855,13 +968,13 @@ FastLoginController::FastLoginController(QObject *parent)
 
 bool FastLoginController::fastLogin()
 {
-    QString session = mPreferences->getSession();
-    if(session.size())
+    const auto session = mPreferences->getSession();
+    const bool hasSession = !session.isEmpty();
+    if (hasSession)
     {
         mMegaApi->fastLogin(session.toUtf8().constData());
-        return true;
     }
-    return false;
+    return hasSession;
 }
 
 void FastLoginController::onLogin(mega::MegaRequest* request, mega::MegaError* e)
@@ -885,11 +998,12 @@ void FastLoginController::onLogin(mega::MegaRequest* request, mega::MegaError* e
         else if (errorCode != mega::MegaError::API_ESID && errorCode != mega::MegaError::API_ESSL)
         //Invalid session or public key, already managed in TYPE_LOGOUT
         {
-            QMegaMessageBox::MessageBoxInfo msgInfo;
-            msgInfo.title = MegaSyncApp->getMEGAString();
-            msgInfo.text = tr("Login error: %1").arg(QCoreApplication::translate("MegaError", e->getErrorString()));
+            MessageDialogInfo msgInfo;
+            msgInfo.descriptionText =
+                tr("Login error: %1")
+                    .arg(QCoreApplication::translate("MegaError", e->getErrorString()));
 
-            QMegaMessageBox::warning(msgInfo);
+            MessageDialogOpener::warning(msgInfo);
         }
 
         //Wrong login -> logout
@@ -906,9 +1020,15 @@ void FastLoginController::onboardingFinished()
 LogoutController::LogoutController(QObject* parent)
     : QObject(parent)
       , mMegaApi(MegaSyncApp->getMegaApi())
-      , mDelegateListener(new mega::QTMegaRequestListener(MegaSyncApp->getMegaApi(), this))
       , mLoginInWithoutSession(false)
-{
+{   
+    ListenerCallbacks lcInfo{
+        this,
+        std::bind(&LogoutController::onRequestStart, this, std::placeholders::_1),
+        std::bind(&LogoutController::onRequestFinish, this, std::placeholders::_1, std::placeholders::_2)
+    };
+
+    mDelegateListener = RequestListenerManager::instance().registerAndGetListener(lcInfo);
     mMegaApi->addRequestListener(mDelegateListener.get());
 }
 
@@ -916,10 +1036,8 @@ LogoutController::~LogoutController()
 {
 }
 
-void LogoutController::onRequestFinish(mega::MegaApi* api, mega::MegaRequest* request, mega::MegaError* e)
+void LogoutController::onRequestFinish(mega::MegaRequest* request, mega::MegaError* e)
 {
-    Q_UNUSED(api)
-
     if(request->getType() == mega::MegaRequest::TYPE_LOGIN)
     {
         mLoginInWithoutSession = false;
@@ -949,49 +1067,57 @@ void LogoutController::onRequestFinish(mega::MegaApi* api, mega::MegaRequest* re
 
         if (paramType == mega::MegaError::API_ESID)
         {
-            QMegaMessageBox::MessageBoxInfo msgInfo;
-            msgInfo.title = MegaSyncApp->getMEGAString();
-            msgInfo.text = tr("You have been logged out on this computer from another location");
-            msgInfo.ignoreCloseAll = true;
+            MessageDialogInfo msgInfo;
+            msgInfo.descriptionText =
+                tr("You have been logged out on this computer from another location");
 
-            QMegaMessageBox::information(msgInfo);
+            MessageDialogOpener::information(msgInfo);
         }
         else if (paramType == mega::MegaError::API_ESSL)
         {
-            QMegaMessageBox::MessageBoxInfo msgInfo;
-            msgInfo.title = MegaSyncApp->getMEGAString();
-            msgInfo.text = tr("Our SSL key can't be verified. You could be affected by a man-in-the-middle attack or your antivirus software "
-                               "could be intercepting your communications and causing this problem. Please disable it and try again.")
-                           + QString::fromUtf8(" (Issuer: %1)").arg(QString::fromUtf8(request->getText() ? request->getText() : "Unknown"));
-            msgInfo.ignoreCloseAll = true;
+            MessageDialogInfo msgInfo;
 
-            QMegaMessageBox::critical(msgInfo);
+            msgInfo.textFormat = Qt::RichText;
+
+            msgInfo.titleText = tr("Secure connection failed");
+            msgInfo.descriptionText =
+                tr("MEGA couldn’t connect securely.[BR]If you’re on public Wi-Fi, try signing in "
+                   "through your browser first. Your ISP or security software might also be "
+                   "interfering with the connection.[BR][A]Learn more[/A]");
+            Text::RichText(ServiceUrls::getSslErrorHelpUrl().toString())
+                .process(msgInfo.descriptionText);
+
+            msgInfo.buttons = QMessageBox::Ok;
+
+            MessageDialogOpener::critical(msgInfo);
             mMegaApi->localLogout();
         }
         else if (paramType != mega::MegaError::API_EACCESS && paramType != mega::MegaError::API_EBLOCKED)
         {
-            QMegaMessageBox::MessageBoxInfo msgInfo;
-            msgInfo.title = MegaSyncApp->getMEGAString();
+            MessageDialogInfo msgInfo;
             if(errorCode != mega::MegaError::API_OK)
             {
-                msgInfo.text =tr("You have been logged out because of this error: %1").arg(QCoreApplication::translate("MegaError", e->getErrorString()));
+                msgInfo.descriptionText =
+                    tr("You have been logged out because of this error: %1")
+                        .arg(QCoreApplication::translate("MegaError", e->getErrorString()));
             }
             else
             {
-                Text::Link link(QString::fromLatin1("mailto:support@mega.nz"));
-                QString text = tr("You have been logged out. Please contact [A]support@mega.nz[/A] if this issue persists.");
+                // URL handled through translations. TODO use placeholder
+                QString text = tr("You have been logged out. Please contact [A]support@mega.nz[/A] "
+                                  "if this issue persists.");
+                Text::Link link(ServiceUrls::getSupportEmail().toString());
                 link.process(text);
-                msgInfo.text = text;
+                msgInfo.descriptionText = text;
                 msgInfo.textFormat = Qt::RichText;
             }
-            msgInfo.ignoreCloseAll = true;
 
-            QMegaMessageBox::information(msgInfo);
+            MessageDialogOpener::information(msgInfo);
         }
         MegaSyncApp->unlink();
     }
 
-           //Check for any sync disabled by logout to warn user on next login with user&password
+    // Check for any sync disabled by logout to warn user on next login with user&password
     const auto syncSettings (SyncInfo::instance()->getAllSyncSettings());
     auto isErrorLoggedOut = [](const std::shared_ptr<SyncSettings> s) {return s->getError() == mega::MegaSync::LOGGED_OUT;};
     if (std::any_of(syncSettings.cbegin(), syncSettings.cend(), isErrorLoggedOut))
@@ -1003,9 +1129,8 @@ void LogoutController::onRequestFinish(mega::MegaApi* api, mega::MegaRequest* re
     mLoginInWithoutSession = false;
 }
 
-void LogoutController::onRequestStart(mega::MegaApi *api, mega::MegaRequest *request)
+void LogoutController::onRequestStart(mega::MegaRequest *request)
 {
-    Q_UNUSED(api)
     if(request->getType() == mega::MegaRequest::TYPE_LOGIN && !request->getSessionKey())
     {
         mLoginInWithoutSession = true;
