@@ -1,19 +1,18 @@
 #ifndef PREFERENCES_H
 #define PREFERENCES_H
 
-#include <megaapi.h>
 #include "EncryptedSettings.h"
-#include "syncs/control/SyncInfo.h"
 #include "EphemeralCredentials.h"
+#include "megaapi.h"
+#include "SyncInfo.h"
 
-#include <QLocale>
-#include <QStringList>
-#include <QMutex>
 #include <QDataStream>
+#include <QLocale>
+#include <QMutex>
+#include <QStringList>
 
-#include <iostream>
-#include <assert.h>
 #include <chrono>
+#include <memory>
 #include <type_traits>
 
 Q_DECLARE_METATYPE(QList<long long>)
@@ -102,13 +101,13 @@ public:
     int bandwidthInterval();
     void setBandwidthInterval(int value);
     bool isTemporalBandwidthValid();
-    long long getMsDiffTimeWithSDK();
-    void setDsDiffTimeWithSDK(long long diffTime);
 
     long long getOverStorageDialogExecution();
     void setOverStorageDialogExecution(long long timestamp);
     long long getOverStorageNotificationExecution();
     void setOverStorageNotificationExecution(long long timestamp);
+    long long getAlmostOverStorageDialogExecution();
+    void setAlmostOverStorageDialogExecution(long long timestamp);
     long long getAlmostOverStorageNotificationExecution();
     void setAlmostOverStorageNotificationExecution(long long timestamp);
     long long getPayWallNotificationExecution();
@@ -195,6 +194,16 @@ public:
 
     //**** END OF Stalled Issues ****/
 
+    //**** Cloud Drive dialog stats ****/
+
+    qint64 cloudDriveDialogLastDateTimeOpened();
+    void cloudDriveDialogOpened();
+
+    qint64 cloudDriveDialogLastDateTimeStatSent();
+    void updateCloudDriveDialogLastDateTimeStatSent();
+
+    //**** END OF Cloud Drive dialog stats ****/
+
     void setTemporalBandwidthValid(bool value);
     long long temporalBandwidth();
     void setTemporalBandwidth(long long value);
@@ -218,10 +227,8 @@ public:
     void setUpdateAutomatically(bool value);
     bool hasDefaultUploadFolder();
     bool hasDefaultDownloadFolder();
-    bool hasDefaultImportFolder();
     void setHasDefaultUploadFolder(bool value);
     void setHasDefaultDownloadFolder(bool value);
-    void setHasDefaultImportFolder(bool value);
     bool canUpdate(QString filePath);
     int uploadLimitKB();
     int downloadLimitKB();
@@ -231,18 +238,6 @@ public:
     int parallelDownloadConnections();
     void setParallelUploadConnections(int value);
     void setParallelDownloadConnections(int value);
-    long long upperSizeLimitValue();
-    void setUpperSizeLimitValue(long long value);
-    long long lowerSizeLimitValue();
-    void setLowerSizeLimitValue(long long value);
-    bool upperSizeLimit();
-    void setUpperSizeLimit(bool value);
-    bool lowerSizeLimit();
-    void setLowerSizeLimit(bool value);
-    int upperSizeLimitUnit();
-    void setUpperSizeLimitUnit(int value);
-    int lowerSizeLimitUnit();
-    void setLowerSizeLimitUnit(int value);     
     bool cleanerDaysLimit();
     void setCleanerDaysLimit(bool value);
     int cleanerDaysLimitValue();
@@ -257,8 +252,8 @@ public:
     void setProxyProtocol(int value);
     QString proxyServer();
     void setProxyServer(const QString &value);
-    int proxyPort();
-    void setProxyPort(int value);
+    unsigned short proxyPort();
+    void setProxyPort(unsigned short value);
     bool proxyRequiresAuth();
     void setProxyRequiresAuth(bool value);
     QString getProxyUsername();
@@ -290,12 +285,16 @@ public:
     void setFatWarningShown(bool value = true);
     QString lastCustomStreamingApp();
     void setLastCustomStreamingApp(const QString &value);
-    long long getMaxMemoryUsage();
-    void setMaxMemoryUsage(long long value);
+    unsigned long long getMaxMemoryUsage();
+    void setMaxMemoryUsage(unsigned long long value);
     long long getMaxMemoryReportTime();
     void setMaxMemoryReportTime(long long timestamp);
     long long lastDailyStatTime();
     void setLastDailyStatTime(long long time);
+    long long lastSyncReminderTime();
+    void setSyncReminderTime(long long time);
+    int lastSyncReminderState();
+    void setLastSyncReminderState(int state);
 
     long long lastUpdateTime();
     void setLastUpdateTime(long long time);
@@ -304,10 +303,10 @@ public:
 
     QString downloadFolder();
     void setDownloadFolder(QString value);
-    long long uploadFolder();
-    void setUploadFolder(long long value);
-    long long importFolder();
-    void setImportFolder(long long value);
+    mega::MegaHandle uploadFolder();
+    void setUploadFolder(mega::MegaHandle value);
+    mega::MegaHandle importFolder();
+    void setImportFolder(mega::MegaHandle value);
 
     bool getImportMegaLinksEnabled();
     void setImportMegaLinksEnabled(const bool value);
@@ -331,11 +330,27 @@ public:
                                        int *cachedStorageState = nullptr, QString email = QString());
     void saveOldCachedSyncs(); //save the old cache (intended to clean them)
 
+    //**** LEGACY EXCLUSION RULES ****//
+    unsigned long long upperSizeLimitValue();
+    void setUpperSizeLimitValue(unsigned long long value);
+    unsigned long long lowerSizeLimitValue();
+    void setLowerSizeLimitValue(unsigned long long value);
+    bool upperSizeLimit();
+    void setUpperSizeLimit(bool value);
+    bool lowerSizeLimit();
+    void setLowerSizeLimit(bool value);
+    int upperSizeLimitUnit();
+    void setUpperSizeLimitUnit(int value);
+    int lowerSizeLimitUnit();
+    void setLowerSizeLimitUnit(int value);
     QStringList getExcludedSyncNames();
     QStringList getExcludedSyncPaths();
     // preloads excluded sync names and adds missing defaults ones in previous versions
     void loadExcludedSyncNames();
 
+    bool hasLegacyExclusionRules();
+    //**** END OF LEGACY EXCLUSION RULES ****/
+
     bool isOneTimeActionDone(int action);
     void setOneTimeActionDone(int action, bool done);
     void setSystemTrayPromptSuppressed(bool suppressed);
@@ -382,6 +397,8 @@ public:
 
     bool isCrashed();
     void setCrashed(bool value);
+    QString crashedUserID();
+    void setCrashedUserID(const QString& value);
     bool getGlobalPaused();
     void setGlobalPaused(bool value);
     bool getUploadsPaused();
@@ -401,6 +418,8 @@ public:
     void disableOverlayIcons(bool value);
     bool leftPaneIconsDisabled();
     void disableLeftPaneIcons(bool value);
+    bool contextMenuDisabled();
+    void disableContextMenu(bool value);
     bool error();
 
     QString getDataPath();
@@ -410,6 +429,53 @@ public:
     void clearAll();
     void sync();
 
+    void setDontShowExportLinkDialog(bool value);
+    bool getDontShowExportLinkDialog();
+
+    enum class ThemeType
+    {
+        UNINITIALIZED = -1,
+        SYSTEM_DEFAULT = 0,
+        LIGHT_THEME = 1,
+        DARK_THEME = 2,
+        LAST
+    };
+    Q_ENUM(ThemeType)
+
+    enum class ThemeAppeareance
+    {
+        UNINITIALIZED = -1,
+        DARK = 0,
+        LIGHT = 1
+    };
+    Q_ENUM(ThemeAppeareance)
+
+    // To store the different color scheme properties we can get from the system:
+    // System scheme (allowed in Windows)
+    // Apps scheme (All platforms)
+    // TODO future:
+    // Accent color
+    // High contrast
+    // ...
+    struct SystemColorScheme
+    {
+        ThemeAppeareance systemScheme = ThemeAppeareance::UNINITIALIZED;
+        ThemeAppeareance appsScheme = ThemeAppeareance::UNINITIALIZED;
+    };
+
+    void setThemeType(ThemeType theme);
+    ThemeType getThemeType();
+
+    static constexpr Preferences::ThemeAppeareance toTheme(bool dark) noexcept
+    {
+        return dark ? Preferences::ThemeAppeareance::DARK : Preferences::ThemeAppeareance::LIGHT;
+    }
+
+#if defined(ENABLE_SDK_ISOLATED_GFX)
+    void setGfxWorkerEndpointInGeneral(const QString& endpoint);
+    QString getGfxWorkerEndpointInGeneral();
+#endif
+
     enum {
         PROXY_TYPE_NONE = 0,
         PROXY_TYPE_AUTO   = 1,
@@ -428,7 +494,9 @@ public:
         GIGA_BYTE_UNIT = 3
     };
 
-    enum AccountType {
+    // These values must be kept in sync with mega::AccountType
+    enum AccountType
+    {
         ACCOUNT_TYPE_FREE = 0,
         ACCOUNT_TYPE_PROI = 1,
         ACCOUNT_TYPE_PROII = 2,
@@ -438,8 +506,9 @@ public:
         ACCOUNT_TYPE_BASIC = 12,
         ACCOUNT_TYPE_ESSENTIAL = 13,
         ACCOUNT_TYPE_BUSINESS = 100,
-        ACCOUNT_TYPE_PRO_FLEXI = 101
+        ACCOUNT_TYPE_PRO_FLEXI = 101,
     };
+    Q_ENUM(AccountType)
 
     enum {
         ONE_TIME_ACTION_DEPRECATED_OPERATING_SYSTEM = 0,
@@ -448,7 +517,9 @@ public:
         ONE_TIME_ACTION_REGISTER_UPDATE_TASK = 3,
         ONE_TIME_ACTION_OS_TOO_OLD = 4,
         ONE_TIME_ACTION_HGFS_WARNING = 5,
-        ONE_TIME_ACTION_ONBOARDING_SHOWN
+        ONE_TIME_ACTION_ONBOARDING_SHOWN = 6,
+        ONE_TIME_ACTION_UNHIDE_TRAY_ICON = 7,
+        ONE_TIME_ACTION_REQUEST_PIN_TASKBAR
     };
 
     enum {
@@ -491,50 +562,49 @@ public:
     static int FINISHED_TRANSFER_REFRESH_INTERVAL_MS;
 
     static long long MIN_UPDATE_NOTIFICATION_INTERVAL_MS;
-    static unsigned int UPDATE_INITIAL_DELAY_SECS;
-    static unsigned int UPDATE_RETRY_INTERVAL_SECS;
-    static unsigned int UPDATE_TIMEOUT_SECS;
-    static unsigned int MAX_LOGIN_TIME_MS;
+    static int UPDATE_INITIAL_DELAY_SECS;
+    static int UPDATE_RETRY_INTERVAL_SECS;
+    static int UPDATE_TIMEOUT_SECS;
+    static int MAX_LOGIN_TIME_MS;
 
     static long long MIN_REBOOT_INTERVAL_MS;
     static long long MIN_EXTERNAL_NODES_WARNING_MS;
     static long long MIN_TRANSFER_NOTIFICATION_INTERVAL_MS;
 
-    static unsigned int PROXY_TEST_TIMEOUT_MS;
+    static int PROXY_TEST_TIMEOUT_MS;
     static unsigned int MAX_IDLE_TIME_MS;
     static unsigned int MAX_COMPLETED_ITEMS;
 
     static unsigned int MUTEX_STEALER_MS; //to create a task that steals the sdk mutex for a while (how long)
     static unsigned int MUTEX_STEALER_PERIOD_MS; //periodicity (how often)
     static unsigned int MUTEX_STEALER_PERIOD_ONLY_ONCE; //if only done once
+    static int CRASH_REPORT_TIMEOUT_MS;
+
+#if defined(ENABLE_SDK_ISOLATED_GFX)
+    static unsigned int GFXWORKER_KEEPALIVE_S;
+#endif
 
     static const QString UPDATE_CHECK_URL;
-    static const QString CRASH_REPORT_URL;
     static const QString UPDATE_FOLDER_NAME;
     static const QString UPDATE_BACKUP_FOLDER_NAME;
-    static const QString PROXY_TEST_URL;
     static const QString PROXY_TEST_SUBSTRING;
     static const char UPDATE_PUBLIC_KEY[];
 
     static const char CLIENT_KEY[];
     static const QString USER_AGENT;
     static const int VERSION_CODE;
+    static const int VERSION_RC;
     static const int BUILD_ID;
     static const QString VERSION_STRING;
     static QString SDK_ID;
     static const QString CHANGELOG;
     static const QString TRANSLATION_FOLDER;
     static const QString TRANSLATION_PREFIX;
-    static const qint16 HTTP_PORT;
+    static const unsigned short HTTP_PORT;
 
-    static const QStringList HTTPS_ALLOWED_ORIGINS;
     static bool HTTPS_ORIGIN_CHECK_ENABLED;
     static const QString FINDER_EXT_BUNDLE_ID;
-    static QString BASE_URL;
-    static QString BASE_MEGA_IO_URL;
-    static QString BASE_MEGA_HELP_URL;
 
-    static void setBaseUrl(const QString &value);
     template<typename T>
     static void overridePreference(const QSettings &settings, QString &&name, T &value);
     static void overridePreferences(const QSettings &settings);
@@ -551,7 +621,6 @@ protected:
     void login(QString account);
     void logout();
 
-
     // sync related:
     void readFolders(); //read sync stored configuration
 
@@ -569,10 +638,9 @@ protected:
     template<typename T>
     T getValueConcurrent(const QString &key, const T &defaultValue);
     void setAndCachedValue(const QString &key, const QVariant &value);
-    //For these two methods, use notifyChange = true if someone needs to detect when it changes
+    //For this method, use notifyChange = true if someone needs to detect when it changes
     //Not all prefeerences need this, that´s why by default it is set to false
-    void setValueAndSyncConcurrent(const QString &key, const QVariant &value, bool notifyChange = false);
-    void setValueConcurrent(const QString &key, const QVariant &value, bool notifyChange = false);
+    void setValueConcurrently(const QString &key, const QVariant &value, bool notifyChange = false);
 
     std::unique_ptr<EncryptedSettings> mSettings;
 
@@ -591,7 +659,6 @@ protected:
     int tempBandwidthInterval;
     bool isTempBandwidthValid;
     QString mDataPath;
-    long long diffTimeWithSDK;
     std::chrono::system_clock::time_point transferOverQuotaDialogDisabledUntil;
     std::chrono::system_clock::time_point transferOverQuotaOsNotificationDisabledUntil;
     std::chrono::system_clock::time_point transferAlmostOverQuotaOsNotificationDisabledUntil;
@@ -633,6 +700,7 @@ protected:
     static const QString usedBandwidthIntervalKey;
     static const QString overStorageDialogExecutionKey;
     static const QString overStorageNotificationExecutionKey;
+    static const QString almostOverStorageDialogExecutionKey;
     static const QString almostOverStorageNotificationExecutionKey;
     static const QString payWallNotificationExecutionKey;
     static const QString almostOverStorageDismissExecutionKey;
@@ -690,7 +758,6 @@ protected:
     static const QString uploadFolderKey;
     static const QString hasDefaultUploadFolderKey;
     static const QString hasDefaultDownloadFolderKey;
-    static const QString hasDefaultImportFolderKey;
     static const QString importFolderKey;
     static const QString localFingerprintKey;
     static const QString lastExecutionTimeKey;
@@ -698,6 +765,7 @@ protected:
     static const QString excludedSyncPathsKey;
     static const QString lastVersionKey;
     static const QString isCrashedKey;
+    static const QString crashedUserIDKey;
     static const QString lastStatsRequestKey;
     static const QString wasUploadsPausedKey;
     static const QString wasDownloadsPausedKey;
@@ -709,6 +777,7 @@ protected:
     static const QString disableOverlayIconsKey;
     static const QString disableFileVersioningKey;
     static const QString disableLeftPaneIconsKey;
+    static const QString disableContextMenuKey;
     static const QString sessionKey;
     static const QString ephemeralSessionKey;
     static const QString firstStartDoneKey;
@@ -743,6 +812,12 @@ protected:
     static const QString systemTrayLastPromptTimestamp;
     static const QString lastDailyStatTimeKey;
     static const QString askOnExclusionRemove;
+    static const QString themeKey;
+    static const QString lastSyncReminderTimeKey;
+    static const QString lastSyncReminderStateKey;
+#if defined(ENABLE_SDK_ISOLATED_GFX)
+    static const QString gfxWorkerEndpointKey;
+#endif
 
     //Sleep mode
     static const QString awakeIfActiveKey;
@@ -761,25 +836,27 @@ protected:
     //Stalled issues event date
     static const QString stalledIssuesEventDateKey;
 
+    // Cloud Drive stats
+    static const QString cloudDriveDialogLastDateTimeOpenedKey;
+    static const QString cloudDriveDialogLastDateTimeStatSentKey;
+
     static const bool defaultStartOnStartup;
     static const bool defaultUpdateAutomatically;
     static const int  defaultUploadLimitKB;
     static const int  defaultDownloadLimitKB;
     static const unsigned long long defaultTransferIdentifier;
-    static const int  defaultParallelUploadConnections;
-    static const int  defaultParallelDownloadConnections;
     static const int  defaultProxyType;
     static const int  defaultProxyProtocol;
     static const long long defaultTimeStamp;
     static const QString  defaultProxyServer;
-    static const int defaultProxyPort;
+    static const unsigned short defaultProxyPort;
     static const bool defaultProxyRequiresAuth;
     static const QString defaultProxyUsername;
     static const QString defaultProxyPassword;
     static const bool defaultUpperSizeLimit;
     static const bool defaultLowerSizeLimit;
-    static const long long defaultUpperSizeLimitValue;
-    static const long long defaultLowerSizeLimitValue;
+    static const unsigned long long defaultUpperSizeLimitValue;
+    static const unsigned long long defaultLowerSizeLimitValue;
     static const int defaultUpperSizeLimitUnit;
     static const int defaultLowerSizeLimitUnit;
     static const bool defaultCleanerDaysLimit;
@@ -797,6 +874,11 @@ protected:
     static const bool defaultDownloadMegaLinksEnabled;
     static const bool defaultSystemTrayPromptSuppressed;
     static const bool defaultAskOnExclusionRemove;
+    static const int defaultLastVersion;
+
+    static const ThemeType defaultTheme;
+
+    static const QString dontShowExportLinkDialogKey;
 
 private:
     void updateFullName();
