@@ -17,20 +17,25 @@ public:
     Q_INVOKABLE void sendTrackedEvent(AppStatsEvents::EventType type,
                                       bool fromInfoDialog = false) override;
 
+    Q_INVOKABLE void sendTrackedEventArg(AppStatsEvents::EventType type,
+                                         const QStringList& args = QStringList(),
+                                         bool fromInfoDialog = false) override;
+
     void sendTrackedEvent(AppStatsEvents::EventType type,
                           const QObject* senderObj,
                           const QObject* expectedObj,
                           bool fromInfoDialog = false) override;
 
 protected:
-    void sendEvent(AppStatsEvents::EventType type,
-                   const char* message,
-                   bool addJourneyId = false,
-                   const char* viewId = nullptr) override;
+    void send(AppStatsEvents::EventType type,
+              const QString& message,
+              bool addJourneyId = false,
+              const char* viewId = nullptr) override;
 
 private:
     bool canSend() const;
-    QByteArray encodeMessage(const QString& msg) const;
+    QString encodeMessage(const QString& msg) const;
+    void updateTrackInfo(bool fromInfoDialog = false);
 
 };
 
