@@ -1,16 +1,24 @@
 #include "SetManager.h"
+
+#include "MegaApplication.h"
+#include "MegaDownloader.h"
+#include "RequestListenerManager.h"
+#include "TransferMetaData.h"
+
 #include <QDir>
 
 using namespace mega;
 
-SetManager::SetManager(MegaApi* megaApi, MegaApi* megaApiFolders)
-    : AsyncHandler()
-    , mMegaApi(megaApi)
-    , mMegaApiFolders(megaApiFolders)
-    , mDelegateListener(std::make_shared<QTMegaRequestListener>(megaApi, this))
-    , mDelegateTransferListener(std::make_shared<QTMegaTransferListener>(megaApi, this))
-    , mSetManagerState(SetManagerState::INIT)
+SetManager::SetManager(MegaApi* megaApi, MegaApi* megaApiFolders):
+    AsyncHandler(),
+    mMegaApi(megaApi),
+    mMegaApiFolders(megaApiFolders),
+    mDownloadedCounter(0),
+    mDownloader(std::make_shared<MegaDownloader>(megaApi, this)),
+    mSetManagerState(SetManagerState::INIT)
 {
+    // Register for SDK Request callbacks
+    mDelegateListener = RequestListenerManager::instance().registerAndGetFinishListener(this);
 }
 
 SetManager::~SetManager()
@@ -151,7 +159,7 @@ void SetManager::handleStates()
 
         // Get the first action in the queue
         mRequestQueue.pop(action);
-        handleStateINIT(action);
+        handleStateInit(action);
         break;
     }
 
@@ -159,7 +167,7 @@ void SetManager::handleStates()
     {
         if (processInternalQueue(action))
         {
-            handleStateWAIT_FOR_PREVIEW_SET_TO_GET_DATA(action);
+            handleStateWaitForPreviewSetToGetData(action);
         }
         break;
     }
@@ -168,7 +176,7 @@ void SetManager::handleStates()
     {
         if (processInternalQueue(action))
         {
-            handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK(action);
+            handleStateWaitForPreviewSetToDownloadFromLink(action);
         }
         break;
     }
@@ -177,7 +185,7 @@ void SetManager::handleStates()
     {
         if (processInternalQueue(action))
         {
-            handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION(action);
+            handleStateWaitForPreviewSetToDownloadCollection(action);
         }
         break;
     }
@@ -186,7 +194,7 @@ void SetManager::handleStates()
     {
         if (processInternalQueue(action))
         {
-            handleStateWAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION(action);
+            handleStateWaitForPreviewSetToImportCollection(action);
         }
         break;
     }
@@ -197,10 +205,10 @@ void SetManager::handleStates()
 }
 
 //!
-//! \brief SetManager::handleStateINIT
+//! \brief SetManager::handleStateInit
 //! \param action: Set of action parameters to handle INIT state
 //!
-void SetManager::handleStateINIT(const ActionParams& action)
+void SetManager::handleStateInit(const ActionParams& action)
 {
     if (mSetManagerState != SetManagerState::INIT) { return; }
 
@@ -230,7 +238,7 @@ void SetManager::handleStateINIT(const ActionParams& action)
         mCurrentSet.link = action.link;
 
         if (!mCurrentSet.link.isEmpty())
-        {
+        {            
             mSetManagerState = SetManagerState::WAIT_FOR_PREVIEW_SET_TO_GET_DATA;
             mMegaApi->fetchPublicSet(mCurrentSet.link.toUtf8().constData(), mDelegateListener.get());
         }
@@ -285,10 +293,10 @@ void SetManager::handleStateINIT(const ActionParams& action)
 }
 
 //!
-//! \brief SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_GET_DATA
+//! \brief SetManager::handleStateWaitForPreviewSetToGetData
 //! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_GET_DATA state
 //!
-void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_GET_DATA(const ActionParams& action)
+void SetManager::handleStateWaitForPreviewSetToGetData(const ActionParams& action)
 {
     if (mSetManagerState != SetManagerState::WAIT_FOR_PREVIEW_SET_TO_GET_DATA)
     {
@@ -325,10 +333,11 @@ void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_GET_DATA(const ActionParams&
 }
 
 //!
-//! \brief SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK
-//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK state
+//! \brief SetManager::handleStateWaitForPreviewSetToDownloadFromLink
+//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK
+//! state
 //!
-void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK(const ActionParams& action)
+void SetManager::handleStateWaitForPreviewSetToDownloadFromLink(const ActionParams& action)
 {
     if (mSetManagerState != SetManagerState::WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK)
     {
@@ -349,7 +358,7 @@ void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK(const Act
         // A new Set Element node has been fetched
         if (!mCurrentSet.nodeList.isEmpty())
         {
-            startDownload(mCurrentSet.nodeList.last().get(), mCurrentDownloadPath);
+            startDownload(mCurrentSet.nodeList, mCurrentDownloadPath);
         }
         break;
 
@@ -359,10 +368,11 @@ void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_FROM_LINK(const Act
 }
 
 //!
-//! \brief SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION
-//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION state
+//! \brief SetManager::handleStateWaitForPreviewSetToDownloadCollection
+//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION
+//! state
 //!
-void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION(const ActionParams& action)
+void SetManager::handleStateWaitForPreviewSetToDownloadCollection(const ActionParams& action)
 {
     if (mSetManagerState != SetManagerState::WAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION)
     {
@@ -378,10 +388,11 @@ void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_DOWNLOAD_COLLECTION(const Ac
 }
 
 //!
-//! \brief SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION
-//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION state
+//! \brief SetManager::handleStateWaitForPreviewSetToImportCollection
+//! \param action: Set of action parameters to handle WAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION
+//! state
 //!
-void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION(const ActionParams& action)
+void SetManager::handleStateWaitForPreviewSetToImportCollection(const ActionParams& action)
 {
     if (mSetManagerState != SetManagerState::WAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION)
     {
@@ -400,7 +411,7 @@ void SetManager::handleStateWAIT_FOR_PREVIEW_SET_TO_IMPORT_COLLECTION(const Acti
 //! \brief SetManager::onRequestFinish
 //! \Callback from SDK: response to a request
 //!
-void SetManager::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* error)
+void SetManager::onRequestFinish(MegaRequest* request, MegaError* error)
 {
     switch (request->getType())
     {
@@ -435,29 +446,17 @@ void SetManager::onRequestFinish(MegaApi*, MegaRequest* request, MegaError* erro
 //!
 void SetManager::onTransferFinish(MegaApi* api, MegaTransfer* transfer, MegaError* error)
 {
-    (void) api;
-    (void) transfer;
+    Q_UNUSED(api);
+    Q_UNUSED(transfer);
+    Q_UNUSED(error);
 
-    if (error->getErrorCode() == MegaError::API_OK)
-    {
-        mSucceededDownloadedElements.push_back(QString::fromUtf8(transfer->getFileName()));
-    }
-    else
-    {
-        mFailedDownloadedElements.push_back(QString::fromUtf8(transfer->getFileName()));
-    }
+    mDownloadedCounter++;
 
     // Check if we are waiting for more Elements to download, or if we are done
-    int nrElementsToDownload = mCurrentSet.elementHandleList.size();
-    int nrDownloadedElements = mSucceededDownloadedElements.size() + mFailedDownloadedElements.size();
-
-    if (nrDownloadedElements == nrElementsToDownload)
+    if (mDownloadedCounter == mCurrentSet.elementHandleList.size())
     {
-        // Notify observers about the successful download: pass Set name and nr downloaded Elements
-        emit onSetDownloadFinished(mCurrentSet.name,
-                                   mSucceededDownloadedElements,
-                                   mFailedDownloadedElements,
-                                   mCurrentDownloadPath);
+        // Reset counter
+        mDownloadedCounter = 0;
 
         // End preview
         mMegaApi->stopPublicSetPreview();
@@ -496,10 +495,7 @@ bool SetManager::handleFetchPublicSetResponseToDownloadCollection()
     if (!createDirectory(mCurrentDownloadPath)) { return false; }
 
     // The requested Set has been put in preview, so download of (selected) Elements can start
-    for (MegaNodeSPtr node : mCurrentSet.nodeList)
-    {
-        startDownload(node.get(), mCurrentDownloadPath);
-    }
+    startDownload(mCurrentSet.nodeList, mCurrentDownloadPath);
 
     return true;
 }
@@ -544,7 +540,7 @@ void SetManager::handleGetPreviewElementNodeResponse(MegaRequest* request, MegaE
 
     // Do not expose the raw pointer in a variable, to prevent 'double-free' vulnerability
     MegaNodeSPtr nodeSPtr(request->getPublicMegaNode());
-    mCurrentSet.nodeList.push_back(nodeSPtr);
+    mCurrentSet.nodeList.push_back(WrappedNode(WrappedNode::FROM_LINK, nodeSPtr, false));
 
     // Delegate to State Machine
     ActionParams action;
@@ -580,12 +576,13 @@ void SetManager::handleCreateFolderResponse(MegaRequest* request, MegaError* err
     }
 
     // Copy (import) all Elements to the Cloud Drive
-    for (const auto& node : mCurrentSet.nodeList)
+    for (const auto& wrappedNode: mCurrentSet.nodeList)
     {
-        if (!copyNode(node, createdNode))
+        if (!copyNode(wrappedNode.getMegaNode(), createdNode))
         {
             // The Element node already exists in the target destination
-            mAlreadyExistingImportElements.push_back(QString::fromUtf8(node->getName()));
+            mAlreadyExistingImportElements.push_back(
+                QString::fromUtf8(wrappedNode.getMegaNode()->getName()));
         }
     }
 
@@ -647,6 +644,12 @@ void SetManager::checkandHandleFinishedImport()
     }
 }
 
+unsigned long long SetManager::getAppDataId()
+{
+    auto data = TransferMetaDataContainer::createImportedLinkTransferMetaData(mCurrentDownloadPath);
+    return data->getAppId();
+}
+
 //!
 //! \brief SetManager::createDirectory
 //! \param path: full path to the folder that needs to be created.
@@ -766,21 +769,39 @@ void SetManager::resetAndHandleStates()
     handleStates();
 }
 
-void SetManager::startDownload(MegaNode* linkNode, const QString& localPath)
+void SetManager::startDownload(QQueue<WrappedNode>& nodes, const QString& localPath)
 {
-    if (!linkNode || localPath.isEmpty()) { return; }
+    if (nodes.isEmpty() || localPath.isEmpty())
+    {
+        if (nodes.isEmpty())
+        {
+            MegaApi::log(MegaApi::LOG_LEVEL_ERROR,
+                         QString::fromUtf8("Download set %1 failed. Nodes not available.")
+                             .arg(localPath)
+                             .toUtf8()
+                             .constData());
+        }
+        else if (localPath.isEmpty())
+        {
+            MegaApi::log(
+                MegaApi::LOG_LEVEL_ERROR,
+                QString::fromUtf8("Download set failed. Local path not available. Nodes number: %1")
+                    .arg(QString::number(nodes.size()))
+                    .toUtf8()
+                    .constData());
+        }
 
-    const bool startFirst = false;
-    QByteArray path = (localPath + QDir::separator()).toUtf8();
-    const char* name = nullptr;
-    const char* appData = nullptr;
-    MegaCancelToken* cancelToken = nullptr; // No cancellation possible
-    const bool undelete = false;
-    mMegaApi->startDownload(linkNode, path.constData(), name, appData, startFirst, cancelToken,
-                            MegaTransfer::COLLISION_CHECK_FINGERPRINT,
-                            MegaTransfer::COLLISION_RESOLUTION_NEW_WITH_N,
-                            undelete,
-                            mDelegateTransferListener.get());
+        return;
+    }
+
+    MegaDownloader::DownloadInfo info;
+    info.appId = getAppDataId();
+    info.checkLocalSpace = false;
+    info.downloadQueue = nodes;
+    info.path = localPath + QDir::separator();
+    mDownloader->processDownloadQueue(info);
+
+    nodes.clear();
 }
 
 //!
@@ -791,13 +812,14 @@ void SetManager::startDownload(MegaNode* linkNode, const QString& localPath)
 //! \size doesn't already exist at the destination.
 //! \Returns true if a copy/import request was made to SDK, false otherwise
 //!
-bool SetManager::copyNode(MegaNodeSPtr linkNode, MegaNodeSPtr importParentNode)
+bool SetManager::copyNode(MegaNode* linkNode, MegaNodeSPtr importParentNode)
 {
     if (!linkNode || !importParentNode) { return false; }
 
     // Returns true if a similar node to @node, with the same name and size,
     // already exists in import folder @importNode. Returns false otherwise.
-    auto alreadyExists = [&](MegaNodeSPtr node, MegaNodeSPtr importNode) -> bool {
+    auto alreadyExists = [&](MegaNode* node, MegaNodeSPtr importNode) -> bool
+    {
         std::unique_ptr<MegaNodeList> children(mMegaApi->getChildren(importNode.get()));
         const char* srcName = node->getName();
         const long long srcSize = node->getSize();
@@ -817,7 +839,7 @@ bool SetManager::copyNode(MegaNodeSPtr linkNode, MegaNodeSPtr importParentNode)
 
     if (!alreadyExists(linkNode, importParentNode))
     {
-        mMegaApi->copyNode(linkNode.get(), importParentNode.get(), mDelegateListener.get());
+        mMegaApi->copyNode(linkNode, importParentNode.get(), mDelegateListener.get());
         return true;
     }
 
