@@ -1,7 +1,7 @@
 #ifndef TEXTDECORATOR_H
 #define TEXTDECORATOR_H
 
-#include "QObject"
+#include <QObject>
 
 namespace Text
 {
@@ -43,6 +43,19 @@ class NewLine : public Decorator
 public:
     explicit NewLine(QObject* parent = nullptr);
     void process(QString& input) const override;
+    void process(QString& input, int brCount) const;
+};
+
+class RichText: public Decorator
+{
+public:
+    explicit RichText(QObject* parent = nullptr);
+    explicit RichText(const QStringList& links, QObject* parent = nullptr);
+    explicit RichText(const QString& link, QObject* parent = nullptr);
+    void process(QString& input) const override;
+
+private:
+    QStringList mLinkAddresses;
 };
 }
 
