@@ -1,4 +1,7 @@
-#pragma once
+#ifndef THREAD_POOL_H
+#define THREAD_POOL_H
+
+#include <QtGlobal>
 
 #include <condition_variable>
 #include <cstddef>
@@ -8,8 +11,6 @@
 #include <thread>
 #include <vector>
 
-#include <QtGlobal>
-
 class ThreadPool
 {
 public:
@@ -35,4 +36,4 @@ private:
     std::condition_variable mCv;
     std::mutex mMutex;
 };
-
+#endif
