@@ -1,7 +1,8 @@
 #include "UpdateTask.h"
 
-#include "Utilities.h"
 #include "Platform.h"
+#include "ServiceUrls.h"
+#include "Utilities.h"
 
 #include <QAuthenticator>
 #include <QDesktopServices>
@@ -73,9 +74,10 @@ void UpdateTask::startUpdateThread()
     connect(m_WebCtrl, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy&, QAuthenticator*)), this, SLOT(onProxyAuthenticationRequired(const QNetworkProxy&, QAuthenticator*)));
 
     string updatePublicKey = Preferences::UPDATE_PUBLIC_KEY;
-    if (getenv("MEGA_UPDATE_PUBLIC_KEY"))
+    QString updatePubKeyEnv = qEnvironmentVariable("MEGA_UPDATE_PUBLIC_KEY");
+    if (!updatePubKeyEnv.isEmpty())
     {
-        updatePublicKey = getenv("MEGA_UPDATE_PUBLIC_KEY");
+        updatePublicKey = updatePubKeyEnv.toUtf8().constData();
     }
 
     signatureChecker = new MegaHashSignature(updatePublicKey.c_str());
@@ -95,21 +97,7 @@ void UpdateTask::tryUpdate()
     running = true;
     initialCleanup();
 
-    QString randomSequence = QString::fromUtf8("?");
-    for (int i = 0; i < 10; i++)
-    {
-        randomSequence += QChar::fromLatin1(static_cast<char>('A'+ (rand() % 26)));
-    }
-
-    QString updateURL = Preferences::UPDATE_CHECK_URL;
-    if (getenv("MEGA_UPDATE_CHECK_URL"))
-    {
-        updateURL = QString::fromUtf8(getenv("MEGA_UPDATE_CHECK_URL"));
-
-        MegaApi::log(MegaApi::LOG_LEVEL_WARNING, "Using environment variable MEGA_UPDATE_CHECK_URL to fetch update file");
-    }
-
-    downloadFile(updateURL + randomSequence);
+    downloadFile(ServiceUrls::getAutoUpdateUrl());
 }
 
 void UpdateTask::onTimeout()
@@ -193,9 +181,13 @@ void UpdateTask::postponeUpdate()
     forceCheck = false;
 }
 
-void UpdateTask::downloadFile(QString url)
+void UpdateTask::downloadFile(const QUrl& url)
 {
-    MegaApi::log(MegaApi::LOG_LEVEL_DEBUG, QString::fromUtf8("Downloading updated file from %1").arg(url).toUtf8().constData());
+    MegaApi::log(MegaApi::LOG_LEVEL_DEBUG,
+                 QString::fromUtf8("Downloading updated file from %1")
+                     .arg(url.toString())
+                     .toUtf8()
+                     .constData());
 
     QNetworkRequest request(url);
     request.setAttribute(QNetworkRequest::CacheLoadControlAttribute,
@@ -232,9 +224,10 @@ bool UpdateTask::processUpdateFile(QNetworkReply *reply)
 
     updateVersion = version.toInt();
     int currentVersion = QApplication::applicationVersion().toInt();
-    if (getenv("MEGA_UPDATE_CHECK_VERSION"))
+    QString updateCheckVersionEnv = qEnvironmentVariable("MEGA_UPDATE_CHECK_VERSION");
+    if (!updateCheckVersionEnv.isEmpty())
     {
-        currentVersion = std::atoi(getenv("MEGA_UPDATE_CHECK_VERSION"));
+        currentVersion = updateCheckVersionEnv.toInt();
 
         MegaApi::log(MegaApi::LOG_LEVEL_WARNING,"Comparing version against environment variable MEGA_UPDATE_CHECK_VERSION");
     }
@@ -460,7 +453,7 @@ void UpdateTask::addToSignature(QString value)
 
 void UpdateTask::addToSignature(QByteArray bytes)
 {
-    signatureChecker->add(bytes.constData(), bytes.length());
+    signatureChecker->add(bytes.constData(), static_cast<unsigned>(bytes.length()));
 }
 
 void UpdateTask::initSignature()
@@ -499,7 +492,7 @@ bool UpdateTask::alreadyExists(QString absolutePath, QString fileSignature)
     }
 
     QByteArray bytes = file.readAll();
-    tmpHash.add(bytes.constData(), bytes.size());
+    tmpHash.add(bytes.constData(), static_cast<unsigned>(bytes.size()));
     file.close();
 
     return tmpHash.checkSignature(fileSignature.toLatin1().constData());
