@@ -3,31 +3,38 @@
 
 #include "megaapi.h"
 #include "ThreadPool.h"
-#include "QTMegaRequestListener.h"
 
-#include <QString>
-#include <QHash>
-#include <QPixmap>
-#include <QProgressDialog>
 #include <QDesktopServices>
-#include <QFuture>
 #include <QDir>
+#include <QEasingCurve>
+#include <QEvent>
+#include <QEventLoop>
+#include <QFuture>
+#include <QHash>
 #include <QIcon>
 #include <QLabel>
-#include <QQueue>
-#include <QEventLoop>
 #include <QMetaEnum>
-
-#include <QEasingCurve>
+#include <QPixmap>
+#include <QPointer>
+#include <QProgressDialog>
+#include <QQueue>
+#include <QString>
+#include <QTimer>
+#include <sys/stat.h>
 
 #include <functional>
 
-#include <sys/stat.h>
-
 #ifdef __APPLE__
-#define MEGA_SET_PERMISSIONS chmod("/Applications/MEGAsync.app/Contents/MacOS/MEGAsync", S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); \
-                             chmod("/Applications/MEGAsync.app/Contents/MacOS/MEGAupdater", S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); \
-                             chmod("/Applications/MEGAsync.app/Contents/PlugIns/MEGAShellExtFinder.appex/Contents/MacOS/MEGAShellExtFinder", S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+#define MEGA_SET_PERMISSIONS \
+chmod("/Applications/MEGAsync.app/Contents/MacOS/MEGAsync", \
+      S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); \
+chmod("/Applications/MEGAsync.app/Contents/MacOS/MEGAupdater", \
+      S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); \
+chmod("/Applications/MEGAsync.app/Contents/MacOS/mega-desktop-app-gfxworker", \
+      S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH); \
+chmod("/Applications/MEGAsync.app/Contents/PlugIns/MEGAShellExtFinder.appex/Contents/MacOS/" \
+      "MEGAShellExtFinder", \
+      S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
 #endif
 
 #define MegaSyncApp (static_cast<MegaApplication *>(QCoreApplication::instance()))
@@ -92,6 +99,33 @@ struct PSA_info
     }
 };
 
+namespace DeviceOs
+{
+Q_NAMESPACE
+
+enum Os
+{
+    UNDEFINED,
+    LINUX,
+    MAC,
+    WINDOWS
+};
+Q_ENUM_NS(Os)
+
+inline DeviceOs::Os getCurrentOS()
+{
+#ifdef Q_OS_WINDOWS
+    return DeviceOs::WINDOWS;
+#endif
+#ifdef Q_OS_MACOS
+    return DeviceOs::MAC;
+#endif
+#ifdef Q_OS_LINUX
+    return DeviceOs::LINUX;
+#endif
+}
+}
+
 class IStorageObserver
 {
 public:
@@ -282,7 +316,9 @@ protected:
 
 struct TimeInterval
 {
-    TimeInterval(long long secs, bool secondPrecision);
+    TimeInterval(long long secs, bool secondPrecision = true);
+
+    TimeInterval& operator=(const TimeInterval& other);
 
     int days;
     int hours;
@@ -291,9 +327,27 @@ struct TimeInterval
     bool useSecondPrecision;
 };
 
-class Utilities
+class Utilities : public QObject
 {
+    Q_OBJECT
+
 public:
+    enum class AttributeType
+    {
+        NONE = 0x0,
+        SMALL = 0x1,
+        MEDIUM = 0x2,
+        LARGE = 0x4,
+        THIN = 0x8,
+        REGULAR = 0x10,
+        OUTLINE = 0x20,
+        SOLID = 0x40,
+        INVERSE = 0x80
+    };
+    Q_DECLARE_FLAGS(AttributeTypes, AttributeType)
+    Q_ENUM(AttributeType)
+    Q_ENUM(AttributeTypes)
+
     enum class FileType
     {
         TYPE_OTHER    = 0x01,
@@ -304,13 +358,24 @@ public:
         TYPE_IMAGE    = 0x20,
     };
     Q_DECLARE_FLAGS(FileTypes, FileType)
-    static const QString SUPPORT_URL;
-    static const QString BACKUP_CENTER_URL;
-    static const QString SYNC_SUPPORT_URL;
 
-    static QString getSizeString(unsigned long long bytes);
+    enum class FolderType
+    {
+        TYPE_NORMAL,
+        TYPE_BACKUP_1,
+        TYPE_BACKUP_2,
+        TYPE_CAMERA_UPLOADS,
+        TYPE_CHAT,
+        TYPE_DROP,
+        TYPE_INCOMING_SHARE,
+        TYPE_OUTGOING_SHARE,
+        TYPE_REQUEST,
+        TYPE_SYNC,
+        TYPE_USERS
+    };
+
     static QString getSizeString(long long bytes);
-    static QString getSizeStringLocalized(quint64 bytes);
+    static QString getSizeStringLocalized(qint64 bytes);
     static int toNearestUnit(long long bytes);
     static QString getTranslatedSeparatorTemplate();
 
@@ -320,23 +385,21 @@ public:
         QString totalBytes;
         QString units;
     };
-    static ProgressSize getProgressSizes(unsigned long long transferredBytes, unsigned long long totalBytes);
+    static ProgressSize getProgressSizes(long long transferredBytes, long long totalBytes);
 
     static QString createSimpleUsedString(long long usedData);
+    static QString createSimpleUsedOfString(long long usedData, long long totalData);
     static QString createSimpleUsedStringWithoutReplacement(long long usedData);
     static QString createCompleteUsedString(long long usedData, long long totalData, int percentage);
-    static QString getTimeString(long long secs, bool secondPrecision = true, bool color = true);
-    static QString getQuantityString(unsigned long long quantity);
+    static QString getTimeString(long long secs, bool secondPrecision = true);
     static QString getAddedTimeString(long long secs);
     static QString extractJSONString(QString json, QString name);
     static QStringList extractJSONStringList(const QString& json, const QString& name);
     static long long extractJSONNumber(QString json, QString name);
     static QString getDefaultBasePath();
-    static void getPROurlWithParameters(QString &url);
     static QString joinLogZipFiles(mega::MegaApi *megaApi, const QDateTime *timestampSince = nullptr, QString appendHashReference = QString());
 
-    static void adjustToScreenFunc(QPoint position, QWidget *what);
-    static QString minProPlanNeeded(std::shared_ptr<mega::MegaPricing> pricing, long long usedStorage);
+    static void adjustToScreenFunc(QPoint position, QWidget* what);
     static QString getReadableStringFromTs(mega::MegaIntegerList* list);
     static QString getReadablePlanFromId(int identifier, bool shortPlan = false);
     static void animateFadeout(QWidget *object, int msecs = 700);
@@ -360,39 +423,147 @@ public:
     //get mega transfer nodepath
     static QString getNodePath(mega::MegaTransfer* transfer);
 
+    // Detect folder case sensitivity utils
+    static Qt::CaseSensitivity isCaseSensitive(const QString& folder);
+
     //Check is current account is business (either business or flexi pro)
     static bool isBusinessAccount();
     static QFuture<bool> openUrl(QUrl url);
+    static void openAppDataPath();
     static void openInMega(mega::MegaHandle handle);
     static void openBackupCenter();
 
     static QString getCommonPath(const QString& path1, const QString& path2, bool cloudPaths);
+    static const QString getPlatformProps(const QString sourceStyleSheet);
 
     static bool isIncommingShare(mega::MegaNode* node);
+    static int getNodeAccess(mega::MegaHandle handle);
+    static int getNodeAccess(mega::MegaNode* handle);
+    static QString getNodeStringAccess(mega::MegaNode* handle);
+    static QString getNodeStringAccess(mega::MegaHandle handle);
+    static QIcon getNodeAccessIcon(mega::MegaHandle handle);
+    static QIcon getNodeAccessIcon(mega::MegaNode* node);
+
+    enum HandlesType
+    {
+        FILES = 0x1,
+        FOLDERS = 0x2
+    };
+    Q_DECLARE_FLAGS(HandlesTypes, HandlesType)
+
+    static HandlesTypes getHandlesType(const QList<mega::MegaHandle>& handles);
+
+    static bool hourHasChanged(qint64 date1, qint64 date2);
 
     static bool dayHasChangedSince(qint64 msecs);
     static bool monthHasChangedSince(qint64 msecs);
 
     static QString getTranslatedError(const mega::MegaError* error);
 
+    static std::shared_ptr<mega::MegaError> removeRemoteFile(const mega::MegaNode* node);
+    static std::shared_ptr<mega::MegaError> removeSyncRemoteFile(const mega::MegaNode* node);
+    static std::shared_ptr<mega::MegaError> removeRemoteFile(const QString& path);
+    static std::shared_ptr<mega::MegaError> removeSyncRemoteFile(const QString& path);
+    static bool removeLocalFile(const QString& path, const mega::MegaHandle& syncId);
+    static bool restoreNode(mega::MegaNode* node,
+                            mega::MegaApi* megaApi,
+                            bool async,
+                            std::function<void(mega::MegaRequest*, mega::MegaError*)> finishFunc);
+
+    static bool restoreNode(const char* nodeName,
+                            const char* nodeDirectoryPath,
+                            mega::MegaApi* megaApi,
+                            std::function<void(mega::MegaRequest*, mega::MegaError*)> finishFunc);
+
+    template<class parentType>
+    static QPointer<parentType> getTopParent(QWidget* widget)
+    {
+        if (!widget)
+        {
+            return nullptr;
+        }
+
+        QWidget* parent = widget->parentWidget();
+        while (parent)
+        {
+            if (parentType* classParent = qobject_cast<parentType*>(parent))
+            {
+                return classParent;
+            }
+
+            parent = parent->parentWidget();
+        }
+
+        return nullptr;
+    }
+
+    static QPointer<QWidget> getParent(QWidget* widget, int level)
+    {
+        if (!widget || level < 0)
+        {
+            return nullptr;
+        }
+
+        int parentCounter(0);
+        QWidget* parent = widget->parentWidget();
+        while (parent)
+        {
+            if (parentCounter == level)
+            {
+                return parent;
+            }
+
+            parentCounter++;
+            parent = parent->parentWidget();
+        }
+
+        return nullptr;
+    }
+
+    static QPointer<QWidget> getParent(QWidget* widget, std::function<bool(QWidget*)> checkFunc)
+    {
+        if (!widget || !checkFunc)
+        {
+            return nullptr;
+        }
+
+        QWidget* parent = widget->parentWidget();
+        while (parent)
+        {
+            if (checkFunc(parent))
+            {
+                return parent;
+            }
+
+            parent = parent->parentWidget();
+        }
+
+        return nullptr;
+    }
+
 private:
     Utilities() {}
     static QHash<QString, QString> extensionIcons;
+    static QMap<FolderType, QString> folderIcons;
     static QHash<QString, FileType> fileTypes;
     static QHash<QString, QString> languageNames;
     static void initializeExtensions();
     static void initializeFileTypes();
-    static QString getExtensionPixmapNameSmall(QString fileName);
-    static QString getExtensionPixmapNameMedium(QString fileName);
+    static void initializeFolderTypes();
     static double toDoubleInUnit(unsigned long long bytes, unsigned long long unit);
     static QString getTimeFormat(const TimeInterval& interval);
-    static QString filledTimeString(const QString& timeFormat, const TimeInterval& interval, bool color);
+    static QString filledTimeString(const QString& timeFormat, const TimeInterval& interval);
 
     static QString cleanedTimeString(const QString& timeString);
 
-    static unsigned long long getNearestUnit(long long bytes);
+    static long long getNearestUnit(long long bytes);
+
+    // Detect folder case sensitivity utils
+    static void createFile(const QDir& path, const QString& filename);
+    static uint8_t getFileCount(QDir path);
+
+    // Platform dependent functions
 
-//Platform dependent functions
 public:
     static QString languageCodeToString(QString code);
     static QString getAvatarPath(QString email);
@@ -407,17 +578,29 @@ public:
     static qreal getDevicePixelRatio();
 
     static QIcon getCachedPixmap(QString fileName);
-    static QIcon getExtensionPixmapSmall(QString fileName);
-    static QIcon getExtensionPixmapMedium(QString fileName);
-    static QString getExtensionPixmapName(QString fileName, QString prefix);
-    static FileType getFileType(QString fileName, QString prefix);
+    static QIcon getIcon(const QString& iconName, AttributeTypes attribute);
+    static QPixmap getColoredPixmap(const QString& iconName,
+                                    AttributeTypes attributes,
+                                    const QString& token,
+                                    const QSize& size);
+    static QPixmap getPixmap(const QString& iconName, AttributeTypes attribute, const QSize& size);
+    static QString getPixmapName(const QString& iconName,
+                                 AttributeTypes attribute,
+                                 bool addResourcePrefix = true);
+    static QIcon getExtensionPixmap(QString fileName, AttributeTypes attribute);
+    static QString getExtensionPixmapName(QString fileName, AttributeTypes attribute);
+    static QString getUndecryptedPixmapName(AttributeTypes attribute);
+    static QIcon getFolderPixmap(FolderType type, AttributeTypes attribute);
+    static QString getFolderPixmapName(FolderType type, AttributeTypes attribute);
+    static void clearIconCache();
+    static FileType getFileType(QString fileName, AttributeTypes prefix);
 
     static long long getSystemsAvailableMemory();
 
     static void sleepMilliseconds(unsigned int milliseconds);
 
     // Compute the part per <ref> of <part> from <total>. Defaults to %
-    static int partPer(unsigned long long part, unsigned long long total, uint ref = 100);
+    static int partPer(long long part, long long total, uint ref = 100);
 
     static QString getFileHash(const QString& filePath);
 
@@ -429,33 +612,15 @@ public:
     static constexpr int ERROR_DISPLAY_TIME_MS = 10000; //10s in milliseconds
 
     static bool isNodeNameValid(const QString& name);
+
+    static bool shouldDisplayUpgradeButton(const bool isTransferOverquota);
+
+    static void propagateCustomEvent(QEvent::Type event);
 };
 
 Q_DECLARE_METATYPE(Utilities::FileType)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Utilities::FileTypes)
-
-template <class EnumType>
-class EnumConversions
-{
-public:
-    EnumConversions()
-        : mMetaEnum(QMetaEnum::fromType<EnumType>())
-    {
-    }
-
-    QString getString(EnumType type)
-    {
-        return QString::fromUtf8(mMetaEnum.valueToKey(static_cast<int>(type)));
-    }
-
-    EnumType  getEnum(const QString& typeAsString)
-    {
-        return static_cast<EnumType>(mMetaEnum.keyToValue(typeAsString.toStdString().c_str()));
-    }
-
-private:
-    QMetaEnum mMetaEnum;
-};
+Q_DECLARE_OPERATORS_FOR_FLAGS(Utilities::AttributeTypes)
 
 // This class encapsulates a MEGA node and adds useful information, like the origin
 // of the transfer.
@@ -463,10 +628,12 @@ class WrappedNode
 {
 public:
     // Enum used to record origin of transfer
-    enum TransferOrigin {
-        FROM_UNKNOWN   = 0,
-        FROM_APP       = 1,
+    enum TransferOrigin
+    {
+        FROM_UNKNOWN = 0,
+        FROM_APP = 1,
         FROM_WEBSERVER = 2,
+        FROM_LINK = 3,
     };
 
     // Constructor with origin and pointer to MEGA node. Default to unknown/nullptr
@@ -476,23 +643,23 @@ public:
                 mega::MegaNode* node = nullptr,
                 bool undelete = false);
 
+    WrappedNode(TransferOrigin from = WrappedNode::TransferOrigin::FROM_UNKNOWN,
+                std::shared_ptr<mega::MegaNode> node = nullptr,
+                bool undelete = false);
+
     // Destructor
-    ~WrappedNode()
-    {
-        // MEGA node should be deleted when this is deleted.
-        delete mNode;
-    }
+    ~WrappedNode() {}
 
     // Get the transfer origin
-    WrappedNode::TransferOrigin getTransferOrigin()
+    WrappedNode::TransferOrigin getTransferOrigin() const
     {
         return mTransfersFrom;
     }
 
     // Get the wrapped MEGA node pointer
-    mega::MegaNode* getMegaNode()
+    mega::MegaNode* getMegaNode() const
     {
-        return mNode;
+        return mNode.get();
     }
 
     bool getUndelete() const
@@ -505,11 +672,11 @@ private:
     WrappedNode::TransferOrigin  mTransfersFrom;
 
     // Wrapped MEGA node
-    mega::MegaNode* mNode;
+    std::shared_ptr<mega::MegaNode> mNode;
 
     bool mUndelete;
 };
 
-Q_DECLARE_METATYPE(QQueue<WrappedNode*>)
+Q_DECLARE_METATYPE(QQueue<WrappedNode>)
 
 #endif // UTILITIES_H
