@@ -2,7 +2,6 @@
 #define IMPORTLISTWIDGETITEM_H
 
 #include <QListWidgetItem>
-#include "megaapi.h"
 
 namespace Ui {
 class ImportListWidgetItem;
@@ -37,6 +36,9 @@ private:
     long long fileSize;
     QString link;
     int id;
+
+protected:
+    void resizeEvent(QResizeEvent* e) override;
 };
 
 #endif // IMPORTLISTWIDGETITEM_H
