@@ -1,10 +1,10 @@
 #ifndef MEGANODENAMES_H
 #define MEGANODENAMES_H
 
-#include <QObject>
-#include <QCoreApplication>
+#include "megaapi.h"
 
-#include <megaapi.h>
+#include <QCoreApplication>
+#include <QObject>
 
 class MegaNodeNames : public QObject
 {
@@ -30,6 +30,21 @@ public:
         return tr("Backups");
     }
 
+    static QString getRubbishName()
+    {
+        return tr("Rubbish bin");
+    }
+
+    static QString getUndecryptedFileName()
+    {
+        return tr("Undecrypted file");
+    }
+
+    static QString getUndecryptedFolderName()
+    {
+        return tr("Undecrypted folder");
+    }
+
     static QString getRootNodeName(mega::MegaNode* node)
     {
         if(node)
@@ -50,17 +65,16 @@ public:
             }
             else
             {
-                return getUndecryptedName();
+                if (node->isFile())
+                {
+                    return getUndecryptedFileName();
+                }
+                return getUndecryptedFolderName();
             }
         }
 
         return QString();
     }
-
-    static QString getUndecryptedName()
-    {
-        return QCoreApplication::translate("MegaError", "Decryption error");
-    }
 };
 
 #endif // MEGANODENAMES_H
