@@ -25,6 +25,8 @@ private:
 
 private slots:
     void permissionsChanged();
+    void on_bUpdate_clicked();
+    void on_bCancel_clicked();
 };
 
 #endif // PERMISSIONSDIALOG_H
