@@ -1,21 +1,22 @@
 #ifndef REMOTEITEMUI_H
 #define REMOTEITEMUI_H
 
-#include <QWidget>
+#include "megaapi.h"
+
 #include <QTableView>
+#include <QWidget>
 
-#include <megaapi.h>
-
-namespace Ui {
+namespace Ui
+{
 class RemoteItemUi;
 }
 
-class RemoteItemUi : public QWidget
+class RemoteItemUi: public QWidget
 {
     Q_OBJECT
 
 public:
-    explicit RemoteItemUi(QWidget *parent = nullptr);
+    explicit RemoteItemUi(QWidget* parent = nullptr);
     ~RemoteItemUi();
 
     void setTitle(const QString& title);
@@ -27,20 +28,19 @@ public:
     void setAddButtonEnabled(bool enabled);
 
 signals:
-    void addClicked(mega::MegaHandle);
-    void deleteClicked();
+    void addClicked(mega::MegaHandle = mega::INVALID_HANDLE);
 
 #ifndef Q_OS_WIN
     void permissionsClicked();
 #endif
 
 protected:
-    void changeEvent(QEvent* event) override;
+    bool event(QEvent* event) override;
 
 private:
     void setTableViewProperties(QTableView* view) const;
 
-    Ui::RemoteItemUi *ui;
+    Ui::RemoteItemUi* ui;
 };
 
 #endif // REMOTEITEMUI_H
