@@ -1,7 +1,7 @@
 #ifndef ADDEXCLUSIONRULE_H
 #define ADDEXCLUSIONRULE_H
 
-#include "qml/QmlDialogWrapper.h"
+#include "QmlDialogWrapper.h"
 
 class AddExclusionRule : public QMLComponent
 {
@@ -12,8 +12,6 @@ public:
 
     QUrl getQmlUrl() override;
 
-    QString contextName() override;
-
     Q_INVOKABLE void appendRuleToFolders(int targetType, int wildCard, QString ruleValue);
 
 private:
