@@ -1,9 +1,9 @@
 #ifndef BACKUPSCONTROLLER_H
 #define BACKUPSCONTROLLER_H
 
-#include "syncs/control/SyncController.h"
+#include "SyncController.h"
 
-class BackupsController : public QObject
+class BackupsController : public SyncController
 {
     Q_OBJECT
 
@@ -11,21 +11,30 @@ public:
     typedef QPair<QString, QString> BackupInfo;
     typedef QList<BackupInfo> BackupInfoList;
 
-    BackupsController(QObject *parent = 0);
+    static BackupsController& instance()
+    {
+        static BackupsController instance;
+        return instance;
+    }
 
-    void addBackups(const BackupInfoList& localPathList, SyncInfo::SyncOrigin origin = SyncInfo::SyncOrigin::NONE);
+    BackupsController(const BackupsController&) = delete;
+    BackupsController& operator=(const BackupsController&) = delete;
+
+    void addBackups(const BackupInfoList& localPathList,
+                    SyncInfo::SyncOrigin origin = SyncInfo::SyncOrigin::NONE);
 
     QSet<QString> getRemoteFolders() const;
 
-    static QString getErrorString(int errorCode, int syncErrorCode);
+    QString getErrorString(int errorCode, int syncErrorCode) const;
 
 signals:
     void backupFinished(const QString& folder, int errorCode, int syncErrorCode);
     void backupsCreationFinished(bool success);
 
 private:
+    BackupsController(QObject *parent = 0);
+
     mega::MegaApi* mMegaApi;
-    SyncController* mBackupController;
     int mBackupsToDoSize;
     int mBackupsProcessedWithError;
 
@@ -34,6 +43,8 @@ private:
     SyncInfo::SyncOrigin mBackupsOrigin;
 
     bool existsName(const QString& name) const;
+    bool hasBackupsWithErrors() const;
+    void showErrorMessage() const;
 
 private slots:
     void onBackupAddRequestStatus(int errorCode, int syncErrorCode, QString name);
