@@ -1,7 +1,7 @@
 #ifndef GUESTCONTENT_H
 #define GUESTCONTENT_H
 
-#include "qml/QmlDialogWrapper.h"
+#include "QmlDialogWrapper.h"
 
 class GuestContent : public QMLComponent
 {
@@ -12,8 +12,6 @@ public:
 
     QUrl getQmlUrl() override;
 
-    QString contextName() override;
-
     Q_INVOKABLE void onAboutMEGAClicked();
     Q_INVOKABLE void onPreferencesClicked();
     Q_INVOKABLE void onExitClicked();
