@@ -1,7 +1,7 @@
 #ifndef ONBOARDINGQMLDIALOG_H
 #define ONBOARDINGQMLDIALOG_H
 
-#include "qml/QmlDialog.h"
+#include "QmlDialog.h"
 
 class OnboardingQmlDialog : public QmlDialog
 {
@@ -36,6 +36,7 @@ private:
     bool mForceClose;
     bool mCreatingAccount;
 
+    void requestPinOnTaskbar() const;
 };
 
 #endif // ONBOARDINGQMLDIALOG_H
