@@ -1,33 +1,34 @@
-#ifndef QMLDIALOGMANAGER_H
-#define QMLDIALOGMANAGER_H
+#ifndef QML_DIALOG_MANAGER_H
+#define QML_DIALOG_MANAGER_H
 
+#include <QQmlComponent>
 #include <QTimer>
 
 #include <memory>
 
-class QmlDialogManager
+class QmlDialogManager: public QObject
 {
+    Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
 public:
+    static QmlDialogManager* getQmlInstance(QQmlEngine* qmlEngine, QJSEngine* jsEngine);
     static std::shared_ptr<QmlDialogManager> instance();
 
     QmlDialogManager(const QmlDialogManager&) = delete;
     QmlDialogManager& operator=(const QmlDialogManager&) = delete;
 
     void openGuestDialog();
-    bool openOnboardingDialog();
-    bool openWhatsNewDialog();
-    void openBackupsDialog(bool fromSettings = false);
+    bool openOnboardingDialog(bool force = false);
 
     bool raiseGuestDialog();
     void raiseOnboardingDialog();
-
     void raiseOrHideInfoGuestDialog(QTimer* dialogTimer, int msec);
-
     void forceCloseOnboardingDialog();
 
 private:
-    QmlDialogManager();
-
+    explicit QmlDialogManager(QObject* parent = nullptr) {}
 };
 
-#endif // QMLDIALOGMANAGER_H
+#endif // QML_DIALOG_MANAGER_H
