@@ -1,32 +1,38 @@
+#ifndef DEBUG
+#include "CrashHandler.h"
+#endif
+
 #include "MegaApplication.h"
-#include "gui/MegaProxyStyle.h"
-#include "platform/Platform.h"
-#include "qtlockedfile/qtlockedfile.h"
-#include "ScaleFactorManager.h"
+#include "MegaProxyStyle.h"
+#include "Platform.h"
 #include "PowerOptions.h"
 #include "ProxyStatsEventHandler.h"
-#include "StatsEventHandler.h"
-#include "CrashHandler.h"
+#include "qtlockedfile/qtlockedfile.h"
 
 #include <QFontDatabase>
-#include <assert.h>
+
+#include <cassert>
+#include <iostream>
 
 #ifdef Q_OS_LINUX
-    #include <signal.h>
-    #include <condition_variable>
+#include <condition_variable>
+#include <signal.h>
 #endif
 
 #ifndef WIN32
-//sleep
 #include <unistd.h>
 #else
-#include <Windows.h>
 #include <Psapi.h>
-#include <Strsafe.h>
 #include <Shellapi.h>
+#include <Strsafe.h>
+#include <Windows.h>
+
+#include <winrt/base.h>
 #endif
 
 #if defined(WIN32) || defined(Q_OS_LINUX)
+#include "ScaleFactorManager.h"
+
 #include <QScreen>
 #endif
 
@@ -68,11 +74,12 @@ void msgHandler(QtMsgType type, const char *msg)
 }
 
 #ifdef Q_OS_LINUX
-MegaApplication *theapp = NULL;
-bool waitForRestartSignal = false;
+MegaApplication* theapp = NULL;
 std::mutex mtxcondvar;
 std::condition_variable condVarRestart;
-QString appToWaitForSignal;
+QString appExecPath;
+QStringList appArgs;
+const QString waitForSignalArg = QLatin1String("--waitforsignal");
 
 void LinuxSignalHandler(int signum)
 {
@@ -83,13 +90,12 @@ void LinuxSignalHandler(int signum)
     }
     else if (signum == SIGUSR1)
     {
-        waitForRestartSignal = true;
-        if (waitForRestartSignal)
-        {
-            appToWaitForSignal.append(QString::fromUtf8(" --waitforsignal"));
-            bool success = QProcess::startDetached(appToWaitForSignal);
-            cout << "Started detached MEGAsync to wait for restart signal: " << appToWaitForSignal.toUtf8().constData() << " " << (success?"OK":"FAILED!") << endl;
-        }
+        appArgs << waitForSignalArg;
+        bool success = QProcess::startDetached(appExecPath, appArgs);
+        cout << "Started detached MEGAsync to wait for restart signal: "
+             << appExecPath.toUtf8().constData() << " "
+             << appArgs.join(QLatin1Char(' ')).toUtf8().constData() << " "
+             << (success ? "OK" : "FAILED!") << endl;
 
         if (theapp)
         {
@@ -191,6 +197,69 @@ void freeStaticResources()
     Platform::destroy();
 }
 
+#if defined(WIN32) || defined(Q_OS_LINUX)
+void sendUninstallEvent(std::shared_ptr<Preferences> preferences)
+{
+    if (preferences->installationTime() != -1)
+    {
+        auto megaApi = std::make_unique<MegaApi>(Preferences::CLIENT_KEY,
+                                                 (char *)NULL,
+                                                 Preferences::USER_AGENT.toUtf8().constData());
+        auto statsEventHandler = std::make_unique<ProxyStatsEventHandler>(megaApi.get());
+        statsEventHandler->sendEvent(AppStatsEvents::EventType::UNINSTALL,
+                                     { QString::number(preferences->installationTime()),
+                                       QString::number(preferences->accountCreationTime()),
+                                       QString::number(preferences->hasLoggedIn()) },
+                                     true);
+        QThread::msleep(500);
+    }
+}
+#endif
+
+void uninstall()
+{
+    auto preferences = Preferences::instance();
+    preferences->initialize(MegaApplication::applicationDataPath());
+    if (!preferences->error())
+    {
+        if (preferences->logged())
+        {
+            preferences->unlink();
+        }
+
+        for (int i = 0; i < preferences->getNumUsers(); i++)
+        {
+            preferences->enterUser(i);
+
+            // we do first for old sync configuration (in case there were remaining for some user)
+            QList<SyncData> syncData = preferences->readOldCachedSyncs();
+            foreach(SyncData osd, syncData)
+            {
+                removeSyncData(osd.mLocalFolder, osd.mName.remove(QLatin1Char(':')), osd.mSyncID);
+            }
+
+            // now for the new syncs cached configurations
+            auto loadedSyncs = preferences->getLoadedSyncsMap();
+            for (auto it = loadedSyncs.begin(); it != loadedSyncs.end(); it++)
+            {
+                removeSyncData(it.value()->getLocalFolder(), it.value()->name(true), it.value()->getSyncID());
+            }
+
+            preferences->leaveUser();
+        }
+    }
+
+    QDir dir(MegaApplication::applicationDataPath());
+    dir.removeRecursively();
+    Platform::getInstance()->uninstall();
+
+#ifdef WIN32
+    sendUninstallEvent(preferences);
+#endif
+
+    freeStaticResources();
+}
+
 void addFonts()
 {
 #if !defined(__APPLE__) && !defined (_WIN32)
@@ -207,6 +276,35 @@ void addFonts()
     QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Lato-Bold.ttf"));
     QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Lato-Regular.ttf"));
     QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Lato-Semibold.ttf"));
+
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Black.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-BlackItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Bold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-BoldItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-ExtraBold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-ExtraBoldItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-ExtraLight.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-ExtraLightItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Italic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Light.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-LightItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Medium.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-MediumItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Regular.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-SemiBold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-SemiBoldItalic.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-Thin.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Poppins-ThinItalic.ttf"));
+
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Bold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Black.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-ExtraBold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-ExtraLight.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Light.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Medium.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Regular.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-SemiBold.ttf"));
+    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Thin.ttf"));
 }
 
 int main(int argc, char *argv[])
@@ -226,58 +324,18 @@ int main(int argc, char *argv[])
 
     if ((argc == 2) && !strcmp("/uninstall", argv[1]))
     {
-        auto preferences = Preferences::instance();
-        preferences->initialize(MegaApplication::applicationDataPath());
-        if (!preferences->error())
-        {
-            if (preferences->logged())
-            {
-                preferences->unlink();
-            }
-
-            for (int i = 0; i < preferences->getNumUsers(); i++)
-            {
-                preferences->enterUser(i);
-
-                // we do first for old sync configuration (in case there were remaining for some user)
-                QList<SyncData> syncData = preferences->readOldCachedSyncs();
-                foreach(SyncData osd, syncData)
-                {
-                    removeSyncData(osd.mLocalFolder, osd.mName.remove(QLatin1Char(':')), osd.mSyncID);
-                }
-
-                // now for the new syncs cached configurations
-                auto loadedSyncs = preferences->getLoadedSyncsMap();
-                for (auto it = loadedSyncs.begin(); it != loadedSyncs.end(); it++)
-                {
-                    removeSyncData(it.value()->getLocalFolder(), it.value()->name(true), it.value()->getSyncID());
-                }
-
-                preferences->leaveUser();
-            }
-        }
-
-        QDir dir(MegaApplication::applicationDataPath());
-        dir.removeRecursively();
-        Platform::getInstance()->uninstall();
-
-#ifdef WIN32
-        if (preferences->installationTime() != -1)
-        {
-            MegaApi *megaApi = new MegaApi(Preferences::CLIENT_KEY, (char *)NULL,
-                                           Preferences::USER_AGENT.toUtf8().constData());
-            StatsEventHandler* statsEventHandler = new ProxyStatsEventHandler(megaApi);
-            statsEventHandler->sendEvent(AppStatsEvents::EventType::UNINSTALL_STATS,
-                                         { QString::number(preferences->installationTime()),
-                                           QString::number(preferences->accountCreationTime()),
-                                           QString::number(preferences->hasLoggedIn()) },
-                                         true);
-            Sleep(5000);
-        }
-#endif
-        freeStaticResources();
+        uninstall();
         return 0;
     }
+#ifdef Q_OS_LINUX
+    else if((argc == 2) && !strcmp("--send-uninstall-event", argv[1]))
+    {
+        auto preferences = Preferences::instance();
+        preferences->initialize(MegaApplication::applicationDataPath());
+        sendUninstallEvent(preferences);
+        return 0;
+    }
+#endif
 
 #ifdef Q_OS_LINUX
 
@@ -293,40 +351,37 @@ int main(int argc, char *argv[])
         cerr << " Failed to register signal SIGUSR1 " << endl;
     }
 
-    for (int i = 1; i < argc ; i++)
+    // Get app parameters
+    for (int i = 0; i < argc; ++i)
     {
-        if (!strcmp(argv[i],"--waitforsignal"))
+        appArgs << QString::fromUtf8(argv[i]);
+    }
+    appExecPath = appArgs.takeFirst();
+
+    // Process "waitforsignal" if passed
+    if (appArgs.contains(waitForSignalArg))
+    {
+        std::unique_lock<std::mutex> lock(mtxcondvar);
+        if (signal(SIGUSR2, LinuxSignalHandler))
         {
-            std::unique_lock<std::mutex> lock(mtxcondvar);
-            if (signal(SIGUSR2, LinuxSignalHandler))
-            {
-                cerr << " Failed to register signal SIGUSR2 " << endl;
-            }
-
-            cout << "Waiting for signal to restart MEGAsync ... "<< endl;
-            if (condVarRestart.wait_for(lock, std::chrono::minutes(30)) == std::cv_status::no_timeout )
-            {
-                QString app;
-
-                for (int j = 0; j < argc; j++)
-                {
-                    if (strcmp(argv[j],"--waitforsignal"))
-                    {
-                        app.append(QString::fromUtf8(" \""));
-                        app.append(QString::fromUtf8(argv[j]));
-                        app.append(QString::fromUtf8("\""));
-                    }
-                }
-
-                bool success = QProcess::startDetached(app);
-                cout << "Restarting MEGAsync: " << app.toUtf8().constData() << " " << (success?"OK":"FAILED!") << endl;
-                freeStaticResources();
-                exit(!success);
-            }
-            cout << "Timed out waiting for restart signal" << endl;
-            freeStaticResources();
-            exit(2);
+            cerr << " Failed to register signal SIGUSR2 " << endl;
         }
+
+        cout << "Waiting for signal to restart MEGAsync ... " << endl;
+        if (condVarRestart.wait_for(lock, std::chrono::minutes(30)) == std::cv_status::no_timeout)
+        {
+            appArgs.removeAll(waitForSignalArg);
+            bool success = QProcess::startDetached(appExecPath, appArgs);
+            cout << "Restarting MEGAsync: " << appExecPath.toUtf8().constData() << " "
+                 << appArgs.join(QLatin1Char(' ')).toUtf8().constData() << " "
+                 << (success ? "OK" : "FAILED!") << endl;
+            freeStaticResources();
+            exit(!success);
+        }
+
+        cout << "Timed out waiting for restart signal" << endl;
+        freeStaticResources();
+        exit(2);
     }
 
     // Block SIGUSR2 for normal execution: we don't want it to kill the process, in case there's a rogue update going on.
@@ -340,11 +395,12 @@ int main(int argc, char *argv[])
     QSslSocket::supportsSsl();
 
 #ifndef Q_OS_MACX
-   const auto autoScreenScaleFactor = getenv("QT_AUTO_SCREEN_SCALE_FACTOR");
-   const bool autoScreenScaleFactorDisabled{autoScreenScaleFactor && autoScreenScaleFactor == std::string("0")};
-   if(autoScreenScaleFactorDisabled)
-   {
-       logMessages.emplace_back(MegaApi::LOG_LEVEL_DEBUG, QStringLiteral("auto screen scale factor disabled because of QT_AUTO_SCREEN_SCALE_FACTOR set to 0"));
+    const QString autoScreenScaleFactor = qEnvironmentVariable("QT_AUTO_SCREEN_SCALE_FACTOR");
+    if (autoScreenScaleFactor == QString::fromUtf8("0"))
+    {
+        logMessages.emplace_back(MegaApi::LOG_LEVEL_DEBUG,
+                                 QStringLiteral("auto screen scale factor disabled because of "
+                                                "QT_AUTO_SCREEN_SCALE_FACTOR set to 0"));
    }
    else
    {
@@ -368,7 +424,7 @@ int main(int argc, char *argv[])
 #endif
 
 #if defined(Q_OS_LINUX)
-    if (!(getenv("DO_NOT_SET_QT_PLUGIN_PATH")))
+    if (!qEnvironmentVariableIsSet("DO_NOT_SET_QT_PLUGIN_PATH"))
     {
         if (QDir(QString::fromUtf8("/opt/mega/plugins")).exists())
         {
@@ -386,32 +442,40 @@ int main(int argc, char *argv[])
     ScaleFactorManager scaleFactorManager(OsType::LINUX);
 #endif
 
-    try {
-        scaleFactorManager.setScaleFactorEnvironmentVariable();
-    } catch (const std::exception& exception)
+    try
     {
-        const QString errorMessage{QString::fromStdString("Error while setting scale factor environment variable: "+
-                    std::string(exception.what()))};
+        scaleFactorManager.setScaleFactorEnvironmentVariable();
+    }
+    catch (const std::exception& exception)
+    {
+        const QString errorMessage{
+            QString::fromStdString("Error while setting scale factor environment variable: " +
+                                   std::string(exception.what()))};
         logMessages.emplace_back(MegaApi::LOG_LEVEL_DEBUG, errorMessage);
     }
 #endif
 
 #if defined(Q_OS_LINUX)
-    if (!(getenv("DO_NOT_UNSET_QT_QPA_PLATFORMTHEME")) && getenv("QT_QPA_PLATFORMTHEME"))
+    if (!qEnvironmentVariableIsSet("DO_NOT_UNSET_QT_QPA_PLATFORMTHEME") &&
+        qEnvironmentVariableIsSet("QT_QPA_PLATFORMTHEME"))
     {
-        if (!unsetenv("QT_QPA_PLATFORMTHEME")) //open folder dialog & similar crashes is fixed with this
+        if (qunsetenv("QT_QPA_PLATFORMTHEME") !=
+            0) // open folder dialog & similar crashes is fixed with this
         {
-            std::cerr <<  "Error unsetting QT_QPA_PLATFORMTHEME vble" << std::endl;
+            std::cerr << "Error unsetting QT_QPA_PLATFORMTHEME vble. errno=" << errno << std::endl;
         }
     }
-    if (!(getenv("DO_NOT_UNSET_SHLVL")) && getenv("SHLVL"))
+
+    if (!qEnvironmentVariableIsSet("DO_NOT_UNSET_SHLVL") && qEnvironmentVariableIsSet("SHLVL"))
     {
-        if (!unsetenv("SHLVL")) // reported failure in mint
+        if (qunsetenv("SHLVL") != 0) // reported failure in mint
         {
-            //std::cerr <<  "Error unsetting SHLVL vble" << std::endl; //Fedora fails to unset this env var ... too verbose error
+            // std::cerr <<  "Error unsetting SHLVL vble" << std::endl; // Fedora fails to unset
+            // this env var ... too verbose error
         }
     }
-    if (!(getenv("DO_NOT_SET_DESKTOP_SETTINGS_UNAWARE")))
+
+    if (!qEnvironmentVariableIsSet("DO_NOT_SET_DESKTOP_SETTINGS_UNAWARE"))
     {
         QApplication::setDesktopSettingsAware(false);
     }
@@ -420,23 +484,16 @@ int main(int argc, char *argv[])
     MegaApplication app(argc, argv);
 #if defined(Q_OS_LINUX)
     theapp = &app;
-    appToWaitForSignal = QString::fromUtf8("\"%1\"").arg(MegaApplication::applicationFilePath());
-    for (int i = 1; i < argc; i++)
-    {
-        appToWaitForSignal.append(QString::fromUtf8(" \""));
-        appToWaitForSignal.append(QString::fromUtf8(argv[i]));
-        appToWaitForSignal.append(QString::fromUtf8("\""));
-    }
 #endif
 
     for(const auto &message : logMessages)
     {
-        MegaApi::log(message.logLevel, message.message.toStdString().c_str());
+        MegaApi::log(message.logLevel, message.message.toUtf8().constData());
     }
 
 #ifdef Q_OS_LINUX
-    auto megaLibGL = getenv("MEGA_LIBGL_ALWAYS_SOFTWARE");
-    if (megaLibGL && !strcmp(megaLibGL, "1"))
+    QByteArray megaLibGL = qgetenv("MEGA_LIBGL_ALWAYS_SOFTWARE");
+    if (!megaLibGL.isEmpty() && megaLibGL == "1")
     {
         MegaApi::log(MegaApi::LOG_LEVEL_INFO, "Setting LIBGL_ALWAYS_SOFTWARE to 1");
         qputenv("LIBGL_ALWAYS_SOFTWARE", "1");
@@ -444,26 +501,24 @@ int main(int argc, char *argv[])
 #endif
 
 #ifndef Q_OS_MACX
-    const auto scaleFactorLogMessages = scaleFactorManager.getLogMessages();
-    for(const auto& message : scaleFactorLogMessages)
+    const QVector<QString> scaleFactorLogMessages = scaleFactorManager.getLogMessages();
+    for (const QString& message: scaleFactorLogMessages)
     {
-        MegaApi::log(MegaApi::LOG_LEVEL_DEBUG, message.c_str());
+        MegaApi::log(MegaApi::LOG_LEVEL_DEBUG, message.toUtf8().constData());
     }
 #endif
 
 #if defined(Q_OS_LINUX)
     for (const auto& screen : app.screens())
     {
-        MegaApi::log(MegaApi::LOG_LEVEL_INFO, ("Device pixel ratio on '" +
-                                               screen->name().toStdString() + "': " +
-                                               std::to_string(screen->devicePixelRatio())).c_str());
+        QString msg = QString::fromUtf8("Device pixel ratio on '") + screen->name() +
+                      QString::fromUtf8("': ") + QString::number(screen->devicePixelRatio());
+        MegaApi::log(MegaApi::LOG_LEVEL_INFO, msg.toUtf8().constData());
     }
 #endif
 
     qInstallMessageHandler(messageHandler);
 
-    app.setStyle(new MegaProxyStyle());
-
 #ifdef Q_OS_MACX
 
     auto current = QOperatingSystemVersion::current();
@@ -500,11 +555,11 @@ int main(int argc, char *argv[])
     {
         avatarsDir.mkpath(QString::fromUtf8("."));
     }
-
+#ifdef USE_BREAKPAD
 #ifndef DEBUG
-    CrashHandler::instance()->Init(QDir::toNativeSeparators(crashPath));
+    CrashHandler::instance()->init(QDir::toNativeSeparators(crashPath));
+#endif
 #endif
-
     QtLockedFile singleInstanceChecker(appLockPath);
     bool alreadyStarted = true;
     for (int i = 0; i < 10; i++)
@@ -547,7 +602,7 @@ int main(int argc, char *argv[])
             QFile fappVersionPath(appVersionPath);
             if (!fappVersionPath.exists())
             {
-                QProcess::startDetached(QString::fromUtf8("/bin/bash -c \"lsof ~/Library/Application\\ Support/Mega\\ Limited/MEGAsync/megasync.lock 2>/dev/null | grep MEGAclien | cut -d' ' -f2 | xargs kill\""));
+                QProcess::startDetached(QString::fromUtf8("/bin/bash -c \"lsof ~/Library/Application\\ Support/Mega\\ Limited/MEGAsync/megasync.lock 2>/dev/null | grep MEGAclien | cut -d' ' -f2 | xargs kill\""), {});
             }
         }
 #endif
@@ -579,18 +634,7 @@ int main(int argc, char *argv[])
 
     addFonts();
 
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Bold.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Black.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-ExtraBold.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-ExtraLight.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Light.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Medium.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Regular.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-SemiBold.ttf"));
-    QFontDatabase::addApplicationFont(QString::fromUtf8("://fonts/Inter-Thin.ttf"));
-
     app.setWindowIcon(QIcon(QString::fromUtf8(":/images/app_ico.ico")));
-
     app.initialize();
     app.start();
 
@@ -726,7 +770,6 @@ int main(int argc, char *argv[])
     QT_TRANSLATE_NOOP("MegaError", "Your account has been suspended due to copyright violations. Please check your email inbox.");
     QT_TRANSLATE_NOOP("MegaError", "Your account was terminated due to a breach of MEGA's Terms of Service, such as abuse of rights of others; sharing and/or importing illegal data; or system abuse.");
     QT_TRANSLATE_NOOP("MegaError", "Storage Quota Exceeded. Upgrade now");
-    QT_TRANSLATE_NOOP("MegaError", "Decryption error");
     QT_TRANSLATE_NOOP("FinderExtensionApp", "Get MEGA link");
     QT_TRANSLATE_NOOP("FinderExtensionApp", "View on MEGA");
     QT_TRANSLATE_NOOP("FinderExtensionApp", "No options available");
