@@ -111,6 +111,7 @@
 #endif
 #endif
 
+#include "mega/log_level.h"
 #include "mega/utils.h"
 
 #if ((defined(ANDROID) || defined(__ANDROID__)) && defined(ENABLE_CRASHLYTICS))
@@ -119,17 +120,6 @@
 
 namespace mega {
 
-// available log levels
-enum LogLevel
-{
-    logFatal = 0, // Very severe error event that will presumably lead the application to abort.
-    logError,     // Error information but will continue application to keep running.
-    logWarning,   // Information representing errors in application but application will keep running
-    logInfo,      // Mainly useful to represent current progress of application.
-    logDebug,     // Informational logs, that are useful for developers. Only applicable if DEBUG is defined.
-    logMax
-};
-
 // Output Log Interface
 class Logger
 {
@@ -195,26 +185,23 @@ class SimpleLogger
 
     std::string getTime();
 #else
-    static thread_local std::array<char, LOGGER_CHUNKS_SIZE> mBuffer;
-    std::array<char, LOGGER_CHUNKS_SIZE>::iterator mBufferIt;
-#ifndef NDEBUG
-    // Detect and warn against multiple instances of this class created in the same thread.
-    // If multiple instances are used in the same thread, the messages from the last created one will
-    // overwrite and corrupt the messages of others, by overwriting mBuffer.
-    //
-    // An alternative approach aiming to allow such cases, would be to overload operator,(), but that will
-    // require to no longer use LoggerVoidify(), and create a "null logger" for the cases when requested
-    // log level needs to be ignored. That would also incur a small performance penalty for the latter case.
-    static thread_local const SimpleLogger* mBufferOwner;
-#endif
+    using Buffer = std::array<char, LOGGER_CHUNKS_SIZE>;
+    static inline thread_local Buffer mBuffer;
+    static inline thread_local Buffer::iterator mBufferIt{mBuffer.begin()};
 
     using DiffType = std::array<char, LOGGER_CHUNKS_SIZE>::difference_type;
     using NumBuf = char[24];
     const char* filenameStr;
     int lineNum;
 
-    std::vector<DirectMessage> mDirectMessages;
-    std::vector<std::string *> mCopiedParts;
+    // True if there is a previous instance in the logging chain.
+    // For example, the first LOG_debug below will have isChained == false:
+    // the second LOG_debug inside f() below will have isChained == true:
+    //     int f() {LOG_DEBUG << "not first in chain"; return 2;}
+    //     LOG_debug << "1" << f();
+    bool isChained{false};
+    static inline thread_local std::vector<DirectMessage> mDirectMessages;
+    static inline thread_local std::vector<std::string> mCopiedParts;
 
     template<typename DataIterator>
     void copyToBuffer(const DataIterator dataIt, DiffType currentSize)
@@ -248,10 +235,9 @@ class SimpleLogger
             }
             else //reached LOGGER_CHUNKS_SIZE, we need to copy mBuffer contents
             {
-                std::string *newStr  = new string(mBuffer.data());
-                mCopiedParts.emplace_back( newStr);
-                string * back = mCopiedParts[mCopiedParts.size()-1];
-                mDirectMessages.push_back(DirectMessage(back->data(), back->size()));
+                mCopiedParts.emplace_back(string(mBuffer.data()));
+                mDirectMessages.push_back(
+                    DirectMessage{mCopiedParts.back().data(), mCopiedParts.back().size()});
             }
         }
         else if (logger)
@@ -364,6 +350,11 @@ class SimpleLogger
         copyToBuffer(value.begin(), static_cast<DiffType>(value.size()));
     }
 
+    void logValue(const std::string_view value)
+    {
+        copyToBuffer(value.begin(), static_cast<DiffType>(value.size()));
+    }
+
     void logValue(const ::mega::Error& value)   // ::mega:: when building MEGAchat on windows, else ambiguity errors
     {
         logValue(error(value));
@@ -392,27 +383,30 @@ class SimpleLogger
 
 public:
     // flag to turn off logging on the log-output thread, to prevent possible deadlock cycles.
-    static thread_local bool mThreadLocalLoggingDisabled;
+    static inline thread_local bool mThreadLocalLoggingDisabled = false;
 
-    SimpleLogger(const LogLevel ll, const char* filename, const int line)
-    : level{ll}
+    SimpleLogger(const LogLevel ll,
+                 const char* filename,
+                 const int line,
+                 [[maybe_unused]] bool noChainAssert = false):
+        level{ll}
 #ifdef ENABLE_LOG_PERFORMANCE
-    , mBufferIt{mBuffer.begin()}
-    , filenameStr(filename)
-    , lineNum(line)
+        ,
+        filenameStr(filename),
+        lineNum(line),
+        isChained(false)
 #endif
     {
         if (mThreadLocalLoggingDisabled) return;
 
 #ifdef ENABLE_LOG_PERFORMANCE
-#ifndef NDEBUG
-        // Multiple instances in the same thread will lead to message corruption!
-        if (!mBufferOwner)
-        {
-            mBufferOwner = this;
-        }
-        assert(mBufferOwner == this);
-#endif
+
+        // isChained if there are logs in the mBuffer or mDirectMessages
+        // Note: such as if the first logger instance doesn't output anything and we treat
+        // the second one as not in chain
+        isChained = mBufferIt != mBuffer.begin() || !mDirectMessages.empty();
+        // logging chain is not well supported, change caller code to avoid it
+        assert(!isChained || noChainAssert);
 #else
         if (!logger)
         {
@@ -444,52 +438,61 @@ public:
             copyToBuffer("]", 1);
         }
 
-        outputBuffer(true);
+        // The last output if it is not chained
+        const auto lastOutput = !isChained;
+        outputBuffer(lastOutput);
 
-        if (!mDirectMessages.empty())
+        // Direct Messages are log on last output
+        if (lastOutput)
         {
-            if (logger)
+            if (!mDirectMessages.empty())
             {
-                std::unique_ptr<const char *[]> dm(new const char *[mDirectMessages.size()]);
-                std::unique_ptr<size_t[]> dms(new size_t[mDirectMessages.size()]);
-                unsigned i = 0;
-                for (const auto & d : mDirectMessages)
+                if (logger)
                 {
-                    dm[i] = d.constChar();
-                    dms[i] = d.size();
-                    i++;
+                    std::unique_ptr<const char*[]> dm(new const char*[mDirectMessages.size()]);
+                    std::unique_ptr<size_t[]> dms(new size_t[mDirectMessages.size()]);
+                    unsigned i = 0;
+                    for (const auto& d: mDirectMessages)
+                    {
+                        dm[i] = d.constChar();
+                        dms[i] = d.size();
+                        i++;
+                    }
+
+                    logger->log(nullptr, level, nullptr, "", dm.get(), dms.get(), i);
                 }
-
-                logger->log(nullptr, level, nullptr, "", dm.get(), dms.get(), i);
             }
-        }
-        for (auto &s: mCopiedParts)
-        {
-            delete s;
+            // Clear
+            mDirectMessages.clear();
+            mCopiedParts.clear();
         }
 
-#ifndef NDEBUG
-        if (mBufferOwner == this)
-        {
-            mBufferOwner = nullptr;
-        }
-#endif
 #else
         if (logger)
-            logger->log(t.c_str(), level, fname.c_str(), ostr.str().c_str());
+        {
+            const auto message = ostr.str();
+            logger->log(/*                                                                     */
+                        t.c_str(), /* time stamp                                               */
+                        level, /* log level                                                    */
+                        fname.c_str(), /* file name and line                                   */
+                        message.c_str()); /* the log message itself                            */
+        }
 #endif
     }
 
+    SimpleLogger(const SimpleLogger&) = delete;
+    SimpleLogger& operator=(const SimpleLogger&) = delete;
+
     static const char *toStr(LogLevel ll)
     {
         switch (ll)
         {
-            case logMax: return "verbose";
             case logDebug: return "debug";
             case logInfo: return "info";
             case logWarning: return "warn";
             case logError: return "err";
             case logFatal: return "FATAL";
+            case logVerbose: return "verbose";
         }
         assert(false);
         return "";
@@ -608,18 +611,18 @@ public:
     SimpleLogger& operator<<(const DirectMessage &obj)
     {
 #ifndef ENABLE_LOG_PERFORMANCE
-        ostr.write(obj.constChar(), obj.size());
+        ostr.write(obj.constChar(), static_cast<std::streamsize>(obj.size()));
 #else
         // careful using constChar() without taking size() into account: *this << obj.constChar(); ended up with 2MB+ lines from fetchnodes.
 
         if (mBufferIt != mBuffer.begin()) //something was appended to the buffer before this direct msg
         {
             *mBufferIt = '\0';
-            std::string *newStr  = new string(mBuffer.data());
-            mCopiedParts.emplace_back( newStr);
-            string * back = mCopiedParts[mCopiedParts.size()-1];
 
-            mDirectMessages.push_back(DirectMessage(back->data(), back->size()));
+            mCopiedParts.emplace_back(string(mBuffer.data()));
+            mDirectMessages.push_back(
+                DirectMessage{mCopiedParts.back().data(), mCopiedParts.back().size()});
+
             mBufferIt = mBuffer.begin();
         }
 
@@ -658,21 +661,15 @@ public:
     // These do not go through the LOG_<level> macros.
     //
     // When ENABLE_LOG_PERFORMANCE is on, this must not be called during the lifetime of an
-    // existing instance created in the same thread. Otherwise this will overwrite the message
-    // of the existing instance. That would be a rare case, but could be achieved by writing
+    // existing instance created in the same thread. Otherwise logs are mixed. That would be
+    // a rare case, but could be avoided by writing:
     // LOG_info << "foo", SimpleLogger::postLog(logInfo, "bar", filename, line);
     static void postLog(LogLevel logLevel, const char *message, const char *filename, int line)
     {
-#ifdef ENABLE_LOG_PERFORMANCE
-#ifndef NDEBUG
-        assert(!mBufferOwner);
-        if (mBufferOwner) return;
-#endif
-#endif
-
         if (logCurrentLevel < logLevel) return;
-        SimpleLogger logger(logLevel, filename ? filename : "", line);
-        if (message) logger << message;
+        SimpleLogger simpleLogger(logLevel, filename ? filename : "", line);
+        if (message)
+            simpleLogger << message;
     }
 };
 
@@ -697,7 +694,7 @@ struct LoggerVoidify
 
 #define LOG_verbose \
     ::mega::SimpleLogger::getLogLevel() < ::mega::logMax ? (void)0 : \
-        ::mega::LoggerVoidify() & ::mega::SimpleLogger(::mega::logMax, ::mega::log_file_leafname(__FILE__), __LINE__)
+        ::mega::LoggerVoidify() & ::mega::SimpleLogger(::mega::logVerbose, ::mega::log_file_leafname(__FILE__), __LINE__)
 
 #define LOG_debug \
     ::mega::SimpleLogger::getLogLevel() < ::mega::logDebug ? (void)0 : \
@@ -748,7 +745,7 @@ inline bool isWithinActivePeriod(const TimeUnit sleepDuration, const TimeUnit ac
     ::mega::LoggerVoidify() & \
         ::mega::SimpleLogger(LOG_LEVEL, ::mega::log_file_leafname(__FILE__), __LINE__)
 
-#define LOG_verbose_timed(SLEEP, ACTIVE) LOG_generic_timed(::mega::logMax, SLEEP, ACTIVE)
+#define LOG_verbose_timed(SLEEP, ACTIVE) LOG_generic_timed(::mega::logVerbose, SLEEP, ACTIVE)
 #define LOG_debug_timed(SLEEP, ACTIVE) LOG_generic_timed(::mega::logDebug, SLEEP, ACTIVE)
 #define LOG_info_timed(SLEEP, ACTIVE) LOG_generic_timed(::mega::logInfo, SLEEP, ACTIVE)
 #define LOG_warn_timed(SLEEP, ACTIVE) LOG_generic_timed(::mega::logWarning, SLEEP, ACTIVE)
@@ -837,4 +834,48 @@ public:
 extern ExternalLogger g_externalLogger;
 extern ExclusiveLogger g_exclusiveLogger;
 
+inline error logAndReturnError(const error e, const std::string_view msg)
+{
+    LOG_err << msg;
+    return e;
+}
+
+/**
+ * @brief A generic version of `logAndReturnError` where you can specify how the return value is
+ * instantiated from the error code
+ *
+ * @param returnGenerator A function that can be invoked with `const error` and generates the
+ * returned value
+ * @return The result from invoking returnGenerator(e)
+ */
+template<typename F>
+auto logAndReturnErrorGeneric(const error e, const std::string_view msg, F&& returnGenerator)
+    -> std::invoke_result_t<F, error>
+{
+    LOG_err << msg;
+    return returnGenerator(e);
+}
+
+/**
+ * @brief A helper factory function to generate callables with the same input parameters as
+ * `logAndReturnError` but with a custom return type, generated by resGenerator.
+ *
+ * @example
+ * const auto logAndError = generateLogAndReturnError(
+ *     [](const error e) -> std::pair<error, std::pair<std::string, unsigned>>
+ *     {
+ *         return {e, {"", 0}};
+ *     });
+ * return logAndError(API_EARGS, "Invalid handle"); // will return the pair
+ */
+template<typename F>
+constexpr auto generateLogAndReturnError(F&& resGenerator)
+    -> std::function<std::invoke_result_t<std::decay_t<F>, error>(error, std::string_view)>
+{
+    return [gen = std::forward<F>(resGenerator)](const error e, const std::string_view msg)
+    {
+        return logAndReturnErrorGeneric(e, msg, gen);
+    };
+}
+
 } // namespace
