@@ -18,28 +18,33 @@
  * You should have received a copy of the license along with this
  * program.
  */
-#include <cctype>
+#include "mega/filesystem.h"
 
 #include "mega.h"
-#include "mega/filesystem.h"
-#include "mega/node.h"
-#include "mega/megaclient.h"
+#include "mega/base64.h"
+#include "mega/localpath.h"
 #include "mega/logging.h"
 #include "mega/mega_utf8proc.h"
+#include "mega/megaclient.h"
+#include "mega/node.h"
 #include "mega/sync.h"
-#include "mega/base64.h"
-
 #include "megafs.h"
 
 #include <cassert>
+#include <cctype>
+#include <regex>
 #include <tuple>
 
 #ifdef TARGET_OS_MAC
 #include "mega/osx/osxutils.h"
 #endif
 
-namespace mega {
+#ifdef __ANDROID__
+#include "mega/android/androidFileSystem.h"
+#endif
 
+namespace mega
+{
 std::atomic<int> FileSystemAccess::mMinimumDirectoryPermissions{0700};
 std::atomic<int> FileSystemAccess::mMinimumFilePermissions{0600};
 
@@ -232,12 +237,7 @@ fsfp_t::fsfp_t(std::uint64_t fingerprint,
 
 fsfp_t::operator bool() const
 {
-    return mFingerprint;
-}
-
-bool fsfp_t::operator!() const
-{
-    return !mFingerprint;
+    return mFingerprint != 0 || !mUUID.empty();
 }
 
 bool fsfp_t::operator==(const fsfp_t& rhs) const
@@ -252,11 +252,6 @@ bool fsfp_t::operator<(const fsfp_t& rhs) const
            < std::tie(rhs.mFingerprint, rhs.mUUID);
 }
 
-bool fsfp_t::operator!=(const fsfp_t& rhs) const
-{
-    return !operator==(rhs);
-}
-
 bool fsfp_t::equivalent(const fsfp_t& rhs) const
 {
     // Only compare legacy fingerprints if UUIDs are unavailable.
@@ -369,26 +364,29 @@ int compareUtf(const string& s1, bool unescaping1, const string& s2, bool unesca
 
 int compareUtf(const string& s1, bool unescaping1, const LocalPath& s2, bool unescaping2, bool caseInsensitive)
 {
-    return detail::compareUtf(
-        unicodeCodepointIterator(s1), unescaping1,
-        unicodeCodepointIterator(s2.localpath), unescaping2,
-        caseInsensitive ? Utils::toUpper: detail::identity);
+    return detail::compareUtf(unicodeCodepointIterator(s1),
+                              unescaping1,
+                              unicodeCodepointIterator(s2.toPath(false)),
+                              unescaping2,
+                              caseInsensitive ? Utils::toUpper : detail::identity);
 }
 
 int compareUtf(const LocalPath& s1, bool unescaping1, const string& s2, bool unescaping2, bool caseInsensitive)
 {
-    return detail::compareUtf(
-        unicodeCodepointIterator(s1.localpath), unescaping1,
-        unicodeCodepointIterator(s2), unescaping2,
-        caseInsensitive ? Utils::toUpper: detail::identity);
+    return detail::compareUtf(unicodeCodepointIterator(s1.toPath(false)),
+                              unescaping1,
+                              unicodeCodepointIterator(s2),
+                              unescaping2,
+                              caseInsensitive ? Utils::toUpper : detail::identity);
 }
 
 int compareUtf(const LocalPath& s1, bool unescaping1, const LocalPath& s2, bool unescaping2, bool caseInsensitive)
 {
-    return detail::compareUtf(
-        unicodeCodepointIterator(s1.localpath), unescaping1,
-        unicodeCodepointIterator(s2.localpath), unescaping2,
-        caseInsensitive ? Utils::toUpper: detail::identity);
+    return detail::compareUtf(unicodeCodepointIterator(s1.toPath(false)),
+                              unescaping1,
+                              unicodeCodepointIterator(s2.toPath(false)),
+                              unescaping2,
+                              caseInsensitive ? Utils::toUpper : detail::identity);
 }
 
 RemotePath::RemotePath(const string& path)
@@ -576,163 +574,6 @@ bool IsContainingLocalPathOf(const string& a, const char* b, size_t bLength)
 #endif
 }
 
-// TODO: may or may not be needed
-void LocalPath::removeTrailingSeparators()
-{
-    assert(invariant());
-
-    // Remove trailing separator if present.
-    while (localpath.size() > 1 &&
-           localpath.back() == localPathSeparator)
-    {
-        localpath.pop_back();
-    }
-
-    assert(invariant());
-}
-
-void LocalPath::normalizeAbsolute()
-{
-    assert(!localpath.empty());
-
-#ifdef USE_IOS
-    // iOS is a tricky case.
-    // We need to be able to use and persist absolute paths.  however on iOS app restart,
-    // our app base path may be different.  So, we only record the path beyond that app
-    // base path.   That is what the app supplies for absolute paths, that's what is persisted.
-    // Actual filesystem functions passed such an "absolute" path will prepend the app base path
-    // unless it already started with /
-    // and that's how it worked before we added the "absolute" feature to LocalPath.
-    // As a result of that though, there's nothing to adjust or check here for iOS.
-    isFromRoot = true;
-
-    // In addition, for iOS, should the app try to use ".", or "./", we interpret that to mean
-    // that really it's relative to the app base path.  So we convert:
-    if (!localpath.empty() && localpath.front() == '.')
-    {
-        if (localpath.size() == 1 || localpath[1] == localPathSeparator)
-        {
-            localpath.erase(0, 1);
-            while (!localpath.empty() &&
-                    localpath.front() == localPathSeparator)
-            {
-                localpath.erase(0, 1);
-            }
-        }
-    }
-
-#elif WIN32
-
-    // Add a drive separator if necessary.
-    // append \ to bare Windows drive letter paths
-    // GetFullPathNameW does all of this for windows.
-    // The documentation says to prepend \\?\ to deal with long names, but it makes the function fail
-    // it seems to work with long names anyway.
-
-    // We also convert to absolute if it isn't already, which GetFullPathNameW does also.
-    // So that when working with LocalPath, we always have the full path.
-    // Historically, relative paths can come into the system, this will convert them.
-
-    if (PathIsRelativeW(localpath.c_str()))
-    {
-        // ms: In the ANSI version of this function, the name is limited to MAX_PATH characters.
-        // ms: To extend this limit to 32,767 wide characters, call the Unicode version of the function (GetFullPathNameW), and prepend "\\?\" to the path
-        WCHAR buffer[32768];
-        DWORD stringLen = GetFullPathNameW(localpath.c_str(), 32768, buffer, NULL);
-        assert(stringLen < 32768);
-
-        localpath = wstring(buffer, stringLen);
-    }
-
-    isFromRoot = true;
-
-    // See https://docs.microsoft.com/en-us/dotnet/standard/io/file-path-formats
-    // Also https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation
-    // Basically, \\?\ is the magic prefix that means "don't mess with the path I gave you",
-    // and lets us access otherwise inaccessible files (trailing ' ', '.', very long names, etc).
-    // "Unless the path starts exactly with \\?\ (note the use of the canonical backslash), it is normalized."
-
-    // TODO:  add long-path-aware manifest? (see 2nd link)
-
-
-    if (localpath.substr(0,2) == L"\\\\")
-    {
-        // The caller already passed in a path that should be precise either with \\?\ or \\.\ or \\<server> etc.
-        // Let's trust they know what they are doing and leave the path alone
-
-        if (localpath.substr(0,4) != L"\\\\?\\" &&
-            localpath.substr(0,4) != L"\\\\.\\")
-        {
-            // However, it turns out, \\?\UNC\<server>\etc  can allow us to operate on paths with trailing spaces and other things Explorer doesn't like, which just \\server\ does not
-            {
-                localpath.insert(2, L"?\\UNC\\");
-            }
-        }
-    }
-    else
-    {
-        localpath.insert(0, L"\\\\?\\");
-    }
-
-#else
-    // convert to absolute if it isn't already
-    if (!localpath.empty() && localpath[0] != localPathSeparator)
-    {
-        LocalPath lp;
-        PosixFileSystemAccess::cwd_static(lp);
-        lp.appendWithSeparator(*this, false);
-        localpath = std::move(lp.localpath);
-    }
-    isFromRoot = true;
-#endif
-
-    assert(invariant());
-}
-
-bool LocalPath::invariant() const
-{
-#ifdef USE_IOS
-    // iOS is a tricky case.
-    // We need to be able to use and persist absolute paths.  however on iOS app restart,
-    // our app base path may be different.  So, we only record the path beyond that app
-    // base path.   That is what the app supplies for absolute paths, that's what is persisted.
-    // Actual filesystem functions passed such an "absolute" path will prepend the app base path
-    // unless it already started with /
-    // and that's how it worked before we added the "absolute" feature to LocalPath.
-    // As a result of that though, there's nothing to adjust or check here for iOS.
-#elif WIN32
-    if (isFromRoot)
-    {
-        // if it starts with \\ then it's absolute, either by us or provided
-        if (localpath.size() >= 2 && localpath[0] == '\\' && localpath[1] == '\\') return true;
-        // otherwise it must contain a drive letter
-        if (localpath.find(L":") == string_type::npos) return false;
-        // ok so probably relative then, but double check:
-        if (PathIsRelativeW(localpath.c_str())) return false;
-    }
-    else
-    {
-        // must not contain a drive letter
-        if (localpath.find(L":") != string_type::npos) return false;
-        // must not start "\\"
-        if (localpath.size() >= 2 &&
-            localpath.substr(0, 2) == L"\\\\") return false;
-     }
-#else
-    if (isFromRoot)
-    {
-        // must start /
-        if (localpath.size() < 1) return false;
-        if (localpath.front() != localPathSeparator) return false;
-    }
-    else
-    {
-        // this could contain /relative for appending etc.
-    }
-#endif
-    return true;
-}
-
 FileSystemAccess::FileSystemAccess()
 {
 }
@@ -828,20 +669,10 @@ FileSystemType FileSystemAccess::getlocalfstype(const LocalPath& path) const
         return FS_UNKNOWN;
     }
 
-    // Where does our name begin?
-    auto index = parentPath.getLeafnameByteIndex();
-
-    // We have a parent.
-    if (index)
+    parentPath = parentPath.parentPath();
+    if (getlocalfstype(parentPath, type))
     {
-        // Remove the current leaf name.
-        parentPath.truncate(index);
-
-        // Try and get our parent's filesystem type.
-        if (getlocalfstype(parentPath, type))
-        {
-            return type;
-        }
+        return type;
     }
 
     return FS_UNKNOWN;
@@ -942,42 +773,6 @@ void FileSystemAccess::unescapefsincompatible(string *name) const
     }
 }
 
-void LocalPath::utf8_normalize(string* filename)
-{
-    if (!filename) return;
-
-    const char* cfilename = filename->c_str();
-    size_t fnsize = filename->size();
-    string result;
-
-    for (size_t i = 0; i < fnsize; )
-    {
-        // allow NUL bytes between valid UTF-8 sequences
-        if (!cfilename[i])
-        {
-            result.append("", 1);
-            i++;
-            continue;
-        }
-
-        const char* substring = cfilename + i;
-        char* normalized = (char*)utf8proc_NFC((uint8_t*)substring);
-
-        if (!normalized)
-        {
-            filename->clear();
-            return;
-        }
-
-        result.append(normalized);
-        free(normalized);
-
-        i += strlen(substring);
-    }
-
-    *filename = std::move(result);
-}
-
 std::unique_ptr<LocalPath> FileSystemAccess::fsShortname(const LocalPath& localname)
 {
     LocalPath s;
@@ -1092,13 +887,23 @@ bool FileAccess::fopen(const LocalPath& name, FSLogging fsl)
 
 bool FileAccess::isfile(const LocalPath& path)
 {
-    return fopen(path, FSLogging::noLogging) && type == FILENODE;
+    auto name = std::move(nonblocking_localname);
+
+    updatelocalname(path, true);
+    sysstat(&mtime, &size, FSLogging::noLogging);
+    updatelocalname(name, true);
+
+    return type == FILENODE;
 }
 
 bool FileAccess::isfolder(const LocalPath& path)
 {
+    auto name = std::move(nonblocking_localname);
+
     updatelocalname(path, true);
     sysstat(&mtime, &size, FSLogging::noLogging);
+    updatelocalname(name, true);
+
     return type == FOLDERNODE;
 }
 
@@ -1346,42 +1151,60 @@ AsyncIOContext *FileAccess::newasynccontext()
     return new AsyncIOContext();
 }
 
-bool FileAccess::fread(string* dst, unsigned len, unsigned pad, m_off_t pos, FSLogging fsl)
+bool FileAccess::fread(string* buffer,
+                       unsigned long length,
+                       unsigned long padding,
+                       m_off_t offset,
+                       FSLogging logging,
+                       bool* cretry)
 {
-    if (!openf(fsl))
-    {
+    // Sanity.
+    assert(buffer);
+
+    // Make sure the file's been opened.
+    if (!openf(logging))
         return false;
-    }
 
-    bool r;
+    // Make sure our buffer is large enough.
+    buffer->resize(length + padding);
 
-    dst->resize(len + pad);
+    // Try and perform the read.
+    auto result = sysread(buffer->data(), length, offset, cretry);
 
-    if ((r = sysread((byte*)dst->data(), len, pos)))
-    {
-        memset((char*)dst->data() + len, 0, pad);
-    }
+    // Read was successful so zero pad bytes.
+    if (result && padding)
+        std::memset(buffer->data() + length, 0, padding);
 
+    // Close the file if necessary.
     closef();
 
-    return r;
+    // Let the caller know if the read was successful.
+    return result;
 }
 
-bool FileAccess::frawread(byte* dst, unsigned len, m_off_t pos, bool caller_opened, FSLogging fsl)
+bool FileAccess::frawread(void* buffer,
+                          unsigned long length,
+                          m_off_t offset,
+                          bool alreadyOpened,
+                          FSLogging logging,
+                          bool* cretry)
 {
-    if (!caller_opened && !openf(fsl))
-    {
+    // Sanity.
+    assert(buffer || !length);
+
+    // Couldn't open the file.
+    if (!alreadyOpened && !openf(logging))
         return false;
-    }
 
-    bool r = sysread(dst, len, pos);
+    // Try and perform the read.
+    auto result = sysread(buffer, length, offset, cretry);
 
-    if (!caller_opened)
-    {
+    // Close the file if necessary.
+    if (!alreadyOpened)
         closef();
-    }
 
-    return r;
+    // Let the caller know if the read was successful.
+    return result;
 }
 
 AsyncIOContext::~AsyncIOContext()
@@ -1445,581 +1268,6 @@ bool FileInputStream::read(byte *buffer, unsigned size)
     return false;
 }
 
-bool LocalPath::empty() const
-{
-    assert(invariant());
-    return localpath.empty();
-}
-
-void LocalPath::clear()
-{
-    assert(invariant());
-    localpath.clear();
-    isFromRoot = false;
-    assert(invariant());
-}
-
-void LocalPath::truncate(size_t bytePos)
-{
-    assert(invariant());
-    localpath.resize(bytePos);
-    assert(invariant());
-}
-
-LocalPath LocalPath::leafName() const
-{
-    assert(invariant());
-
-    auto p = localpath.find_last_of(localPathSeparator);
-    p = p == string::npos ? 0 : p + 1;
-    LocalPath result;
-    result.localpath = localpath.substr(p, localpath.size() - p);
-    assert(result.invariant());
-    return result;
-}
-
-string LocalPath::leafOrParentName() const
-{
-    assert(invariant());
-
-    LocalPath name;
-    // win32: normalizeAbsolute() does not work with paths like "D:\\foo\\..\\bar.txt". TODO ?
-    FSACCESS_CLASS().expanselocalpath(*this, name);
-    name.removeTrailingSeparators();
-
-    if (name.empty())
-    {
-        return string();
-    }
-
-#ifdef WIN32
-    if (name.localpath.back() == L':')
-    {
-        // drop trailing ':'
-        string n = name.toPath(true);
-        n.pop_back();
-        return n;
-    }
-#endif
-
-    return name.leafName().toPath(true);
-}
-
-void LocalPath::append(const LocalPath& additionalPath)
-{
-    assert(invariant());
-    localpath.append(additionalPath.localpath);
-    assert(invariant());
-}
-
-std::string LocalPath::platformEncoded() const
-{
-    assert(invariant());
-#ifdef WIN32
-    // this function is typically used where we need to pass a file path to the client app, which expects utf16 in a std::string buffer
-    // some other backwards compatible cases need this format also, eg. serialization
-    std::string outstr;
-
-    if (localpath.size() >= 4 && 0 == localpath.compare(0, 4, L"\\\\?\\", 4))
-    {
-        if (0 == localpath.compare(4, 4, L"UNC\\", 4))
-        {
-            // when a path leaves LocalPath, we can remove prefix which is only needed internally
-            outstr.resize((localpath.size() - 6) * sizeof(wchar_t));
-            memcpy(const_cast<char*>(outstr.data()), localpath.data() + 0, 2 * sizeof(wchar_t));  // "\\\\"
-            memcpy(const_cast<char*>(outstr.data()) + 4, localpath.data() + 8, (localpath.size() - 8) * sizeof(wchar_t));
-        }
-        else
-        {
-            // when a path leaves LocalPath, we can remove prefix which is only needed internally
-            outstr.resize((localpath.size() - 4) * sizeof(wchar_t));
-            memcpy(const_cast<char*>(outstr.data()), localpath.data() + 4, (localpath.size() - 4) * sizeof(wchar_t));
-        }
-    }
-    else
-    {
-        outstr.resize(localpath.size() * sizeof(wchar_t));
-        memcpy(const_cast<char*>(outstr.data()), localpath.data(), localpath.size() * sizeof(wchar_t));
-    }
-    return outstr;
-#else
-    // for non-windows, it's just the same utf8 string we use anyway
-    return localpath;
-#endif
-}
-
-
-void LocalPath::appendWithSeparator(const LocalPath& additionalPath, bool separatorAlways)
-{
-#ifdef USE_IOS
-    bool originallyUsesAppBasePath = isFromRoot &&
-        (localpath.empty() || localpath.front() != localPathSeparator);
-#endif
-
-    assert(!additionalPath.isFromRoot);
-    if (separatorAlways || localpath.size())
-    {
-        // still have to be careful about appending a \ to F:\ for example, on windows, which produces an invalid path
-        if (!(endsInSeparator() || additionalPath.beginsWithSeparator()))
-        {
-            localpath.append(1, localPathSeparator);
-        }
-    }
-
-    localpath.append(additionalPath.localpath);
-
-#ifdef USE_IOS
-    if (originallyUsesAppBasePath)
-    {
-        while (!localpath.empty() && localpath.front() == localPathSeparator)
-        {
-            localpath.erase(0, 1);
-        }
-    }
-#endif
-
-    assert(invariant());
-}
-
-void LocalPath::prependWithSeparator(const LocalPath& additionalPath)
-{
-    assert(!isFromRoot);
-    assert(invariant());
-    // no additional separator if there is already one after
-    if (!localpath.empty() && localpath[0] != localPathSeparator)
-    {
-        // no additional separator if there is already one before
-        if (!(beginsWithSeparator() || additionalPath.endsInSeparator()))
-        {
-            localpath.insert(0, 1, localPathSeparator);
-        }
-    }
-    localpath.insert(0, additionalPath.localpath);
-    isFromRoot = additionalPath.isFromRoot;
-    assert(invariant());
-}
-
-LocalPath LocalPath::prependNewWithSeparator(const LocalPath& additionalPath) const
-{
-    assert(!isFromRoot);
-    LocalPath lp = *this;
-    lp.prependWithSeparator(additionalPath);
-    assert(lp.invariant());
-    return lp;
-}
-
-void LocalPath::trimNonDriveTrailingSeparator()
-{
-    assert(invariant());
-    if (endsInSeparator())
-    {
-        // ok so the last character is a directory separator.  But don't remove it for eg. F:\ on windows
-        #ifdef WIN32
-        if (localpath.size() > 1 &&
-            localpath[localpath.size() - 2] == L':')
-        {
-            return;
-        }
-        #endif
-
-        localpath.resize(localpath.size() - 1);
-    }
-    assert(invariant());
-}
-
-bool LocalPath::findNextSeparator(size_t& separatorBytePos) const
-{
-    assert(invariant());
-    separatorBytePos = localpath.find(localPathSeparator, separatorBytePos);
-    return separatorBytePos != string::npos;
-}
-
-bool LocalPath::findPrevSeparator(size_t& separatorBytePos, const FileSystemAccess& fsaccess) const
-{
-    assert(invariant());
-    separatorBytePos = localpath.rfind(LocalPath::localPathSeparator, separatorBytePos);
-    return separatorBytePos != string::npos;
-}
-
-bool LocalPath::endsInSeparator() const
-{
-    assert(invariant());
-    return !localpath.empty() && localpath.back() == localPathSeparator;
-}
-
-bool LocalPath::beginsWithSeparator() const
-{
-    assert(invariant());
-    return !localpath.empty() && localpath.front() == localPathSeparator;
-}
-
-size_t LocalPath::getLeafnameByteIndex() const
-{
-    assert(invariant());
-    size_t p = localpath.size();
-
-    while (p && (p -= 1))
-    {
-        if (localpath[p] == LocalPath::localPathSeparator)
-        {
-            p += 1;
-            break;
-        }
-    }
-    return p;
-}
-
-LocalPath LocalPath::subpathFrom(size_t bytePos) const
-{
-    assert(invariant());
-    LocalPath result;
-    result.localpath = localpath.substr(bytePos);
-    assert(result.invariant());
-    return result;
-}
-
-LocalPath LocalPath::subpathTo(size_t bytePos) const
-{
-    assert(invariant());
-    LocalPath p;
-    p.localpath = localpath.substr(0, bytePos);
-    p.isFromRoot = isFromRoot;
-    assert(p.invariant());
-    return p;
-}
-
-LocalPath LocalPath::parentPath() const
-{
-    assert(invariant());
-    return subpathTo(getLeafnameByteIndex());
-}
-
-LocalPath LocalPath::insertFilenameSuffix(const std::string& suffix) const
-{
-    assert(invariant());
-
-    size_t dotindex = localpath.find_last_of('.');
-    size_t sepindex = localpath.find_last_of(LocalPath::localPathSeparator);
-
-    LocalPath result, extension;
-
-    if (dotindex == string::npos || (sepindex != string::npos && sepindex > dotindex))
-    {
-        result.localpath = localpath;
-        result.isFromRoot = isFromRoot;
-    }
-    else
-    {
-        result.localpath = localpath.substr(0, dotindex);
-        result.isFromRoot = isFromRoot;
-        extension.localpath = localpath.substr(dotindex);
-    }
-
-    result.localpath += LocalPath::fromRelativePath(suffix).localpath + extension.localpath;
-    assert(result.invariant());
-    return result;
-}
-
-
-string LocalPath::toPath(bool normalize) const
-{
-    assert(invariant());
-    string path;
-    local2path(&localpath, &path, normalize);
-
-#ifdef WIN32
-    if (path.size() >= 4 && 0 == path.compare(0, 4, "\\\\?\\", 4))
-    {
-        if (0 == localpath.compare(4, 4, L"UNC\\", 4))
-        {
-            // when a path leaves LocalPath, we can remove prefix which is only needed internally
-            path.erase(2, 6);
-        }
-        else
-        {
-            // when a path leaves LocalPath, we can remove prefix which is only needed internally
-            path.erase(0, 4);
-        }
-    }
-#endif
-
-    return path;
-}
-
-string LocalPath::toName(const FileSystemAccess& fsaccess) const
-{
-    string name = toPath(true);
-    fsaccess.unescapefsincompatible(&name);
-    return name;
-}
-
-LocalPath LocalPath::fromAbsolutePath(const string& path)
-{
-    assert(!path.empty());
-    LocalPath p;
-    path2local(&path, &p.localpath);
-    p.normalizeAbsolute();
-    return p;
-}
-
-LocalPath LocalPath::fromRelativePath(const string& path)
-{
-    LocalPath p;
-    path2local(&path, &p.localpath);
-    assert(p.invariant());
-    return p;
-}
-
-LocalPath LocalPath::fromRelativeName(string path, const FileSystemAccess& fsaccess, FileSystemType fsType)
-{
-    fsaccess.escapefsincompatible(&path, fsType);
-    return fromRelativePath(path);
-}
-
-LocalPath LocalPath::fromPlatformEncodedRelative(string path)
-{
-    LocalPath p;
-#if defined(_WIN32)
-    assert(!(path.size() % 2));
-    p.localpath.resize(path.size() / sizeof(wchar_t));
-    memcpy(const_cast<wchar_t*>(p.localpath.data()), path.data(), p.localpath.size() * sizeof(wchar_t));
-#else
-    p.localpath = std::move(path);
-#endif
-    assert(p.invariant());
-    return p;
-}
-
-LocalPath LocalPath::fromPlatformEncodedAbsolute(string path)
-{
-    LocalPath p;
-#if defined(_WIN32)
-    assert(!(path.size() % 2));
-    p.localpath.resize(path.size() / sizeof(wchar_t));
-    memcpy(const_cast<wchar_t*>(p.localpath.data()), path.data(), p.localpath.size() * sizeof(wchar_t));
-#else
-    p.localpath = std::move(path);
-#endif
-    p.normalizeAbsolute();
-    return p;
-}
-
-
-#if defined(_WIN32)
-LocalPath LocalPath::fromPlatformEncodedRelative(wstring&& wpath)
-{
-    LocalPath p;
-    p.localpath = std::move(wpath);
-    assert(p.invariant());
-    return p;
-}
-
-LocalPath LocalPath::fromPlatformEncodedAbsolute(wstring&& wpath)
-{
-    LocalPath p;
-    p.localpath = std::move(wpath);
-    p.normalizeAbsolute();
-    return p;
-}
-
-wchar_t LocalPath::driveLetter()
-{
-    assert(isFromRoot);
-    assert(invariant());
-    auto drivepos = localpath.find(L':');
-    return drivepos == wstring::npos || drivepos < 1 ? 0 : localpath[drivepos-1];
-}
-
-
-// convert UTF-8 to Windows Unicode
-void LocalPath::path2local(const string* path, string* local)
-{
-    // make space for the worst case
-    local->resize((path->size() + 1) * sizeof(wchar_t));
-
-    int len = MultiByteToWideChar(CP_UTF8, 0,
-        path->c_str(),
-        -1,
-        (wchar_t*)local->data(),
-        int(local->size() / sizeof(wchar_t) + 1));
-    if (len)
-    {
-        // resize to actual result
-        local->resize(sizeof(wchar_t) * (len - 1));
-    }
-    else
-    {
-        local->clear();
-    }
-}
-
-// convert UTF-8 to Windows Unicode
-void LocalPath::path2local(const string* path, std::wstring* local)
-{
-    // make space for the worst case
-    local->resize(path->size() + 2);
-
-    int len = MultiByteToWideChar(CP_UTF8, 0,
-        path->c_str(),
-        -1,
-        const_cast<wchar_t*>(local->data()),
-        int(local->size()));
-    if (len)
-    {
-        // resize to actual result
-        local->resize(len - 1);
-    }
-    else
-    {
-        local->clear();
-    }
-}
-
-// convert Windows Unicode to UTF-8
-void LocalPath::local2path(const string* local, string* path, bool normalize)
-{
-    path->resize((local->size() + 1) * 4 / sizeof(wchar_t) + 1);
-
-    path->resize(WideCharToMultiByte(CP_UTF8, 0, (wchar_t*)local->data(),
-        int(local->size() / sizeof(wchar_t)),
-        (char*)path->data(),
-        int(path->size()),
-        NULL, NULL));
-
-    if (normalize) utf8_normalize(path);
-}
-
-void LocalPath::local2path(const std::wstring* local, string* path, bool normalize)
-{
-    path->resize((local->size() * sizeof(wchar_t) + 1) * 4 / sizeof(wchar_t) + 1);
-
-    path->resize(WideCharToMultiByte(CP_UTF8, 0, local->data(),
-        int(local->size()),
-        (char*)path->data(),
-        int(path->size()),
-        NULL, NULL));
-
-    if (normalize) utf8_normalize(path);
-}
-
-#else
-
-void LocalPath::path2local(const string* path, string* local)
-{
-#ifdef __MACH__
-    path2localMac(path, local);
-#else
-    *local = *path;
-#endif
-}
-
-void LocalPath::local2path(const string* local, string* path, bool normalize)
-{
-    *path = *local;
-    if (normalize) LocalPath::utf8_normalize(path);
-}
-
-#endif
-
-
-std::atomic<unsigned> LocalPath_tmpNameLocal_counter{};
-
-LocalPath LocalPath::tmpNameLocal()
-{
-    char buf[128];
-    snprintf(buf, sizeof(buf), ".getxfer.%lu.%u.mega", getCurrentPid(), ++LocalPath_tmpNameLocal_counter);
-    return LocalPath::fromRelativePath(buf);
-}
-
-bool LocalPath::isContainingPathOf(const LocalPath& path, size_t* subpathIndex) const
-{
-    assert(!empty());
-    assert(!path.empty());
-    assert(isFromRoot == path.isFromRoot);
-    assert(invariant());
-    assert(path.invariant());
-
-    if (path.localpath.size() >= localpath.size()
-        && !Utils::pcasecmp(path.localpath, localpath, localpath.size()))
-    {
-       if (path.localpath.size() == localpath.size())
-       {
-           if (subpathIndex) *subpathIndex = localpath.size();
-           return true;
-       }
-       else if (path.localpath[localpath.size()] == localPathSeparator)
-       {
-           if (subpathIndex) *subpathIndex = localpath.size() + 1;
-           return true;
-       }
-       else if (!localpath.empty() &&
-                path.localpath[localpath.size() - 1] == localPathSeparator)
-       {
-           if (subpathIndex) *subpathIndex = localpath.size();
-           return true;
-       }
-    }
-    return false;
-}
-
-bool LocalPath::nextPathComponent(size_t& subpathIndex, LocalPath& component) const
-{
-    assert(invariant());
-
-    while (subpathIndex < localpath.size() && localpath[subpathIndex] == localPathSeparator)
-    {
-        ++subpathIndex;
-    }
-    size_t start = subpathIndex;
-    if (start >= localpath.size())
-    {
-        return false;
-    }
-    else if (findNextSeparator(subpathIndex))
-    {
-        component.localpath = localpath.substr(start, subpathIndex - start);
-        assert(component.invariant());
-        return true;
-    }
-    else
-    {
-        component.localpath = localpath.substr(start, localpath.size() - start);
-        subpathIndex = localpath.size();
-        assert(component.invariant());
-        return true;
-    }
-}
-
-bool LocalPath::hasNextPathComponent(size_t index) const
-{
-    assert(invariant());
-    return index < localpath.size();
-}
-
-bool LocalPath::related(const LocalPath& other) const
-{
-    // Sanity: Compare like for like paths.
-    assert(isFromRoot == other.isFromRoot);
-
-    // This path is shorter: It may contain other.
-    if (localpath.size() <= other.localpath.size())
-        return isContainingPathOf(other);
-
-    // Other is shorter: It may contain this path.
-    return other.isContainingPathOf(*this);
-}
-
-ScopedLengthRestore::ScopedLengthRestore(LocalPath& p)
-    : path(p)
-    , length(path.localpath.size())
-{
-    assert(path.invariant());
-}
-ScopedLengthRestore::~ScopedLengthRestore()
-{
-    path.localpath.resize(length);
-};
-
 #ifdef ENABLE_SYNC
 bool Notification::fromDebris(const Sync& sync) const
 {
@@ -2052,7 +1300,30 @@ LocalPath FileNameGenerator::suffixWithN(FileAccess* fa, const LocalPath& localn
 
 LocalPath FileNameGenerator::suffixWithOldN(FileAccess* fa, const LocalPath& localname)
 {
-    return suffix(fa, localname, [](unsigned num) { return ".old" + std::to_string(num); });
+    LocalPath currentLeafName;
+    // We need to get current name, in case of unsensitive case systems, suffix is added to current
+    // name without any change case
+    fa->fopen(localname,
+              true,
+              false,
+              FSLogging::logExceptFileNotFound,
+              nullptr,
+              false,
+              false,
+              &currentLeafName);
+
+    LocalPath path = localname;
+    if (!currentLeafName.empty() && !currentLeafName.isAbsolute() && !currentLeafName.isURI())
+    {
+        path.changeLeaf(currentLeafName);
+    }
+
+    return suffix(fa,
+                  path,
+                  [](unsigned num)
+                  {
+                      return ".old" + std::to_string(num);
+                  });
 }
 
 LocalPath FileNameGenerator::suffix(FileAccess* fa, const LocalPath& localname, std::function<std::string(unsigned)> suffixF)
@@ -2085,7 +1356,14 @@ FileDistributor::~FileDistributor()
     assert(numTargets == 0);
 }
 
-bool FileDistributor::moveTo(const LocalPath& source, LocalPath& target, TargetNameExistsResolution method, FileSystemAccess& fsAccess, bool& transient_error, bool& name_too_long, Sync* syncForDebris, const FileFingerprint& confirmFingerprint)
+bool FileDistributor::moveTo(const LocalPath& source,
+                             LocalPath& target,
+                             TargetNameExistsResolution method,
+                             FileSystemAccess& fsAccess,
+                             bool& transient_error,
+                             bool& name_too_long,
+                             [[maybe_unused]] Sync* syncForDebris,
+                             [[maybe_unused]] const FileFingerprint& confirmFingerprint)
 {
     assert (!!syncForDebris == (method == MoveReplacedFileToSyncDebris));
 
@@ -2175,9 +1453,14 @@ bool FileDistributor::moveToForMethod_RenameExistingToOldN(const LocalPath& sour
 }
 
 #ifdef ENABLE_SYNC
-bool FileDistributor::moveToForMethod_MoveReplacedFileToSyncDebris(const LocalPath& source, LocalPath& target, FileSystemAccess& fsAccess,
-                        bool& transient_error, bool& name_too_long, Sync* syncForDebris,
-                        const FileFingerprint& confirmFingerprint)
+bool FileDistributor::moveToForMethod_MoveReplacedFileToSyncDebris(
+    const LocalPath& source,
+    LocalPath& target,
+    FileSystemAccess& fsAccess,
+    bool& transient_error,
+    bool& name_too_long,
+    Sync* syncForDebris,
+    [[maybe_unused]] const FileFingerprint& confirmFingerprint)
 {
         // Move the obstruction to the local debris.
         if (!syncForDebris->movetolocaldebris(target))
@@ -2199,9 +1482,15 @@ bool FileDistributor::moveToForMethod_MoveReplacedFileToSyncDebris(const LocalPa
         return result;
 }
 
-bool FileDistributor::copyToForMethod_MoveReplacedFileToSyncDebris(const LocalPath& source, LocalPath& target, m_time_t mtime, FileSystemAccess& fsAccess,
-                        bool& transient_error, bool& name_too_long, Sync* syncForDebris,
-                        const FileFingerprint& confirmFingerprint)
+bool FileDistributor::copyToForMethod_MoveReplacedFileToSyncDebris(
+    const LocalPath& source,
+    LocalPath& target,
+    m_time_t mtime,
+    FileSystemAccess& fsAccess,
+    bool& transient_error,
+    bool& name_too_long,
+    Sync* syncForDebris,
+    [[maybe_unused]] const FileFingerprint& confirmFingerprint)
 {
         // Move the obstruction to the local debris.
         if (!syncForDebris->movetolocaldebris(target))
@@ -2278,8 +1567,14 @@ bool FileDistributor::copyToForMethod_RenameExistingToOldN(const LocalPath& sour
         }
 }
 
-bool FileDistributor::copyToForMethod_OverwriteTarget(const LocalPath& source, LocalPath& target, m_time_t mtime,
-                        FileSystemAccess& fsAccess, bool& transient_error, bool& name_too_long, const FileFingerprint& confirmFingerprint)
+bool FileDistributor::copyToForMethod_OverwriteTarget(
+    const LocalPath& source,
+    LocalPath& target,
+    m_time_t mtime,
+    FileSystemAccess& fsAccess,
+    bool& transient_error,
+    bool& name_too_long,
+    [[maybe_unused]] const FileFingerprint& confirmFingerprint)
 {
     if (fsAccess.copylocal(source, target, mtime))//copylocal is implemented as always overwrite
     {
@@ -2294,7 +1589,15 @@ bool FileDistributor::copyToForMethod_OverwriteTarget(const LocalPath& source, L
     }
 }
 
-bool FileDistributor::copyTo(const LocalPath& source, LocalPath& target, m_time_t mtime, TargetNameExistsResolution method, FileSystemAccess& fsAccess, bool& transient_error, bool& name_too_long, Sync* syncForDebris, const FileFingerprint& confirmFingerprint)
+bool FileDistributor::copyTo(const LocalPath& source,
+                             LocalPath& target,
+                             m_time_t mtime,
+                             TargetNameExistsResolution method,
+                             FileSystemAccess& fsAccess,
+                             bool& transient_error,
+                             bool& name_too_long,
+                             [[maybe_unused]] Sync* syncForDebris,
+                             const FileFingerprint& confirmFingerprint)
 {
     assert (!!syncForDebris == (method == MoveReplacedFileToSyncDebris));
     assert(FSNode::debugConfirmOnDiskFingerprintOrLogWhy(fsAccess, source, confirmFingerprint));
@@ -2641,6 +1944,12 @@ bool FSNode::debugConfirmOnDiskFingerprintOrLogWhy(FileSystemAccess& fsAccess, c
     if (unique_ptr<FSNode> od = fromPath(fsAccess, path, false, FSLogging::logOnError))
     {
         if (od->fingerprint == ff) return true;
+#ifdef __ANDROID__
+        if (od->fingerprint.equalExceptMtime(ff))
+        {
+            return true;
+        }
+#endif
 
         LOG_debug << "fingerprint mismatch at path: " << path;
         LOG_debug << "size: " << od->fingerprint.size << " should have been " << ff.size;
@@ -2655,7 +1964,22 @@ bool FSNode::debugConfirmOnDiskFingerprintOrLogWhy(FileSystemAccess& fsAccess, c
     return false;
 }
 
+auto FileSystemAccess::getFileSize(const LocalPath& path)
+    -> std::optional<std::pair<std::uint64_t, std::uint64_t>>
+{
+    // Create an FA so we can access the file at path.
+    auto fileAccess = newfileaccess(false);
 
+    // Couldn't create an FA.
+    if (!fileAccess)
+        return std::nullopt;
+
+    // Couldn't open the file.
+    if (!fileAccess->fopen(path, true, false, FSLogging::logOnError))
+        return std::nullopt;
+
+    // Return the file's allocated and reported size to our caller.
+    return fileAccess->getFileSize();
+}
 
 } // namespace
-
