@@ -20,14 +20,116 @@
  */
 
 #include "mega/nodemanager.h"
-#include "mega/megaclient.h"
+
 #include "mega/base64.h"
 #include "mega/megaapp.h"
+#include "mega/megaclient.h"
 #include "mega/share.h"
 
-
 namespace mega {
 
+NodeManager::NoKeyLogger NodeManager::mNoKeyLogger{};
+
+bool NodeSearchFilter::isValidNodeType(const nodetype_t nodeType) const
+{
+    return mNodeType == nodeType;
+}
+
+bool NodeSearchFilter::isValidCreationTime(const int64_t time) const
+{
+    if (mCreationLowerLimit && time <= mCreationLowerLimit)
+        return false;
+    if (mCreationUpperLimit && time >= mCreationUpperLimit)
+        return false;
+    return true;
+}
+
+bool NodeSearchFilter::isValidModificationTime(const int64_t time) const
+{
+    if (mModificationLowerLimit && time <= mModificationLowerLimit)
+        return false;
+    if (mModificationUpperLimit && (time <= 0 || time >= mModificationUpperLimit))
+        return false;
+    return true;
+}
+
+bool NodeSearchFilter::isValidCategory(const MimeType_t category, const nodetype_t nodeType) const
+{
+    if (nodeType != FILENODE)
+        return false;
+    if (mMimeCategory == MIME_TYPE_ALL_DOCS && isDocType(category))
+        return true;
+    return category == mMimeCategory;
+}
+
+bool NodeSearchFilter::isValidName(const uint8_t* testName) const
+{
+    const auto& pattern = mNameFilter.getPattern();
+    if (pattern.empty() || !testName)
+        return true;
+    return likeCompare(pattern.c_str(), reinterpret_cast<const char*>(testName));
+}
+
+bool NodeSearchFilter::isValidDescription(const uint8_t* testDescription) const
+{
+    const auto& pattern = mDescriptionFilter.getPattern();
+    if (pattern.empty())
+        return true;
+    return testDescription &&
+           likeCompare(pattern.c_str(), reinterpret_cast<const char*>(testDescription));
+}
+
+bool NodeSearchFilter::isValidTagSequence(const uint8_t* tagSequence) const
+{
+    if (!hasTag())
+        return true;
+    // we know tags can not contain delimiter
+    if (!tagSequence || mTagFilterContainsSeparator)
+        return false;
+    auto tokens = splitString(reinterpret_cast<const char*>(tagSequence), TAG_DELIMITER);
+    return getTagPosition(tokens, mTagFilter.getPattern()) != tokens.end();
+}
+
+bool NodeSearchFilter::isValidFav(const bool isNodeFav) const
+{
+    return !hasFav() || (mFavouriteFilterOption == BoolFilter::onlyTrue) == isNodeFav;
+}
+
+bool NodeSearchFilter::isValidSensitivity(const bool isNodeSensitive) const
+{
+    if (!hasSensitive())
+        return true;
+    if (isNodeSensitive && mExcludeSensitive == BoolFilter::onlyFalse)
+        return true;
+    if (!isNodeSensitive && mExcludeSensitive == BoolFilter::onlyTrue)
+        return true;
+    return false;
+}
+
+bool NodeSearchFilter::isDocType(const MimeType_t t)
+{
+    switch (t)
+    {
+        case MIME_TYPE_DOCUMENT:
+        case MIME_TYPE_PDF:
+        case MIME_TYPE_PRESENTATION:
+        case MIME_TYPE_SPREADSHEET:
+            return true;
+        default:
+            return false;
+    }
+}
+
+// Log the first 256 entries and then every 256 entries
+void NodeManager::NoKeyLogger::log(const Node& n) const
+{
+    auto current = mCount.fetch_add(1);
+    if (current <= 256 || (current % 256 == 0))
+    {
+        LOG_debug << "Storing an encrypted node[" << current << "]: " << n.type << " " << n.size
+                  << " " << Base64Str<MegaClient::NODEHANDLE>(n.nodehandle);
+    }
+}
 
 NodeManager::NodeManager(MegaClient& client)
     : mClient(client)
@@ -58,6 +160,7 @@ void NodeManager::reset_internal()
     assert(mMutex.owns_lock());
     setTable_internal(nullptr);
     cleanNodes_internal();
+    mNullRootNodesReported = false;
 }
 
 bool NodeManager::setrootnode(std::shared_ptr<Node> node)
@@ -140,8 +243,11 @@ void NodeManager::notifyNode_internal(std::shared_ptr<Node> n, sharedNode_vector
 
             int attrlen = int(n->attrstring->size());
             string base64attrstring;
-            base64attrstring.resize(attrlen * 4 / 3 + 4);
-            base64attrstring.resize(Base64::btoa((byte *)n->attrstring->data(), int(n->attrstring->size()), (char *)base64attrstring.data()));
+            base64attrstring.resize(static_cast<size_t>(attrlen * 4 / 3 + 4));
+            base64attrstring.resize(
+                static_cast<size_t>(Base64::btoa((byte*)n->attrstring->data(),
+                                                 int(n->attrstring->size()),
+                                                 (char*)base64attrstring.data())));
 
             char report[512];
             Base64::btoa((const byte *)&n->nodehandle, MegaClient::NODEHANDLE, report);
@@ -188,7 +294,7 @@ bool NodeManager::addNode_internal(std::shared_ptr<Node> node, bool notify, bool
     // 'notify' is false when loading nodes from API or DB. True when node is received from
     // actionpackets and/or from response of CommandPutnodes
 
-    bool rootNode = node->type == ROOTNODE || node->type == RUBBISHNODE || node->type == VAULTNODE;
+    bool rootNode = isFromRootNodeType(*node.get());
 
     // getRootNodeFiles() is always set for folder links before adding any node (upon login)
     bool isFolderLink = rootnodes.files == node->nodeHandle();
@@ -243,6 +349,24 @@ bool NodeManager::updateNode_internal(Node *node)
     return true;
 }
 
+void NodeManager::reportNullRootNodes(const size_t rootNodesSize)
+{
+    if (mNullRootNodesReported)
+    {
+        return;
+    }
+
+    rootNodesSize >= rootnodes.MIN_NUM_ROOT_NODES ?
+        mClient.sendevent(99490, "Null rootnode/s detected", 0) :
+        mClient.sendevent(99491,
+                          "Null rootnode/s detected and wrong number of root nodes retrieved",
+                          0);
+    LOG_err << "getNodeCount_internal: Null rootnode/s detected. Number of root nodes: "
+            << rootNodesSize;
+    mNullRootNodesReported = true;
+    assert(false);
+}
+
 std::shared_ptr<Node> NodeManager::getNodeByHandle(NodeHandle handle)
 {
     LockGuard g(mMutex);
@@ -268,13 +392,17 @@ std::shared_ptr<Node> NodeManager::getNodeByHandle_internal(NodeHandle handle)
     return node;
 }
 
-sharedNode_list NodeManager::getChildren(const Node *parent, CancelToken cancelToken)
+sharedNode_list NodeManager::getChildren(const Node* parent,
+                                         CancelToken cancelToken,
+                                         bool includeVersions)
 {
     LockGuard g(mMutex);
-    return getChildren_internal(parent, cancelToken);
+    return getChildren_internal(parent, cancelToken, includeVersions);
 }
 
-sharedNode_list NodeManager::getChildren_internal(const Node *parent, CancelToken cancelToken)
+sharedNode_list NodeManager::getChildren_internal(const Node* parent,
+                                                  CancelToken cancelToken,
+                                                  bool includeVersions)
 {
     assert(mMutex.owns_lock());
 
@@ -333,6 +461,7 @@ sharedNode_list NodeManager::getChildren_internal(const Node *parent, CancelToke
 
         std::vector<std::pair<NodeHandle, NodeSerialized>> nodesFromTable;
         NodeSearchFilter nf;
+        nf.includeVersions(includeVersions);
         nf.byAncestors({parent->nodehandle, UNDEF, UNDEF});
         mTable->getChildren(nf,
                             0 /*Order none*/,
@@ -425,13 +554,56 @@ sharedNode_vector NodeManager::getChildren_internal(const NodeSearchFilter& filt
     return nodes;
 }
 
-sharedNode_vector NodeManager::getRecentNodes(unsigned maxcount, m_time_t since)
+sharedNode_vector NodeManager::getRecentNodes(unsigned maxcount,
+                                              m_time_t since,
+                                              bool excludeSensitives)
 {
     LockGuard g(mMutex);
-    return getRecentNodes_internal(maxcount, since);
+
+    sharedNode_vector result = getRecentNodes_internal(NodeSearchPage{0, maxcount}, since);
+    if (!excludeSensitives)
+        return result;
+
+    const auto isSensitive = [](const std::shared_ptr<Node>& node) -> bool
+    {
+        return node && node->isSensitiveInherited();
+    };
+    const auto filterSensitives = [&isSensitive](sharedNode_vector& v) -> void
+    {
+        auto it = std::remove_if(std::begin(v), std::end(v), isSensitive);
+        v.erase(it, std::end(v));
+    };
+
+    filterSensitives(result);
+    if (result.size() == maxcount)
+        return result;
+
+    // Keep asking for more no sensitive nodes to the db
+    unsigned start = maxcount;
+    unsigned querySize = maxcount;
+    while (true)
+    {
+        auto moreResults = getRecentNodes_internal(NodeSearchPage{start, querySize}, since);
+        if (moreResults.empty()) // No more potential results
+            return result;
+        filterSensitives(moreResults);
+        if (const auto remaining = maxcount - result.size(); moreResults.size() > remaining)
+        {
+            result.insert(std::end(result),
+                          std::begin(moreResults),
+                          std::begin(moreResults) + static_cast<long>(remaining));
+            return result;
+        }
+        result.insert(std::end(result), std::begin(moreResults), std::end(moreResults));
+
+        start += querySize;
+        constexpr unsigned MAX_QUERY_SIZE = 100000U;
+        if (querySize < MAX_QUERY_SIZE)
+            querySize *= 2;
+    }
 }
 
-sharedNode_vector NodeManager::getRecentNodes_internal(unsigned maxcount, m_time_t since)
+sharedNode_vector NodeManager::getRecentNodes_internal(const NodeSearchPage& page, m_time_t since)
 {
     assert(mMutex.owns_lock());
 
@@ -441,7 +613,7 @@ sharedNode_vector NodeManager::getRecentNodes_internal(unsigned maxcount, m_time
     }
 
     std::vector<std::pair<NodeHandle, NodeSerialized>> nodesFromTable;
-    mTable->getRecentNodes(maxcount, since, nodesFromTable);
+    mTable->getRecentNodes(page, since, nodesFromTable);
 
     return processUnserializedNodes(nodesFromTable);
 }
@@ -464,10 +636,17 @@ uint64_t NodeManager::getNodeCount_internal()
     uint64_t count = 0;
     sharedNode_vector roots = getRootNodesAndInshares();
 
-    for (auto& node : roots)
+    for (auto& node: roots)
     {
-        NodeCounter nc = node->getCounter();
-        count += nc.files + nc.folders + nc.versions;
+        if (node)
+        {
+            NodeCounter nc = node->getCounter();
+            count += nc.files + nc.folders + nc.versions;
+        }
+        else
+        {
+            reportNullRootNodes(roots.size());
+        }
     }
 
     // add roots to the count if logged into account (and fetchnodes is done <- roots are ready)
@@ -507,37 +686,50 @@ uint64_t NodeManager::getNodeCount_internal()
     return count;
 }
 
-std::set<std::string> NodeManager::getAllNodeTags(const char* searchString, CancelToken cancelFlag)
+auto NodeManager::getNodeTagsBelow(CancelToken cancelToken,
+                                   const std::set<NodeHandle>& handles,
+                                   const std::string& pattern)
+    -> std::optional<std::set<std::string>>
 {
-    LockGuard g(mMutex);
-    return getAllNodeTags_internal(searchString, cancelFlag);
-}
+    // Make sure we have exclusive access to the database.
+    LockGuard guard(mMutex);
 
-std::set<std::string> NodeManager::getAllNodeTags_internal(const char* searchString,
-                                                           CancelToken cancelFlag)
-{
-    assert(mMutex.owns_lock());
-    // validation
+    // Convenience.
+    static const auto warning = [](const char* message)
+    {
+        LOG_warn << "getNodeTagsBelow: " << message;
+        return std::nullopt;
+    }; // warning
+
+    // Make sure the database is sane and that some nodes exist.
     if (!mTable || mNodes.empty())
-    {
-        LOG_warn
-            << "getAllNodeTags_internal: The database is not opened or there are no nodes loaded";
-        assert(mTable && !mNodes.empty());
-        return {};
-    }
-    const std::string auxSearchString = searchString ? searchString : "";
-    // ',' cannot be in the string
-    if (auxSearchString.find(MegaClient::TAG_DELIMITER) != std::string::npos)
-    {
-        LOG_warn << "getAllNodeTags_internal: The search string (" << auxSearchString
-                 << ") contains an invalid character (,)";
-        return {};
-    }
-    std::set<std::string> result;
-    if (!mTable->getAllNodeTags(auxSearchString, result, cancelFlag))
-        return {};
+        return warning("The database hasn't been opened or there are no nodes present");
 
-    return result;
+    // Make sure the caller isn't trying to filter by multiple tags simultaneously.
+    if (pattern.find(MegaClient::TAG_DELIMITER) != std::string::npos)
+        return warning("You can't filter by multiple tags at the same time");
+
+    std::optional<std::set<std::string>> accumulatedTags;
+
+    // Try and retrieve the tags below the specified nodes.
+    for (const auto& handle: handles)
+    {
+        // Try and retrieve tags below this node.
+        auto tags = mTable->getNodeTagsBelow(cancelToken, handle, pattern);
+
+        // Couldn't get tags.
+        if (!tags)
+            continue;
+
+        // Merge tags into accumulated result if possible.
+        if (accumulatedTags)
+            accumulatedTags->merge(*tags);
+        else
+            accumulatedTags = std::move(tags);
+    }
+
+    // Return accumulated tags to caller.
+    return accumulatedTags;
 }
 
 sharedNode_vector NodeManager::searchNodes(const NodeSearchFilter& filter, int order, CancelToken cancelFlag, const NodeSearchPage& page)
@@ -608,13 +800,13 @@ sharedNode_vector NodeManager::getNodesWithLinks()
     return getNodesWithSharesOrLink_internal(ShareType_t::LINK);
 }
 
-sharedNode_vector NodeManager::getNodesByFingerprint(FileFingerprint &fingerprint)
+sharedNode_vector NodeManager::getNodesByFingerprint(const FileFingerprint& fingerprint)
 {
     LockGuard g(mMutex);
     return getNodesByFingerprint_internal(fingerprint);
 }
 
-sharedNode_vector NodeManager::getNodesByFingerprint_internal(FileFingerprint &fingerprint)
+sharedNode_vector NodeManager::getNodesByFingerprint_internal(const FileFingerprint& fingerprint)
 {
     assert(mMutex.owns_lock());
 
@@ -630,7 +822,7 @@ sharedNode_vector NodeManager::getNodesByFingerprint_internal(FileFingerprint &f
     auto p = mFingerPrints.equal_range(&fingerprint);
     for (auto it = p.first; it != p.second; ++it)
     {
-        Node* node = static_cast<Node*>(*it);
+        const auto node = static_cast<const Node*>(*it);
         fpLoaded.emplace(node->nodeHandle());
         std::shared_ptr<Node> sharedNode = node->mNodePosition->second.getNodeInRam();
         assert(sharedNode && "Node loaded at fingerprint map should have a node in RAM ");
@@ -725,7 +917,7 @@ std::shared_ptr<Node> NodeManager::getNodeByFingerprint_internal(FileFingerprint
     auto it = mFingerPrints.find(&fingerprint);
     if (it != mFingerPrints.end())
     {
-        Node *n = static_cast<Node*>(*it);
+        const auto n = static_cast<const Node*>(*it);
         assert(n);
         return n->mNodePosition->second.getNodeInRam();
     }
@@ -830,25 +1022,52 @@ sharedNode_vector NodeManager::getRootNodes_internal()
     {
         const auto loadVault = [this, &nodes]() -> void
         {
-            std::shared_ptr<Node> inBox = rootnodes.mRootNodes[VAULTNODE];
-            assert(inBox && "Vault node should be defined (except logged into folder link)");
-            nodes.push_back(std::move(inBox));
+            auto vaultIt = rootnodes.mRootNodes.find(VAULTNODE);
+            if (vaultIt != rootnodes.mRootNodes.end() && vaultIt->second)
+            {
+                nodes.push_back(vaultIt->second);
+            }
+            else
+            {
+                LOG_err << "Vault node should be defined (except logged into folder link)";
+                mClient.sendevent(800032, "Vault node is not present", 0);
+                assert(vaultIt != rootnodes.mRootNodes.end() && "Vault node is not defined");
+                assert(vaultIt->second && "Vault node is defined but it is null");
+            }
         };
 
         if (mClient.isClientType(MegaClient::ClientType::DEFAULT))
         {
-            std::shared_ptr<Node> rootNode = rootnodes.mRootNodes[ROOTNODE];
-            assert(rootNode && "Root node should be defined");
-            nodes.push_back(std::move(rootNode));
+            auto rootIt = rootnodes.mRootNodes.find(ROOTNODE);
+            if (rootIt != rootnodes.mRootNodes.end() && rootIt->second)
+            {
+                nodes.push_back(rootIt->second);
+            }
+            else
+            {
+                LOG_err << "Root node should be defined";
+                mClient.sendevent(800031, "Root node is not present", 0);
+                assert(rootIt != rootnodes.mRootNodes.end() && "Root node is not defined");
+                assert(rootIt->second && "Root node is defined but it is null");
+            }
 
             if (!mClient.loggedIntoFolder())
             {
                 loadVault();
 
-                std::shared_ptr<Node> rubbish = rootnodes.mRootNodes[RUBBISHNODE];
-                assert(rubbish &&
-                       "Rubbishbin node should be defined (except logged into folder link)");
-                nodes.push_back(std::move(rubbish));
+                auto rubbishIt = rootnodes.mRootNodes.find(RUBBISHNODE);
+                if (rubbishIt != rootnodes.mRootNodes.end() && rubbishIt->second)
+                {
+                    nodes.push_back(rubbishIt->second);
+                }
+                else
+                {
+                    LOG_err << "Rubbishbin node should be defined (except logged into folder link)";
+                    mClient.sendevent(800033, "Rubbishbin node is not present", 0);
+                    assert(rubbishIt != rootnodes.mRootNodes.end() &&
+                           "Rubbishbin node is not defined");
+                    assert(rubbishIt->second && "Rubbishbin node is defined but it is null");
+                }
             }
         }
         else if (mClient.isClientType(MegaClient::ClientType::PASSWORD_MANAGER))
@@ -1100,7 +1319,7 @@ size_t NodeManager::getNumberOfChildrenFromNode_internal(NodeHandle parentHandle
         return parentIt->second.mChildren ? parentIt->second.mChildren->size() : 0;
     }
 
-    return mTable->getNumberOfChildren(parentHandle);
+    return static_cast<size_t>(mTable->getNumberOfChildren(parentHandle));
 }
 
 size_t NodeManager::getNumberOfChildrenByType(NodeHandle parentHandle, nodetype_t nodeType)
@@ -1121,7 +1340,7 @@ size_t NodeManager::getNumberOfChildrenByType_internal(NodeHandle parentHandle,
 
     assert(nodeType == FILENODE || nodeType == FOLDERNODE);
 
-    return mTable->getNumberOfChildrenByType(parentHandle, nodeType);
+    return static_cast<size_t>(mTable->getNumberOfChildrenByType(parentHandle, nodeType));
 }
 
 bool NodeManager::isAncestor(NodeHandle nodehandle, NodeHandle ancestor, CancelToken cancelFlag)
@@ -1153,13 +1372,28 @@ void NodeManager::removeChanges_internal()
 {
     assert(mMutex.owns_lock());
 
-    for (auto& it : mNodes)
+    if (mNodesInRam <= mCacheLRU.size() + rootnodes.mRootNodes.size())
     {
-        std::shared_ptr<Node> node = it.second.getNodeInRam(false);
-        if (node)
+        for (auto& node: mCacheLRU)
         {
             memset(&(node->changed), 0, sizeof node->changed);
         }
+
+        for (auto& [_, node]: rootnodes.mRootNodes)
+        {
+            memset(&(node->changed), 0, sizeof node->changed);
+        }
+    }
+    else
+    {
+        for (auto& [_, nodeManagerNode]: mNodes)
+        {
+            std::shared_ptr<Node> node = nodeManagerNode.getNodeInRam(false);
+            if (node)
+            {
+                memset(&(node->changed), 0, sizeof node->changed);
+            }
+        }
     }
 }
 
@@ -1176,15 +1410,18 @@ void NodeManager::cleanNodes_internal()
     mFingerPrints.clear();
     mNodes.clear();
     mCacheLRU.clear();
-    mNodesInRam = 0;
     mNodeToWriteInDb.reset();
     mNodeNotify.clear();
+    mNodePendingApplyKeys.clear();
 
     rootnodes.clear();
 
     if (mTable) mTable->removeNodes();
 
     mInitialized = false;
+
+    mAppliedKeyNodeCount = 0;
+    mNodesInRam = 0;
 }
 
 std::shared_ptr<Node> NodeManager::getNodeFromBlob(const std::string* nodeSerialized)
@@ -1235,30 +1472,55 @@ shared_ptr<Node> NodeManager::unserializeNode(const std::string *d, bool fromOld
     return nullptr;
 }
 
-void NodeManager::applyKeys(uint32_t appliedKeys)
+void NodeManager::applyKeys()
 {
     LockGuard g(mMutex);
-    applyKeys_internal(appliedKeys);
+    applyKeys_internal();
 }
 
-void NodeManager::applyKeys_internal(uint32_t appliedKeys)
+void NodeManager::addNodePendingApplykey(std::shared_ptr<Node> node)
+{
+    LockGuard g(mMutex);
+    if (isFromRootNodeType(*node.get()))
+    {
+        return;
+    }
+
+    mNodePendingApplyKeys.push_back(node);
+}
+
+void NodeManager::applyKeys_internal()
 {
     assert(mMutex.owns_lock());
 
-    if (mNodes.size() > appliedKeys)
+    for (auto it = mNodePendingApplyKeys.begin(); it != mNodePendingApplyKeys.end();)
     {
-        for (auto& it : mNodes)
+        if (it->get()->applykey() || it->get()->keyApplied())
         {
-            if (shared_ptr<Node> node = it.second.getNodeInRam(false))
-            {
-               node->applykey();
-            }
+            it = mNodePendingApplyKeys.erase(it);
+        }
+        else
+        {
+            it++;
         }
     }
+
+#ifdef DEBUG
+    // In case of folder links, root node is not from type rootnode and it is decryptable
+    unsigned rootNodeUndecrypted =
+        (!getRootNodeFiles().isUndef() && rootnodes.mRootNodes[ROOTNODE]->type == ROOTNODE) ? 1 : 0;
+    unsigned noKeyExpected = rootNodeUndecrypted + (getRootNodeVault().isUndef() ? 0 : 1) +
+                             (getRootNodeRubbish().isUndef() ? 0 : 1);
+
+    assert(mNodesInRam - (static_cast<uint64_t>(mAppliedKeyNodeCount) + noKeyExpected) ==
+           mNodePendingApplyKeys.size());
+#endif
 }
 
 void NodeManager::notifyPurge()
 {
+    mClient.applykeys();
+
     // only lock to get the nodes to report
     sharedNode_vector nodesToReport;
     {
@@ -1270,8 +1532,6 @@ void NodeManager::notifyPurge()
 
     if (!nodesToReport.empty())
     {
-        mClient.applykeys();
-
         if (!mClient.fetchingnodes)
         {
             assert(!mMutex.owns_lock());
@@ -1279,9 +1539,10 @@ void NodeManager::notifyPurge()
         }
 
         LockGuard g(mMutex);
-		
-        // Let FUSE know that nodes have been updated.
-        mClient.mFuseClientAdapter.updated(nodesToReport);
+
+        // Let the client adapter know that nodes have been updated.
+        // FIXME: uncomment the following line as part of SDK-5665 once it's safe to call
+        // mClient.mClientAdapter.updated(nodesToReport);
 
         TransferDbCommitter committer(mClient.tctable);
 
@@ -1293,13 +1554,6 @@ void NodeManager::notifyPurge()
         {
             std::shared_ptr<Node> n = nodesToReport[i];
 
-            if (n->attrstring)
-            {
-                // make this just a warning to avoid auto test failure
-                // this can happen if another client adds a folder in our share and the key for us is not available yet
-                LOG_warn << "NO_KEY node: " << n->type << " " << n->size << " " << toNodeHandle(n->nodehandle) << " " << n->nodekeyUnchecked().size();
-            }
-
             if (n->changed.removed)
             {
                 // remove inbound share
@@ -1380,7 +1634,7 @@ void NodeManager::notifyPurge()
 bool NodeManager::hasCacheLoaded()
 {
     LockGuard g(mMutex);
-    return mNodes.size();
+    return mNodes.size() > 0;
 }
 
 bool NodeManager::loadNodes()
@@ -1399,12 +1653,8 @@ bool NodeManager::loadNodes_internal()
         return false;
     }
 
-    sharedNode_vector rootnodes = getRootNodes_internal();
-    // We can't base in `user.sharing` because it's set yet. We have to get from DB
-    sharedNode_vector inshares =
-        getNodesWithSharesOrLink_internal(ShareType_t::IN_SHARES); // it includes nested inshares
-
-    for (auto &node : rootnodes)
+    sharedNode_vector allRootNodes = getRootNodes_internal();
+    for (const auto& node: allRootNodes)
     {
         getChildren_internal(node.get());
     }
@@ -1549,10 +1799,11 @@ void NodeManager::checkOrphanNodes(MissingParentNodes& nodesWithMissingParent)
                 mClient.proctree(orphan, &td);
 
                 // TODO: Change this warning to an error when Speculative Instant Completion (SIC) is gone
-               LOG_warn << "Detected orphan node: " << toNodeHandle(orphan->nodehandle)
-                        << " Parent: " << toNodeHandle(orphan->parentHandle());
+                LOG_warn << mClient.clientname
+                         << "Detected orphan node: " << toNodeHandle(orphan->nodehandle)
+                         << " Parent: " << toNodeHandle(orphan->parentHandle());
 
-               mClient.sendevent(99455, "Orphan node(s) detected");
+                mClient.sendevent(99455, "Orphan node(s) detected");
 
                 // If we didn't get all the parents of all the (not inshare) nodes,
                 // then the API is sending us inconsistent data,
@@ -1569,6 +1820,18 @@ void NodeManager::initCompleted()
     initCompleted_internal();
 }
 
+void NodeManager::dropSearchDBIndexes()
+{
+    assert(mNodeNotify.empty());
+    if (!mTable || mNodesInRam > 0)
+    {
+        LOG_err << "DB isn't opened yet or nodes has been already loaded";
+        return;
+    }
+
+    mTable->dropSearchDBIndexes();
+}
+
 std::shared_ptr<Node> NodeManager::getNodeFromNodeManagerNode(NodeManagerNode& nodeManagerNode)
 {
     LockGuard g(mMutex);
@@ -1589,12 +1852,22 @@ void NodeManager::insertNodeCacheLRU(std::shared_ptr<Node> node)
 
 void NodeManager::increaseNumNodesInRam()
 {
-    mNodesInRam++;
+    ++mNodesInRam;
 }
 
 void NodeManager::decreaseNumNodesInRam()
 {
-    mNodesInRam--;
+    --mNodesInRam;
+}
+
+void NodeManager::increaseNumNodesAppliedKey()
+{
+    ++mAppliedKeyNodeCount;
+}
+
+void NodeManager::decreaseNumNodesAppliedKey()
+{
+    --mAppliedKeyNodeCount;
 }
 
 uint64_t NodeManager::getCacheLRUMaxSize() const
@@ -1627,12 +1900,19 @@ void NodeManager::initCompleted_internal()
     }
 
     sharedNode_vector rootNodes = getRootNodesAndInshares();
-    for (auto& node : rootNodes)
+    for (auto& node: rootNodes)
     {
-        calculateNodeCounter(node->nodeHandle(), TYPE_UNKNOWN, node, node->type == RUBBISHNODE);
+        if (node)
+        {
+            calculateNodeCounter(node->nodeHandle(), TYPE_UNKNOWN, node, node->type == RUBBISHNODE);
+        }
+        else
+        {
+            reportNullRootNodes(rootNodes.size());
+        }
     }
 
-    mTable->createIndexes();
+    mTable->createIndexes(mClient.mEnableSearchDBIndexes);
     mInitialized = true;
 }
 
@@ -1641,6 +1921,11 @@ bool NodeManager::ready()
     return mInitialized;
 }
 
+bool NodeManager::isFromRootNodeType(const Node& node) const
+{
+    return node.type == ROOTNODE || node.type == RUBBISHNODE || node.type == VAULTNODE;
+}
+
 void NodeManager::insertNodeCacheLRU_internal(std::shared_ptr<Node> node)
 {
     assert(mMutex.owns_lock() && "Mutex should be locked by this thread");
@@ -1696,9 +1981,16 @@ NodeCounter NodeManager::getCounterOfRootNodes_internal()
     }
 
     sharedNode_vector rootNodes = getRootNodes_internal();
-    for (auto& node : rootNodes)
+    for (auto& node: rootNodes)
     {
-        c += node->getCounter();
+        if (node)
+        {
+            c += node->getCounter();
+        }
+        else
+        {
+            reportNullRootNodes(rootNodes.size());
+        }
     }
 
     return c;
@@ -1824,7 +2116,7 @@ void NodeManager::dumpNodes_internal()
         }
     }
 
-    mTable->createIndexes();
+    mTable->createIndexes(mClient.mEnableSearchDBIndexes);
     mInitialized = true;
 }
 
@@ -1849,7 +2141,6 @@ void NodeManager::saveNodeInDb_internal(Node *node)
     if (mNodeToWriteInDb)   // not to be kept in memory
     {
         assert(mNodeToWriteInDb.get() == node);
-        assert(mNodeToWriteInDb.use_count() == 2);
         mNodeToWriteInDb.reset();
     }
 }
@@ -1860,6 +2151,12 @@ uint64_t NodeManager::getNumberNodesInRam() const
     return mNodesInRam;
 }
 
+long long NodeManager::getNumNodesKeyApplied() const
+{
+    LockGuard g(mMutex);
+    return mAppliedKeyNodeCount;
+}
+
 void NodeManager::addChild(NodeHandle parent, NodeHandle child, Node* node)
 {
     LockGuard g(mMutex);
@@ -1927,16 +2224,16 @@ shared_ptr<Node> NodeManager::getNodeFromDataBase(NodeHandle handle)
 sharedNode_vector NodeManager::getRootNodesAndInshares()
 {
     assert(mMutex.owns_lock());
-    sharedNode_vector rootnodes;
+    sharedNode_vector allRootNodes;
 
-    rootnodes = getRootNodes_internal();
+    allRootNodes = getRootNodes_internal();
     if (!mClient.loggedIntoFolder()) // logged into user's account: incoming shared folders
     {
         sharedNode_vector inshares = mClient.getInShares();
-        rootnodes.insert(rootnodes.end(), inshares.begin(), inshares.end());
+        allRootNodes.insert(allRootNodes.end(), inshares.begin(), inshares.end());
     }
 
-    return rootnodes;
+    return allRootNodes;
 }
 
 sharedNode_vector NodeManager::processUnserializedNodes(const vector<pair<NodeHandle, NodeSerialized>>& nodesFromTable, CancelToken cancelFlag)
@@ -2014,13 +2311,12 @@ void NodeManager::putNodeInDb(Node* node) const
     if (node->attrstring)
     {
         // Last attempt to decrypt the node before storing it.
-        LOG_debug << "Trying to store an encrypted node";
         node->applykey();
         node->setattr();
 
         if (node->attrstring)
         {
-            LOG_debug << "Storing an encrypted node.";
+            mNoKeyLogger.log(*node);
         }
     }
 
