@@ -27,18 +27,23 @@
 #endif // ! __APPLE__
 
 #include "mega.h"
-#include <sys/utsname.h>
+#include "mega/scoped_helpers.h"
+
 #include <sys/ioctl.h>
-#include <sys/statvfs.h>
 #include <sys/resource.h>
+#include <sys/statvfs.h>
 #include <sys/types.h>
+#include <sys/utsname.h>
 #ifdef TARGET_OS_MAC
 #include "mega/osx/osxutils.h"
 #endif
 
 #ifdef __ANDROID__
+#include <mega/android/androidFileSystem.h>
+
 #include <jni.h>
-extern JavaVM *MEGAjvm;
+extern JavaVM* MEGAjvm;
+extern jclass fileWrapper;
 #endif
 
 #if defined(__MACH__) && !(TARGET_OS_IPHONE)
@@ -137,14 +142,14 @@ AdjustBasePathResult adjustBasePath(const LocalPath& path)
 
     // No base path.
     if (basePath.empty())
-        return path.rawValue();
+        return path.asPlatformEncoded(false);
 
     // Path is absolute.
     if (path.beginsWithSeparator())
-        return path.rawValue();
+        return path.asPlatformEncoded(false);
 
     // Compute absolute path.
-    basePath.append(path.rawValue());
+    basePath.append(path.asPlatformEncoded(false));
 
     // Return absolute path to caller.
     return basePath;
@@ -154,7 +159,7 @@ AdjustBasePathResult adjustBasePath(const LocalPath& path)
 
 AdjustBasePathResult adjustBasePath(const LocalPath& path)
 {
-    return path.rawValue();
+    return path.asPlatformEncoded(false);
 }
 
 #endif // ! USE_IOS
@@ -179,18 +184,6 @@ void FileSystemAccess::setMinimumFilePermissions(int permissions)
     mMinimumFilePermissions = permissions & 07777;
 }
 
-auto LocalPath::asPlatformEncoded(bool) const -> string_type
-{
-    return localpath;
-}
-bool LocalPath::isRootPath() const
-{
-    if (isFromRoot)
-        return localpath.size() == 1 && localpath.back() == '/';
-
-    return false;
-}
-
 int platformCompareUtf(const string& p1, bool unescape1, const string& p2, bool unescape2)
 {
     return compareUtf(p1, unescape1, p2, unescape2, false);
@@ -257,6 +250,41 @@ PosixFileAccess::~PosixFileAccess()
     fclose();
 }
 
+bool PosixFileAccess::setSparse()
+{
+    // Whether or not a file is sparse is up to the filesystem.
+    return true;
+}
+
+auto PosixFileAccess::getFileSize() const -> std::optional<std::pair<std::uint64_t, std::uint64_t>>
+{
+    // File isn't open.
+    if (fd < 0)
+        return std::nullopt;
+
+    struct stat attributes;
+
+    // Couldn't retrieve the file's attributes.
+    if (::fstat(fd, &attributes) < 0)
+    {
+        // Latch error code.
+        auto error = errno;
+
+        // Let debuggers know why we couldn't get the file's sizes.
+        LOG_err << "Unable to stat descriptor: " << fd << ". Error was: " << error;
+
+        return std::nullopt;
+    }
+
+    // Note that st_blocks is reported in units of 512B sectors.
+    auto allocatedSize = static_cast<std::uint64_t>(attributes.st_blocks) * 512ul;
+
+    // st_size is reported in units of bytes.
+    auto reportedSize = static_cast<std::uint64_t>(attributes.st_size);
+
+    return std::make_pair(allocatedSize, reportedSize);
+}
+
 bool PosixFileAccess::sysstat(m_time_t* mtime, m_off_t* size, FSLogging)
 {
     AdjustBasePathResult nameStr = adjustBasePath(nonblocking_localname);
@@ -512,15 +540,28 @@ void PosixFileAccess::updatelocalname(const LocalPath& name, bool force)
     }
 }
 
-bool PosixFileAccess::sysread(byte* dst, unsigned len, m_off_t pos)
+bool PosixFileAccess::sysread(void* buffer, unsigned long length, m_off_t offset, bool* cretry)
 {
-    retry = false;
-#ifndef __ANDROID__
-    return pread(fd, (char*)dst, len, pos) == len;
-#else
-    lseek64(fd, pos, SEEK_SET);
-    return read(fd, (char*)dst, len) == len;
-#endif
+    // Sanity.
+    assert(buffer || !length);
+    assert(offset >= 0);
+
+    // Keeps logic simple.
+    if (!cretry)
+        cretry = &retry;
+
+    // Reads are never retriable on POSIX systems.
+    *cretry = false;
+
+    // Perform the read.
+    auto result = pread(fd, buffer, length, offset);
+
+    // Read failed.
+    if (result < 0)
+        return false;
+
+    // Read was successful if all bytes were read.
+    return static_cast<unsigned long>(result) == length;
 }
 
 void PosixFileAccess::fclose()
@@ -538,15 +579,43 @@ void PosixFileAccess::fclose()
     fd = -1;
 }
 
-bool PosixFileAccess::fwrite(const byte* data, unsigned len, m_off_t pos)
+bool PosixFileAccess::fwrite(const void* buffer,
+                             unsigned long length,
+                             m_off_t offset,
+                             unsigned long* numWritten,
+                             bool* cretry)
 {
-    retry = false;
-#ifndef __ANDROID__
-    return pwrite(fd, data, len, pos) == len;
-#else
-    lseek64(fd, pos, SEEK_SET);
-    return write(fd, data, len) == len;
-#endif
+    // Sanity.
+    assert(buffer || !length);
+    assert(offset >= 0);
+
+    // Keeps logic simple.
+    if (!cretry)
+        cretry = &retry;
+
+    auto numWritten_ = 0ul;
+
+    if (!numWritten)
+        numWritten = &numWritten_;
+
+    // Assume we can't write any data to file.
+    *numWritten = 0;
+
+    // Write failures are not retriable on POSIX systems.
+    *cretry = false;
+
+    // Try and perform the write.
+    auto result = pwrite(fd, buffer, length, offset);
+
+    // Couldn't perform the write.
+    if (result < 0)
+        return false;
+
+    // Let the user know how many bytes were written.
+    *numWritten = static_cast<unsigned long>(result);
+
+    // Write is successful if all bytes were be written.
+    return length == *numWritten;
 }
 
 bool PosixFileAccess::fstat(m_time_t& modified, m_off_t& size)
@@ -595,7 +664,14 @@ int PosixFileAccess::stealFileDescriptor()
     return toret;
 }
 
-bool PosixFileAccess::fopen(const LocalPath& f, bool read, bool write, FSLogging fsl, DirAccess* iteratingDir, bool, bool skipcasecheck, LocalPath* actualLeafNameIfDifferent)
+bool PosixFileAccess::fopen(const LocalPath& f,
+                            bool read,
+                            bool write,
+                            FSLogging fsl,
+                            DirAccess* iteratingDir,
+                            bool,
+                            [[maybe_unused]] bool skipcasecheck,
+                            [[maybe_unused]] LocalPath* actualLeafNameIfDifferent)
 {
     struct stat statbuf;
 
@@ -615,17 +691,15 @@ bool PosixFileAccess::fopen(const LocalPath& f, bool read, bool write, FSLogging
     if (!write)
     {
         char resolved_path[PATH_MAX];
-        if (memcmp(fstr.c_str(), ".", 2) && memcmp(fstr.c_str(), "..", 3)
-                && (statok || !lstat(fstr.c_str(), &statbuf) )
-                && !S_ISLNK(statbuf.st_mode)
-                && realpath(fstr.c_str(), resolved_path) == resolved_path)
+        if (fstr != "." && fstr != ".." && (statok || !lstat(fstr.c_str(), &statbuf)) &&
+            !S_ISLNK(statbuf.st_mode) && realpath(fstr.c_str(), resolved_path) == resolved_path)
         {
             const char *fname;
             size_t fnamesize;
             if ((fname = strrchr(fstr.c_str(), '/')))
             {
                 fname++;
-                fnamesize = fstr.size() - (fname - fstr.c_str());
+                fnamesize = fstr.size() - (static_cast<size_t>(fname - fstr.c_str()));
             }
             else
             {
@@ -646,18 +720,25 @@ bool PosixFileAccess::fopen(const LocalPath& f, bool read, bool write, FSLogging
             }
             rnamesize = strlen(rname) + 1;
 
+            bool equalNameAndCapitalization =
+                (rnamesize == fnamesize && !memcmp(fname, rname, fnamesize));
+
+            if (actualLeafNameIfDifferent && !equalNameAndCapitalization)
+            {
+                *actualLeafNameIfDifferent = LocalPath::fromRelativePath(std::string(rname));
+            }
+
             if (!skipcasecheck)
             {
                 if (rnamesize == fnamesize && memcmp(fname, rname, fnamesize))
                 {
-                    LOG_warn << "fopen failed due to invalid case: " << fstr;
+                    LOG_warn << "fopen failed due to invalid case: " << fstr
+                             << "   File system real name: " << rname;
                     return false;
                 }
             }
         }
     }
-#else
-    (void)skipcasecheck; // avoid unused parameter warning
 #endif
 
 #ifndef HAVE_FDOPENDIR
@@ -719,11 +800,34 @@ bool PosixFileAccess::fopen(const LocalPath& f, bool read, bool write, FSLogging
 
     assert(fd < 0 && "There should be no opened file descriptor at this point");
     sysclose();
-    // if mFollowSymLinks is true (open normally: it will open the targeted file/folder),
-    // otherwise, get the file descriptor for symlinks in case it is a sync link (notice O_PATH invalidates read/only flags)
+
+    // Compute open flags.
+    auto openFlags = [&]()
+    {
+        // We're dealing with a symlink that isn't being followed.
+        if (!mFollowSymLinks && mIsSymLink)
+            return O_NOFOLLOW | O_PATH;
+
+        // Sanity.
+        assert(read || write);
+
+        // Caller only wants to read the file.
+        if (!write)
+            return O_RDONLY;
+
+        // Assume caller wants to read and write the file.
+        auto flags = O_CREAT | O_RDWR;
+
+        // Caller only wants to write the file.
+        if (!read)
+            flags = (flags & ~O_RDWR) | O_WRONLY;
+
+        // Return flags to caller.
+        return flags;
+    }();
 
     errorcode = 0;
-    fd = open(fstr.c_str(), (!mFollowSymLinks && mIsSymLink) ? (O_PATH | O_NOFOLLOW) : (write ? (read ? O_RDWR : O_WRONLY | O_CREAT) : O_RDONLY), defaultfilepermissions);
+    fd = open(fstr.c_str(), openFlags, defaultfilepermissions);
     if (fd < 0)
     {
         errorcode = errno; // streaming may set errno
@@ -872,7 +976,7 @@ bool PosixFileSystemAccess::cwd_static(LocalPath& path)
 // wake up from filesystem updates
 
 #ifdef __linux__
-void LinuxFileSystemAccess::addevents(Waiter* waiter, int flags)
+void LinuxFileSystemAccess::addevents([[maybe_unused]] Waiter* waiter, int /*flags*/)
 {
 #ifdef ENABLE_SYNC
 
@@ -890,7 +994,7 @@ void LinuxFileSystemAccess::addevents(Waiter* waiter, int flags)
 }
 
 // read all pending inotify events and queue them for processing
-int LinuxFileSystemAccess::checkevents(Waiter* waiter)
+int LinuxFileSystemAccess::checkevents([[maybe_unused]] Waiter* waiter)
 {
     int result = 0;
 
@@ -948,7 +1052,10 @@ int LinuxFileSystemAccess::checkevents(Waiter* waiter)
             }
 
             auto localName = LocalPath::fromPlatformEncodedRelative(name);
-            notifier.notify(notifier.fsEventq, &node, Notification::NEEDS_PARENT_SCAN, move(localName));
+            notifier.notify(notifier.fsEventq,
+                            &node,
+                            Notification::NEEDS_PARENT_SCAN,
+                            std::move(localName));
 
             // We need to rescan the directory if it's changed permissions.
             //
@@ -1019,6 +1126,22 @@ bool PosixFileSystemAccess::renamelocal(const LocalPath& oldname, const LocalPat
     AdjustBasePathResult oldnamestr = adjustBasePath(oldname);
     AdjustBasePathResult newnamestr = adjustBasePath(newname);
 
+#ifdef __MACH__
+    // On case-insensitive file systems, renaming a file to a name that differs
+    // only by case (e.g., "B" → "A" when "a" already exists) does not actually change the case of
+    // the target filename. Instead, the existing "a" is overwritten in content but its original
+    // casing is preserved.
+    //
+    // To ensure that the renamed file takes the exact desired casing ("A" in this case), we
+    // explicitly remove the existing target using unlink() before calling rename(). This guarantees
+    // that the resulting file name matches the requested casing exactly, regardless of file system
+    // case sensitivity.
+    if (override)
+    {
+        unlinklocal(newname);
+    }
+#endif
+
     bool existingandcare = !override && (0 == access(newnamestr.c_str(), F_OK));
     if (!existingandcare && !rename(oldnamestr.c_str(), newnamestr.c_str()))
     {
@@ -1028,7 +1151,7 @@ bool PosixFileSystemAccess::renamelocal(const LocalPath& oldname, const LocalPat
 
     target_exists = existingandcare  || errno == EEXIST || errno == EISDIR || errno == ENOTEMPTY || errno == ENOTDIR;
     target_name_too_long = errno == ENAMETOOLONG;
-    transient_error = !existingandcare && (errno == ETXTBSY || errno == EBUSY);
+    transient_error = !existingandcare && isTransient(errno);
 
     int e = errno;
     if (e != EEXIST  || !skip_targetexists_errorreport)
@@ -1066,7 +1189,9 @@ bool PosixFileSystemAccess::copylocal(const LocalPath& oldname, const LocalPath&
         if ((tfd = open(newnamestr.c_str(), O_WRONLY | O_CREAT | O_TRUNC, defaultfilepermissions)) >= 0)
         {
             umask(mode);
-            while (((t = read(sfd, buf, sizeof buf)) > 0) && write(tfd, buf, t) == t);
+            while (((t = read(sfd, buf, sizeof buf)) > 0) &&
+                   write(tfd, buf, static_cast<size_t>(t)) == t)
+                ;
 #endif
             close(tfd);
         }
@@ -1075,7 +1200,7 @@ bool PosixFileSystemAccess::copylocal(const LocalPath& oldname, const LocalPath&
             umask(mode);
             target_exists = errno == EEXIST;
             target_name_too_long = errno == ENAMETOOLONG;
-            transient_error = errno == ETXTBSY || errno == EBUSY;
+            transient_error = isTransient(errno);
 
             int e = errno;
             LOG_warn << "Unable to copy file. Error code: " << e;
@@ -1109,11 +1234,20 @@ bool PosixFileSystemAccess::unlinklocal(const LocalPath& name)
         return true;
     }
 
-    transient_error = errno == ETXTBSY || errno == EBUSY;
+    auto error = errno;
+
+    transient_error = isTransient(error);
+
+    LOG_debug << "Couldn't unlink file: " << name.toPath(false) << ": " << strerror(error);
 
     return false;
 }
 
+bool PosixFileSystemAccess::isTransient(const int e)
+{
+    return e == ETXTBSY || e == EBUSY;
+}
+
 // delete all files, folders and symlinks contained in the specified folder
 // (does not recurse into mounted devices)
 void PosixFileSystemAccess::emptydirlocal(const LocalPath& nameParam, dev_t basedev)
@@ -1150,17 +1284,18 @@ void PosixFileSystemAccess::emptydirlocal(const LocalPath& nameParam, dev_t base
                  || *d->d_name != '.'
                  || (d->d_name[1] && (d->d_name[1] != '.' || d->d_name[2])))
                 {
-                    ScopedLengthRestore restore(name);
+                    LocalPath newpath{name};
 
-                    name.appendWithSeparator(LocalPath::fromPlatformEncodedRelative(d->d_name), true);
+                    newpath.appendWithSeparator(LocalPath::fromPlatformEncodedRelative(d->d_name),
+                                                true);
 
-                    AdjustBasePathResult nameStr = adjustBasePath(name);
+                    AdjustBasePathResult nameStr = adjustBasePath(newpath);
 
                     if (!lstat(nameStr.c_str(), &statbuf))
                     {
                         if (!S_ISLNK(statbuf.st_mode) && S_ISDIR(statbuf.st_mode) && statbuf.st_dev == basedev)
                         {
-                            emptydirlocal(name, basedev);
+                            emptydirlocal(newpath, basedev);
                             removed |= !rmdir(nameStr.c_str());
                         }
                         else
@@ -1218,7 +1353,7 @@ bool PosixFileSystemAccess::rmdirlocal(const LocalPath& name)
         return true;
     }
 
-    transient_error = errno == ETXTBSY || errno == EBUSY;
+    transient_error = isTransient(errno);
 
     return false;
 }
@@ -1228,7 +1363,7 @@ bool PosixFileSystemAccess::mkdirlocal(const LocalPath& name, bool, bool logAlre
     AdjustBasePathResult nameStr = adjustBasePath(name);
 
     mode_t mode = umask(0);
-    bool r = !mkdir(nameStr.c_str(), defaultfolderpermissions);
+    bool r = !mkdir(nameStr.c_str(), static_cast<mode_t>(defaultfolderpermissions));
     umask(mode);
 
     if (!r)
@@ -1247,7 +1382,7 @@ bool PosixFileSystemAccess::mkdirlocal(const LocalPath& name, bool, bool logAlre
         {
             LOG_err << "Error creating local directory: " << nameStr << " errno: " << errno;
         }
-        transient_error = errno == ETXTBSY || errno == EBUSY;
+        transient_error = isTransient(errno);
     }
 
     return r;
@@ -1263,7 +1398,7 @@ bool PosixFileSystemAccess::setmtimelocal(const LocalPath& name, m_time_t mtime)
     if (!success)
     {
         LOG_err << "Error setting mtime: " << nameStr <<" mtime: "<< mtime << " errno: " << errno;
-        transient_error = errno == ETXTBSY || errno == EBUSY;
+        transient_error = isTransient(errno);
     }
 
     return success;
@@ -1286,7 +1421,7 @@ bool PosixFileSystemAccess::expanselocalpath(const LocalPath& source, LocalPath&
     if (!source.isAbsolute())
     {
         // Sanity.
-        assert(source.localpath[0] != '/');
+        assert(source.toPath(false)[0] != '/');
 
         // Retrieve current working directory.
         if (!cwd(destination))
@@ -1300,18 +1435,18 @@ bool PosixFileSystemAccess::expanselocalpath(const LocalPath& source, LocalPath&
 
     // Sanity.
     assert(destination.isAbsolute());
-    assert(destination.localpath[0] == '/');
+    assert(destination.toPath(false)[0] == '/');
 
     // Canonicalize the path.
     char buffer[PATH_MAX];
 
-    if (!realpath(destination.localpath.c_str(), buffer))
+    if (!realpath(destination.toPath(false).c_str(), buffer))
     {
         destination = source;
         return false;
     }
 
-    destination.localpath.assign(buffer);
+    destination = LocalPath::fromAbsolutePath(buffer);
 
     return true;
 }
@@ -1619,7 +1754,7 @@ void PosixFileSystemAccess::statsid(string *id) const
 
     if (len > 0)
     {
-        id->append(buff, len);
+        id->append(buff, static_cast<size_t>(len));
     }
 #endif
 }
@@ -1628,11 +1763,11 @@ void PosixFileSystemAccess::statsid(string *id) const
 #if defined(__linux__)
 
 LinuxDirNotify::LinuxDirNotify(LinuxFileSystemAccess& owner,
-    LocalNode& root,
-    const LocalPath& rootPath)
-    : DirNotify(rootPath)
-    , mOwner(owner)
-    , mNotifiersIt(owner.mNotifiers.insert(owner.mNotifiers.end(), this))
+                               LocalNode& /*root*/,
+                               const LocalPath& rootPath):
+    DirNotify(rootPath),
+    mOwner(owner),
+    mNotifiersIt(owner.mNotifiers.insert(owner.mNotifiers.end(), this))
 {
     // Assume our owner couldn't initialize.
     setFailed(-owner.mNotifyFd, "Unable to create filesystem monitor.");
@@ -1664,16 +1799,10 @@ AddWatchResult LinuxDirNotify::addWatch(LocalNode& node,
 
     auto handle =
         inotify_add_watch(mOwner.mNotifyFd,
-            path.localpath.c_str(),
-            IN_ATTRIB
-            | IN_CLOSE_WRITE
-            | IN_CREATE
-            | IN_DELETE
-            | IN_DELETE_SELF
-            | IN_EXCL_UNLINK
-            | IN_MOVED_FROM // event->cookie set as IN_MOVED_TO
-            | IN_MOVED_TO
-            | IN_ONLYDIR);
+                          path.toPath(false).c_str(),
+                          IN_ATTRIB | IN_CLOSE_WRITE | IN_CREATE | IN_DELETE | IN_DELETE_SELF |
+                              IN_EXCL_UNLINK | IN_MOVED_FROM // event->cookie set as IN_MOVED_TO
+                              | IN_MOVED_TO | IN_ONLYDIR);
 
     if (handle >= 0)
     {
@@ -1686,11 +1815,8 @@ AddWatchResult LinuxDirNotify::addWatch(LocalNode& node,
     }
 
     LOG_warn << "Unable to monitor path for filesystem notifications: "
-        << path.localpath.c_str()
-        << ": Descriptor: "
-        << mOwner.mNotifyFd
-        << ": Error: "
-        << errno;
+             << path.toPath(false).c_str() << ": Descriptor: " << mOwner.mNotifyFd
+             << ": Error: " << errno;
 
     if (errno == ENOMEM || errno == ENOSPC)
         return make_pair(watches.end(), WR_FATAL);
@@ -1850,7 +1976,7 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
 
     // Try and get information about the scan target.
     bool scanTarget_followSymLink = true; // Follow symlink for the parent directory, so we retrieve the stats of the path that the symlinks points to
-    if (!stat(targetPath.localpath.c_str(), metadata, &scanTarget_followSymLink))
+    if (!stat(targetPath.toPath(false).c_str(), metadata, &scanTarget_followSymLink))
     {
         LOG_warn << "Failed to directoryScan: "
                  << "Unable to stat(...) scan target: "
@@ -1884,7 +2010,7 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
     }
 
     // Try and open the directory for iteration.
-    auto directory = opendir(targetPath.localpath.c_str());
+    auto directory = opendir(targetPath.toPath(false).c_str());
 
     if (!directory)
     {
@@ -1920,18 +2046,15 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
         result.localname = LocalPath::fromPlatformEncodedRelative(entry->d_name);
 
         // Compute this entry's absolute name.
-        ScopedLengthRestore restorer(path);
+        LocalPath newpath{path};
 
-        path.appendWithSeparator(result.localname, false);
+        newpath.appendWithSeparator(result.localname, false);
 
         // Try and get information about this entry.
-        if (!stat(path.localpath.c_str(), metadata))
+        if (!stat(newpath.toPath(false).c_str(), metadata))
         {
             LOG_warn << "directoryScan: "
-                     << "Unable to stat(...) file: "
-                     << path
-                     << ". Error code was: "
-                     << errno;
+                     << "Unable to stat(...) file: " << newpath << ". Error code was: " << errno;
 
             // Entry's unknown if we can't determine otherwise.
             result.type = TYPE_UNKNOWN;
@@ -1958,16 +2081,11 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
 
                 // Leave a trail for debuggers.
                 LOG_warn << "directoryScan: "
-                         << "Encountered a nested mount: "
-                         << path
-                         << ". Expected device "
-                         << major(device)
-                         << ":"
-                         << minor(device)
-                         << ", got device "
-                         << major(metadata.st_dev)
-                         << ":"
-                         << minor(metadata.st_dev);
+                         << "Encountered a nested mount: " << newpath << ". Expected device "
+                         << major(static_cast<unsigned>(device)) << ":"
+                         << minor(static_cast<unsigned>(device)) << ", got device "
+                         << major(static_cast<unsigned>(metadata.st_dev)) << ":"
+                         << minor(static_cast<unsigned>(metadata.st_dev));
             }
 
             continue;
@@ -1979,10 +2097,8 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
         if (!S_ISREG(metadata.st_mode))
         {
             LOG_warn << "directoryScan: "
-                     << "Encountered a special file: "
-                     << path
-                     << ". Mode flags were: "
-                     << (metadata.st_mode & S_IFMT);
+                     << "Encountered a special file: " << newpath
+                     << ". Mode flags were: " << (metadata.st_mode & S_IFMT);
 
             result.isSymlink = S_ISLNK(metadata.st_mode);
             result.type = result.isSymlink ? TYPE_SYMLINK: TYPE_SPECIAL;
@@ -2010,8 +2126,7 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
         if (result.isBlocked)
         {
             LOG_warn << "directoryScan: "
-                     << "Finder has marked this file as busy: "
-                     << path;
+                     << "Finder has marked this file as busy: " << newpath;
             continue;
         }
 #endif // __MACH__
@@ -2027,17 +2142,14 @@ ScanResult PosixFileSystemAccess::directoryScan(const LocalPath& targetPath,
         }
 
         // Try and open the file for reading.
-        UnixStreamAccess isAccess(path.localpath.c_str(),
-                                  result.fingerprint.size);
+        UnixStreamAccess isAccess(newpath.toPath(false).c_str(), result.fingerprint.size);
 
         // Only fingerprint the file if we could actually open it.
         if (!isAccess)
         {
             LOG_warn << "directoryScan: "
-                     << "Unable to open file for fingerprinting: "
-                     << path
-                     << ". Error was: "
-                     << errno;
+                     << "Unable to open file for fingerprinting: " << newpath
+                     << ". Error was: " << errno;
             continue;
         }
 
@@ -2178,7 +2290,7 @@ static std::string deviceOf(const std::string& database,
                  << device
                  << ". Error was: "
                  << std::strerror(error);
-                 
+
         return std::string();
     }
 
@@ -2331,14 +2443,14 @@ static std::string uuidOf(const std::string& device)
 fsfp_t FileSystemAccess::fsFingerprint(const LocalPath& path) const
 {
     // Try and compute legacy filesystem fingerprint.
-    auto fingerprint = fingerprintOf(path.localpath);
+    auto fingerprint = fingerprintOf(path.toPath(false));
 
     // Couldn't compute legacy fingerprint.
     if (!fingerprint)
         return fsfp_t();
 
     // What device contains the specified path?
-    auto device = deviceOf(path.localpath);
+    auto device = deviceOf(path.toPath(false));
 
     // We know what device contains path.
     if (!device.empty())
@@ -2351,9 +2463,6 @@ fsfp_t FileSystemAccess::fsFingerprint(const LocalPath& path) const
             return fsfp_t(fingerprint, std::move(uuid));
     }
 
-    LOG_warn << "Falling back to legacy filesystem fingerprint: "
-             << path;
-
     // Couldn't determine filesystem UUID.
     return fsfp_t(fingerprint, std::string());
 }
@@ -2404,12 +2513,36 @@ bool PosixFileSystemAccess::hardLink(const LocalPath& source, const LocalPath& t
 
 std::unique_ptr<FileAccess> PosixFileSystemAccess::newfileaccess(bool followSymLinks)
 {
+#ifndef __ANDROID__
     return std::unique_ptr<FileAccess>{new PosixFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+#else
+    if (fileWrapper != nullptr)
+    {
+        return std::unique_ptr<FileAccess>{
+            new AndroidFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+    }
+    else
+    {
+        return std::unique_ptr<FileAccess>{
+            new PosixFileAccess{waiter, defaultfilepermissions, followSymLinks}};
+    }
+#endif
 }
 
 unique_ptr<DirAccess>  PosixFileSystemAccess::newdiraccess()
 {
+#ifndef __ANDROID__
     return unique_ptr<DirAccess>(new PosixDirAccess());
+#else
+    if (fileWrapper != nullptr)
+    {
+        return unique_ptr<DirAccess>(new AndroidDirAccess());
+    }
+    else
+    {
+        return unique_ptr<DirAccess>(new PosixDirAccess());
+    }
+#endif
 }
 
 #ifdef __linux__
@@ -2442,7 +2575,7 @@ bool PosixFileSystemAccess::getlocalfstype(const LocalPath& path, FileSystemType
 #if defined(__linux__) || defined(__ANDROID__)
     struct statfs statbuf;
 
-    if (!statfs(path.localpath.c_str(), &statbuf))
+    if (!statfs(path.toPath(false).c_str(), &statbuf))
     {
         switch (static_cast<unsigned long>(statbuf.f_type))
         {
@@ -2511,7 +2644,7 @@ bool PosixFileSystemAccess::getlocalfstype(const LocalPath& path, FileSystemType
 
     struct statfs statbuf;
 
-    if (!statfs(path.localpath.c_str(), &statbuf))
+    if (!statfs(path.toPath(false).c_str(), &statbuf))
     {
         auto it = filesystemTypes.find(statbuf.f_fstypename);
 
@@ -2593,13 +2726,13 @@ bool PosixDirAccess::dnext(LocalPath& path, LocalPath& name, bool followsymlinks
 
     while ((d = readdir(dp)))
     {
-        ScopedLengthRestore restore(path);
+        LocalPath newpath{path};
 
         if (*d->d_name != '.' || (d->d_name[1] && (d->d_name[1] != '.' || d->d_name[2])))
         {
-            path.appendWithSeparator(LocalPath::fromPlatformEncodedRelative(d->d_name), true);
+            newpath.appendWithSeparator(LocalPath::fromPlatformEncodedRelative(d->d_name), true);
 
-            AdjustBasePathResult pathStr = adjustBasePath(path);
+            AdjustBasePathResult pathStr = adjustBasePath(newpath);
 
             bool statOk = !lstat(pathStr.c_str(), &statbuf);
             if (followsymlinks && statOk && S_ISLNK(statbuf.st_mode))
