@@ -57,9 +57,9 @@ string Request::get(MegaClient* client, char reqidCounter[10], string& idempoten
         for (int i = 0; i < (int)cmds.size(); i++)
         {
             req.append(i ? ",{" : "{");
-            req.append(cmds[i]->getJSON(client));
+            req.append(cmds[static_cast<size_t>(i)]->getJSON(client));
             req.append("}");
-            ++counts[cmds[i]->commandStr];
+            ++counts[cmds[static_cast<size_t>(i)]->commandStr];
         }
 
         req.append("]");
@@ -92,24 +92,24 @@ string Request::get(MegaClient* client, char reqidCounter[10], string& idempoten
     return cachedJSON;
 }
 
-bool Request::processCmdJSON(Command* cmd, bool couldBeError, JSON& json)
+bool Request::processCmdJSON(Command* cmd, bool couldBeError, JSON& jsonResponse)
 {
     Error e;
-    if (couldBeError && cmd->checkError(e, json))
+    if (couldBeError && cmd->checkError(e, jsonResponse))
     {
-        return cmd->procresult(Command::Result(Command::CmdError, e), json);
+        return cmd->procresult(Command::Result(Command::CmdError, e), jsonResponse);
     }
-    else if (json.enterobject())
+    else if (jsonResponse.enterobject())
     {
-        return cmd->procresult(Command::CmdObject, json) && json.leaveobject();
+        return cmd->procresult(Command::CmdObject, jsonResponse) && jsonResponse.leaveobject();
     }
-    else if (json.enterarray())
+    else if (jsonResponse.enterarray())
     {
-        return cmd->procresult(Command::CmdArray, json) && json.leavearray();
+        return cmd->procresult(Command::CmdArray, jsonResponse) && jsonResponse.leavearray();
     }
     else
     {
-        return cmd->procresult(Command::CmdItem, json);
+        return cmd->procresult(Command::CmdItem, jsonResponse);
     }
 }
 
@@ -245,12 +245,13 @@ void Request::process(MegaClient* client)
 
         if (cmd->mSeqtagArray && processingJson.enterarray())
         {
-            // Some commands need to return seqtag and also some JSON,
-            // in which case they are in an array with `st` first, and the JSON second
-            // Some commands might or might not produce `st`.  And might return a string.
-            // So in the case of success with a string return, but no `st`, the array is [0, "returnValue"]
+            // Responses parsed here:
+            //      -> ["<st>", <json>] : Sequence Tag + a json
+            //      -> [0, "string"] : API_OK + a no-json-string
+            // Commands should be marked as mSeqtagArray=true to be differentiated from a json array
+            // like ["ES","FR","IE","GB"] If a command returning a string fails, there is no array,
+            // just the error code
             // If the command failed, there is no array, just the error code
-            assert(cmd->mV3);
             assert(*processingJson.pos == '0' || *processingJson.pos == '\"');
             if (*processingJson.pos == '0' && *(processingJson.pos+1) == ',')
             {
@@ -270,9 +271,10 @@ void Request::process(MegaClient* client)
                 parsedOk = false;
             }
         }
-        else if (mV3 && *processingJson.pos == '"')
+        else if (*processingJson.pos == '"')
         {
-            // For v3 commands, a string result is a string which is a seqtag.
+            // Responses parsed here:
+            //      -> "<st>" : Only a Sequence Tag.
             if (!processSeqTag(cmd, false, parsedOk, false, processingJson))
             {
                 // we need to wait for sc processing to catch up with the seqtag we just read
@@ -282,7 +284,9 @@ void Request::process(MegaClient* client)
         }
         else
         {
-            // straightforward case - plain JSON response, no seqtag
+            // Responses parsed here:
+            //      -> 0 or -N : Error codes, plain numeric values.
+            //      -> {..} or [..,..,] : Json response, could be an object or an array.
             parsedOk = processCmdJSON(cmd, true, processingJson);
         }
 
@@ -325,7 +329,7 @@ Command* Request::getCurrentCommand()
     return cmds[processindex].get();
 }
 
-void Request::serverresponse(std::string&& movestring, MegaClient* client)
+void Request::serverresponse(std::string&& movestring, MegaClient*)
 {
     assert(processindex == 0);
     jsonresponse = std::move(movestring);
@@ -370,7 +374,6 @@ void Request::swap(Request& r)
 {
     // we use swap to move between queues, but process only after it gets into the completedreqs
     cmds.swap(r.cmds);
-    std::swap(mV3, r.mV3);
 
     std::swap(cachedJSON, r.cachedJSON);
     std::swap(cachedIdempotenceId, r.cachedIdempotenceId);
@@ -425,16 +428,6 @@ void RequestDispatcher::add(Command *c)
         nextreqs.push_back(Request());
     }
 
-    if (!nextreqs.back().empty() && nextreqs.back().mV3 != c->mV3)
-    {
-        LOG_debug << "Starting an additional Request for v3 transition " << c->mV3;
-        nextreqs.push_back(Request());
-    }
-    if (nextreqs.back().empty())
-    {
-        nextreqs.back().mV3 = c->mV3;
-    }
-
     nextreqs.back().add(c);
     if (c->batchSeparately)
     {
@@ -458,8 +451,16 @@ bool RequestDispatcher::readyToSend() const
 
 bool RequestDispatcher::cmdsInflight() const
 {
-    // stays true even through network errors, -3, retries, etc until we get that response
-    return !inflightreq.empty();
+    // stays true even through network errors, retries, etc until we get that response
+    // but not when the API has notified that commands were not applied
+    return !inflightreq.empty() && inflightFailReason != RETRY_API_LOCK &&
+           inflightFailReason != RETRY_RATE_LIMIT;
+}
+
+bool RequestDispatcher::retryReasonIsApi() const
+{
+    return !inflightreq.empty() &&
+           (inflightFailReason == RETRY_API_LOCK || inflightFailReason == RETRY_RATE_LIMIT);
 }
 
 Command* RequestDispatcher::getCurrentCommand(bool currSeqtagSeen)
@@ -467,7 +468,9 @@ Command* RequestDispatcher::getCurrentCommand(bool currSeqtagSeen)
     return currSeqtagSeen ? inflightreq.getCurrentCommand() : nullptr;
 }
 
-string RequestDispatcher::serverrequest(bool &includesFetchingNodes, bool& v3, MegaClient* client, string& idempotenceId)
+string RequestDispatcher::serverrequest(bool& includesFetchingNodes,
+                                        MegaClient* client,
+                                        string& idempotenceId)
 {
     if (!inflightreq.empty() && inflightFailReason != RETRY_NONE)
     {
@@ -487,7 +490,6 @@ string RequestDispatcher::serverrequest(bool &includesFetchingNodes, bool& v3, M
     }
     string requestJSON = inflightreq.get(client, reqid, idempotenceId);
     includesFetchingNodes = inflightreq.isFetchNodes();
-    v3 = inflightreq.mV3;
 #ifdef MEGA_MEASURE_CODE
     csRequestsSent += inflightreq.size();
     csBatchesSent += 1;
