@@ -1,16 +1,16 @@
-#include "megaapi.h"
-#include "CommonMessages.h"
 #include "DesktopNotifications.h"
+
+#include "CommonMessages.h"
 #include "EmailRequester.h"
+#include "FullName.h"
+#include "megaapi.h"
 #include "MegaApplication.h"
-#include "QTMegaRequestListener.h"
-#include "mega/user.h"
-#include "Platform.h"
-#include "UserAttributesRequests/FullName.h"
-#include "MegaApplication.h"
-#include "transfers/model/TransferMetaData.h"
-#include "TransferNotificationBuilder.h"
 #include "Notificator.h"
+#include "Platform.h"
+#include "RequestListenerManager.h"
+#include "ServiceUrls.h"
+#include "TransferMetaData.h"
+#include "TransferNotificationBuilder.h"
 
 #include <QCoreApplication>
 #include <QtConcurrent/QtConcurrent>
@@ -138,7 +138,7 @@ int DesktopNotifications::countUnseenAlerts(mega::MegaUserAlertList *alertList)
     return count;
 }
 
-void DesktopNotifications::addUserAlertList(mega::MegaUserAlertList *alertList)
+void DesktopNotifications::onUserAlertsUpdated(mega::MegaUserAlertList* alertList)
 {
     if(mPreferences->isAnyNotificationEnabled())
     {
@@ -293,10 +293,12 @@ void DesktopNotifications::processAlert(mega::MegaUserAlert* alert, const QStrin
     {
         if(mPreferences->isNotificationEnabled(Preferences::NotificationsTypes::CONTACT_ESTABLISHED))
         {
-            auto notification = CreateContactNotification(tr("New Contact Established"),
-                                                         tr("New contact with [A] has been established").replace(QString::fromUtf8("[A]"), fullName),
-                                                         email,
-                                                         QStringList() << tr("Accept") << tr("Chat"));
+            auto notification =
+                CreateContactNotification(tr("New Contact Established"),
+                                          tr("New contact with [A] has been established")
+                                              .replace(QString::fromUtf8("[A]"), fullName),
+                                          email,
+                                          QStringList() << tr("Chat"));
 
             QObject::connect(notification, &DesktopAppNotification::activated, this, &DesktopNotifications::viewContactOnWebClient);
         }
@@ -389,10 +391,10 @@ DesktopAppNotification* DesktopNotifications::CreateContactNotification(const QS
     return notification;
 }
 
-void DesktopNotifications::replyIncomingPendingRequest(DesktopAppNotification::Action action) const
+void DesktopNotifications::replyIncomingPendingRequest(DesktopAppNotification::Action action)
 {
     DesktopAppNotification *notification = qobject_cast<DesktopAppNotification *>(QObject::sender());
-    if(!notification)
+    if (!notification)
     {
         return;
     }
@@ -400,22 +402,28 @@ void DesktopNotifications::replyIncomingPendingRequest(DesktopAppNotification::A
     const auto requestList = megaApp->getMegaApi()->getIncomingContactRequests();
     const auto sourceEmail = notification->getData().toString();
 
-    for(int iRequest=0; iRequest < requestList->size(); iRequest++)
+    for (int iRequest=0; iRequest < requestList->size(); iRequest++)
     {
         const auto request = requestList->get(iRequest);
-        if(QString::fromUtf8(request->getSourceEmail()) == sourceEmail)
+        if (QString::fromUtf8(request->getSourceEmail()) == sourceEmail)
         {
-            if(action == DesktopAppNotification::Action::firstButton)
+            if (action == DesktopAppNotification::Action::firstButton)
             {
-                megaApp->getMegaApi()->replyContactRequest(request, mega::MegaContactRequest::REPLY_ACTION_ACCEPT,
-                                                           new mega::OnFinishOneShot(megaApp->getMegaApi(), this, [=](bool isContextValid, const mega::MegaRequest&, const mega::MegaError& e){
-                    if (isContextValid && e.getErrorCode() == mega::MegaError::API_OK)
-                    {
-                        UserAttributes::UserAttributesManager::instance().updateEmptyAttributesByUser(sourceEmail.toStdString().c_str());
-                    }
-                }));
+                auto listener = RequestListenerManager::instance().registerAndGetCustomFinishListener(
+                    this,
+                    [=](mega::MegaRequest* request, mega::MegaError* e) {
+                        if (e->getErrorCode() == mega::MegaError::API_OK)
+                        {
+                            UserAttributes::UserAttributesManager::instance()
+                                .updateEmptyAttributesByUser(sourceEmail.toUtf8().constData());
+                        }
+                    });
+
+                megaApp->getMegaApi()->replyContactRequest(request,
+                                                           mega::MegaContactRequest::REPLY_ACTION_ACCEPT,
+                                                           listener.get());
             }
-            else if(action == DesktopAppNotification::Action::secondButton)
+            else if (action == DesktopAppNotification::Action::secondButton)
             {
                 megaApp->getMegaApi()->replyContactRequest(request, mega::MegaContactRequest::REPLY_ACTION_DENY);
             }
@@ -558,7 +566,7 @@ void DesktopNotifications::viewContactOnWebClient(DesktopAppNotification::Action
     }
     const auto megaApi = static_cast<MegaApplication*>(qApp)->getMegaApi();
     const auto userMail = megaApi->getContact(notification->getData().toString().toUtf8());
-    auto url = QUrl(QString::fromUtf8("mega://#fm/contacts"));
+    auto url = ServiceUrls::getContactsUrl();
     const auto userVisible = userMail && userMail->getVisibility() == mega::MegaUser::VISIBILITY_VISIBLE;
     if (userVisible)
     {
@@ -567,11 +575,11 @@ void DesktopNotifications::viewContactOnWebClient(DesktopAppNotification::Action
         const bool actionIsOpenChat{activationButton == DesktopAppNotification::Action::secondButton};
         if(actionIsViewContact)
         {
-            url = QUrl(QString::fromUtf8("mega://#fm/%1").arg(userHandle));
+            url = ServiceUrls::getContactUrl(userHandle);
         }
         else if(actionIsOpenChat)
         {
-            url = QUrl(QString::fromUtf8("mega://#fm/chat/p/%1").arg(userHandle));
+            url = ServiceUrls::getChatUrl(userHandle);
         }
     }
     Utilities::openUrl(url);
@@ -637,9 +645,11 @@ void DesktopNotifications::sendFinishedTransferNotification(unsigned long long a
     auto data = TransferMetaDataContainer::getAppDataById(appDataId);
     if (data)
     {
+        const auto megaApi = static_cast<MegaApplication*>(qApp)->getMegaApi();
+
         mPreferences->setLastTransferNotificationTimestamp();
         TransferNotificationBuilder messageBuilder(data);
-        auto info = messageBuilder.buildNotification();
+        auto info = messageBuilder.buildNotification(megaApi->isLoggedIn());
 
         auto notification = new DesktopAppNotification();
         notification->setText(info.message);
@@ -837,9 +847,7 @@ void DesktopNotifications::redirectToPayBusiness(DesktopAppNotification::Action
 {
     if (checkIfActionIsValid(activationButton))
     {
-        QString url = QString::fromUtf8("mega://#repay");
-        Utilities::getPROurlWithParameters(url);
-        Utilities::openUrl(QUrl(url));
+        Utilities::openUrl(ServiceUrls::instance()->getRepayUrl());
     }
 }
 
@@ -902,6 +910,8 @@ void DesktopNotifications::actionPressedOnDownloadFinishedTransferNotification(D
 
                     break;
                 }
+                default:
+                    break;
             }
         }
     }
@@ -964,6 +974,8 @@ void DesktopNotifications::actionPressedOnUploadFinishedTransferNotification(Des
                 }
                 break;
             }
+            default:
+                break;
             }
         }
     }
@@ -984,16 +996,16 @@ void DesktopNotifications::getRemoteNodeLink(const QList<std::shared_ptr<mega::M
             }
             else
             {
-                char *handle = node->getBase64Handle();
-                char *key = node->getBase64Key();
+                std::unique_ptr<char[]> handle(node->getBase64Handle());
+                std::unique_ptr<char[]> key(node->getBase64Key());
                 if (handle && key)
                 {
-                    QString link = Preferences::BASE_URL + QString::fromUtf8("/#!%1!%2")
-                            .arg(QString::fromUtf8(handle), QString::fromUtf8(key));
+                    auto link = ServiceUrls::instance()
+                                    ->getRemoteNodeLinkUrl(QString::fromUtf8(handle.get()),
+                                                           QString::fromUtf8(key.get()))
+                                    .toString();
                     linkList.push_back(link);
                 }
-                delete [] key;
-                delete [] handle;
             }
         }
 
@@ -1019,8 +1031,7 @@ void DesktopNotifications::viewShareOnWebClientByHandle(const QString& nodeBase6
 {
     if(!nodeBase64Handle.isEmpty())
     {
-        const auto url = QUrl(QString::fromUtf8("mega://#fm/%1").arg(nodeBase64Handle));
-        Utilities::openUrl(url);
+        Utilities::openUrl(ServiceUrls::getNodeUrl(nodeBase64Handle));
     }
 }
 
@@ -1060,7 +1071,8 @@ void DesktopNotifications::replyNewShareReceived(DesktopAppNotification::Action
     {
         if(!base64Handle.isEmpty())
         {
-            const auto megaFolderHandle = mega::MegaApi::base64ToUserHandle(base64Handle.toStdString().c_str());
+            const auto megaFolderHandle =
+                mega::MegaApi::base64ToUserHandle(base64Handle.toUtf8().constData());
             const auto megaApp = static_cast<MegaApplication*>(qApp);
             megaApp->openSettingsAddSync(megaFolderHandle);
         }
