@@ -1,19 +1,27 @@
 #include "PlatformImplementation.h"
 
-#include <QProgressBar>
-#include <QSet>
-#include <QX11Info>
-#include <QScreen>
+#include "DolphinFileManager.h"
+#include "MessageDialogOpener.h"
+#include "NautilusFileManager.h"
+
 #include <QHostInfo>
+#include <QObject>
+#include <QProgressBar>
+#include <QScreen>
+#include <QSet>
+#include <QVariantMap>
+#include <QX11Info>
+#include <sys/statvfs.h>
+
+#ifndef QT_NO_DBUS
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QDBusInterface>
+#include <QDBusVariant>
+#endif
 
 #include <cstdlib>
 #include <cstring>
-#include <map>
-#include <sys/statvfs.h>
-
-#include "DolphinFileManager.h"
-#include "NautilusFileManager.h"
-#include "QMegaMessageBox.h"
 
 using namespace std;
 using namespace mega;
@@ -24,14 +32,19 @@ PlatformImplementation::PlatformImplementation()
 {
     autostart_dir = QDir::homePath() + QString::fromLatin1("/.config/autostart/");
     desktop_file = autostart_dir + QString::fromLatin1("megasync.desktop");
-    set_icon = QString::fromUtf8("gio set -t string \"%1\" metadata::custom-icon file://%2");
-    remove_icon = QString::fromUtf8("gio set -t unset \"%1\" metadata::custom-icon");
     custom_icon = QString::fromUtf8("/usr/share/icons/hicolor/256x256/apps/mega.png");
 }
 
+PlatformImplementation::~PlatformImplementation()
+{
+    stopThemeMonitor();
+}
+
 void PlatformImplementation::initialize(int /*argc*/, char** /*argv*/)
 {
     mShellNotifier = std::make_shared<SignalShellNotifier>();
+
+    startThemeMonitor();
 }
 
 void PlatformImplementation::notifyItemChange(const QString& path, int)
@@ -40,8 +53,7 @@ void PlatformImplementation::notifyItemChange(const QString& path, int)
     {
         if (notify_server && !Preferences::instance()->overlayIconsDisabled())
         {
-            std::string stdPath = path.toStdString();
-            notify_server->notifyItemChange(&stdPath);
+            notify_server->notifyItemChange(path);
         }
         mShellNotifier->notify(path);
     }
@@ -110,6 +122,7 @@ bool PlatformImplementation::isTilingWindowManager()
     static const QSet<QString> tiling_wms = {
         QString::fromUtf8("sway"),
         QString::fromUtf8("wayfire"),
+        QString::fromUtf8("Hyprland"),
         QString::fromUtf8("i3")
     };
 
@@ -132,9 +145,25 @@ bool PlatformImplementation::showInFolder(QString pathIn)
     if (itFoundAppParams != showInFolderCallMap.constEnd())
     {
         params << *itFoundAppParams;
+        return QProcess::startDetached(fileBrowser, params << QUrl::fromLocalFile(pathIn).toString());
+    }
+    else
+    {
+        QString folderToOpen;
+        QFileInfo file(pathIn);
+        if(file.isFile())
+        {
+            //xdg-open open folders, so we choose the file parent folder
+            folderToOpen = file.absolutePath();
+        }
+        else
+        {
+            folderToOpen = pathIn;
+        }
+
+        return QProcess::startDetached(QLatin1String("xdg-open"), params << QUrl::fromLocalFile(folderToOpen).toString());
     }
 
-    return QProcess::startDetached(fileBrowser, params << QUrl::fromLocalFile(pathIn).toString());
 }
 
 void PlatformImplementation::startShellDispatcher(MegaApplication *receiver)
@@ -172,7 +201,8 @@ void PlatformImplementation::syncFolderAdded(QString syncPath, QString /*syncNam
         QFile *folder = new QFile(syncPath);
         if (folder->exists())
         {
-            QProcess::startDetached(set_icon.arg(folder->fileName()).arg(custom_icon));
+            NautilusFileManager::changeFolderIcon(syncPath, custom_icon);
+            DolphinFileManager::changeFolderIcon(syncPath, custom_icon);
         }
         delete folder;
 
@@ -189,7 +219,8 @@ void PlatformImplementation::syncFolderRemoved(QString syncPath, QString /*syncN
     QFile *folder = new QFile(syncPath);
     if (folder->exists())
     {
-        QProcess::startDetached(remove_icon.arg(folder->fileName()));
+        NautilusFileManager::changeFolderIcon(syncPath);
+        DolphinFileManager::changeFolderIcon(syncPath);
     }
     delete folder;
 
@@ -214,9 +245,45 @@ void PlatformImplementation::notifyAllSyncFoldersRemoved()
 
 }
 
-void PlatformImplementation::processSymLinks()
+QString PlatformImplementation::preparePathForSync(const QString& path)
 {
+    return QDir::toNativeSeparators(QDir::cleanPath(path));
+}
 
+void PlatformImplementation::processSymLinks() {}
+
+bool PlatformImplementation::loadThemeResource(const QString& theme)
+{
+    static QString currentTheme = QString();
+
+    if (!currentTheme.isEmpty())
+    {
+        QResource::unregisterResource(currentTheme);
+    }
+
+    QString execPath = MegaApplication::applicationDirPath();
+    QString resourcePath = execPath + QString::fromUtf8("/../share/megasync/resources");
+
+    if (QDir(resourcePath).exists())
+    {
+        // Look for installed files, if not, use rcc files from same path as binary.
+        execPath = resourcePath;
+    }
+
+    QStringList rccFiles =
+        QStringList() << execPath + QString::fromUtf8("/Resources_common.rcc")
+                      << execPath + QString::fromUtf8("/Resources_qml.rcc")
+                      << execPath + QString::fromUtf8("/qml.rcc")
+                      << execPath + QString::fromUtf8("/Resources_%1.rcc").arg(theme.toLower());
+
+    bool allLoaded = loadRccResources(rccFiles);
+
+    if (allLoaded)
+    {
+        currentTheme = execPath + QString::fromUtf8("/Resources_%1.rcc").arg(theme.toLower());
+    }
+
+    return allLoaded;
 }
 
 QString PlatformImplementation::getDefaultFileBrowserApp()
@@ -238,11 +305,14 @@ QString PlatformImplementation::getDefaultOpenApp(QString extension)
 
 QString PlatformImplementation::getDefaultOpenAppByMimeType(QString mimeType)
 {
-    QString getDefaultAppDesktopFileName = QString::fromUtf8("xdg-mime query default ") + mimeType;
+    QString exe = QLatin1String("xdg-mime");
+    QStringList args;
+    args << QLatin1String("query");
+    args << QLatin1String("default");
+    args << mimeType;
 
     QProcess process;
-    process.start(getDefaultAppDesktopFileName,
-                  QIODevice::ReadWrite | QIODevice::Text);
+    process.start(exe, args, QIODevice::ReadWrite | QIODevice::Text);
     if(!process.waitForFinished(5000))
     {
         return QString();
@@ -288,22 +358,26 @@ QString PlatformImplementation::getDefaultOpenAppByMimeType(QString mimeType)
 
 bool PlatformImplementation::getValue(const char * const name, const bool default_value)
 {
-    const char * const value = getenv(name);
+    QString value = qEnvironmentVariable(name);
 
-    if (!value)
+    if (value.isEmpty())
+    {
         return default_value;
+    }
 
-    return strcmp(value, "0") != 0;
+    return value != QString::fromUtf8("0");
 }
 
 std::string PlatformImplementation::getValue(const char * const name, const std::string &default_value)
 {
-    const char * const value = getenv(name);
+    QString value = qEnvironmentVariable(name);
 
-    if (!value)
+    if (value.isEmpty())
+    {
         return default_value;
+    }
 
-    return value;
+    return value.toUtf8().constData();
 }
 
 QString PlatformImplementation::getWindowManagerName()
@@ -316,51 +390,65 @@ QString PlatformImplementation::getWindowManagerName()
 
     if (!cached)
     {
-        const int maxLen = 1024;
-        const auto connection = QX11Info::connection();
-        const auto appRootWindow = static_cast<xcb_window_t>(QX11Info::appRootWindow());
-
-        auto wmCheckAtom = getAtom(QX11Info::connection(), "_NET_SUPPORTING_WM_CHECK");
-        // Get window manager
-        auto reply = xcb_get_property_reply(connection,
-                                            xcb_get_property(connection,
-                                                             false,
-                                                             appRootWindow,
-                                                             wmCheckAtom,
-                                                             XCB_ATOM_WINDOW,
-                                                             0,
-                                                             maxLen),
-                                            nullptr);
-
-        if (reply && reply->format == 32 && reply->type == XCB_ATOM_WINDOW)
+        if (QX11Info::isPlatformX11())
         {
-            // Get window manager name
-            const xcb_window_t windowManager = *(static_cast<xcb_window_t*>(xcb_get_property_value(reply)));
+            const int maxLen = 1024;
+            const auto connection = QX11Info::connection();
+            const auto appRootWindow = static_cast<xcb_window_t>(QX11Info::appRootWindow());
 
-            if (windowManager != XCB_WINDOW_NONE)
+            if (connection != nullptr)
             {
-                const auto utf8StringAtom = getAtom(connection, "UTF8_STRING");
-                const auto wmNameAtom = getAtom(connection, "_NET_WM_NAME");
+                auto wmCheckAtom = getAtom(connection, "_NET_SUPPORTING_WM_CHECK");
+                // Get window manager
+                auto reply = xcb_get_property_reply(connection,
+                                                    xcb_get_property(connection,
+                                                                     false,
+                                                                     appRootWindow,
+                                                                     wmCheckAtom,
+                                                                     XCB_ATOM_WINDOW,
+                                                                     0,
+                                                                     maxLen),
+                                                    nullptr);
 
-                auto wmReply = xcb_get_property_reply(connection,
-                                                      xcb_get_property(connection,
-                                                                       false,
-                                                                       windowManager,
-                                                                       wmNameAtom,
-                                                                       utf8StringAtom,
-                                                                       0,
-                                                                       maxLen),
-                                                      nullptr);
-                if (wmReply && wmReply->format == 8 && wmReply->type == utf8StringAtom)
+                if (reply && reply->format == 32 && reply->type == XCB_ATOM_WINDOW)
                 {
-                    wmName = QString::fromUtf8(static_cast<const char*>(xcb_get_property_value(wmReply)),
-                                                                        xcb_get_property_value_length(wmReply));
+                    // Get window manager name
+                    const xcb_window_t windowManager = *(static_cast<xcb_window_t*>(xcb_get_property_value(reply)));
+
+                    if (windowManager != XCB_WINDOW_NONE)
+                    {
+                        const auto utf8StringAtom = getAtom(connection, "UTF8_STRING");
+                        const auto wmNameAtom = getAtom(connection, "_NET_WM_NAME");
+
+                        auto wmReply = xcb_get_property_reply(connection,
+                                                              xcb_get_property(connection,
+                                                                               false,
+                                                                               windowManager,
+                                                                               wmNameAtom,
+                                                                               utf8StringAtom,
+                                                                               0,
+                                                                               maxLen),
+                                                              nullptr);
+                        if (wmReply && wmReply->format == 8 && wmReply->type == utf8StringAtom)
+                        {
+                            wmName = QString::fromUtf8(static_cast<const char*>(xcb_get_property_value(wmReply)),
+                                                                                xcb_get_property_value_length(wmReply));
+                        }
+                        free(wmReply);
+                    }
                 }
-                free(wmReply);
+                free(reply);
+                cached = true;
             }
         }
-        free(reply);
-        cached = true;
+
+        if (!cached)
+        {
+            // The previous method failed. We are most probably on Wayland.
+            // Try to get info from environment.
+            wmName = qEnvironmentVariable("XDG_CURRENT_DESKTOP");
+            cached = !wmName.isEmpty();
+        }
     }
     return wmName;
 }
@@ -370,33 +458,6 @@ bool PlatformImplementation::registerUpdateJob()
     return true;
 }
 
-// Check if it's needed to start the local HTTP server
-// for communications with the webclient
-bool PlatformImplementation::shouldRunHttpServer()
-{
-    QStringList data = getListRunningProcesses();
-    if (data.size() > 1)
-    {
-        for (int i = 1; i < data.size(); i++)
-        {
-            // The MEGA webclient sends request to MEGAsync to improve the
-            // user experience. We check if web browsers are running because
-            // otherwise it isn't needed to run the local web server for this purpose.
-            // Here is the list or web browsers that allow HTTP communications
-            // with 127.0.0.1 inside HTTPS webs.
-            QString command = data.at(i).trimmed();
-            if (command.contains(QString::fromUtf8("firefox"), Qt::CaseInsensitive)
-                    || command.contains(QString::fromUtf8("chrome"), Qt::CaseInsensitive)
-                    || command.contains(QString::fromUtf8("chromium"), Qt::CaseInsensitive)
-                    )
-            {
-                return true;
-            }
-        }
-    }
-    return false;
-}
-
 bool PlatformImplementation::isUserActive()
 {
     return true;
@@ -475,8 +536,7 @@ void PlatformImplementation::fileAndFolderSelector(const SelectorInfo& info)
 
 void PlatformImplementation::streamWithApp(const QString &app, const QString &url)
 {
-    QString command = QString::fromUtf8("%1 \"%2\"").arg(QDir::toNativeSeparators(app)).arg(url);
-    QProcess::startDetached(command);
+    QProcess::startDetached(QDir::toNativeSeparators(app), {url});
 }
 
 DriveSpaceData PlatformImplementation::getDriveData(const QString& path)
@@ -486,39 +546,17 @@ DriveSpaceData PlatformImplementation::getDriveData(const QString& path)
     struct statvfs statData;
     const int result = statvfs(path.toUtf8().constData(), &statData);
     data.mIsReady = (result == 0);
-    data.mTotalSpace = static_cast<qint64>(statData.f_blocks * statData.f_bsize);
-    data.mAvailableSpace = static_cast<qint64>(statData.f_bfree * statData.f_bsize);
+    data.mTotalSpace = static_cast<long long>(statData.f_blocks * statData.f_bsize);
+    data.mAvailableSpace = static_cast<long long>(statData.f_bfree * statData.f_bsize);
     return data;
 }
 
-QStringList PlatformImplementation::getListRunningProcesses()
+#if defined(ENABLE_SDK_ISOLATED_GFX)
+QString PlatformImplementation::getGfxProviderPath()
 {
-    QProcess p;
-    p.start(QString::fromUtf8("ps ax -o comm"));
-    p.waitForFinished(2000);
-    QString output = QString::fromUtf8(p.readAllStandardOutput().constData());
-    QString e = QString::fromUtf8(p.readAllStandardError().constData());
-    if (e.size())
-    {
-        MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Error for \"ps ax -o comm\" command:");
-        MegaApi::log(MegaApi::LOG_LEVEL_ERROR, e.toUtf8().constData());
-    }
-
-    p.start(QString::fromUtf8("/bin/bash -c \"readlink /proc/*/exe\""));
-    p.waitForFinished(2000);
-    QString output2 = QString::fromUtf8(p.readAllStandardOutput().constData());
-    e = QString::fromUtf8(p.readAllStandardError().constData());
-    if (e.size())
-    {
-        MegaApi::log(MegaApi::LOG_LEVEL_ERROR, "Error for \"readlink /proc/*/exe\" command:");
-        MegaApi::log(MegaApi::LOG_LEVEL_ERROR, e.toUtf8().constData());
-    }
-    QStringList data = output.split(QString::fromUtf8("\n"));
-
-    data.append(output2.split(QString::fromUtf8("\n")));
-
-    return data;
+    return QCoreApplication::applicationDirPath() + QLatin1String("/mega-desktop-app-gfxworker");
 }
+#endif
 
 xcb_atom_t PlatformImplementation::getAtom(xcb_connection_t * const connection, const char *name)
 {
@@ -599,16 +637,22 @@ bool PlatformImplementation::isFedoraWithGnome()
 
 void PlatformImplementation::promptFedoraGnomeUser()
 {
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Install notification area icon");
-    msgInfo.text = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "For a better experience on Fedora with GNOME, we recommend you enable the notification area icon.\n"
-                                                                                    "Would you like to install the necessary components now?");
+    MessageDialogInfo msgInfo;
+    msgInfo.titleText = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon",
+                                                    "Install notification area icon");
+    msgInfo.descriptionText =
+        QCoreApplication::translate("LinuxPlatformNotificationAreaIcon",
+                                    "For a better experience on Fedora with GNOME, we recommend "
+                                    "you enable the notification area icon.\n"
+                                    "Would you like to install the necessary components now?");
     msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
     msgInfo.defaultButton = QMessageBox::Yes;
-    msgInfo.checkboxText = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Do not show again");
-    msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+    msgInfo.checkboxText =
+        QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Do not show again");
+    msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
     {
-        if (!msg) return;
+        if (!msg)
+            return;
 
         bool isInstallationAttempted = (msg->result() == QMessageBox::Yes);
         bool isInstallationSuccessful = false;
@@ -619,13 +663,14 @@ void PlatformImplementation::promptFedoraGnomeUser()
             isInstallationSuccessful = installAppIndicatorForFedoraGnome();
         }
 
-        if (msg->checkBox() && msg->checkBox()->isChecked())
+        if (msg->isChecked())
         {
             Preferences::instance()->setSystemTrayPromptSuppressed(true);
         }
         else
         {
-            Preferences::instance()->setSystemTrayLastPromptTimestamp(QDateTime::currentSecsSinceEpoch());
+            Preferences::instance()->setSystemTrayLastPromptTimestamp(
+                QDateTime::currentSecsSinceEpoch());
         }
 
         // Set the one-time action done if the user clicked "No" or if the installation was successful.
@@ -635,7 +680,7 @@ void PlatformImplementation::promptFedoraGnomeUser()
         }
     };
 
-    QMegaMessageBox::question(msgInfo);
+    MessageDialogOpener::question(msgInfo);
 }
 
 bool PlatformImplementation::installAppIndicatorForFedoraGnome()
@@ -678,10 +723,13 @@ bool PlatformImplementation::installAppIndicatorForFedoraGnome()
             installProcess.waitForFinished();
             progressDialog.close();
 
-            QMegaMessageBox::MessageBoxInfo msgWarnInfo;
-            msgWarnInfo.title = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Installation Cancelled");
-            msgWarnInfo.text = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "The notification area icon installation was cancelled.");
-            QMegaMessageBox::warning(msgWarnInfo);
+            MessageDialogInfo msgWarnInfo;
+            msgWarnInfo.titleText = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon",
+                                                                "Installation Cancelled");
+            msgWarnInfo.descriptionText = QCoreApplication::translate(
+                "LinuxPlatformNotificationAreaIcon",
+                "The notification area icon installation was cancelled.");
+            MessageDialogOpener::warning(msgWarnInfo);
 
             loop.exit(1);
         }
@@ -704,14 +752,17 @@ bool PlatformImplementation::installAppIndicatorForFedoraGnome()
             loop.exit(0);
         } else
         {
-            QMegaMessageBox::MessageBoxInfo errorInfo;
-            errorInfo.title = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Error installing components");
-            errorInfo.text = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Failed to install the necessary components.");
-            errorInfo.informativeText = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "To install manually, please run the following commands:\n\n"
-                                                       "sudo dnf install gnome-shell-extensions\n"
-                                                       "sudo dnf install gnome-shell-extension-appindicator\n"
-                                                       "gnome-extensions enable appindicatorsupport@rgcjonas.gmail.com");
-            QMegaMessageBox::critical(errorInfo);
+            MessageDialogInfo errorInfo;
+            errorInfo.titleText =
+                QCoreApplication::translate("LinuxPlatformNotificationAreaIcon",
+                                            "Failed to install the necessary components.");
+            errorInfo.descriptionText = QCoreApplication::translate(
+                "LinuxPlatformNotificationAreaIcon",
+                "To install manually, please run the following commands:\n\n"
+                "sudo dnf install gnome-shell-extensions\n"
+                "sudo dnf install gnome-shell-extension-appindicator\n"
+                "gnome-extensions enable appindicatorsupport@rgcjonas.gmail.com");
+            MessageDialogOpener::critical(errorInfo);
             loop.exit(1);
         }
     });
@@ -729,11 +780,14 @@ bool PlatformImplementation::installAppIndicatorForFedoraGnome()
     enableProcess.start(QString::fromUtf8(GNOME_EXTENSIONS_CMD), { QStringLiteral("enable"), QString::fromUtf8(APP_INDICATOR_EXTENSION_ID) });
     enableProcess.waitForFinished(PROCESS_TIMEOUT_MS);
 
-    QMegaMessageBox::MessageBoxInfo successInfo;
-    successInfo.title = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Install complete");
-    successInfo.text = QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "The notification area icon was installed successfully.\n"
-                                                                                        "Please log out of your computer to complete the installation.");
-    QMegaMessageBox::information(successInfo);
+    MessageDialogInfo successInfo;
+    successInfo.titleText =
+        QCoreApplication::translate("LinuxPlatformNotificationAreaIcon", "Install complete");
+    successInfo.descriptionText = QCoreApplication::translate(
+        "LinuxPlatformNotificationAreaIcon",
+        "The notification area icon was installed successfully.\n"
+        "Please log out of your computer to complete the installation.");
+    MessageDialogOpener::information(successInfo);
 
     return true;
 }
@@ -877,3 +931,319 @@ void PlatformImplementation::calculateInfoDialogCoordinates(const QRect& rect, i
     QString otherInfo = QString::fromUtf8("dialog rect = %1, posx = %2, posy = %3").arg(rectToString(rect)).arg(*posx).arg(*posy);
     logInfoDialogCoordinates("Final", screenGeometry, otherInfo);
 }
+
+Preferences::SystemColorScheme PlatformImplementation::getCurrentThemeAppearance() const
+{
+    const auto theme = effectiveTheme();
+    return {theme, theme};
+}
+
+void PlatformImplementation::startThemeMonitor()
+{
+#ifndef QT_NO_DBUS
+    // Setup portal monitoring
+    setupSettingsPortalMonitor();
+#endif
+    if (!mIsSettingsPortalActive)
+    {
+        // Setup gsettings monitoring
+        setupGSettingsThemeCli();
+    }
+}
+
+void PlatformImplementation::stopThemeMonitor()
+{
+    mIsSettingsPortalActive = false;
+    mThemeMonitor.close();
+    mCurrentPortalTheme = Preferences::ThemeAppeareance::UNINITIALIZED;
+    mCurrentGSettingsTheme = Preferences::ThemeAppeareance::UNINITIALIZED;
+    mLastEmittedTheme = Preferences::ThemeAppeareance::UNINITIALIZED;
+    mUseGtkTheme = false;
+}
+
+#ifndef QT_NO_DBUS
+
+void PlatformImplementation::setupSettingsPortalMonitor()
+{
+    const QString DBUS_SERVICE = QLatin1String("org.freedesktop.portal.Desktop");
+    const QString DBUS_PATH = QLatin1String("/org/freedesktop/portal/desktop");
+    const QString DBUS_CONNECTION = QLatin1String("org.freedesktop.portal.Settings");
+    const QString DBUS_NAME = QLatin1String("SettingChanged");
+    const QString DBUS_SIG = QLatin1String("ssv");
+
+    auto sessionbus = QDBusConnection::connectToBus(QDBusConnection::BusType::SessionBus,
+                                                    QLatin1String("session"));
+
+    mIsSettingsPortalActive = sessionbus.interface()->isServiceRegistered(DBUS_SERVICE);
+    if (mIsSettingsPortalActive)
+    {
+        // Connect to setting changed signal.
+        mIsSettingsPortalActive = sessionbus.connect(DBUS_SERVICE,
+                                                     DBUS_PATH,
+                                                     DBUS_CONNECTION,
+                                                     DBUS_NAME,
+                                                     DBUS_SIG,
+                                                     this,
+                                                     SLOT(onSettingsPortalChanged(QDBusMessage)));
+
+        if (mIsSettingsPortalActive)
+        {
+            // Init value
+            mCurrentPortalTheme = readSettingsPortal();
+            mIsSettingsPortalActive =
+                mCurrentPortalTheme != Preferences::ThemeAppeareance::UNINITIALIZED;
+        }
+    }
+}
+
+Preferences::ThemeAppeareance PlatformImplementation::themeFromVariant(const QVariant& var)
+{
+    // Documentation:
+    // https://flatpak.github.io/xdg-desktop-portal/docs/doc-org.freedesktop.portal.Settings.html
+    static const unsigned int PORTAL_NO_PREFERENCE = 0;
+    static const unsigned int PORTAL_DARK = 1;
+    static const unsigned int PORTAL_LIGHT = 2;
+
+    auto theme = Preferences::ThemeAppeareance::UNINITIALIZED;
+
+    bool ok = false;
+    auto extractedValue = var.toUInt(&ok);
+
+    if (ok)
+    {
+        switch (extractedValue)
+        {
+            case PORTAL_DARK:
+            {
+                theme = Preferences::ThemeAppeareance::DARK;
+                break;
+            }
+            case PORTAL_LIGHT:
+                [[fallthrough]];
+            case PORTAL_NO_PREFERENCE:
+                [[fallthrough]];
+            default:
+            {
+                // Default to light theme
+                theme = Preferences::ThemeAppeareance::LIGHT;
+            }
+        }
+    }
+
+    return theme;
+}
+
+Preferences::ThemeAppeareance PlatformImplementation::readSettingsPortal()
+{
+    const QString DBUS_SERVICE = QLatin1String("org.freedesktop.portal.Desktop");
+    const QString DBUS_PATH = QLatin1String("/org/freedesktop/portal/desktop");
+    const QString DBUS_CONNECTION = QLatin1String("org.freedesktop.portal.Settings");
+    const QString DBUS_NS = QLatin1String("org.freedesktop.appearance");
+    const QString DBUS_KEY = QLatin1String("color-scheme");
+
+    auto settingsPortal = QDBusInterface(DBUS_SERVICE, DBUS_PATH, DBUS_CONNECTION);
+
+    // Prefer ReadOne(ns, key); fall back to Read(ns, [keys])
+    QString DBUS_CMD = QLatin1String("ReadOne");
+    QDBusMessage dbusMsg = settingsPortal.call(DBUS_CMD, DBUS_NS, DBUS_KEY);
+
+    QVariant val;
+    if (dbusMsg.type() == QDBusMessage::ReplyMessage && !dbusMsg.arguments().isEmpty())
+    {
+        val = dbusMsg.arguments().at(0);
+    }
+    else
+    {
+        DBUS_CMD = QLatin1String("Read");
+        dbusMsg = settingsPortal.call(DBUS_CMD, DBUS_NS, QStringList{DBUS_KEY});
+
+        if (dbusMsg.type() == QDBusMessage::ReplyMessage && !dbusMsg.arguments().isEmpty())
+        {
+            val = dbusMsg.arguments().at(0);
+            if (val.canConvert<QVariantMap>())
+            {
+                val = val.toMap().value(DBUS_KEY);
+            }
+        }
+    }
+
+    if (val.userType() == qMetaTypeId<QDBusVariant>())
+    {
+        val = qvariant_cast<QDBusVariant>(val).variant();
+    }
+
+    return themeFromVariant(val);
+}
+
+void PlatformImplementation::onSettingsPortalChanged(const QDBusMessage& msg)
+{
+    const QString DBUS_NS = QLatin1String("org.freedesktop.appearance");
+    const QString DBUS_KEY = QLatin1String("color-scheme");
+
+    const auto args = msg.arguments();
+    if (args.size() == 3)
+    {
+        const QString ns = args.at(0).toString();
+        const QString key = args.at(1).toString();
+
+        if (ns == DBUS_NS && key == DBUS_KEY)
+        {
+            QVariant val = args.at(2);
+
+            if (val.userType() == qMetaTypeId<QDBusVariant>())
+            {
+                val = qvariant_cast<QDBusVariant>(val).variant();
+            }
+            mCurrentPortalTheme = themeFromVariant(val);
+            maybeEmitTheme();
+        }
+    }
+}
+
+#endif
+
+Preferences::ThemeAppeareance
+    PlatformImplementation::themeFromColorSchemeString(const QString& schemeStr)
+{
+    // Documentation: https://gitlab.gnome.org/GNOME/Initiatives/-/wikis/Dark-Style-Preference
+    static const QString GSETTINGS_PREFER_LIGHT = QLatin1String("prefer-light");
+    static const QString GSETTINGS_PREFER_DARK = QLatin1String("prefer-dark");
+    static const QString GSETTINGS_DEFAULT = QLatin1String("default");
+
+    auto theme = Preferences::ThemeAppeareance::UNINITIALIZED;
+
+    // Default to light theme
+    if (schemeStr.contains(GSETTINGS_DEFAULT, Qt::CaseInsensitive) ||
+        schemeStr.contains(GSETTINGS_PREFER_LIGHT, Qt::CaseInsensitive))
+    {
+        theme = Preferences::ThemeAppeareance::LIGHT;
+    }
+    else if (schemeStr.contains(GSETTINGS_PREFER_DARK, Qt::CaseInsensitive))
+    {
+        theme = Preferences::ThemeAppeareance::DARK;
+    }
+    return theme;
+}
+
+Preferences::ThemeAppeareance
+    PlatformImplementation::themeFromGtkThemeString(const QString& themeStr)
+{
+    // Dark theme usually end with "-dark", like "Adwaita-dark"
+    static const QString GTK_DARK_ID = QLatin1String("-dark");
+    auto theme = Preferences::ThemeAppeareance::UNINITIALIZED;
+
+    if (themeStr.contains(GTK_DARK_ID, Qt::CaseInsensitive))
+    {
+        theme = Preferences::ThemeAppeareance::DARK;
+    }
+    else
+    {
+        // Default to light theme
+        theme = Preferences::ThemeAppeareance::LIGHT;
+    }
+    return theme;
+}
+
+Preferences::ThemeAppeareance PlatformImplementation::effectiveTheme() const
+{
+    auto theme = Preferences::ThemeAppeareance::UNINITIALIZED;
+    // Portal wins when it is initialized; otherwise use GSettings if initialized.
+    if (mCurrentPortalTheme != theme)
+    {
+        theme = mCurrentPortalTheme;
+    }
+    // Use fallback
+    else if (mCurrentGSettingsTheme != theme)
+    {
+        theme = mCurrentGSettingsTheme;
+    }
+    return theme;
+}
+
+void PlatformImplementation::maybeEmitTheme()
+{
+    const auto effTheme = effectiveTheme();
+    if (effTheme != mLastEmittedTheme)
+    {
+        mLastEmittedTheme = effTheme;
+        emit themeChanged({effTheme, effTheme});
+    }
+}
+
+void PlatformImplementation::setupGSettingsThemeCli()
+{
+    const QString GSETTINGS_BIN = QLatin1String("gsettings");
+    const QString GSETTINGS_DI_PATH = QLatin1String("org.gnome.desktop.interface");
+    const QString GSETTINGS_GET_CMD = QLatin1String("get");
+    const QString GSETTINGS_MONITOR_CMD = QLatin1String("monitor");
+    // Gnome 42+ key
+    const QString GSETTINGS_COLOR_SCHEME_KEY = QLatin1String("color-scheme");
+    // GTK theme for fallback
+    const QString GSETTINGS_GTK_THEME_KEY = QLatin1String("gtk-theme");
+
+    const int PROCESS_TIMEOUT_MS = 1500;
+
+    QProcess process;
+    process.start(GSETTINGS_BIN,
+                  {GSETTINGS_GET_CMD, GSETTINGS_DI_PATH, GSETTINGS_COLOR_SCHEME_KEY});
+    bool gSettingsAvailable =
+        process.waitForFinished(PROCESS_TIMEOUT_MS) && process.exitCode() == 0;
+    if (gSettingsAvailable)
+    {
+        const QString output = QString::fromUtf8(process.readAllStandardOutput());
+        mCurrentGSettingsTheme = themeFromColorSchemeString(output);
+    }
+
+    // gtk-theme fallback
+    if (mCurrentGSettingsTheme == Preferences::ThemeAppeareance::UNINITIALIZED)
+    {
+        process.start(GSETTINGS_BIN,
+                      {GSETTINGS_GET_CMD, GSETTINGS_DI_PATH, GSETTINGS_GTK_THEME_KEY});
+        gSettingsAvailable = process.waitForFinished(PROCESS_TIMEOUT_MS) && process.exitCode() == 0;
+        if (gSettingsAvailable)
+        {
+            const QString output = QString::fromUtf8(process.readAllStandardOutput());
+            mCurrentGSettingsTheme = themeFromGtkThemeString(output);
+            mUseGtkTheme = true;
+        }
+    }
+
+    if (mCurrentGSettingsTheme != Preferences::ThemeAppeareance::UNINITIALIZED)
+    {
+        mThemeMonitor.start(GSETTINGS_BIN,
+                            {GSETTINGS_MONITOR_CMD,
+                             GSETTINGS_DI_PATH,
+                             mUseGtkTheme ? GSETTINGS_GTK_THEME_KEY : GSETTINGS_COLOR_SCHEME_KEY});
+        QObject::connect(&mThemeMonitor,
+                         &QProcess::readyReadStandardOutput,
+                         this,
+                         &PlatformImplementation::onGsettingsThemeReadyRead);
+    }
+}
+
+void PlatformImplementation::onGsettingsThemeReadyRead()
+{
+    QRegularExpression themeRx;
+    if (mUseGtkTheme)
+    {
+        themeRx = QRegularExpression(QLatin1String(R"(.*gtk-theme.*'([^']+)')"));
+    }
+    else
+    {
+        themeRx = QRegularExpression(QLatin1String(R"(.*color-scheme.*'([^']+)')"));
+    }
+
+    while (mThemeMonitor.canReadLine())
+    {
+        const QString line = QString::fromUtf8(mThemeMonitor.readLine()).trimmed();
+        const auto match = themeRx.match(line);
+
+        if (match.hasMatch())
+        {
+            const QString val = match.captured(1);
+            mCurrentGSettingsTheme =
+                mUseGtkTheme ? themeFromGtkThemeString(val) : themeFromColorSchemeString(val);
+            maybeEmitTheme();
+        }
+    }
+}
