@@ -1,35 +1,23 @@
 #include "StalledIssueDelegate.h"
 
-#include "StalledIssueFilePath.h"
-#include "StalledIssueBaseDelegateWidget.h"
-#include "StalledIssueHeader.h"
-#include "StalledIssuesView.h"
+#include "DialogOpener.h"
 #include "MegaDelegateHoverManager.h"
 #include "StalledIssue.h"
-#include <DialogOpener.h>
-#include <StalledIssuesDialog.h>
+#include "StalledIssueBaseDelegateWidget.h"
+#include "StalledIssueHeader.h"
+#include "StalledIssuesDialog.h"
+#include "StalledIssuesView.h"
+#include "ThemeManager.h"
+#include "TokenParserWidgetManager.h"
 
-#include <QPainter>
-#include <QMouseEvent>
 #include <QDebug>
 #include <QElapsedTimer>
+#include <QMouseEvent>
+#include <QPainter>
 #include <QPainterPath>
 
-#ifdef Q_OS_MACOS
-const QColor HOVER_COLOR = QColor("#F7F7F7");
-#else
-const QColor HOVER_COLOR = QColor("#FAFAFA");
-#endif
-const QColor SELECTED_BORDER_COLOR = QColor("#E9E9E9");
-const float LEFT_MARGIN = 4.0;
-const float RIGHT_MARGIN = 6.0;
-const float RECT_BORDERS_MARGIN = 20.0;
-const float BOTTON_MARGIN = 6.0;
-const float TOP_MARGIN = 2.0;
-const float CORNER_RADIUS = 10.0;
 const int PEN_WIDTH = 2;
 const int UPDATE_SIZE_TIMER = 50;
-
 const QSize DEFAULT_SIZE = QSize(100,60);
 
 StalledIssueDelegate::StalledIssueDelegate(StalledIssuesProxyModel* proxyModel,  StalledIssuesView *view)
@@ -79,6 +67,8 @@ StalledIssueDelegate::StalledIssueDelegate(StalledIssuesProxyModel* proxyModel,
     });
 
     mView->installEventFilter(this);
+
+    updateColors();
 }
 
 void StalledIssueDelegate::updateVisibleIndexesSizeHint(int updateDelay, bool forceUpdate)
@@ -131,6 +121,15 @@ void StalledIssueDelegate::updateVisibleIndexesSizeHint(int updateDelay, bool fo
     }
 }
 
+void StalledIssueDelegate::onSelectionChanged(const QItemSelection&,
+                                              const QItemSelection& itemsDeselected)
+{
+    for (auto& index: itemsDeselected.indexes())
+    {
+        updateRelativeIndex(index);
+    }
+}
+
 QSize StalledIssueDelegate::sizeHint(const QStyleOptionViewItem& option, const QModelIndex& index) const
 {
     if(mSizeHintRequested > 0)
@@ -255,8 +254,11 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
 
         bool isExpanded(mView->isExpanded(index) && stalledIssueItem.consultData()->isExpandable());
 
+        updateColorsCheckingTheme();
+
         auto pos (option.rect.topLeft());
         QRect geometry(option.rect);
+        QRectF realGeometry(geometry);
 
 #ifdef __APPLE__
         auto width = mView->size().width();
@@ -266,10 +268,8 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
         {
             width -= mView->verticalScrollBar()->width();
         }
-        geometry.setWidth(std::min(width, option.rect.width()));
+        geometry.setWidth(std::min(width, geometry.width()));
 #endif
-
-        auto rowColor = Qt::white;
         auto backgroundRect = isExpanded ? geometry : option.rect.adjusted(0,0,0,-1);
 
         QStyle::State state = option.state;
@@ -299,46 +299,16 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
 
         if(state & (QStyle::State_MouseOver | QStyle::State_Selected))
         {
-            path.setFillRule( Qt::WindingFill );
-            path.addRoundedRect(QRectF(geometry.x() + LEFT_MARGIN,
-                                       geometry.y() + TOP_MARGIN,
-                                       geometry.width() - RIGHT_MARGIN,
-                                       geometry.height() - BOTTON_MARGIN), CORNER_RADIUS, CORNER_RADIUS);
+            path.setFillRule(Qt::WindingFill);
+            path.addRect(realGeometry);
 
-            if(index.parent().isValid())
+            if (state & (QStyle::State_MouseOver | QStyle::State_Selected))
             {
-                path.addRect(QRectF(geometry.x() + LEFT_MARGIN,
-                                    geometry.y(),
-                                    geometry.width() - RIGHT_MARGIN,
-                                    RECT_BORDERS_MARGIN)); // Bottom corners not rounded
-            }
-            else if(isExpanded)
-            {
-                path.addRect(QRectF(geometry.x() + LEFT_MARGIN,
-                                    geometry.y() + RECT_BORDERS_MARGIN,
-                                    geometry.width() - RIGHT_MARGIN,
-                                    geometry.height() - RECT_BORDERS_MARGIN)); // Bottom corners not rounded
-            }
-
-            if(state & QStyle::State_MouseOver && state & QStyle::State_Selected)
-            {
-                pen.setColor(SELECTED_BORDER_COLOR);
-                pen.setWidth(PEN_WIDTH);
-                fillColor = HOVER_COLOR;
+                fillColor = mSelectedColor;
             }
             else
             {
-                if(state & QStyle::State_MouseOver)
-                {
-                    pen.setColor(HOVER_COLOR);
-                    fillColor = HOVER_COLOR;
-                }
-                else if (state & QStyle::State_Selected)
-                {
-                    pen.setColor(SELECTED_BORDER_COLOR);
-                    pen.setWidth(PEN_WIDTH);
-                    fillColor = Qt::white;
-                }
+                fillColor = mActiveColor;
             }
 
             if(pen != QPen())
@@ -347,11 +317,10 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
             }
         }
 
-        painter->fillRect(backgroundRect, rowColor);
+        painter->fillRect(backgroundRect, mActiveColor);
         if(!path.isEmpty())
         {
             painter->fillPath(path.simplified(), fillColor);
-            painter->drawPath(path.simplified());
         }
 
         //REMOVE SEPARATION LINE WHEN EXPANDED
@@ -361,17 +330,16 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
 
             if(index.parent().isValid())
             {
-                collision = QRectF(geometry.x() + LEFT_MARGIN + PEN_WIDTH/2 /* next pixel*/,
-                                   geometry.y() - PEN_WIDTH/2,
-                                   geometry.x() + geometry.width() - RIGHT_MARGIN - PEN_WIDTH,
+                collision = QRectF(realGeometry.x() + PEN_WIDTH / 2 /* next pixel*/,
+                                   realGeometry.y() - PEN_WIDTH / 2,
+                                   realGeometry.x() + realGeometry.width() - PEN_WIDTH,
                                    PEN_WIDTH);
-
             }
             else if(isExpanded)
             {
-                collision = QRectF(geometry.x() + LEFT_MARGIN + PEN_WIDTH/2 /* next pixel*/,
-                                   geometry.y() + geometry.height() - PEN_WIDTH/2,
-                                   geometry.x() + geometry.width() - RIGHT_MARGIN - PEN_WIDTH,
+                collision = QRectF(realGeometry.x() + PEN_WIDTH / 2 /* next pixel*/,
+                                   realGeometry.y() + realGeometry.height() - PEN_WIDTH / 2,
+                                   realGeometry.x() + realGeometry.width() - PEN_WIDTH,
                                    PEN_WIDTH);
             }
 
@@ -400,7 +368,7 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
             w->setGeometry(geometry);
 
             auto pixmap = w->grab();
-            painter->drawPixmap(0,0,w->width(), w->height(),pixmap);
+            painter->drawPixmap(0, 0, w->width(), w->height(), pixmap);
 
             painter->restore();
         }
@@ -427,7 +395,7 @@ void StalledIssueDelegate::paint(QPainter *painter, const QStyleOptionViewItem &
         if(drawBottomLine)
         {
             painter->setRenderHint(QPainter::Antialiasing, false);
-            painter->setPen(QPen(Qt::black, 1));
+            painter->setPen(QPen(mBottomBorderColor, 1));
             painter->setOpacity(0.2);
             painter->drawLine(QPoint(0 - geometry.x(), geometry.height()-1),QPoint(geometry.width(), geometry.height()-1));
         }
@@ -475,40 +443,35 @@ void StalledIssueDelegate::updateEditorGeometry(QWidget *editor, const QStyleOpt
 
 bool StalledIssueDelegate::event(QEvent *event)
 {
-    if(auto hoverEvent = dynamic_cast<MegaDelegateHoverEvent*>(event))
+    if (event)
     {
-        if(hoverEvent->type() == QEvent::Enter)
+        if (auto hoverEvent = dynamic_cast<MegaDelegateHoverEvent*>(event))
         {
-            auto headerIndex(getHeaderIndex(hoverEvent->index()));
-            auto relativeIndex(getRelativeIndex(hoverEvent->index()));
-
-            mLastHoverIndex = headerIndex;
-            QTimer::singleShot(0, this, [this, relativeIndex](){
-                mView->update(relativeIndex);
-            });
-
-            onHoverEnter(hoverEvent->index());
-        }
-        else if(hoverEvent->type() == QEvent::MouseMove)
-        {
-            onHoverEnter(hoverEvent->index());
-        }
-        else if(hoverEvent->type() == QEvent::Leave)
-        {
-            auto headerIndex(getHeaderIndex(hoverEvent->index()));
-            if(mLastHoverIndex == headerIndex)
+            if (hoverEvent->type() == MegaDelegateHoverEvent::Enter)
             {
-                auto relativeIndex(getRelativeIndex(hoverEvent->index()));
-                QTimer::singleShot(0, this, [this, relativeIndex](){
-                    mView->update(relativeIndex);
-                });
+                auto headerIndex(getHeaderIndex(hoverEvent->index()));
+                mLastHoverIndex = headerIndex;
 
-                mLastHoverIndex = QModelIndex();
+                updateRelativeIndex(hoverEvent->index());
+
+                onHoverEnter(hoverEvent->index());
+            }
+            else if (hoverEvent->type() == MegaDelegateHoverEvent::MouseMove)
+            {
+                onHoverEnter(hoverEvent->index());
+            }
+            else if (hoverEvent->type() == MegaDelegateHoverEvent::Leave)
+            {
+                auto headerIndex(getHeaderIndex(hoverEvent->index()));
+                if (mLastHoverIndex == headerIndex)
+                {
+                    updateRelativeIndex(hoverEvent->index());
+                    mLastHoverIndex = QModelIndex();
+                }
+                onHoverLeave(hoverEvent->index());
             }
-            onHoverLeave(hoverEvent->index());
         }
     }
-
     return QStyledItemDelegate::event(event);
 }
 
@@ -611,25 +574,6 @@ void StalledIssueDelegate::onHoverLeave(const QModelIndex& index)
     mView->update(index);
 }
 
-QColor StalledIssueDelegate::getRowColor(const QModelIndex &index) const
-{
-    QColor rowColor;
-
-    if(index.row()%2 == 0)
-    {
-        rowColor = Qt::white;
-    }
-    else
-    {
-        rowColor.setRed(0);
-        rowColor.setGreen(0);
-        rowColor.setBlue(0);
-        rowColor.setAlphaF(0.05);
-    }
-
-    return rowColor;
-}
-
 QModelIndex StalledIssueDelegate::getEditorCurrentIndex() const
 {
     if(mEditor)
@@ -650,7 +594,7 @@ QModelIndex StalledIssueDelegate::getRelativeIndex(const QModelIndex& index) con
     {
         if(auto model = index.model())
         {
-            model->index(0,0,index);
+            return model->index(0, 0, index);
         }
     }
     return QModelIndex();
@@ -661,6 +605,21 @@ QModelIndex StalledIssueDelegate::getHeaderIndex(const QModelIndex &index) const
     return index.parent().isValid() ? index.parent() : index;
 }
 
+void StalledIssueDelegate::updateRelativeIndex(const QModelIndex& parentIndex) const
+{
+    QPersistentModelIndex relativeIndex(getRelativeIndex(parentIndex));
+
+    QTimer::singleShot(0,
+                       this,
+                       [this, relativeIndex]()
+                       {
+                           if (relativeIndex.isValid())
+                           {
+                               mView->update(relativeIndex);
+                           }
+                       });
+}
+
 StalledIssueBaseDelegateWidget *StalledIssueDelegate::getStalledIssueItemWidget(const QModelIndex& proxyIndex, const StalledIssueVariant& data, const QSize& size) const
 {
     StalledIssueBaseDelegateWidget* item(nullptr);
@@ -683,3 +642,21 @@ StalledIssueBaseDelegateWidget *StalledIssueDelegate::getStalledIssueItemWidget(
 
     return item;
 }
+
+void StalledIssueDelegate::updateColorsCheckingTheme() const
+{
+    static auto theme = ThemeManager::instance()->currentColorScheme();
+    if (theme != ThemeManager::instance()->currentColorScheme())
+    {
+        theme = ThemeManager::instance()->currentColorScheme();
+        updateColors();
+    }
+}
+
+void StalledIssueDelegate::updateColors() const
+{
+    mActiveColor = TokenParserWidgetManager::instance()->getColor(QLatin1String("page-background"));
+    mSelectedColor = TokenParserWidgetManager::instance()->getColor(QLatin1String("surface-1"));
+    mBottomBorderColor =
+        TokenParserWidgetManager::instance()->getColor(QLatin1String("border-subtle"));
+}
