@@ -1,16 +1,19 @@
 #ifndef STALLEDISSUEFILEPATH_H
 #define STALLEDISSUEFILEPATH_H
 
-#include "StalledIssueBaseDelegateWidget.h"
+#include "StalledIssue.h"
 
-#include <QWidget>
-#include <QLabel>
 #include <QEvent>
+#include <QIcon>
+#include <QLabel>
+#include <QWidget>
 
 namespace Ui {
 class StalledIssueFilePath;
 }
 
+class IconLabel;
+
 class StalledIssueFilePath : public QWidget
 {
     Q_OBJECT
@@ -32,9 +35,11 @@ private slots:
     void onHelpIconClicked();
 
 private:
-    void showHoverAction(QEvent::Type type, QLabel *actionWidget, const QString &path);
+    void showHoverAction(QEvent::Type type, QWidget* actionWidget, const QString& path);
     void updateFileIcons();
     void updateMoveFileIcons();
+    void updateCornerArrows();
+    void updateLocalOrMegaTitle();
 
     void fillFilePath();
     QString getFilePath() const;
@@ -47,12 +52,11 @@ private:
 
     QString getSyncPathProblemString(mega::MegaSyncStall::SyncPathProblem pathProblem);
     bool showError(mega::MegaSyncStall::SyncPathProblem pathProblem);
-    QString getHelpLink(mega::MegaSyncStall::SyncPathProblem pathProblem);
+    QUrl getHelpLink(mega::MegaSyncStall::SyncPathProblem pathProblem);
 
     Ui::StalledIssueFilePath *ui;
     StalledIssueDataPtr mData;
     bool mShowFullPath;
-    QIcon mOpenIcon;
 };
 
 #endif // STALLEDISSUEFILEPATH_H
