@@ -1,25 +1,19 @@
 #include "LocalAndRemoteDifferentWidget.h"
-#include "TextDecorator.h"
-#include <QDialogButtonBox>
-#include "ui_LocalAndRemoteDifferentWidget.h"
-#include <QCheckBox>
 
+#include "LocalOrRemoteUserMustChooseStalledIssue.h"
 #include "MegaApplication.h"
-#include "StalledIssuesModel.h"
-#include "StalledIssueHeader.h"
-#include <DialogOpener.h>
-#include <StalledIssuesDialog.h>
-#include <PlatformStrings.h>
-#include <QMegaMessageBox.h>
-#include <LocalOrRemoteUserMustChooseStalledIssue.h>
+#include "MessageDialogOpener.h"
+#include "Preferences.h"
 #include "StalledIssueChooseWidget.h"
-#include <Preferences/Preferences.h>
+#include "StalledIssueHeader.h"
+#include "StalledIssuesModel.h"
+#include "TextDecorator.h"
+#include "ui_LocalAndRemoteDifferentWidget.h"
 
-#include "mega/types.h"
-
-#include <QMessageBox>
+#include <QCheckBox>
+#include <QDialogButtonBox>
 #include <QFile>
-
+#include <QMessageBox>
 
 const QList<mega::MegaSyncStall::SyncStallReason> ReasonsToCheck
     = QList<mega::MegaSyncStall::SyncStallReason>() << mega::MegaSyncStall::LocalAndRemoteChangedSinceLastSyncedState_userMustChoose
@@ -86,7 +80,11 @@ void LocalAndRemoteDifferentWidget::refreshUi()
         QString bothInfoTitle = tr("[B]Keep both[/B]");
         StalledIssuesBoldTextDecorator::boldTextDecorator.process(bothInfoTitle);
         bothInfo.title = bothInfoTitle;
-        bothInfo.icon = QLatin1String(":/images/copy.png");
+        bothInfo.icon = Utilities::getPixmapName(QLatin1String("keep-both"),
+                                                 Utilities::AttributeType::SMALL |
+                                                     Utilities::AttributeType::THIN |
+                                                     Utilities::AttributeType::OUTLINE);
+        bothInfo.iconToken = QLatin1String("icon-primary");
         bothInfo.solvedText = ui->keepBothOption->chosenString();
         ui->keepBothOption->setInfo(bothInfo);
 
@@ -104,7 +102,11 @@ void LocalAndRemoteDifferentWidget::refreshUi()
 
         StalledIssuesBoldTextDecorator::boldTextDecorator.process(lastModifiedInfoTitle);
         lastModifiedInfo.title = lastModifiedInfoTitle;
-        lastModifiedInfo.icon = QLatin1String(":/images/clock_ico.png");
+        lastModifiedInfo.icon = Utilities::getPixmapName(QLatin1String("clock"),
+                                                         Utilities::AttributeType::SMALL |
+                                                             Utilities::AttributeType::THIN |
+                                                             Utilities::AttributeType::OUTLINE);
+        lastModifiedInfo.iconToken = QLatin1String("icon-primary");
         lastModifiedInfo.solvedText = ui->keepLastModifiedOption->chosenString();
         ui->keepLastModifiedOption->setInfo(lastModifiedInfo);
     }
@@ -115,6 +117,8 @@ void LocalAndRemoteDifferentWidget::refreshUi()
         ui->keepLastModifiedOption->hide();
     }
 
+    ui->selectLabel->setVisible(!issue->isSolved());
+
     if (issue->isSolved())
     {
         unSetFailedChooseWidget();
@@ -127,9 +131,8 @@ void LocalAndRemoteDifferentWidget::refreshUi()
             ui->chooseLocalCopy->setActionButtonVisibility(false);
             ui->chooseRemoteCopy->setActionButtonVisibility(false);
         }
-
     }
-    else if(issue->isFailed())
+    else if (issue->isFailed())
     {
         ui->keepBothOption->setSolved(false, false);
         ui->keepLastModifiedOption->show();
@@ -138,27 +141,31 @@ void LocalAndRemoteDifferentWidget::refreshUi()
 
         unSetFailedChooseWidget();
 
-        switch(issue->getChosenSide())
+        switch (issue->getChosenSide())
         {
             case LocalOrRemoteUserMustChooseStalledIssue::ChosenSide::REMOTE:
             {
-                auto errorStr = issue->consultLocalData()->isFile() ? tr("Unable to remove the local file") : tr("Unable to remove the local folder");
+                auto errorStr = issue->consultLocalData()->isFile() ?
+                                    tr("Unable to remove the local file") :
+                                    tr("Unable to remove the local folder");
                 ui->chooseRemoteCopy->setFailed(true, errorStr);
                 mFailedItem = ui->chooseRemoteCopy;
                 break;
             }
             case LocalOrRemoteUserMustChooseStalledIssue::ChosenSide::LOCAL:
             {
-                auto errorStr = issue->consultLocalData()->isFile() ? tr("Unable to remove the file stored in MEGA")
-                                                                    : tr("Unable to remove the folder stored in MEGA");
+                auto errorStr = issue->consultLocalData()->isFile() ?
+                                    tr("Unable to remove the file stored in MEGA") :
+                                    tr("Unable to remove the folder stored in MEGA");
                 ui->chooseLocalCopy->setFailed(true, errorStr);
                 mFailedItem = ui->chooseLocalCopy;
                 break;
             }
             case LocalOrRemoteUserMustChooseStalledIssue::ChosenSide::BOTH:
             {
-                auto errorStr = issue->consultLocalData()->isFile() ? tr("Unable to update both local and MEGA files")
-                                                                    : tr("Unable to update both local and MEGA folders");
+                auto errorStr = issue->consultLocalData()->isFile() ?
+                                    tr("Unable to update both local and MEGA files") :
+                                    tr("Unable to update both local and MEGA folders");
                 ui->keepBothOption->setFailed(true, errorStr);
                 mFailedItem = ui->keepBothOption;
                 break;
@@ -168,7 +175,6 @@ void LocalAndRemoteDifferentWidget::refreshUi()
                 break;
             }
         }
-
     }
 
     updateSizeHint();
@@ -248,8 +254,7 @@ void LocalAndRemoteDifferentWidget::onLocalButtonClicked(int)
     stringInfo.isFile = localInfo.isFile();
     stringInfo.itemName = ui->chooseLocalCopy->data()->getFileName();
     stringInfo.numberOfIssues = info.selection.size();
-    info.msgInfo.text = keepLocalSideString(stringInfo);
-    StalledIssuesBoldTextDecorator::boldTextDecorator.process(info.msgInfo.text);
+    info.msgInfo.titleText = keepLocalSideString(stringInfo);
 
     if(node->isFile())
     {
@@ -257,19 +262,19 @@ void LocalAndRemoteDifferentWidget::onLocalButtonClicked(int)
         {
             if(info.selection.size() == 1)
             {
-                info.msgInfo.informativeText =
+                info.msgInfo.descriptionText =
                     tr("The [B]local file[/B] %1 will be uploaded to MEGA and replace the "
-                        "current file, which will be moved to the SyncDebris folder in your MEGA "
-                        "Rubbish bin.")
-                            .arg(localInfo.fileName()) +
+                       "current file, which will be moved to the SyncDebris folder in your MEGA "
+                       "Rubbish bin.")
+                        .arg(localInfo.fileName()) +
                     QString::fromUtf8("[BR]");
             }
             else
             {
-                info.msgInfo.informativeText =
+                info.msgInfo.descriptionText =
                     tr("The [B]local files[/B] will be uploaded to MEGA and replace the "
-                        "current files, which will be moved to the SyncDebris folder in your MEGA "
-                        "Rubbish bin.") +
+                       "current files, which will be moved to the SyncDebris folder in your MEGA "
+                       "Rubbish bin.") +
                     QString::fromUtf8("[BR]");
             }
         }
@@ -277,46 +282,51 @@ void LocalAndRemoteDifferentWidget::onLocalButtonClicked(int)
         {
             if (info.selection.size() > 1)
             {
-
-                info.msgInfo.informativeText =
+                info.msgInfo.descriptionText =
                     (tr("The [B]local files[/B] will be uploaded to MEGA and added as a version to "
                         "the "
                         "remote files.\nPlease wait for the upload to complete.")
-                            .arg(localInfo.fileName())) +
+                         .arg(localInfo.fileName())) +
                     QString::fromUtf8("[BR]");
             }
             else
             {
-                info.msgInfo.informativeText =
+                info.msgInfo.descriptionText =
                     (tr("The [B]local file[/B] %1 will be uploaded to MEGA and added as a version "
                         "to "
                         "the remote file.\nPlease wait for the upload to complete.")
-                            .arg(localInfo.fileName())) +
+                         .arg(localInfo.fileName())) +
                     QString::fromUtf8("[BR]");
             }
         }
     }
     else
     {
-        info.msgInfo.informativeText = tr("The [B]remote folder[/B] %1 will be moved to MEGA Rubbish Bin.[BR]You will be able to retrieve the folder from there.[/BR]").arg(localInfo.fileName());
+        info.msgInfo.descriptionText =
+            tr("The [B]remote folder[/B] %1 will be moved to MEGA Rubbish Bin.[BR]You will be able "
+               "to retrieve the folder from there.[/BR]")
+                .arg(localInfo.fileName());
         if(info.selection.size() > 1)
         {
-            info.msgInfo.informativeText = tr("The [B]remote folders[/B] will be moved to MEGA Rubbish Bin.[BR]You will be able to retrieve the folders from there.[/BR]");
+            info.msgInfo.descriptionText =
+                tr("The [B]remote folders[/B] will be moved to MEGA Rubbish Bin.[BR]You will be "
+                   "able to retrieve the folders from there.[/BR]");
         }
     }
 
     if(MegaSyncApp->getTransfersModel()->areAllPaused())
     {
-        info.msgInfo.informativeText.append(QString::fromUtf8("[BR]") + tr("[B]Please, resume your transfers to fix the issue[/B]", "", info.selection.size()) + QString::fromUtf8("[BR]"));
+        info.msgInfo.descriptionText.append(
+            QString::fromUtf8("[BR]") +
+            tr("[B]Please, resume your transfers to fix the issue[/B]", "", info.selection.size()) +
+            QString::fromUtf8("[BR]"));
     }
-    StalledIssuesBoldTextDecorator::boldTextDecorator.process(info.msgInfo.informativeText);
-    StalledIssuesNewLineTextDecorator::newLineTextDecorator.process(info.msgInfo.informativeText);
 
-    info.msgInfo.finishFunc = [info](QMessageBox* msgBox)
+    info.msgInfo.finishFunc = [info](QPointer<MessageDialogResult> msgBox)
     {
-        if(msgBox->result() == QDialogButtonBox::Ok)
+        if (msgBox->result() == QMessageBox::Ok)
         {
-            if(msgBox->checkBox() && msgBox->checkBox()->isChecked())
+            if (msgBox->isChecked())
             {
                 MegaSyncApp->getStalledIssuesModel()->chooseSideManually(false, info.similarSelection);
             }
@@ -327,7 +337,7 @@ void LocalAndRemoteDifferentWidget::onLocalButtonClicked(int)
         }
     };
 
-    QMegaMessageBox::warning(info.msgInfo);
+    MessageDialogOpener::warning(info.msgInfo);
 }
 
 void LocalAndRemoteDifferentWidget::onRemoteButtonClicked(int)
@@ -352,34 +362,40 @@ void LocalAndRemoteDifferentWidget::onRemoteButtonClicked(int)
         stringInfo.isFile = node->isFile();
         stringInfo.itemName = ui->chooseRemoteCopy->data()->getFileName();
         stringInfo.numberOfIssues = info.selection.size();
-        info.msgInfo.text = keepRemoteSideString(stringInfo);
+        info.msgInfo.titleText = keepRemoteSideString(stringInfo);
     }
     else
     {
-        info.msgInfo.text = tr("Are you sure you want to keep the [B]remote item[/B] %1?").arg(ui->chooseRemoteCopy->data()->getFileName());
+        info.msgInfo.titleText = tr("Are you sure you want to keep the [B]remote item[/B] %1?")
+                                     .arg(ui->chooseRemoteCopy->data()->getFileName());
         if (info.selection.size() > 1)
         {
-            info.msgInfo.text = tr("Keep the [B]remote items[/B]?");
+            info.msgInfo.titleText = tr("Keep the [B]remote items[/B]?");
         }
     }
-    StalledIssuesBoldTextDecorator::boldTextDecorator.process(info.msgInfo.text);
     //For the moment, TYPE_TWOWAY or TYPE_UNKNOWN
     if(getData().consultData()->getSyncType() != mega::MegaSync::SyncType::TYPE_BACKUP)
     {
         if(localInfo.isFile())
         {
-            info.msgInfo.informativeText = tr("The [B]local file[/B] %1 will be moved to the sync debris folder").arg(localInfo.fileName());
+            info.msgInfo.descriptionText =
+                tr("The [B]local file[/B] %1 will be moved to the sync debris folder")
+                    .arg(localInfo.fileName());
             if (info.selection.size() > 1)
             {
-                info.msgInfo.informativeText = tr("The [B]local files[/B] will be moved to the sync debris folder");
+                info.msgInfo.descriptionText =
+                    tr("The [B]local files[/B] will be moved to the sync debris folder");
             }
         }
         else
         {
-            info.msgInfo.informativeText = tr("The [B]local folder[/B] %1 will be moved to the sync debris folder").arg(localInfo.fileName());
+            info.msgInfo.descriptionText =
+                tr("The [B]local folder[/B] %1 will be moved to the sync debris folder")
+                    .arg(localInfo.fileName());
             if (info.selection.size() > 1)
             {
-                info.msgInfo.informativeText = tr("The [B]local folders[/B] will be moved to the sync debris folder");
+                info.msgInfo.descriptionText =
+                    tr("The [B]local folders[/B] will be moved to the sync debris folder");
             }
         }
     }
@@ -387,30 +403,35 @@ void LocalAndRemoteDifferentWidget::onRemoteButtonClicked(int)
     {
         if(localInfo.isFile())
         {
-            info.msgInfo.informativeText = tr("The backup will be disabled in order to protect the local file %1").arg(localInfo.fileName());
+            info.msgInfo.descriptionText =
+                tr("The backup will be disabled in order to protect the local file %1")
+                    .arg(localInfo.fileName());
             if (info.selection.size() > 1)
             {
-                info.msgInfo.informativeText = tr("The backup will be disabled in order to protect the local files");
+                info.msgInfo.descriptionText =
+                    tr("The backup will be disabled in order to protect the local files");
             }
         }
         else
         {
-            info.msgInfo.informativeText = tr("The backup will be disabled in order to protect the local folder %1").arg(localInfo.fileName());
+            info.msgInfo.descriptionText =
+                tr("The backup will be disabled in order to protect the local folder %1")
+                    .arg(localInfo.fileName());
             if (info.selection.size() > 1)
             {
-                info.msgInfo.informativeText = tr("The backup will be disabled in order to protect the local folders");
+                info.msgInfo.descriptionText =
+                    tr("The backup will be disabled in order to protect the local folders");
             }
         }
     }
-    StalledIssuesBoldTextDecorator::boldTextDecorator.process(info.msgInfo.informativeText);
 
-    info.msgInfo.finishFunc = [this, info](QMessageBox* msgBox)
+    info.msgInfo.finishFunc = [this, info](QPointer<MessageDialogResult> msgBox)
     {
         if(getData().consultData()->getSyncType() == mega::MegaSync::SyncType::TYPE_TWOWAY)
         {
-            if(msgBox->result() == QDialogButtonBox::Ok)
+            if (msgBox->result() == QMessageBox::Ok)
             {
-                if(msgBox->checkBox() && msgBox->checkBox()->isChecked())
+                if (msgBox->isChecked())
                 {
                     MegaSyncApp->getStalledIssuesModel()->chooseSideManually(true, info.similarSelection);
                 }
@@ -422,9 +443,9 @@ void LocalAndRemoteDifferentWidget::onRemoteButtonClicked(int)
         }
         else
         {
-            if(msgBox->result() == QDialogButtonBox::Ok)
+            if (msgBox->result() == QMessageBox::Ok)
             {
-                if(msgBox->checkBox() && msgBox->checkBox()->isChecked())
+                if (msgBox->isChecked())
                 {
                     MegaSyncApp->getStalledIssuesModel()->chooseRemoteForBackups(info.similarSelection);
                 }
@@ -436,7 +457,7 @@ void LocalAndRemoteDifferentWidget::onRemoteButtonClicked(int)
         }
     };
 
-    QMegaMessageBox::warning(info.msgInfo);
+    MessageDialogOpener::warning(info.msgInfo);
 }
 
 void LocalAndRemoteDifferentWidget::onKeepBothButtonClicked(int)
@@ -457,18 +478,18 @@ void LocalAndRemoteDifferentWidget::onKeepBothButtonClicked(int)
     QFileInfo localInfo(ui->chooseLocalCopy->data()->getFilePath());
         if(localInfo.isFile())
     {
-        info.msgInfo.text = tr("Keep both files?");
+        info.msgInfo.titleText = tr("Keep both files?");
         if(info.selection.size())
         {
-            info.msgInfo.text = tr("Keep all files?");
+            info.msgInfo.titleText = tr("Keep all files?");
         }
     }
     else
     {
-        info.msgInfo.text = tr("Keep both folders");
+        info.msgInfo.titleText = tr("Keep both folders");
         if(info.selection.size())
         {
-            info.msgInfo.text = tr("Keep all folders");
+            info.msgInfo.titleText = tr("Keep all folders");
         }
     }
 
@@ -479,19 +500,24 @@ void LocalAndRemoteDifferentWidget::onKeepBothButtonClicked(int)
 
         if(node->isFile())
         {
-            info.msgInfo.informativeText = tr("The [B]remote file[/B] will have a suffix like (1) added", "", info.selection.size());
+            info.msgInfo.descriptionText =
+                tr("The [B]remote file[/B] will have a suffix like (1) added",
+                   "",
+                   info.selection.size());
         }
         else
         {
-            info.msgInfo.informativeText = tr("The [B]remote folder[/B] will have a suffix like (1) added", "", info.selection.size());
+            info.msgInfo.descriptionText =
+                tr("The [B]remote folder[/B] will have a suffix like (1) added",
+                   "",
+                   info.selection.size());
         }
-        StalledIssuesBoldTextDecorator::boldTextDecorator.process(info.msgInfo.informativeText);
 
-        info.msgInfo.finishFunc = [info](QMessageBox* msgBox)
+        info.msgInfo.finishFunc = [info](QPointer<MessageDialogResult> msgBox)
         {
-            if(msgBox->result() == QDialogButtonBox::Ok)
+            if (msgBox->result() == QMessageBox::Ok)
             {
-                if(msgBox->checkBox() && msgBox->checkBox()->isChecked())
+                if (msgBox->isChecked())
                 {
                     MegaSyncApp->getStalledIssuesModel()->chooseBothSides(info.similarSelection);
                 }
@@ -502,7 +528,7 @@ void LocalAndRemoteDifferentWidget::onKeepBothButtonClicked(int)
             }
         };
 
-        QMegaMessageBox::warning(info.msgInfo);
+        MessageDialogOpener::warning(info.msgInfo);
     }
 }
 
@@ -516,32 +542,34 @@ void LocalAndRemoteDifferentWidget::onKeepLastModifiedTimeButtonClicked(int)
         return;
     }
 
-    info.msgInfo.text = tr("Are you sure you want to choose the latest modified side?");
+    info.msgInfo.titleText = tr("Are you sure you want to choose the latest modified side?");
     if(issue->lastModifiedSide() == LocalOrRemoteUserMustChooseStalledIssue::ChosenSide::LOCAL)
     {
-        info.msgInfo.informativeText = tr("This action will choose the local side");
+        info.msgInfo.descriptionText = tr("This action will choose the local side");
     }
     else
     {
-        info.msgInfo.informativeText = tr("This action will choose the remote side");
+        info.msgInfo.descriptionText = tr("This action will choose the remote side");
     }
 
-    info.msgInfo.finishFunc = [info](QMessageBox* msgBox)
+    info.msgInfo.finishFunc = [info](QPointer<MessageDialogResult> msgBox)
     {
-        if(msgBox->result() == QDialogButtonBox::Ok)
+        if (msgBox->result() == QMessageBox::Ok)
         {
-            if(msgBox->checkBox() && msgBox->checkBox()->isChecked())
+            if (msgBox->isChecked())
             {
-                MegaSyncApp->getStalledIssuesModel()->semiAutoSolveLocalRemoteIssues(info.similarSelection);
+                MegaSyncApp->getStalledIssuesModel()->chooseLastModifiedLocalRemoteIssues(
+                    info.similarSelection);
             }
             else
             {
-                MegaSyncApp->getStalledIssuesModel()->semiAutoSolveLocalRemoteIssues(info.selection);
+                MegaSyncApp->getStalledIssuesModel()->chooseLastModifiedLocalRemoteIssues(
+                    info.selection);
             }
         }
     };
 
-    QMegaMessageBox::warning(info.msgInfo);
+    MessageDialogOpener::warning(info.msgInfo);
 }
 
 void LocalAndRemoteDifferentWidget::unSetFailedChooseWidget()
