@@ -1,21 +1,18 @@
 #include "NameConflict.h"
-#include "ui_NameConflict.h"
 
-#include "StalledIssueHeader.h"
-
-#include <MegaApplication.h>
-#include <StalledIssuesModel.h>
-#include "NodeNameSetterDialog/RenameNodeDialog.h"
-#include "QMegaMessageBox.h"
 #include "DialogOpener.h"
-#include "StalledIssuesDialog.h"
+#include "megaapi.h"
+#include "MegaApplication.h"
 #include "MegaNodeNames.h"
-#include "DateTimeFormatter.h"
+#include "MessageDialogOpener.h"
+#include "RenameNodeDialog.h"
+#include "StalledIssuesDialog.h"
+#include "StalledIssuesModel.h"
 #include "StalledIssuesUtilities.h"
 #include "StatsEventHandler.h"
 #include "TextDecorator.h"
+#include "ui_NameConflict.h"
 
-#include <megaapi.h>
 #include <QDialogButtonBox>
 #include <QPainter>
 
@@ -26,7 +23,7 @@ const char* TITLE_FILENAME = "TITLE_FILENAME";
 const char* TITLE_INDEX = "TITLE_INDEX";
 
 //NAME DUPLICATED
-void NameDuplicatedContainer::paintEvent(QPaintEvent*)
+void NameDuplicatedContainer::paintEvent(QPaintEvent* event)
 {
     QPainter painter(this);
 
@@ -51,6 +48,8 @@ void NameDuplicatedContainer::paintEvent(QPaintEvent*)
 
     QRect horRect2(0, height() - 7, 10, 2);
     painter.fillRect(horRect2, Qt::gray);
+
+    QWidget::paintEvent(event);
 }
 
 //NAME CONFLICT
@@ -164,40 +163,66 @@ void NameConflict::updateUi(std::shared_ptr<const NameConflictedStalledIssue> is
             title->setActionButtonVisibility(RENAME_ID, false);
             title->setActionButtonVisibility(REMOVE_ID, false);
 
-            QIcon icon;
+            QString iconName;
+            QString iconToken;
             QString titleText;
 
             if (info->getSolvedType() == NameConflictedStalledIssue::ConflictedNameInfo::SolvedType::REMOVE)
             {
-                icon.addFile(QString::fromUtf8(":/images/StalledIssues/remove_default.png"));
+                iconName = Utilities::getPixmapName(QLatin1String("cross"),
+                                                    Utilities::AttributeType::SMALL |
+                                                        Utilities::AttributeType::THIN |
+                                                        Utilities::AttributeType::OUTLINE);
+                iconToken = QLatin1String("support-error");
+
                 titleText = tr("Removed");
             }
             else if (info->getSolvedType() ==
                      NameConflictedStalledIssue::ConflictedNameInfo::SolvedType::RENAME)
             {
-                icon.addFile(QString::fromUtf8(":/images/StalledIssues/check_default.png"));
+                iconName = Utilities::getPixmapName(QLatin1String("check"),
+                                                    Utilities::AttributeType::SMALL |
+                                                        Utilities::AttributeType::THIN |
+                                                        Utilities::AttributeType::OUTLINE);
+                iconToken = QLatin1String("support-success");
+
                 titleText = tr("Renamed to \"%1\"").arg(info->mRenameTo);
             }
             else if (info->getSolvedType() ==
                      NameConflictedStalledIssue::ConflictedNameInfo::SolvedType::MERGED)
             {
-                icon.addFile(QString::fromUtf8(":/images/StalledIssues/check_default.png"));
+                iconName = Utilities::getPixmapName(QLatin1String("check"),
+                                                    Utilities::AttributeType::SMALL |
+                                                        Utilities::AttributeType::THIN |
+                                                        Utilities::AttributeType::OUTLINE);
+                iconToken = QLatin1String("support-success");
+
                 titleText = tr("Merged");
             }
             else if (info->getSolvedType() ==
                      NameConflictedStalledIssue::ConflictedNameInfo::SolvedType::CHANGED_EXTERNALLY)
             {
-                icon.addFile(QString::fromUtf8(":/images/StalledIssues/check_default.png"));
+                iconName = Utilities::getPixmapName(QLatin1String("check"),
+                                                    Utilities::AttributeType::SMALL |
+                                                        Utilities::AttributeType::THIN |
+                                                        Utilities::AttributeType::OUTLINE);
+                iconToken = QLatin1String("support-success");
+
                 titleText = tr("Modified externally");
             }
             else
             {
-                icon.addFile(QString::fromUtf8(":/images/StalledIssues/check_default.png"));
+                iconName = Utilities::getPixmapName(QLatin1String("check"),
+                                                    Utilities::AttributeType::SMALL |
+                                                        Utilities::AttributeType::THIN |
+                                                        Utilities::AttributeType::OUTLINE);
+                iconToken = QLatin1String("support-success");
+
                 titleText = tr("No action needed");
             }
 
             titleLayout->activate();
-            title->setMessage(titleText, icon.pixmap(16, 16));
+            title->setMessage(titleText, iconName, iconToken);
         }
         else if(info->isFailed())
         {
@@ -254,10 +279,16 @@ void NameConflict::initTitle(StalledIssueActionTitle* title, int index, const QS
 
 void NameConflict::initActionButtons(StalledIssueActionTitle* title)
 {
-    QIcon renameIcon(QString::fromUtf8("://images/StalledIssues/rename_node_default.png"));
-    QIcon removeIcon(QString::fromUtf8("://images/StalledIssues/remove_default.png"));
-    title->addActionButton(renameIcon, tr("Rename"), RENAME_ID, false);
-    title->addActionButton(removeIcon, QString(), REMOVE_ID, false);
+    QIcon renameIcon(Utilities::getIcon(QLatin1String("pen-2"),
+                                        Utilities::AttributeType::SMALL |
+                                            Utilities::AttributeType::THIN |
+                                            Utilities::AttributeType::OUTLINE));
+    QIcon removeIcon(Utilities::getIcon(QLatin1String("cross"),
+                                        Utilities::AttributeType::SMALL |
+                                            Utilities::AttributeType::THIN |
+                                            Utilities::AttributeType::OUTLINE));
+    title->addActionButton(renameIcon, tr("Rename"), RENAME_ID, false, QLatin1String("secondary"));
+    title->addActionButton(removeIcon, QString(), REMOVE_ID, false, QLatin1String("ghost"));
 }
 
 void NameConflict::onRawInfoChecked()
@@ -278,10 +309,12 @@ void NameConflict::updateTitleExtraInfo(StalledIssueActionTitle* title, std::sha
 {
     auto index = title->property(TITLE_INDEX).toInt();
 
-    info->mItemAttributes->requestModifiedTime(title, [this, index](const QDateTime& time)
+    if (!isCloud() || info->mIsFile)
     {
-        mTitlesByIndex.value(index)->updateLastTimeModified(time);
-    });
+        info->mItemAttributes->requestModifiedTime(title, [this, index](const QDateTime& time) {
+            mTitlesByIndex.value(index)->updateLastTimeModified(time);
+        });
+    }
 
     if(isCloud())
     {
@@ -332,9 +365,9 @@ void NameConflict::updateTitleExtraInfo(StalledIssueActionTitle* title, std::sha
             }
         });
 
-        cloudAttributes->requestUser(title, MegaSyncApp->getMegaApi()->getMyUserHandleBinary(), [this, index](QString user, bool showAttribute)
+        cloudAttributes->requestUser(title, [this, cloudAttributes, index](QString user)
         {
-            if(mTitlesByIndex.value(index)->updateUser(user, showAttribute))
+            if(mTitlesByIndex.value(index)->updateUser(user, !cloudAttributes->isCurrentUser()))
             {
                 mDelegateWidget->updateSizeHint();
             }
@@ -412,13 +445,12 @@ void NameConflict::onActionClicked(int actionId)
                 }
                 else
                 {
-                    QMegaMessageBox::MessageBoxInfo msgInfo;
+                    MessageDialogInfo msgInfo;
                     msgInfo.parent = dialog ? dialog->getDialog() : nullptr;
-                    msgInfo.title = MegaSyncApp->getMEGAString();
                     msgInfo.textFormat = Qt::RichText;
-                    msgInfo.buttons = QMessageBox::Ok;
-                    msgInfo.text = tr("%1 no longer exists.\nPlease refresh the view").arg(info.fileName());
-                    QMegaMessageBox::warning(msgInfo);
+                    msgInfo.descriptionText =
+                        tr("%1 no longer exists.\nPlease refresh the view").arg(info.fileName());
+                    MessageDialogOpener::warning(msgInfo);
                     return;
                 }
             }
@@ -491,54 +523,65 @@ void NameConflict::onActionClicked(int actionId)
                 }
                 else
                 {
-                    QMegaMessageBox::MessageBoxInfo msgInfo;
+                    MessageDialogInfo msgInfo;
                     msgInfo.parent = dialog ? dialog->getDialog() : nullptr;
-                    msgInfo.title = MegaSyncApp->getMEGAString();
                     msgInfo.textFormat = Qt::RichText;
-                    msgInfo.buttons = QMessageBox::Ok;
-                    msgInfo.text = tr("%1 no longer exists.\nPlease refresh the view").arg(info.fileName());
-                    QMegaMessageBox::warning(msgInfo);
+                    msgInfo.descriptionText =
+                        tr("%1 no longer exists.\nPlease refresh the view").arg(info.fileName());
+                    MessageDialogOpener::warning(msgInfo);
                     return;
                 }
             }
 
-            QMegaMessageBox::MessageBoxInfo msgInfo;
+            MessageDialogInfo msgInfo;
             msgInfo.parent = dialog ? dialog->getDialog() : nullptr;
-            msgInfo.title = MegaSyncApp->getMEGAString();
             msgInfo.textFormat = Qt::RichText;
             msgInfo.buttons = QMessageBox::Yes | QMessageBox::Cancel;
+            msgInfo.defaultButton = QMessageBox::Yes;
 
             if(isCloud())
             {
                 if(isFile)
                 {
-                    msgInfo.text = tr("Are you sure you want to remove the remote file %1?").arg(fileName);
-                    msgInfo.informativeText = tr("It will be moved to the SyncDebris folder on the MEGA Rubbish Bin along with its versions.[BR]You will be able to retrieve the file and its versions from there.[/BR]");
+                    msgInfo.titleText =
+                        tr("Are you sure you want to remove the remote file %1?").arg(fileName);
+                    msgInfo.descriptionText =
+                        tr("It will be moved to the SyncDebris folder on the MEGA Rubbish Bin "
+                           "along with its versions.[BR]You will be able to retrieve the file and "
+                           "its versions from there.[/BR]");
                 }
                 else
                 {
-                    msgInfo.text = tr("Are you sure you want to remove the remote folder %1?").arg(fileName);
-                    msgInfo.informativeText = tr("It will be moved to the SyncDebris folder on the MEGA Rubbish Bin.[BR]You will be able to retrieve the folder from there.[/BR]");
+                    msgInfo.titleText =
+                        tr("Are you sure you want to remove the remote folder %1?").arg(fileName);
+                    msgInfo.descriptionText =
+                        tr("It will be moved to the SyncDebris folder on the MEGA Rubbish "
+                           "Bin.[BR]You will be able to retrieve the folder from there.[/BR]");
                 }
             }
             else
             {
                 if(isFile)
                 {
-                    msgInfo.text = tr("Are you sure you want to remove the local file %1?").arg(fileName);
-                    msgInfo.informativeText = tr("It will be moved to the sync rubbish folder.[BR]You will be able to retrieve the file from there.[/BR]");
+                    msgInfo.titleText =
+                        tr("Are you sure you want to remove the local file %1?").arg(fileName);
+                    msgInfo.descriptionText =
+                        tr("It will be moved to the sync rubbish folder.[BR]You will be able to "
+                           "retrieve the file from there.[/BR]");
                 }
                 else
                 {
-                    msgInfo.text = tr("Are you sure you want to remove the local folder %1?").arg(fileName);
-                    msgInfo.informativeText = tr("It will be moved to the sync rubbish folder.[BR]You will be able to retrieve the folder from there.[/BR]");
+                    msgInfo.titleText =
+                        tr("Are you sure you want to remove the local folder %1?").arg(fileName);
+                    msgInfo.descriptionText =
+                        tr("It will be moved to the sync rubbish folder.[BR]You will be able to "
+                           "retrieve the folder from there.[/BR]");
                 }
             }
 
-            msgInfo.finishFunc = [=](
-                                     QMessageBox* msgBox)
+            msgInfo.finishFunc = [=](QPointer<MessageDialogResult> msgBox)
             {
-                if (msgBox->result() == QDialogButtonBox::Yes)
+                if (msgBox->result() == QMessageBox::Yes)
                 {
                     bool areAllSolved(false);
 
@@ -550,12 +593,12 @@ void NameConflict::onActionClicked(int actionId)
                             std::unique_ptr<mega::MegaNode> node(MegaSyncApp->getMegaApi()->getNodeByHandle(handle));
                             if(node)
                             {
-                                error = mUtilities.removeRemoteFile(node.get());
+                                error = Utilities::removeSyncRemoteFile(node.get());
                             }
                         }
                         else
                         {
-                            error = mUtilities.removeRemoteFile(filePath);
+                            error = Utilities::removeSyncRemoteFile(filePath);
                         }
 
                         if(error)
@@ -572,7 +615,7 @@ void NameConflict::onActionClicked(int actionId)
                     else
                     {
                         auto syncId = mIssue->syncIds().isEmpty() ? mega::INVALID_HANDLE : mIssue->firstSyncId();
-                        auto failed = !mUtilities.removeLocalFile(QDir::toNativeSeparators(filePath), syncId);
+                        auto failed = !Utilities::removeLocalFile(QDir::toNativeSeparators(filePath), syncId);
 
                         if(failed)
                         {
@@ -598,10 +641,7 @@ void NameConflict::onActionClicked(int actionId)
                     }
                 }
             };
-            StalledIssuesBoldTextDecorator::boldTextDecorator.process(msgInfo.informativeText);
-            StalledIssuesBoldTextDecorator::boldTextDecorator.process(msgInfo.text);
-            StalledIssuesNewLineTextDecorator::newLineTextDecorator.process(msgInfo.informativeText);
-            QMegaMessageBox::warning(msgInfo);
+            MessageDialogOpener::warning(msgInfo);
         }
     }
 }
