@@ -1,28 +1,30 @@
 #include "NameConflictStalledIssue.h"
 
-#include "mega/types.h"
-#include "Utilities.h"
-#include "StatsEventHandler.h"
+#include "MegaApiSynchronizedRequest.h"
+#include "MergeMEGAFolders.h"
+#include "MoveToMEGABin.h"
+#include "RenameNodeDialog.h"
 #include "StalledIssuesModel.h"
-#include "gui/NodeNameSetterDialog/RenameNodeDialog.h"
+#include "StatsEventHandler.h"
+#include "SyncController.h"
+#include "Utilities.h"
 
-#include <MegaApiSynchronizedRequest.h>
-
-NameConflictedStalledIssue::NameConflictedStalledIssue(const mega::MegaSyncStall *stallIssue)
-    : StalledIssue(stallIssue)
+NameConflictedStalledIssue::NameConflictedStalledIssue(const mega::MegaSyncStall* stallIssue):
+    StalledIssue(stallIssue),
+    mCloudConflictedNames(this)
 {}
 
 void NameConflictedStalledIssue::fillIssue(const mega::MegaSyncStall *stall)
 {
-    auto localConflictNames = stall->pathCount(false);
+    auto localConflictNames = static_cast<int>(stall->pathCount(false));
 
     if(localConflictNames > 0)
     {
         initLocalIssue();
 
-        for(unsigned int index = 0; index < localConflictNames; ++index)
+        for(int index = 0; index < localConflictNames; ++index)
         {
-            QFileInfo localPath(QString::fromUtf8(stall->path(false,index)));
+            QFileInfo localPath(QString::fromUtf8(stall->path(false, index)));
 
             if(consultLocalData()->mPath.isEmpty())
             {
@@ -39,7 +41,7 @@ void NameConflictedStalledIssue::fillIssue(const mega::MegaSyncStall *stall)
         }
     }
 
-    auto cloudConflictNames = stall->pathCount(true);
+    auto cloudConflictNames = static_cast<int>(stall->pathCount(true));
 
     if(cloudConflictNames > 0)
     {
@@ -54,7 +56,7 @@ void NameConflictedStalledIssue::fillIssue(const mega::MegaSyncStall *stall)
             getCloudData()->mPathHandle = stall->cloudNodeHandle(0);
         }
 
-        for(unsigned int index = 0; index < cloudConflictNames; ++index)
+        for(int index = 0; index < cloudConflictNames; ++index)
         {
             auto cloudHandle(stall->cloudNodeHandle(index));
             auto cloudPath = QString::fromUtf8(stall->path(true, index));
@@ -63,7 +65,8 @@ void NameConflictedStalledIssue::fillIssue(const mega::MegaSyncStall *stall)
                 cloudPath = cloudPath.remove(0,1);
             }
 
-            std::unique_ptr<mega::MegaNode> node(MegaSyncApp->getMegaApi()->getNodeByHandle(cloudHandle));
+            std::unique_ptr<mega::MegaNode> node(
+                MegaSyncApp->getMegaApi()->getNodeByHandle(cloudHandle));
             if(node)
             {
                 QFileInfo cloudPathInfo(cloudPath);
@@ -72,7 +75,12 @@ void NameConflictedStalledIssue::fillIssue(const mega::MegaSyncStall *stall)
 
                 if(node->isFile())
                 {
-                    mCloudConflictedNames.addFileConflictedName(node->getModificationTime(), node->getSize(), node->getCreationTime(), QString::fromUtf8(node->getFingerprint()), info);
+                    mCloudConflictedNames.addFileConflictedName(
+                        static_cast<unsigned long long>(node->getModificationTime()),
+                        node->getSize(),
+                        static_cast<unsigned long long>(node->getCreationTime()),
+                        QString::fromUtf8(node->getFingerprint()),
+                        info);
                     mFiles++;
                 }
                 else
@@ -121,38 +129,36 @@ bool NameConflictedStalledIssue::shouldBeIgnored() const
 
 void NameConflictedStalledIssue::showRemoteRenameHasFailedMessageBox(const mega::MegaError& error, bool isFile)
 {
-    QMegaMessageBox::MessageBoxInfo failedInfo;
-    failedInfo.title = MegaSyncApp->getMEGAString();
+    MessageDialogInfo failedInfo;
     failedInfo.textFormat = Qt::RichText;
-    failedInfo.buttons = QMessageBox::Yes;
+    failedInfo.buttons = QMessageBox::Ok;
     if(isFile)
     {
-        failedInfo.text = StalledIssuesStrings::RemoveFileFailedTitle();
+        failedInfo.titleText = StalledIssuesStrings::RemoveFileFailedTitle();
     }
     else
     {
-        failedInfo.text = StalledIssuesStrings::RemoveFolderFailedTitle();
+        failedInfo.titleText = StalledIssuesStrings::RemoveFolderFailedTitle();
     }
-    failedInfo.informativeText = StalledIssuesStrings::RemoveRemoteFailedDescription(&error);
+    failedInfo.descriptionText = StalledIssuesStrings::RemoveRemoteFailedDescription(&error);
 
     StalledIssuesModel::runMessageBox(failedInfo);
 }
 
 void NameConflictedStalledIssue::showLocalRenameHasFailedMessageBox(bool isFile)
 {
-    QMegaMessageBox::MessageBoxInfo failedInfo;
-    failedInfo.title = MegaSyncApp->getMEGAString();
+    MessageDialogInfo failedInfo;
     failedInfo.textFormat = Qt::RichText;
-    failedInfo.buttons = QMessageBox::Yes;
+    failedInfo.buttons = QMessageBox::Ok;
     if(isFile)
     {
-        failedInfo.text = StalledIssuesStrings::RemoveFileFailedTitle();
-        failedInfo.informativeText = StalledIssuesStrings::RemoveLocalFileFailedDescription();
+        failedInfo.titleText = StalledIssuesStrings::RemoveFileFailedTitle();
+        failedInfo.descriptionText = StalledIssuesStrings::RemoveLocalFileFailedDescription();
     }
     else
     {
-        failedInfo.text = StalledIssuesStrings::RemoveFolderFailedTitle();
-        failedInfo.informativeText = StalledIssuesStrings::RemoveLocalFolderFailedDescription();
+        failedInfo.titleText = StalledIssuesStrings::RemoveFolderFailedTitle();
+        failedInfo.descriptionText = StalledIssuesStrings::RemoveLocalFolderFailedDescription();
     }
 
     StalledIssuesModel::runMessageBox(failedInfo);
@@ -242,9 +248,15 @@ void NameConflictedStalledIssue::updateHandle(mega::MegaHandle handle)
         {
             mLastModifiedNode.cloudItem->mHandle = handle;
 
-            if(node->isFile())
+            if (node->isFile())
             {
-                mCloudConflictedNames.updateFileConflictedName(node->getModificationTime(), node->getSize(), oldNode->getCreationTime(), node->getCreationTime(), QString::fromUtf8(node->getFingerprint()), mLastModifiedNode.cloudItem);
+                mCloudConflictedNames.updateFileConflictedName(
+                    static_cast<unsigned long long>(node->getModificationTime()),
+                    node->getSize(),
+                    static_cast<unsigned long long>(oldNode->getCreationTime()),
+                    static_cast<unsigned long long>(node->getCreationTime()),
+                    QString::fromUtf8(node->getFingerprint()),
+                    mLastModifiedNode.cloudItem);
             }
         }
 
@@ -390,33 +402,11 @@ bool NameConflictedStalledIssue::solveLocalConflictedNameByRename(int conflictIn
         auto siblingItem(findOtherSideItem(cloudConflictedNames, conflictName));
         if(siblingItem)
         {
-            result = renameCloudSibling(siblingItem, renameTo);
+            siblingItem->solveByRename(renameTo);
         }
 
-        //Undo the local name change
-        if(!result)
-        {
-            QFileInfo originalFileInfo(conflictName->mConflictedPath);
-            QFileInfo newFileInfo(originalFileInfo.path(), renameTo);
-            QFile file(newFileInfo.filePath());
-            if(file.exists())
-            {
-                file.rename(QDir::toNativeSeparators(originalFileInfo.filePath()));
-            }
-
-            //Fail string pending
-            conflictName->setFailed(tr("Unable to rename the file in MEGA"));
-        }
-
-        if(result)
-        {
-            conflictName->solveByRename(renameTo);
-        }
-
-        if(!siblingItem || result)
-        {
-            result = checkAndSolveConflictedNamesSolved();
-        }
+        conflictName->solveByRename(renameTo);
+        result = checkAndSolveConflictedNamesSolved();
     }
 
     return result;
@@ -436,35 +426,12 @@ bool NameConflictedStalledIssue::solveCloudConflictedNameByRename(int conflictIn
             auto siblingItem(findOtherSideItem(mLocalConflictedNames, conflictName));
             if(siblingItem)
             {
-                result = renameLocalSibling(siblingItem, renameTo);
+                siblingItem->solveByRename(renameTo);
             }
 
-             //Undo the local name change
-            if(!result)
-            {
-                std::unique_ptr<mega::MegaNode> conflictedNode(MegaSyncApp->getMegaApi()->getNodeByHandle(conflictName->mHandle));
-                if(conflictedNode)
-                {
-                    MegaApiSynchronizedRequest::runRequest(
-                        &mega::MegaApi::renameNode,
-                        MegaSyncApp->getMegaApi(),
-                        conflictedNode.get(),
-                        renameFrom.toStdString().c_str());
+            conflictName->solveByRename(renameTo);
 
-                    //Fail string pending
-                    conflictName->setFailed(tr("Unable to rename the local file"));
-                }
-            }
-
-            if(result)
-            {
-                conflictName->solveByRename(renameTo);
-            }
-
-            if(!siblingItem || result)
-            {
-                result = checkAndSolveConflictedNamesSolved();
-            }
+            result = checkAndSolveConflictedNamesSolved();
         }
     }
 
@@ -473,10 +440,16 @@ bool NameConflictedStalledIssue::solveCloudConflictedNameByRename(int conflictIn
 
 bool NameConflictedStalledIssue::renameNodesAutomatically()
 {
-    auto sortLogic = [](QList<std::shared_ptr<ConflictedNameInfo>>& names){
-        std::sort(names.begin(), names.end(), [](const std::shared_ptr<ConflictedNameInfo>& check1, const std::shared_ptr<ConflictedNameInfo>& check2){
-            return check1->mItemAttributes->modifiedTime() > check2->mItemAttributes->modifiedTime();
-        });
+    auto sortLogic = [](QList<std::shared_ptr<ConflictedNameInfo>>& names)
+    {
+        std::sort(names.begin(),
+                  names.end(),
+                  [](const std::shared_ptr<ConflictedNameInfo>& check1,
+                     const std::shared_ptr<ConflictedNameInfo>& check2)
+                  {
+                      return check1->mItemAttributes->modifiedTimeInMSecs() >
+                             check2->mItemAttributes->modifiedTimeInMSecs();
+                  });
     };
 
     auto result(true);
@@ -501,8 +474,8 @@ bool NameConflictedStalledIssue::renameNodesAutomatically()
         auto lastModifiedCloudName = cloudConflictedNames.first();
         auto lastModifiedLocalName = localConflictedNames.first();
 
-        if(lastModifiedCloudName->mItemAttributes->modifiedTime() >
-            lastModifiedLocalName->mItemAttributes->modifiedTime())
+        if (lastModifiedCloudName->mItemAttributes->modifiedTimeInMSecs() >
+            lastModifiedLocalName->mItemAttributes->modifiedTimeInMSecs())
         {
             if((result = renameCloudNodesAutomatically(
                    cloudConflictedNames, localConflictedNames, true, itemsBeingRenamed)))
@@ -560,15 +533,15 @@ bool NameConflictedStalledIssue::renameCloudNodesAutomatically(const QList<std::
                     MegaApiSynchronizedRequest::runRequestWithResult(
                         &mega::MegaApi::renameNode,
                         MegaSyncApp->getMegaApi(),
-                        [this, &error](const mega::MegaRequest&, const mega::MegaError& e)
+                        [&error](mega::MegaRequest*, mega::MegaError* e)
                         {
-                            if(e.getErrorCode() != mega::MegaError::API_OK)
+                            if(e->getErrorCode() != mega::MegaError::API_OK)
                             {
-                                error.reset(e.copy());
+                                error.reset(e->copy());
                             }
                         },
                         conflictedNode.get(),
-                        newName.toStdString().c_str());
+                        newName.toUtf8().constData());
 
                     if(error)
                     {
@@ -644,29 +617,35 @@ bool NameConflictedStalledIssue::renameLocalItemsAutomatically(const QList<std::
     return result;
 }
 
-bool NameConflictedStalledIssue::renameCloudSibling(std::shared_ptr<ConflictedNameInfo> item, const QString &newName)
+bool NameConflictedStalledIssue::renameCloudSibling(std::shared_ptr<ConflictedNameInfo> item,
+                                                    const QString& newName)
 {
     std::shared_ptr<mega::MegaError> error(nullptr);
-    if(item)
+    if (item)
     {
-        std::unique_ptr<mega::MegaNode> conflictedNode(MegaSyncApp->getMegaApi()->getNodeByHandle(item->mHandle));
-        if(conflictedNode)
+        std::unique_ptr<mega::MegaNode> conflictedNode(
+            MegaSyncApp->getMegaApi()->getNodeByHandle(item->mHandle));
+        if (conflictedNode)
         {
-            std::unique_ptr<mega::MegaNode> parentNode(MegaSyncApp->getMegaApi()->getNodeByHandle(conflictedNode->getParentHandle()));
+            std::unique_ptr<mega::MegaNode> parentNode(
+                MegaSyncApp->getMegaApi()->getNodeByHandle(conflictedNode->getParentHandle()));
             MegaApiSynchronizedRequest::runRequestWithResult(
                 &mega::MegaApi::renameNode,
                 MegaSyncApp->getMegaApi(),
-                [&error](const mega::MegaRequest&, const mega::MegaError& e)
+                [&error](mega::MegaRequest*, mega::MegaError* e)
                 {
-                    if(e.getErrorCode() != mega::MegaError::API_OK)
+                    if (e->getErrorCode() != mega::MegaError::API_OK)
                     {
-                        error.reset(e.copy());
+                        error.reset(e->copy());
                     }
                 },
                 conflictedNode.get(),
-                newName.toStdString().c_str());
+                newName.toUtf8().constData());
 
-            error ? item->setFailed(RenameRemoteNodeDialog::renamedFailedErrorString(error.get(), conflictedNode->isFile())) : item->solveByRename(newName);
+            error ? item->setFailed(RenameRemoteNodeDialog::renamedFailedErrorString(
+                        error.get(),
+                        conflictedNode->isFile())) :
+                    item->solveByRename(newName);
         }
     }
 
@@ -705,11 +684,11 @@ std::shared_ptr<NameConflictedStalledIssue::ConflictedNameInfo> NameConflictedSt
             auto sameName(check->getConflictedName().compare(fileIt->getConflictedName(), Qt::CaseSensitive) == 0);
             if(check->mItemAttributes->size() != 0 && (check->mItemAttributes->size() == check->mItemAttributes->size()))
             {
-                auto fp1(check->mItemAttributes->fingerprint());
-                auto fp2(fileIt->mItemAttributes->fingerprint());
-                if(!fp1.isEmpty() && !fp2.isEmpty())
+                auto crc1(check->mItemAttributes->getCRC());
+                auto crc2(fileIt->mItemAttributes->getCRC());
+                if(!crc1.isEmpty() && !crc2.isEmpty())
                 {
-                    sameFingerprint = (fp1.compare(fp2) == 0);
+                    sameFingerprint = (crc1.compare(crc2) == 0);
                 }
             }
 
@@ -799,7 +778,7 @@ bool NameConflictedStalledIssue::semiAutoSolveIssue(ActionsSelected option)
 }
 
 //This code is never called. NameConflict, for the moment, are not autosolvable.
-bool NameConflictedStalledIssue::autoSolveIssue()
+StalledIssue::AutoSolveIssueResult NameConflictedStalledIssue::autoSolveIssue()
 {
     setAutoResolutionApplied(true);
     ActionsSelected options(ActionSelected::RemoveDuplicated | ActionSelected::Rename | ActionSelected::MergeFolders);
@@ -807,10 +786,10 @@ bool NameConflictedStalledIssue::autoSolveIssue()
     if(result)
     {
         MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::SI_NAMECONFLICT_SOLVED_AUTOMATICALLY);
-        return true;
+        return StalledIssue::AutoSolveIssueResult::SOLVED;
     }
 
-    return false;
+    return StalledIssue::AutoSolveIssueResult::FAILED;
 }
 
 bool NameConflictedStalledIssue::isAutoSolvable() const
@@ -825,16 +804,16 @@ bool NameConflictedStalledIssue::solveIssue(ActionsSelected option)
     if(option & ActionSelected::MergeFolders && foldersCount() > 1)
     {
         auto errorInfo = mCloudConflictedNames.mergeFolders();
-        if(!errorInfo.error.isEmpty())
-        {
-            setCloudFailed(errorInfo.conflictIndex, errorInfo.error);
-            result= false;
-        }
+        result = errorInfo.error.isEmpty();
 
         if(result)
         {
             result = checkAndSolveConflictedNamesSolved();
         }
+        else
+        {
+            setCloudFailed(errorInfo.conflictIndex, errorInfo.error);
+        }
     }
 
     if(!result && option & ActionSelected::RemoveDuplicated)
@@ -884,29 +863,20 @@ NameConflictedStalledIssue::CloudConflictedNames::removeDuplicatedNodes()
                         NameConflictedStalledIssue::ConflictedNameInfo::SolvedType::UNSOLVED &&
                     conflictedName != (*(conflictedNamesGroup.conflictedNames.end() - 1)))
                 {
-                    auto moveToBinErrors = MoveToMEGABin::moveToBin(
-                        conflictedName->mHandle, QLatin1String("SyncDuplicated"), true);
-                    if(!moveToBinErrors.binFolderCreationError && !moveToBinErrors.moveError)
+                    auto error = MoveToMEGABin()(conflictedName->mHandle,
+                                                 QLatin1String("SyncDuplicated"),
+                                                 true);
+                    if (error)
                     {
-                        conflictedName->solveByRemove();
-                    }
-                    else
-                    {
-                        std::shared_ptr<mega::MegaError> error;
-                        if(moveToBinErrors.binFolderCreationError)
-                        {
-                            error = moveToBinErrors.binFolderCreationError;
-                        }
-                        else
-                        {
-                            error = moveToBinErrors.moveError;
-                        }
-
                         auto errorStr = StalledIssuesStrings::RemoveRemoteFailedFile(error.get());
                         conflictedName->setFailed(errorStr);
 
                         return error;
                     }
+                    else
+                    {
+                        conflictedName->solveByRemove();
+                    }
                 }
             }
 
@@ -929,8 +899,7 @@ NameConflictedStalledIssue::CloudConflictedNames::keepMostRecentlyModifiedNode()
         std::unique_ptr<mega::MegaNode> conflictNode(MegaSyncApp->getMegaApi()->getNodeByHandle(conflictedName->mHandle));
         if(conflictNode)
         {
-            auto error = StalledIssuesUtilities::removeRemoteFile(
-                conflictNode.get());
+            auto error = Utilities::removeSyncRemoteFile(conflictNode.get());
             if(error)
             {
                 auto errorStr = StalledIssuesStrings::RemoveRemoteFailedFile(
@@ -1032,12 +1001,18 @@ NameConflictedStalledIssue::CloudConflictedNames::mergeFolders()
             std::unique_ptr<mega::MegaNode> folderToMerge(MegaSyncApp->getMegaApi()->getNodeByHandle(conflictedFolder->mHandle));
             if(folderToMerge && folderToMerge->isFolder())
             {
-                MergeMEGAFolders mergeItem(targetFolder.get(), folderToMerge.get());
-                auto error = mergeItem.merge(MergeMEGAFolders::ActionForDuplicates::IgnoreAndMoveToBin);
-                if(error)
+                MergeMEGAFolders foldersMerger(
+                    MergeMEGAFolders::ActionForDuplicates::IgnoreAndMoveToBin,
+                    SyncController::instance().isSyncCaseSensitive(mIssue->firstSyncId()));
+
+                auto error = foldersMerger.merge(targetFolder.get(), folderToMerge.get());
+
+                if (error != mega::MegaError::API_OK)
                 {
                     errorInfo.conflictIndex = index;
-                    errorInfo.error = tr("Unable to merge this folder.");
+                    errorInfo.error = QCoreApplication::translate(
+                        "NameConflictedStalledIssue::CloudConflictedNames",
+                        "Unable to merge this folder.");
                     break;
                 }
                 else
