@@ -1,11 +1,15 @@
 
 #include "SyncInfo.h"
-#include "platform/Platform.h"
-#include "QMegaMessageBox.h"
-#include "UserAttributesRequests/MyBackupsHandle.h"
-#include <MegaNodeNames.h>
-#include <mega/types.h>
+
+#include "MessageDialogOpener.h"
+#include "MyBackupsHandle.h"
+#include "Platform.h"
+#include "ServiceUrls.h"
 #include "StatsEventHandler.h"
+#include "TextDecorator.h"
+#include <MegaNodeNames.h>
+
+#include <mega/types.h>
 
 #include <assert.h>
 
@@ -137,15 +141,16 @@ void SyncInfo::activateSync(std::shared_ptr<SyncSettings> syncSetting)
         {
             if (mSyncToCreateOrigin == SyncOrigin::ONBOARDING_ORIGIN)
             {
-                MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::FIRST_SYNC_FROM_ONBOARDING);
+                MegaSyncApp->getStatsEventHandler()->sendEvent(
+                    AppStatsEvents::EventType::FIRST_SYNC_FROM_ONBOARDING);
             }
             else
             {
-                MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::FIRST_SYNC);
+                MegaSyncApp->getStatsEventHandler()->sendEvent(
+                    AppStatsEvents::EventType::FIRST_SYNC);
             }
 
             mSyncToCreateOrigin = SyncOrigin::NONE;
-
         }
         mIsFirstTwoWaySyncDone = true;
         break;
@@ -157,11 +162,13 @@ void SyncInfo::activateSync(std::shared_ptr<SyncSettings> syncSetting)
         {
             if (mSyncToCreateOrigin == SyncOrigin::ONBOARDING_ORIGIN)
             {
-                MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::FIRST_BACKUP_FROM_ONBOARDING);
+                MegaSyncApp->getStatsEventHandler()->sendEvent(
+                    AppStatsEvents::EventType::FIRST_BACKUP_FROM_ONBOARDING);
             }
             else
             {
-                MegaSyncApp->getStatsEventHandler()->sendEvent(AppStatsEvents::EventType::FIRST_BACKUP);
+                MegaSyncApp->getStatsEventHandler()->sendEvent(
+                    AppStatsEvents::EventType::FIRST_BACKUP);
             }
 
             mSyncToCreateOrigin = SyncOrigin::NONE;
@@ -173,38 +180,44 @@ void SyncInfo::activateSync(std::shared_ptr<SyncSettings> syncSetting)
         break;
     }
 
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title =  MegaSyncApp->getMEGAString();
+    MessageDialogInfo msgInfo;
+    msgInfo.textFormat = Qt::RichText;
 
-    // TODO: extract the QMegaMessageBoxes from the model, use signal to send message
+    // TODO: extract the MessageDialogOpeneres from the model, use signal to send message
 
     // TODO: this never would have worked, comparing an error code to a warning code.
     // maybe implement properly, not as warning but specifically is-on-fat or not, etc.
 
     if (!preferences->isFatWarningShown() && syncSetting->getWarning() == MegaSync::Warning::LOCAL_IS_FAT)
     {
-        msgInfo.text = tr("You are syncing a local folder formatted with a FAT filesystem. "
-                          "That filesystem has deficiencies managing big files and modification"
-                          " times that can cause synchronization problems (e.g. when daylight "
-                          "saving changes), so it's strongly recommended that you only sync "
-                          "folders formatted with more reliable filesystems like NTFS (more information [A]here[/A]).")
-                .replace(QString::fromUtf8("[A]"), QString::fromUtf8("<a href=\"https://help.mega.nz/megasync/syncing.html#can-i-sync-fat-fat32-partitions-under-windows\">"))
-                .replace(QString::fromUtf8("[/A]"), QString::fromUtf8("</a>"));
-        msgInfo.finishFunc = [this](QPointer<QMessageBox>)
+        msgInfo.descriptionText =
+            tr("You are syncing a local folder formatted with a FAT filesystem. That filesystem "
+               "has deficiencies managing big files and modification times that can cause "
+               "synchronization problems (e.g. when daylight saving changes), so it's strongly "
+               "recommended that you only sync folders formatted with more reliable filesystems "
+               "like NTFS (more information [A]here[/A]).");
+        Text::Link link(ServiceUrls::getSyncFat32HelpUrl().toString());
+        link.process(msgInfo.descriptionText);
+
+        msgInfo.finishFunc = [this](QPointer<MessageDialogResult>)
         {
             preferences->setFatWarningShown();
         };
 
-        QMegaMessageBox::warning(msgInfo);
+        MessageDialogOpener::warning(msgInfo);
     }
     else if (!preferences->isOneTimeActionDone(Preferences::ONE_TIME_ACTION_HGFS_WARNING) && syncSetting->getError() == MegaSync::Warning::LOCAL_IS_HGFS)
     {
-        msgInfo.text = tr("You are syncing a local folder shared with VMWare. Those folders do not support filesystem notifications so MEGAsync will have to be continuously scanning to detect changes in your files and folders. Please use a different folder if possible to reduce the CPU usage.");
-        msgInfo.finishFunc = [this](QPointer<QMessageBox>)
+        msgInfo.descriptionText =
+            tr("You are syncing a local folder shared with VMWare. Those folders do not support "
+               "filesystem notifications so MEGAsync will have to be continuously scanning to "
+               "detect changes in your files and folders. Please use a different folder if "
+               "possible to reduce the CPU usage.");
+        msgInfo.finishFunc = [this](QPointer<MessageDialogResult>)
         {
             preferences->setOneTimeActionDone(Preferences::ONE_TIME_ACTION_HGFS_WARNING, true);
         };
-        QMegaMessageBox::warning(msgInfo);
+        MessageDialogOpener::warning(msgInfo);
     }
 }
 
@@ -317,6 +330,20 @@ void SyncInfo::rewriteSyncSettings()
     }
 }
 
+void SyncInfo::onboardingFinished(bool onboardingShown)
+{
+    // write sync settings into user's preferences
+    rewriteSyncSettings();
+
+    // If the onboarding has finished and more than one sync has been created, send an event
+    if (onboardingShown && configuredSyncs.contains(SyncType::TYPE_TWOWAY) &&
+        configuredSyncs[SyncType::TYPE_TWOWAY].size() > 1)
+    {
+        MegaSyncApp->getStatsEventHandler()->sendEvent(
+            AppStatsEvents::EventType::MULTIPLE_SYNCS_CREATED_ON_ONBOARDING);
+    }
+}
+
 void SyncInfo::pickInfoFromOldSync(const SyncData &osd, MegaHandle backupId, bool loadedFromPreviousSessions)
 {
     QMutexLocker qm(&syncMutex);
@@ -407,7 +434,7 @@ QStringList SyncInfo::getCloudDriveSyncMegaFolders(bool cloudDrive)
         for (auto &cs : configuredSyncs[type])
         {
             QString megaFolder = configuredSyncsMap[cs]->getMegaFolder();
-            auto parent_node = std::unique_ptr<MegaNode>(megaApi->getNodeByPath(megaFolder.toStdString().data()));
+            auto parent_node = std::unique_ptr<MegaNode>(megaApi->getNodeByPath(megaFolder.toUtf8().constData()));
 
             while(parent_node && parent_node->getParentHandle() != INVALID_HANDLE)
             {
@@ -611,8 +638,6 @@ void SyncInfo::setSyncToCreateOrigin(SyncOrigin newSyncToCreate)
     mSyncToCreateOrigin = newSyncToCreate;
 }
 
-
-
 void SyncInfo::addUnattendedDisabledSync(MegaHandle tag, mega::MegaSync::SyncType type)
 {
     unattendedDisabledSyncs[type].insert(tag);
@@ -732,6 +757,12 @@ void SyncInfo::onSyncStatsUpdated(mega::MegaApi*, mega::MegaSyncStats *syncStats
     emit syncStatsUpdated(sp);
 }
 
+void SyncInfo::onSyncRemoteRootChanged(mega::MegaApi*, mega::MegaSync* sync)
+{
+    auto sc = updateSyncSettings(sync);
+    emit syncRemoteRootChanged(sc);
+}
+
 void SyncInfo::showSingleSyncDisabledNotification(std::shared_ptr<SyncSettings> syncSetting)
 {
     if (syncSetting)
