@@ -1,21 +1,23 @@
 #ifndef BACKUPSETTINGSELEMENTS_H
 #define BACKUPSETTINGSELEMENTS_H
 
-#include <QObject>
-#include <megaapi.h>
+#include "megaapi.h"
 
-namespace Ui {
+#include <QObject>
+
+namespace Ui
+{
 class OpenBackupsFolder;
 }
 
 class SyncSettingsUIBase;
 
-class BackupSettingsElements : public QObject
+class BackupSettingsElements: public QObject
 {
     Q_OBJECT
 
 public:
-    explicit BackupSettingsElements(QObject *parent = nullptr);
+    explicit BackupSettingsElements(QObject* parent = nullptr);
     ~BackupSettingsElements();
 
     void initElements(SyncSettingsUIBase* syncSettingsUi);
