@@ -1,44 +1,42 @@
 #ifndef BACKUPSSETTINGSUI_H
 #define BACKUPSSETTINGSUI_H
 
-#include <QWidget>
+#include "BackupSettingsElements.h"
+#include "SyncSettingsUIBase.h"
+
 #include <QPointer>
+#include <QWidget>
 
-#include <syncs/gui/SyncSettingsUIBase.h>
-#include <syncs/gui/Backups/BackupTableView.h>
-#include <syncs/gui/Backups/BackupSettingsElements.h>
-
-class BackupSettingsUI : public SyncSettingsUIBase
+class BackupSettingsUI: public SyncSettingsUIBase
 {
     Q_OBJECT
 
 public:
-    explicit BackupSettingsUI(QWidget *parent = nullptr);
+    explicit BackupSettingsUI(QWidget* parent = nullptr);
     ~BackupSettingsUI() override;
 
-    void addButtonClicked(mega::MegaHandle megaFolderHandle = mega::INVALID_HANDLE) override;
+    void addButtonClicked(mega::MegaHandle = mega::INVALID_HANDLE) override;
 
 protected:
     QString getFinishWarningIconString() const override;
     QString getFinishIconString() const override;
     QString disableString() const override;
 
-    //Operation failed
+    // Operation failed
     QString getOperationFailTitle() const override;
     QString getOperationFailText(std::shared_ptr<SyncSettings> sync) override;
 
-    //Error adding
+    // Error adding
     QString getErrorAddingTitle() const override;
 
-    //Error removing
+    // Error removing
     QString getErrorRemovingTitle() const override;
     QString getErrorRemovingText(std::shared_ptr<mega::MegaError> err) override;
 
     void setBackupsTitle();
-    void changeEvent(QEvent* event);
+    bool event(QEvent* event) override;
 
 protected slots:
-    void reqRemoveSync(std::shared_ptr<SyncSettings> backup) override;
     void removeSync(std::shared_ptr<SyncSettings> backup) override;
 
 private:
