@@ -1,16 +1,13 @@
 #include "BackupTableView.h"
 
-#include "platform/Platform.h"
-#include "PlatformStrings.h"
-#include "MenuItemAction.h"
-#include "syncs/model/BackupItemModel.h"
+#include "BackupItemModel.h"
 
 #include <QHeaderView>
 #include <QMenu>
 #include <QtConcurrent/QtConcurrent>
 
-BackupTableView::BackupTableView(QWidget *parent)
-    : SyncTableView(parent)
+BackupTableView::BackupTableView(QWidget* parent):
+    SyncTableView(parent)
 {
     mType = mega::MegaSync::TYPE_BACKUP;
     mContextMenuName = "BackupContextMenu";
