@@ -1,92 +1,21 @@
 #include "MegaTransferView.h"
-#include "MegaApplication.h"
-#include "platform/Platform.h"
-#include "Utilities.h"
-#include "gui/QMegaMessageBox.h"
-#include "TransfersWidget.h"
 
+#include "MegaApplication.h"
+#include "MegaMenuItemAction.h"
+#include "MessageDialogOpener.h"
+#include "Platform.h"
+#include "TransfersWidget.h"
+#include "Utilities.h"
+
+#include <QIcon>
 #include <QScrollBar>
 #include <QtConcurrent/QtConcurrent>
-#include <QIcon>
 
 using namespace mega;
 
 const int MAX_ITEMS_FOR_CONTEXT_MENU = 10;
 
-QString MegaTransferView::cancelAllAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "All your transfers will be cancelled.");
-}
-
-QString MegaTransferView::cancelAndClearAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "All your transfers in this category will be cancelled and cleared.");
-}
-
-QString MegaTransferView::cancelAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "All your transfers in this category will be cancelled.");
-}
-
-QString MegaTransferView::cancelWithSyncAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "Your incomplete sync transfers won't be cancelled.");
-}
-
-QString MegaTransferView::cancelAndClearWithSyncAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "Your incomplete sync transfers won't be cancelled\n"
-              "All the other transfers will be cancelled and cleared.");
-}
-
-QString MegaTransferView::clearAllCompletedAskActionText()
-{
-    return tr("Clear transfers?\n"
-              "All your completed transfers will be cleared.");
-}
-
-QString MegaTransferView::clearCompletedAskActionText()
-{
-    return tr("Clear transfers?\n"
-              "All your completed transfers in this category will be cleared.");
-}
-
-//Multiple seletion
-QString MegaTransferView::cancelSelectedAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "All your selected transfers will be cancelled.");
-}
-
-QString MegaTransferView::cancelAndClearSelectedAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "All your selected transfers will be cancelled and cleared.");
-}
-
-QString MegaTransferView::cancelSelectedWithSyncAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "Your selected incomplete sync transfers won't be cancelled.");
-}
-
-QString MegaTransferView::cancelAndClearSelectedWithSyncAskActionText()
-{
-    return tr("Cancel transfers?\n"
-              "Your selected incomplete sync transfers won't be cancelled\n"
-              "All the other selected transfers will be cancelled and cleared.");
-}
-
-QString MegaTransferView::clearSelectedCompletedAskActionText()
-{
-    return tr("Clear transfers?\n"
-              "All the selected completed transfers in this category will be cleared.");
-}
+// Multiple seletion
 
 //Single seletion
 QString MegaTransferView::cancelSingleActionText()
@@ -245,28 +174,33 @@ MegaTransferView::SelectedIndexesInfo MegaTransferView::getVisibleCancelOrClearI
             {
                 if(info.areAllCancellable || !isAnyCompleted)
                 {
-                    info.actionText = cancelAskActionText();
+                    info.titleText = cancelTransfersMultiSelectionTitleText();
+                    info.actionText = cancelCategoryTransfersDescriptionText();
                 }
                 else
                 {
-                    info.actionText = cancelAndClearAskActionText();
+                    info.titleText = cancelTransfersMultiSelectionTitleText();
+                    info.actionText = cancelAndClearDescriptionText();
                 }
             }
             else
             {
                 if(isAnyCompleted)
                 {
-                    info.actionText = cancelAndClearWithSyncAskActionText();
+                    info.titleText = cancelTransfersMultiSelectionTitleText();
+                    info.actionText = cancelAndClearTransfersWithSyncDescriptionText();
                 }
                 else
                 {
-                    info.actionText = cancelWithSyncAskActionText();
+                    info.titleText = cancelTransfersMultiSelectionTitleText();
+                    info.actionText = cancelTransfersWithSyncDescriptionText();
                 }
             }
         }
         else
         {
-            info.actionText = clearCompletedAskActionText();
+            info.titleText = clearTransfersMultiSelectionTitleText();
+            info.actionText = clearCategoryTransfersDescriptionText();
             info.buttonsText = getClearDialogButtons();
         }
     }
@@ -321,11 +255,16 @@ MegaTransferView::SelectedIndexesInfo MegaTransferView::getSelectedCancelOrClear
         {
             if(isAnyCompleted)
             {
-                info.actionText = info.isAnyCancellable ? cancelAndClearSelectedWithSyncAskActionText() : clearSelectedCompletedAskActionText();
+                info.titleText = info.isAnyCancellable ? cancelTransfersMultiSelectionTitleText() :
+                                                         clearTransfersMultiSelectionTitleText();
+                info.actionText = info.isAnyCancellable ?
+                                      cancelAndClearSelectedWithSyncDescriptionText() :
+                                      clearSelectedCompletedTransfersDescriptionText();
             }
             else if(info.isAnyCancellable)
             {
-                info.actionText = cancelSelectedWithSyncAskActionText();
+                info.titleText = cancelTransfersMultiSelectionTitleText();
+                info.actionText = cancelSelectedWithSyncsDescriptionText();
             }
 
         }
@@ -335,17 +274,20 @@ MegaTransferView::SelectedIndexesInfo MegaTransferView::getSelectedCancelOrClear
             {
                 if(info.isAnyCancellable)
                 {
-                    info.actionText = cancelAndClearSelectedAskActionText();
+                    info.titleText = cancelTransfersMultiSelectionTitleText();
+                    info.actionText = cancelAndClearSelectedDescriptionText();
                 }
                 else
                 {
-                    info.actionText = clearSelectedCompletedAskActionText();
+                    info.titleText = clearTransfersMultiSelectionTitleText();
+                    info.actionText = clearSelectedCompletedTransfersDescriptionText();
                     info.buttonsText = getClearDialogButtons();
                 }
             }
             else if(info.isAnyCancellable)
             {
-                info.actionText = cancelSelectedAskActionText();
+                info.titleText = cancelTransfersMultiSelectionTitleText();
+                info.actionText = cancelSelectedDescriptionText();
             }
         }
     }
@@ -393,15 +335,15 @@ void MegaTransferView::onCancelVisibleTransfers()
 
     if(!info.areAllSync)
     {
-        QMegaMessageBox::MessageBoxInfo msgInfo;
-        msgInfo.title = MegaSyncApp->getMEGAString();
-        msgInfo.text = info.actionText;
+        MessageDialogInfo msgInfo;
+        msgInfo.titleText = info.titleText;
+        msgInfo.descriptionText = info.actionText;
         msgInfo.parent = this;
         msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
         msgInfo.defaultButton = QMessageBox::No;
         msgInfo.buttonsText = info.buttonsText;
 
-        msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+        msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
         {
             if(msg->result() == QMessageBox::Yes)
             {
@@ -412,8 +354,7 @@ void MegaTransferView::onCancelVisibleTransfers()
             }
         };
 
-
-        QMegaMessageBox::warning(msgInfo);
+        MessageDialogOpener::warning(msgInfo);
     }
 }
 
@@ -421,15 +362,15 @@ void MegaTransferView::onCancelSelectedTransfers()
 {
     auto info = getSelectedCancelOrClearInfo();
 
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title = MegaSyncApp->getMEGAString();
-    msgInfo.text = info.actionText;
+    MessageDialogInfo msgInfo;
+    msgInfo.titleText = info.titleText;
+    msgInfo.descriptionText = info.actionText;
     msgInfo.parent = this;
     msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
     msgInfo.defaultButton = QMessageBox::No;
     msgInfo.buttonsText = info.buttonsText;
 
-    msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+    msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
     {
         if(msg->result() == QMessageBox::Yes)
         {
@@ -442,7 +383,79 @@ void MegaTransferView::onCancelSelectedTransfers()
         }
     };
 
-    QMegaMessageBox::warning(msgInfo);
+    MessageDialogOpener::warning(msgInfo);
+}
+
+QString MegaTransferView::cancelTransfersMultiSelectionTitleText()
+{
+    return tr("Cancel transfers?");
+}
+
+QString MegaTransferView::clearTransfersMultiSelectionTitleText()
+{
+    return tr("Clear transfers?");
+}
+
+QString MegaTransferView::cancelAllDescriptionText()
+{
+    return tr("All your transfers will be cancelled.");
+}
+
+QString MegaTransferView::cancelAndClearDescriptionText()
+{
+    return tr("All your transfers in this category will be cancelled and cleared.");
+}
+
+QString MegaTransferView::cancelCategoryTransfersDescriptionText()
+{
+    return tr("All your transfers in this category will be cancelled.");
+}
+
+QString MegaTransferView::cancelTransfersWithSyncDescriptionText()
+{
+    return tr("Your incomplete sync transfers won't be cancelled.");
+}
+
+QString MegaTransferView::cancelAndClearTransfersWithSyncDescriptionText()
+{
+    return tr("Your incomplete sync transfers won't be cancelled\n"
+              "All the other transfers will be cancelled and cleared.");
+}
+
+QString MegaTransferView::clearAllCompletedDescriptionText()
+{
+    return tr("All your completed transfers will be cleared.");
+}
+
+QString MegaTransferView::clearCategoryTransfersDescriptionText()
+{
+    return tr("All your completed transfers in this category will be cleared.");
+}
+
+QString MegaTransferView::cancelSelectedDescriptionText()
+{
+    return tr("All your selected transfers will be cancelled.");
+}
+
+QString MegaTransferView::cancelAndClearSelectedDescriptionText()
+{
+    return tr("All your selected transfers will be cancelled and cleared.");
+}
+
+QString MegaTransferView::cancelSelectedWithSyncsDescriptionText()
+{
+    return tr("Your selected incomplete sync transfers won't be cancelled.");
+}
+
+QString MegaTransferView::cancelAndClearSelectedWithSyncDescriptionText()
+{
+    return tr("Your selected incomplete sync transfers won't be cancelled\n"
+              "All the other selected transfers will be cancelled and cleared.");
+}
+
+QString MegaTransferView::clearSelectedCompletedTransfersDescriptionText()
+{
+    return tr("All the selected completed transfers in this category will be cleared.");
 }
 
 void MegaTransferView::onCancelAllTransfers()
@@ -451,15 +464,16 @@ void MegaTransferView::onCancelAllTransfers()
 
     if (proxy)
     {
-        QMegaMessageBox::MessageBoxInfo msgInfo;
-        msgInfo.title = MegaSyncApp->getMEGAString();
-        msgInfo.text = proxy->isAnySync() ?  cancelWithSyncAskActionText() : cancelAllAskActionText();
+        MessageDialogInfo msgInfo;
+        msgInfo.titleText = cancelTransfersMultiSelectionTitleText();
+        msgInfo.descriptionText = proxy->isAnySync() ? cancelTransfersWithSyncDescriptionText() :
+                                                       cancelAllDescriptionText();
         msgInfo.parent = this;
         msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
         msgInfo.defaultButton = QMessageBox::No;
         msgInfo.buttonsText = getCancelDialogButtons();
 
-        msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+        msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
         {
             if(msg->result() == QMessageBox::Yes)
             {
@@ -468,21 +482,21 @@ void MegaTransferView::onCancelAllTransfers()
             }
         };
 
-        QMegaMessageBox::warning(msgInfo);
+        MessageDialogOpener::warning(msgInfo);
     }
 }
 
 void MegaTransferView::onClearAllTransfers()
 {
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title = MegaSyncApp->getMEGAString();
-    msgInfo.text = clearAllCompletedAskActionText();
+    MessageDialogInfo msgInfo;
+    msgInfo.titleText = clearTransfersMultiSelectionTitleText();
+    msgInfo.descriptionText = clearAllCompletedDescriptionText();
     msgInfo.parent = this;
     msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
     msgInfo.defaultButton = QMessageBox::No;
     msgInfo.buttonsText = getClearDialogButtons();
 
-    msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+    msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
     {
         if(msg->result() == QMessageBox::Yes)
         {
@@ -490,22 +504,22 @@ void MegaTransferView::onClearAllTransfers()
         }
     };
 
-    QMegaMessageBox::warning(msgInfo);
+    MessageDialogOpener::warning(msgInfo);
 }
 
 void MegaTransferView::onCancelAndClearVisibleTransfers()
 {
     auto info = getVisibleCancelOrClearInfo();
 
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title = MegaSyncApp->getMEGAString();
-    msgInfo.text = info.actionText;
+    MessageDialogInfo msgInfo;
+    msgInfo.titleText = info.titleText;
+    msgInfo.descriptionText = info.actionText;
     msgInfo.parent = this;
     msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
     msgInfo.defaultButton = QMessageBox::No;
     msgInfo.buttonsText = info.buttonsText;
 
-    msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+    msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
     {
         if(msg->result() == QMessageBox::Yes)
         {
@@ -520,20 +534,20 @@ void MegaTransferView::onCancelAndClearVisibleTransfers()
         }
     };
 
-    QMegaMessageBox::warning(msgInfo);
+    MessageDialogOpener::warning(msgInfo);
 }
 
 void MegaTransferView::onClearVisibleTransfers()
 {
-    QMegaMessageBox::MessageBoxInfo msgInfo;
-    msgInfo.title = MegaSyncApp->getMEGAString();
-    msgInfo.text = clearCompletedAskActionText();
+    MessageDialogInfo msgInfo;
+    msgInfo.titleText = clearTransfersMultiSelectionTitleText();
+    msgInfo.descriptionText = clearCategoryTransfersDescriptionText();
     msgInfo.parent = this;
     msgInfo.buttons = QMessageBox::Yes | QMessageBox::No;
     msgInfo.defaultButton = QMessageBox::No;
     msgInfo.buttonsText = getClearDialogButtons();
 
-    msgInfo.finishFunc = [this](QPointer<QMessageBox> msg)
+    msgInfo.finishFunc = [this](QPointer<MessageDialogResult> msg)
     {
         if(msg->result() == QMessageBox::Yes)
         {
@@ -544,7 +558,7 @@ void MegaTransferView::onClearVisibleTransfers()
         }
     };
 
-    QMegaMessageBox::warning(msgInfo);
+    MessageDialogOpener::warning(msgInfo);
 }
 
 void MegaTransferView::clearAllTransfers()
@@ -608,9 +622,9 @@ void MegaTransferView::onCustomContextMenu(const QPoint& point)
 QMenu* MegaTransferView::createContextMenu()
 {
     auto contextMenu = new QMenu(this);
-    contextMenu->setWindowFlags(contextMenu->windowFlags() | Qt::NoDropShadowWindowHint);
+    contextMenu->setProperty("class", QLatin1String("MegaMenu"));
+    contextMenu->setProperty("icon-token", QLatin1String("icon-primary"));
     contextMenu->setAttribute(Qt::WA_DeleteOnClose);
-    Platform::getInstance()->initMenu(contextMenu, "ContextMenu");
 
     QModelIndexList indexes = selectedIndexes();
     auto modelSize = model()->rowCount();
@@ -629,16 +643,18 @@ QMenu* MegaTransferView::createContextMenu()
     QList<QDir> localFoldersToOpenByContextMenu;
     QList<QFileInfo> localFilesToOpenByContextMenu;
 
+    auto proxy(qobject_cast<TransfersManagerSortFilterProxyModel*>(model()));
+
     for (auto index : qAsConst(indexes))
     {
         if(index.row() == 0)
         {
-            isTopIndex = true;
+            isTopIndex = proxy->getSortCriterion() == static_cast<int>(SortCriterion::PRIORITY);
         }
 
         if(index.row() == (modelSize -1))
         {
-            isBottomIndex = true;
+            isBottomIndex = proxy->getSortCriterion() == static_cast<int>(SortCriterion::PRIORITY);
         }
 
         auto d (qvariant_cast<TransferItem>(index.data()).getTransferData());
@@ -714,7 +730,7 @@ QMenu* MegaTransferView::createContextMenu()
             }
             case TransferData::TRANSFER_PAUSED:
                 isMovableRow();
-
+                [[fallthrough]];
             default:
                 overallState |= d->getState();
         }
@@ -795,11 +811,18 @@ QMenu* MegaTransferView::createContextMenu()
 
     bool addSeparator(false);
 
-    if(actionFlag & EnableAction::PAUSE)
+    if (actionFlag & EnableAction::PAUSE)
     {
-        auto pauseAction = new MenuItemAction(pauseActionText(indexes.size()),QLatin1String(":/images/transfer_manager/context_menu/pause_ico.png"), contextMenu);
-        connect(pauseAction, &QAction::triggered,
-                this, &MegaTransferView::pauseSelectedClicked);
+        auto pauseAction =
+            new MegaMenuItemAction(pauseActionText(indexes.size()),
+                                   Utilities::getPixmapName(QLatin1String("pause"),
+                                                            Utilities::AttributeType::SMALL |
+                                                                Utilities::AttributeType::THIN |
+                                                                Utilities::AttributeType::OUTLINE,
+                                                            false),
+                                   0,
+                                   contextMenu);
+        connect(pauseAction, &QAction::triggered, this, &MegaTransferView::pauseSelectedClicked);
 
         contextMenu->addAction(pauseAction);
         addSeparator = true;
@@ -807,7 +830,15 @@ QMenu* MegaTransferView::createContextMenu()
 
     if(actionFlag & EnableAction::RESUME)
     {
-        auto resumeAction = new MenuItemAction(resumeActionText(indexes.size()), QLatin1String(":/images/transfer_manager/context_menu/resume_ico.png"), contextMenu);
+        auto resumeAction =
+            new MegaMenuItemAction(resumeActionText(indexes.size()),
+                                   Utilities::getPixmapName(QLatin1String("play"),
+                                                            Utilities::AttributeType::SMALL |
+                                                                Utilities::AttributeType::THIN |
+                                                                Utilities::AttributeType::OUTLINE,
+                                                            false),
+                                   0,
+                                   contextMenu);
         connect(resumeAction, &QAction::triggered,
                 this, &MegaTransferView::resumeSelectedClicked);
 
@@ -822,7 +853,15 @@ QMenu* MegaTransferView::createContextMenu()
     {
         if(localFilesToOpenByContextMenu.size() <= MAX_ITEMS_FOR_CONTEXT_MENU)
         {
-            auto openItemAction = new MenuItemAction(tr("Open"), QLatin1String(":/images/transfer_manager/context_menu/open_file_ico.png"), contextMenu);
+            auto openItemAction = new MegaMenuItemAction(
+                tr("Open"),
+                Utilities::getPixmapName(QLatin1String("external_link"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
             connect(openItemAction, &QAction::triggered, this, &MegaTransferView::openItemClicked);
 
             contextMenu->addAction(openItemAction);
@@ -831,8 +870,15 @@ QMenu* MegaTransferView::createContextMenu()
         if(localFoldersToOpenByContextMenu.size() <= MAX_ITEMS_FOR_CONTEXT_MENU)
         {
             //Ico not included in transfer manager folder as it is also used by settingsDialog
-            auto showInFolderAction = new MenuItemAction(tr("Show in folder"), QLatin1String(":/images/show_in_folder_ico.png"),
-                                                         contextMenu);
+            auto showInFolderAction = new MegaMenuItemAction(
+                tr("Show in folder"),
+                Utilities::getPixmapName(QLatin1String("folder-open"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
             connect(showInFolderAction, &QAction::triggered, this, &MegaTransferView::showInFolderClicked);
 
             contextMenu->addAction(showInFolderAction);
@@ -849,14 +895,30 @@ QMenu* MegaTransferView::createContextMenu()
             if (handlesToOpenByContextMenu.size() <= MAX_ITEMS_FOR_CONTEXT_MENU)
             {
                 //Ico not included in transfer manager folder as it is also used by settingsDialog
-                auto openInMEGAAction = new MenuItemAction(tr("Open in MEGA"), QLatin1String(":/images/ico_open_MEGA.png"), contextMenu);
+                auto openInMEGAAction = new MegaMenuItemAction(
+                    tr("Open in MEGA"),
+                    Utilities::getPixmapName(QLatin1String("MEGA"),
+                                             Utilities::AttributeType::SMALL |
+                                                 Utilities::AttributeType::THIN |
+                                                 Utilities::AttributeType::OUTLINE,
+                                             false),
+                    0,
+                    contextMenu);
                 connect(openInMEGAAction, &QAction::triggered, this, &MegaTransferView::openInMEGAClicked);
 
                 contextMenu->addAction(openInMEGAAction);
             }
             if (!containsIncomingShares)
             {
-                auto getLinkAction = new MenuItemAction(tr("Get link"), QLatin1String(":/images/transfer_manager/context_menu/get_link_ico.png"), contextMenu);
+                auto getLinkAction = new MegaMenuItemAction(
+                    tr("Get link"),
+                    Utilities::getPixmapName(QLatin1String("link-01"),
+                                             Utilities::AttributeType::SMALL |
+                                                 Utilities::AttributeType::THIN |
+                                                 Utilities::AttributeType::OUTLINE,
+                                             false),
+                    0,
+                    contextMenu);
                 connect(getLinkAction, &QAction::triggered, this, &MegaTransferView::getLinkClicked);
 
                 contextMenu->addAction(getLinkAction);
@@ -868,14 +930,33 @@ QMenu* MegaTransferView::createContextMenu()
 
     addSeparatorToContextMenu(addSeparator, contextMenu);
 
-    if(actionFlag & EnableAction::MOVE)
+    if (actionFlag & EnableAction::MOVE)
     {
-        if(!isTopIndex)
+        if (!isTopIndex)
         {
-            auto moveToTopAction = new MenuItemAction(tr("Move to top"), QLatin1String(":/images/transfer_manager/context_menu/move_top_ico.png"), contextMenu);
-            connect(moveToTopAction, &QAction::triggered, this, &MegaTransferView::moveToTopClicked);
+            auto moveToTopAction = new MegaMenuItemAction(
+                tr("Move to top"),
+                Utilities::getPixmapName(QLatin1String("move-top"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
+            connect(moveToTopAction,
+                    &QAction::triggered,
+                    this,
+                    &MegaTransferView::moveToTopClicked);
 
-            auto moveUpAction = new MenuItemAction(tr("Move up"), QLatin1String(":/images/transfer_manager/context_menu/move_up_ico.png"), contextMenu);
+            auto moveUpAction = new MegaMenuItemAction(
+                tr("Move up"),
+                Utilities::getPixmapName(QLatin1String("move-up"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
             connect(moveUpAction, &QAction::triggered, this, &MegaTransferView::moveUpClicked);
 
             contextMenu->addAction(moveToTopAction);
@@ -884,13 +965,32 @@ QMenu* MegaTransferView::createContextMenu()
             addSeparator = true;
         }
 
-        if(!isBottomIndex)
+        if (!isBottomIndex)
         {
-            auto moveDownAction = new MenuItemAction(tr("Move down"), QLatin1String(":/images/transfer_manager/context_menu/move_down_ico.png"), contextMenu);
+            auto moveDownAction = new MegaMenuItemAction(
+                tr("Move down"),
+                Utilities::getPixmapName(QLatin1String("move-down"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
             connect(moveDownAction, &QAction::triggered, this, &MegaTransferView::moveDownClicked);
 
-            auto moveToBottomAction = new MenuItemAction(tr("Move to bottom"), QLatin1String(":/images/transfer_manager/context_menu/move_bottom_ico.png"), contextMenu);
-            connect(moveToBottomAction, &QAction::triggered, this, &MegaTransferView::moveToBottomClicked);
+            auto moveToBottomAction = new MegaMenuItemAction(
+                tr("Move to bottom"),
+                Utilities::getPixmapName(QLatin1String("move-bottom"),
+                                         Utilities::AttributeType::SMALL |
+                                             Utilities::AttributeType::THIN |
+                                             Utilities::AttributeType::OUTLINE,
+                                         false),
+                0,
+                contextMenu);
+            connect(moveToBottomAction,
+                    &QAction::triggered,
+                    this,
+                    &MegaTransferView::moveToBottomClicked);
 
             contextMenu->addAction(moveDownAction);
             contextMenu->addAction(moveToBottomAction);
@@ -903,8 +1003,16 @@ QMenu* MegaTransferView::createContextMenu()
 
     if(actionFlag & EnableAction::CANCEL)
     {
-        auto cancelAction = new MenuItemAction(actionFlag & EnableAction::CLEAR ? cancelAndClearActionText(indexes.size()) : cancelActionText(indexes.size()),
-                                               QLatin1String(":/images/transfer_manager/context_menu/cancel_transfer_ico.png"), contextMenu);
+        auto cancelAction = new MegaMenuItemAction(
+            actionFlag & EnableAction::CLEAR ? cancelAndClearActionText(indexes.size()) :
+                                               cancelActionText(indexes.size()),
+            Utilities::getPixmapName(QLatin1String("x"),
+                                     Utilities::AttributeType::SMALL |
+                                         Utilities::AttributeType::THIN |
+                                         Utilities::AttributeType::OUTLINE,
+                                     false),
+            0,
+            contextMenu);
         connect(cancelAction, &QAction::triggered,
                 this, &MegaTransferView::cancelSelectedClicked);
 
@@ -915,7 +1023,15 @@ QMenu* MegaTransferView::createContextMenu()
     }
     else if(actionFlag & EnableAction::CLEAR)
     {
-        auto clearAction = new MenuItemAction(tr("Clear"), QLatin1String(":/images/transfer_manager/context_menu/ico_clear.png"), contextMenu);
+        auto clearAction =
+            new MegaMenuItemAction(tr("Clear"),
+                                   Utilities::getPixmapName(QLatin1String("eraser"),
+                                                            Utilities::AttributeType::SMALL |
+                                                                Utilities::AttributeType::THIN |
+                                                                Utilities::AttributeType::OUTLINE,
+                                                            false),
+                                   0,
+                                   contextMenu);
         connect(clearAction, &QAction::triggered,
                 this, &MegaTransferView::clearSelectedClicked);
 
@@ -943,7 +1059,7 @@ void MegaTransferView::mouseReleaseEvent(QMouseEvent* event)
         clearSelection();
     }
 
-    QTreeView::mouseReleaseEvent(event);
+    LoadingSceneView::mouseReleaseEvent(event);
 }
 
 void MegaTransferView::mouseMoveEvent(QMouseEvent *event)
@@ -953,17 +1069,17 @@ void MegaTransferView::mouseMoveEvent(QMouseEvent *event)
         return;
     }
 
-    QTreeView::mouseMoveEvent(event);
+    LoadingSceneView::mouseMoveEvent(event);
 }
 
-void MegaTransferView::changeEvent(QEvent* event)
+bool MegaTransferView::event(QEvent* event)
 {
-    if(event->type() == QEvent::LanguageChange)
+    if (event->type() == QEvent::LanguageChange)
     {
         viewport()->update();
     }
 
-    QTreeView::changeEvent(event);
+    return LoadingSceneView::event(event);
 }
 
 void MegaTransferView::dropEvent(QDropEvent* event)
@@ -984,7 +1100,7 @@ void MegaTransferView::keyPressEvent(QKeyEvent *event)
         mKeyNavigation = true;
     }
 
-    QTreeView::keyPressEvent(event);
+    LoadingSceneView::keyPressEvent(event);
 
     if(mKeyNavigation)
     {
@@ -1009,7 +1125,7 @@ void MegaTransferView::selectionChanged(const QItemSelection &selected, const QI
         }
     }
 
-    QTreeView::selectionChanged(selected, deselected);
+    LoadingSceneView::selectionChanged(selected, deselected);
 }
 
 bool MegaTransferView::eventFilter(QObject *object, QEvent *event)
@@ -1026,7 +1142,7 @@ bool MegaTransferView::eventFilter(QObject *object, QEvent *event)
         }
     }
 
-    return QTreeView::eventFilter(object, event);
+    return LoadingSceneView::eventFilter(object, event);
 }
 
 void MegaTransferView::paintEvent(QPaintEvent *event)
@@ -1043,12 +1159,12 @@ void MegaTransferView::paintEvent(QPaintEvent *event)
         }
     }
 
-    QTreeView::paintEvent(event);
+    LoadingSceneView::paintEvent(event);
 }
 
 void MegaTransferView::moveToTopClicked()
 {
-    auto proxy(qobject_cast<QSortFilterProxyModel*>(model()));
+    auto proxy(qobject_cast<TransfersManagerSortFilterProxyModel*>(model()));
     if(proxy)
     {
         if(!selectionModel())
@@ -1061,25 +1177,7 @@ void MegaTransferView::moveToTopClicked()
         {
             auto sourceModel = MegaSyncApp->getTransfersModel();
 
-            // Sort to keep items in the same order
-            std::sort(indexes.begin(), indexes.end(),[](QModelIndex check1, QModelIndex check2){
-                return check1.row() > check2.row();
-            });
-
-            auto firstToMove = indexes.first();
-
-            if(firstToMove.row() == 0)
-            {
-                return;
-            }
-
-            for (int item = 0; item < indexes.size(); ++item)
-            {
-                auto index = indexes.at(item);
-                auto sourceIndex = proxy->mapToSource(index);
-
-                sourceModel->moveTransferPriority(QModelIndex(), QList<int>() << sourceIndex.row(), QModelIndex(), -1);
-            }
+            sourceModel->moveTransferPriorityToTop(proxy->mapToSourceList(indexes));
         }
     }
 
@@ -1088,112 +1186,32 @@ void MegaTransferView::moveToTopClicked()
 
 void MegaTransferView::moveUpClicked()
 {
-    QModelIndexList indexes;
-    if(selectionModel())
+    auto proxy(qobject_cast<TransfersManagerSortFilterProxyModel*>(model()));
+    if (proxy && selectionModel())
     {
-        indexes = selectionModel()->selectedRows();
-    }
-
-    if(!indexes.isEmpty())
-    {
-        auto proxy(qobject_cast<QSortFilterProxyModel*>(model()));
-        if(proxy)
+        auto indexes = selectionModel()->selectedRows();
+        if (!indexes.isEmpty())
         {
             auto sourceModel = MegaSyncApp->getTransfersModel();
 
-            // Sort to keep items in the same order
-            std::sort(indexes.begin(), indexes.end(),[](QModelIndex check1, QModelIndex check2){
-                return check1.row() < check2.row();
-            });
-
-            auto firstToMove = indexes.first();
-
-            if(firstToMove.row() == 0)
-            {
-                return;
-            }
-
-            auto proxyTargetIndex = proxy->index(indexes.first().row() - 1,0);
-            auto sourceTargetIndex = proxy->mapToSource(proxyTargetIndex);
-            auto rowTarget = sourceTargetIndex.row();
-
-            for (int item = 0; item < indexes.size(); ++item)
-            {
-                auto index = indexes.at(item);
-                auto sourceIndex = proxy->mapToSource(index);
-
-                if(item != 0)
-                {
-                    auto previousIndex = indexes.at(item-1);
-
-                    if(index.row() - previousIndex.row() != 1)
-                    {
-                        proxyTargetIndex = proxy->index(index.row()-1,0);
-                        sourceTargetIndex = proxy->mapToSource(proxyTargetIndex);
-                        rowTarget = sourceTargetIndex.row();
-                    }
-                }
-
-                sourceModel->moveTransferPriority(QModelIndex(), QList<int>() << sourceIndex.row(), QModelIndex(), rowTarget);
-            }
+            sourceModel->moveTransferPriorityUp(proxy->mapToSourceList(indexes));
         }
     }
+
     clearSelection();
 }
 
 void MegaTransferView::moveDownClicked()
 {
-    QModelIndexList indexes;
-    if(selectionModel())
+    auto proxy(qobject_cast<TransfersManagerSortFilterProxyModel*>(model()));
+    if (proxy && selectionModel())
     {
-        indexes = selectionModel()->selectedRows();
-    }
-
-    if(!indexes.isEmpty())
-    {
-        auto proxy(qobject_cast<QSortFilterProxyModel*>(model()));
-        if(proxy)
+        auto indexes = selectionModel()->selectedRows();
+        if (!indexes.isEmpty())
         {
             auto sourceModel = MegaSyncApp->getTransfersModel();
 
-            // Sort to keep items in the same order
-            std::sort(indexes.begin(), indexes.end(),[](QModelIndex check1, QModelIndex check2){
-                return check1.row() > check2.row();
-            });
-
-            auto firstToMove = indexes.first();
-
-            if(firstToMove.row() == model()->rowCount())
-            {
-                return;
-            }
-
-            auto proxyTargetIndex = proxy->index(indexes.first().row() + 1,0);
-            auto sourceTargetIndex = proxy->mapToSource(proxyTargetIndex);
-            auto rowTarget = sourceTargetIndex.row();
-
-            sourceModel->inverseMoveRowsSignal(true);
-
-            for (int item = 0; item < indexes.size(); ++item)
-            {
-                auto index = indexes.at(item);
-                auto sourceIndex = proxy->mapToSource(index);
-
-                if(item != 0)
-                {
-                    auto previousIndex = indexes.at(item-1);
-
-                    if(previousIndex.row() - index.row() != 1)
-                    {
-                        proxyTargetIndex = proxy->index(index.row() + 1,0);
-                        sourceTargetIndex = proxy->mapToSource(proxyTargetIndex);
-                        rowTarget = sourceTargetIndex.row();
-                    }
-                }
-                sourceModel->moveTransferPriority(QModelIndex(), QList<int>() << rowTarget, QModelIndex(), sourceIndex.row());
-            }
-
-            sourceModel->inverseMoveRowsSignal(false);
+            sourceModel->moveTransferPriorityDown(proxy->mapToSourceList(indexes));
         }
     }
 
@@ -1202,38 +1220,15 @@ void MegaTransferView::moveDownClicked()
 
 void MegaTransferView::moveToBottomClicked()
 {
-    QModelIndexList indexes;
-    if(selectionModel())
+    auto proxy(qobject_cast<TransfersManagerSortFilterProxyModel*>(model()));
+    if (proxy && selectionModel())
     {
-        indexes = selectionModel()->selectedRows();
-    }
-
-    if(!indexes.isEmpty())
-    {
-        auto proxy(qobject_cast<QSortFilterProxyModel*>(model()));
-        if(proxy)
+        auto indexes = selectionModel()->selectedRows();
+        if (!indexes.isEmpty())
         {
             auto sourceModel = MegaSyncApp->getTransfersModel();
 
-            // Sort to keep items in the same order
-            std::sort(indexes.begin(), indexes.end(),[](QModelIndex check1, QModelIndex check2){
-                return check1.row() < check2.row();
-            });
-
-            auto firstToMove = indexes.last();
-
-            if(firstToMove.row() == model()->rowCount())
-            {
-                return;
-            }
-
-            for (int item = 0; item < indexes.size(); ++item)
-            {
-                auto index = indexes.at(item);
-                auto sourceIndex = proxy->mapToSource(index);
-
-                sourceModel->moveTransferPriority(QModelIndex(), QList<int>() << sourceIndex.row(), QModelIndex(), -2);
-            }
+            sourceModel->moveTransferPriorityToBottom(proxy->mapToSourceList(indexes));
         }
     }
 
@@ -1361,7 +1356,7 @@ void MegaTransferView::showInMegaClicked()
 
             if (d->mParentHandle != mega::INVALID_HANDLE)
             {
-                qobject_cast<MegaApplication*>(qApp)->shellViewOnMega(d->mParentHandle, false);
+                MegaSyncApp->shellViewOnMegaByHandle(d->mParentHandle, false);
             }
         }
     }
@@ -1409,9 +1404,8 @@ void MegaTransferView::onOpenUrlFinished()
 
 void MegaTransferView::showOpeningFileError()
 {
-    QMegaMessageBox::MessageBoxInfo msgInfo;
+    MessageDialogInfo msgInfo;
     msgInfo.parent = this;
-    msgInfo.title =  QMegaMessageBox::errorTitle();
-    msgInfo.text =   errorOpeningFileText();
-    QMegaMessageBox::warning(msgInfo);
+    msgInfo.descriptionText = errorOpeningFileText();
+    MessageDialogOpener::warning(msgInfo);
 }
