@@ -1,10 +1,8 @@
 #include "SomeIssuesOccurredMessage.h"
-#include "ui_SomeIssuesOccurredMessage.h"
 
-#include <StalledIssuesDialog.h>
-#include <DialogOpener.h>
-#include <Platform.h>
-#include <StalledIssuesModel.h>
+#include "DialogOpener.h"
+#include "StalledIssuesDialog.h"
+#include "ui_SomeIssuesOccurredMessage.h"
 
 SomeIssuesOccurredMessage::SomeIssuesOccurredMessage(QWidget *parent) :
     QWidget(parent),
@@ -18,17 +16,22 @@ SomeIssuesOccurredMessage::~SomeIssuesOccurredMessage()
     delete ui;
 }
 
-void SomeIssuesOccurredMessage::changeEvent(QEvent *event)
+bool SomeIssuesOccurredMessage::event(QEvent* event)
 {
-    if(event->type() == QEvent::LanguageChange)
+    if (event->type() == QEvent::LanguageChange)
     {
         ui->retranslateUi(this);
     }
 
-    QWidget::changeEvent(event);
+    return QWidget::event(event);
 }
 
 void SomeIssuesOccurredMessage::on_viewIssuesButton_clicked()
+{
+    showStalledIssuesDialog();
+}
+
+void SomeIssuesOccurredMessage::showStalledIssuesDialog()
 {
     auto stalledIssuesDialog = DialogOpener::findDialog<StalledIssuesDialog>();
     if(stalledIssuesDialog)
