@@ -18,8 +18,10 @@ public:
     explicit SomeIssuesOccurredMessage(QWidget *parent = nullptr);
     ~SomeIssuesOccurredMessage();
 
+    static void showStalledIssuesDialog();
+
 protected:
-    void changeEvent(QEvent *) override;
+    bool event(QEvent* event) override;
 
 private slots:
     void on_viewIssuesButton_clicked();
