@@ -1,17 +1,14 @@
-#include <TransferMetaData.h>
+#include "TransferMetaData.h"
+
+#include "megaapi.h"
+#include "MegaApplication.h"
+#include "MegaNodeNames.h"
+#include "Notificator.h"
+#include "Preferences.h"
 
 #include <QDir>
 
-#include <MegaApplication.h>
-#include "Preferences.h"
-#include <Notificator.h>
-#include <MegaNodeNames.h>
-
-#include <megaapi.h>
-#include <mega/types.h>
-
-
-//CLASS TRANSFERMETADATAITEMID
+// CLASS TRANSFERMETADATAITEMID
 bool TransferMetaDataItemId::operator==(const TransferMetaDataItemId& item) const
 {
     if((item.tag < 0) && (item.handle == mega::INVALID_HANDLE))
@@ -30,7 +27,6 @@ bool TransferMetaDataItemId::operator<(const TransferMetaDataItemId &item) const
 }
 
 /////////////////////////////////
-
 TransferMetaData::TransferMetaData(int direction, unsigned long long id)
     : mInitialTopLevelTransfers(-1), mInitialPendingFolderTransfersFromOtherSession(0), mFinishedTopLevelTransfers(0), mStartedTopLevelTransfers(0), mTransferDirection(direction), mCreateRootFolder(false),
       mAppId(id), mCreatedFromOtherSession(false), mProcessCancelled(false),mTotalFileCount(0), mNotification(nullptr), mNonExistsFailAppId(0)
@@ -115,9 +111,8 @@ bool TransferMetaData::finish(mega::MegaTransfer *transfer, mega::MegaError* e)
             value->id = id;
             TransferData::TransferState state(TransferData::TRANSFER_NONE);
 
-            if(transfer->getState() == mega::MegaTransfer::STATE_FAILED
-                 || e->getErrorCode() == mega::MegaError::API_EINCOMPLETE
-                 || e->getErrorCode() == mega::MegaError::API_EACCESS)
+            if (transfer->getState() == mega::MegaTransfer::STATE_FAILED ||
+                e->getErrorCode() != mega::MegaError::API_OK)
             {
                 state = TransferData::TRANSFER_FAILED;
                 value->failedTransfer = std::shared_ptr<mega::MegaTransfer>(transfer->copy());
@@ -270,17 +265,12 @@ void TransferMetaData::increaseFinishedTopLevelTransfers(mega::MegaTransfer *tra
 
 bool TransferMetaData::isSingleTransfer() const
 {
-    if(isNonExistData())
-    {
-        return mFiles.nonExistFailedTransfers.size() == 1;
-    }
-
-    return (mFiles.completedTransfers.size() + mFiles.failedTransfers.size() + getTotalEmptyFolders()) == 1;
+    return getTransfersCount() == 1;
 }
 
 int TransferMetaData::getTotalFiles() const
 {
-    return mFiles.size();
+    return static_cast<int>(mFiles.size());
 }
 
 int TransferMetaData::getPendingFiles() const
@@ -368,15 +358,23 @@ int TransferMetaData::getNonExistentCount() const
     return mFiles.nonExistFailedTransfers.size();
 }
 
-TransferMetaDataItemId TransferMetaData::getFirstTransferIdByState(TransferData::TransferState state) const
+std::shared_ptr<TransferMetaDataItem>
+    TransferMetaData::getFirstTransferByState(TransferData::TransferState state) const
 {
-    TransferMetaDataItemId id = mFiles.getFirstTransferIdByState(state);
-    if(!id.isValid())
+    auto item = mFiles.getFirstTransferByState(state);
+    if (!item || !item->id.isValid())
     {
-        id = mEmptyFolders.getFirstTransferIdByState(state);
+        item = mEmptyFolders.getFirstTransferByState(state);
     }
 
-    return id;
+    return item;
+}
+
+TransferMetaDataItemId
+    TransferMetaData::getFirstTransferIdByState(TransferData::TransferState state) const
+{
+    auto item = getFirstTransferByState(state);
+    return item ? item->id : TransferMetaDataItemId();
 }
 
 QList<TransferMetaDataItemId> TransferMetaData::getTransferIdsByState(TransferData::TransferState state) const
@@ -454,10 +452,10 @@ void TransferMetaData::addFileFromFolder(int folderTag, int fileTag)
     folderItem->files.pendingTransfers.insert(fileId, fileItem);
 }
 
-void TransferMetaData::topLevelFolderScanningFinished(int filecount)
+void TransferMetaData::topLevelFolderScanningFinished(unsigned int filecount)
 {
     mTotalFileCount += filecount;
-    QString message = QString::fromUtf8("Folder Controller: Folder scanning finished. Containing %1 files").arg(filecount);
+    QString message = QString::fromUtf8("Folder Controller: Folder scanning finished. Containing %1 files").arg(static_cast<int>(filecount));
     mega::MegaApi::log(mega::MegaApi::LOG_LEVEL_DEBUG, message.toUtf8().constData());
     mStartedTopLevelTransfers++;
     checkScanningState();
@@ -582,6 +580,16 @@ bool TransferMetaData::isEmpty() const
     return mFiles.size() == 0 && mEmptyFolders.size() == 0;
 }
 
+int TransferMetaData::getTransfersCount() const
+{
+    if(isNonExistData())
+    {
+        return mFiles.nonExistFailedTransfers.size();
+    }
+
+    return (mFiles.completedTransfers.size() + mFiles.failedTransfers.size() + getTotalEmptyFolders());
+}
+
 unsigned long long TransferMetaData::getAppId() const
 {
     return mAppId;
@@ -647,7 +655,7 @@ void TransferMetaData::start(mega::MegaTransfer *transfer)
     }
 }
 
-void TransferMetaData::retryFileFromFolderFailingItem(int fileTag, int folderTag, int nodeHandle)
+void TransferMetaData::retryFileFromFolderFailingItem(int fileTag, int folderTag, mega::MegaHandle nodeHandle)
 {
     TransferMetaDataItemId fileId(fileTag, nodeHandle);
 
@@ -701,9 +709,10 @@ void TransferMetaData::retryAllPressed()
     }
 }
 
-DownloadTransferMetaData::DownloadTransferMetaData(unsigned long long appId, const QString &path)
-    : TransferMetaData(mega::MegaTransfer::TYPE_DOWNLOAD, appId),
-      mLocalTargetPath(QDir::toNativeSeparators(path))
+DownloadTransferMetaData::DownloadTransferMetaData(unsigned long long appId, const QString& path):
+    TransferMetaData(mega::MegaTransfer::TYPE_DOWNLOAD, appId),
+    mLocalTargetPath(QDir::toNativeSeparators(path)),
+    mIsImportedLink(false)
 {
 }
 
@@ -759,8 +768,8 @@ QStringList DownloadTransferMetaData::getLocalPaths() const
         QFileInfo fileInfo(id.path);
         QString localPath(fileInfo.path());
 
-        char *escapedName = MegaSyncApp->getMegaApi()->escapeFsIncompatible(id.name.toStdString().c_str(),
-                                                                            localPath.toStdString().c_str());
+        char *escapedName = MegaSyncApp->getMegaApi()->escapeFsIncompatible(id.name.toUtf8().constData(),
+                                                                            localPath.toUtf8().constData());
         QString nodeName = QString::fromUtf8(escapedName);
         delete [] escapedName;
 
@@ -812,6 +821,26 @@ bool DownloadTransferMetaData::isNonExistTransfer(mega::MegaTransfer *transfer)
     //    return true;
 }
 
+void DownloadTransferMetaData::setIsImportedLink()
+{
+    mIsImportedLink = true;
+}
+
+std::shared_ptr<TransferMetaDataItem> DownloadTransferMetaData::containsAFailedImportedLink() const
+{
+    if (mIsImportedLink)
+    {
+        auto item = getFirstTransferByState(TransferData::TRANSFER_FAILED);
+
+        if (item && item->getErrorCode() != mega::MegaError::API_OK)
+        {
+            return item;
+        }
+    }
+
+    return nullptr;
+}
+
 UploadTransferMetaData::UploadTransferMetaData(unsigned long long appId, const mega::MegaHandle handle)
  : TransferMetaData(mega::MegaTransfer::TYPE_UPLOAD, appId),
    mNodeTargetHandle(handle)
@@ -934,7 +963,8 @@ QString UploadTransferMetaData::getDestinationNodePathByData(const std::shared_p
     {
         QString path;
 
-        if(destinationNode->getHandle() == MegaSyncApp->getMegaApi()->getRootNode()->getHandle())
+        if (MegaSyncApp->getRootNode() &&
+            destinationNode->getHandle() == MegaSyncApp->getRootNode()->getHandle())
         {
             path = MegaNodeNames::getRootNodeName(destinationNode.get());
         }
@@ -945,7 +975,7 @@ QString UploadTransferMetaData::getDestinationNodePathByData(const std::shared_p
 
         return path;
     }
-    return QCoreApplication::translate("MegaError", "Decryption error");
+    return MegaNodeNames::getUndecryptedFolderName();
 }
 
 bool UploadTransferMetaData::finish(mega::MegaTransfer *transfer, mega::MegaError *e)
