@@ -1,25 +1,28 @@
 #include "TransfersManagerSortFilterProxyModel.h"
-#include "TransfersModel.h"
-#include "MegaTransferView.h"
-#include "MegaApplication.h"
-#include "TransferManagerDelegateWidget.h"
-#include <megaapi.h>
 
-#include <QMutexLocker>
+#include "MegaApplication.h"
+#include "MegaTransferView.h"
+#include "TransferManagerDelegateWidget.h"
+#include "TransfersModel.h"
+#include "TransferWidgetColumnsManager.h"
+
 #include <QElapsedTimer>
+#include <QMutexLocker>
 #include <QRunnable>
 #include <QTimer>
 
-TransfersManagerSortFilterProxyModel::TransfersManagerSortFilterProxyModel(QObject* parent)
-    : TransfersSortFilterProxyBaseModel(parent),
-      mTransferStates (TransferData::STATE_MASK),
-      mTransferTypes (TransferData::TYPE_MASK),
-      mFileTypes (~Utilities::FileTypes()),
-      mNextTransferStates (mTransferStates),
-      mNextTransferTypes (mTransferTypes),
-      mNextFileTypes (mFileTypes),
-      mSortCriterion (SortCriterion::PRIORITY),
-      mThreadPool (ThreadPoolSingleton::getInstance())
+#include <megaapi.h>
+
+TransfersManagerSortFilterProxyModel::TransfersManagerSortFilterProxyModel(QObject* parent):
+    TransfersSortFilterProxyBaseModel(parent),
+    mTransferStates(TransferData::STATE_MASK),
+    mTransferTypes(TransferData::TYPE_MASK),
+    mFileTypes(~Utilities::FileTypes()),
+    mNextTransferStates(mTransferStates),
+    mNextTransferTypes(mTransferTypes),
+    mNextFileTypes(mFileTypes),
+    mSortCriterion(SortCriterion::PRIORITY),
+    mThreadPool(ThreadPoolSingleton::getInstance())
 {
     connect(&mFilterWatcher, &QFutureWatcher<void>::finished,
             this, &TransfersManagerSortFilterProxyModel::onModelSortedFiltered);
@@ -166,6 +169,23 @@ void TransfersManagerSortFilterProxyModel::onModelSortedFiltered()
     emit searchNumbersChanged();
 }
 
+void TransfersManagerSortFilterProxyModel::setColumnManager(
+    QPointer<TransferWidgetColumnsManager> newColumnManager)
+{
+    mColumnManager = newColumnManager;
+}
+
+QModelIndexList
+    TransfersManagerSortFilterProxyModel::mapToSourceList(const QModelIndexList& proxyIndexes)
+{
+    QModelIndexList sourceIndexes;
+    for (auto proxyIndex: proxyIndexes)
+    {
+        sourceIndexes.append(mapToSource(proxyIndex));
+    }
+    return sourceIndexes;
+}
+
 void TransfersManagerSortFilterProxyModel::setFilters(const TransferData::TransferTypes transferTypes,
                                                const TransferData::TransferStates transferStates,
                                                const Utilities::FileTypes fileTypes)
@@ -221,9 +241,14 @@ void TransfersManagerSortFilterProxyModel::resetTransfersStateCounters()
     mPermanentFailedTransfers.clear();
 }
 
-TransferBaseDelegateWidget *TransfersManagerSortFilterProxyModel::createTransferManagerItem(QWidget*)
+TransferBaseDelegateWidget*
+    TransfersManagerSortFilterProxyModel::createTransferManagerItem(QWidget* view)
 {
-    auto item = new TransferManagerDelegateWidget(nullptr);
+    auto item = new TransferManagerDelegateWidget(view);
+    if (mColumnManager)
+    {
+        item->setColumnManager(mColumnManager);
+    }
 
     connect(item, &TransferManagerDelegateWidget::cancelClearTransfer,
             this, &TransfersManagerSortFilterProxyModel::onCancelClearTransfer);
@@ -387,7 +412,7 @@ bool TransfersManagerSortFilterProxyModel::lessThan(const QModelIndex &left, con
             }
             else if(leftItem->isFinished() && rightItem->isFinished())
             {
-                return leftItem->getRawFinishedTime() < rightItem->getRawFinishedTime();
+                return leftItem->getFinishedDateTime() < rightItem->getFinishedDateTime();
             }
         }
         default:
@@ -398,7 +423,6 @@ bool TransfersManagerSortFilterProxyModel::lessThan(const QModelIndex &left, con
     return QSortFilterProxyModel::lessThan(left, right);
 }
 
-
 //It is called from a QtConcurrent thread
 void TransfersManagerSortFilterProxyModel::onRowsAboutToBeRemoved(const QModelIndex &parent, int first, int last)
 {
@@ -519,30 +543,36 @@ bool TransfersManagerSortFilterProxyModel::dropMimeData(const QMimeData *data, Q
             QModelIndex proxyIndex = index(destRow, column,parent);
             auto sourceIndex = mapToSource(proxyIndex);
 
-            sourceM->moveTransferPriority(parent,rows,parent, sourceIndex.row());
+            sourceM->moveTransferPriorityByDrag(parent, rows, parent, sourceIndex.row());
 
             sourceM->ignoreMoveRowsSignal(true);
-            sourceM->moveTransferPriority(parent, QList<int>() << sourceIndex.row(), parent, rows.first());
+            sourceM->moveTransferPriorityByDrag(parent,
+                                                QList<int>() << sourceIndex.row(),
+                                                parent,
+                                                rows.first());
             sourceM->ignoreMoveRowsSignal(false);
         }
         else if(destRow == 0)
         {
             QModelIndex proxyIndex = index(destRow, column,parent);
             auto sourceIndex = mapToSource(proxyIndex);
-            sourceM->moveTransferPriority(parent, rows , parent, sourceIndex.row());
+            sourceM->moveTransferPriorityByDrag(parent, rows, parent, sourceIndex.row());
         }
         else
         {
             QModelIndex proxyIndex = index(destRow, column,parent);
             auto sourceIndex = mapToSource(proxyIndex);
-            sourceM->moveTransferPriority(parent, rows , parent, sourceIndex.row());
+            sourceM->moveTransferPriorityByDrag(parent, rows, parent, sourceIndex.row());
 
             if(mSortCriterion != SortCriterion::PRIORITY)
             {
                 sourceM->ignoreMoveRowsSignal(true);
                 auto previousProxyIndex = index(destRow -1, column, parent);
                 auto previousSourceIndex = mapToSource(previousProxyIndex);
-                sourceM->moveTransferPriority(parent, QList<int>() << previousSourceIndex.row() , parent, rows.first());
+                sourceM->moveTransferPriorityByDrag(parent,
+                                                    QList<int>() << previousSourceIndex.row(),
+                                                    parent,
+                                                    rows.first());
                 sourceM->ignoreMoveRowsSignal(false);
             }
         }
@@ -659,8 +689,10 @@ bool TransfersManagerSortFilterProxyModel::moveRows(const QModelIndex &proxyPare
             destRow = mapToSource(index(destinationChild, 0, destinationParent)).row();
         }
 
-        moveOk = sourceM->moveTransferPriority(sourceIndex.parent(), QList<int>() << sourceIndex.row(),
-                                         sourceIndex.parent(), destRow);
+        moveOk = sourceM->moveTransferPriorityByDrag(sourceIndex.parent(),
+                                                     QList<int>() << sourceIndex.row(),
+                                                     sourceIndex.parent(),
+                                                     destRow);
         row++;
     }
 
