@@ -1,33 +1,36 @@
 #ifndef TRANSFERSMODEL_H
 #define TRANSFERSMODEL_H
 
+#include "megaapi.h"
+#include "Preferences.h"
 #include "QTMegaTransferListener.h"
 #include "TransferItem.h"
 #include "TransferMetaData.h"
 #include "TransferRemainingTime.h"
-#include "Preferences.h"
-
-#include <megaapi.h>
+#include "TransferTrack.h"
 
 #include <QAbstractItemModel>
-#include <QLinkedList>
-#include <QtConcurrent/QtConcurrent>
 #include <QFutureWatcher>
+#include <QLinkedList>
 #include <QReadWriteLock>
+#include <QtConcurrent/QtConcurrent>
 
-#include <set>
 #include <memory>
+#include <set>
 
 struct TransfersCount
 {
-    int totalUploads;
-    int totalDownloads;
+    uint totalUploads;
+    uint totalDownloads;
 
-    int pendingUploads;
-    int pendingDownloads;
+    uint pendingUploads;
+    uint pendingDownloads;
 
-    int failedUploads;
-    int failedDownloads;
+    // Used for throttling logic
+    uint pendingSyncTransfers;
+
+    uint failedUploads;
+    uint failedDownloads;
 
     long long completedUploadBytes;
     long long completedDownloadBytes;
@@ -35,14 +38,15 @@ struct TransfersCount
     long long totalUploadBytes;
     long long totalDownloadBytes;
 
-    QMap<Utilities::FileType, long long> transfersByType;
-    QMap<Utilities::FileType, long long> transfersFinishedByType;
+    QMap<Utilities::FileType, uint> transfersByType;
+    QMap<Utilities::FileType, uint> transfersFinishedByType;
 
     TransfersCount():
         totalUploads(0),
         totalDownloads(0),
         pendingUploads(0),
         pendingDownloads(0),
+        pendingSyncTransfers(0),
         failedUploads(0),
         failedDownloads(0),
         completedUploadBytes(0),
@@ -51,11 +55,51 @@ struct TransfersCount
         totalDownloadBytes(0)
     {}
 
-    int completedDownloads()const {return totalDownloads - pendingDownloads - failedDownloads;}
-    int completedUploads() const {return totalUploads - pendingUploads - failedUploads;}
-    int pendingTransfers() const {return pendingDownloads + pendingUploads;}
+    void addPendingUpload(mega::MegaTransfer* transfer)
+    {
+        if (transfer->isSyncTransfer())
+        {
+            pendingSyncTransfers++;
+        }
 
-    long long totalFailedTransfers() const {return failedUploads + failedDownloads;}
+        pendingUploads++;
+    }
+
+    void addPendingDownload(mega::MegaTransfer* transfer)
+    {
+        if (transfer->isSyncTransfer())
+        {
+            pendingSyncTransfers++;
+        }
+
+        pendingDownloads++;
+    }
+
+    void removePendingUpload(mega::MegaTransfer* transfer)
+    {
+        if (transfer->isSyncTransfer())
+        {
+            pendingSyncTransfers--;
+        }
+
+        pendingUploads--;
+    }
+
+    void removePendingDownload(mega::MegaTransfer* transfer)
+    {
+        if (transfer->isSyncTransfer())
+        {
+            pendingSyncTransfers--;
+        }
+
+        pendingDownloads--;
+    }
+
+    uint completedDownloads()const {return totalDownloads - pendingDownloads - failedDownloads;}
+    uint completedUploads() const {return totalUploads - pendingUploads - failedUploads;}
+    uint pendingTransfers() const {return pendingDownloads + pendingUploads;}
+
+    uint totalFailedTransfers() const {return failedUploads + failedDownloads;}
 
     void clear()
     {
@@ -63,6 +107,7 @@ struct TransfersCount
         totalDownloads = 0;
         pendingUploads = 0;
         pendingDownloads = 0;
+        pendingSyncTransfers = 0;
         failedUploads = 0;
         failedDownloads = 0;
         completedUploadBytes = 0;
@@ -124,15 +169,19 @@ public:
     TransfersCount getTransfersCount();
     LastTransfersCount getLastTransfersCount();
 
-    void resetCompletedUploads(QList<QExplicitlySharedDataPointer<TransferData> > transfersToReset);
-    void resetCompletedDownloads(QList<QExplicitlySharedDataPointer<TransferData>> transfersToReset);
+    void resetUploads(QList<QExplicitlySharedDataPointer<TransferData>> transfersToReset);
+    void resetDownloads(QList<QExplicitlySharedDataPointer<TransferData>> transfersToReset);
     void resetCompletedTransfers();
 
-    void setMaxTransfersToProcess(uint16_t max);
+    void setMaxTransfersToProcess(int max);
+
+    std::shared_ptr<TransferTrack> addTrackToTransfer(const QString& id,
+                                                      TransferData::TransferType type);
+    std::shared_ptr<TransferTrack> getTrackToTransfer(const QString& id);
+    void removeTrackToTransfer(const QString& id);
 
     TransfersToProcess processTransfers();
     void clear();
-    void clearTransfersCount();
 
 public slots:
     void onTransferStart(mega::MegaApi*, mega::MegaTransfer* transfer);
@@ -149,6 +198,9 @@ private:
     void updateFailedTransfer(QExplicitlySharedDataPointer<TransferData> data, mega::MegaTransfer* transfer,
                               mega::MegaError* e);
 
+    void trackTransfer(QExplicitlySharedDataPointer<TransferData> data);
+    void removeFinishedTracks(const QString& id);
+
     QExplicitlySharedDataPointer<TransferData> createData(mega::MegaTransfer* transfer, mega::MegaError *e);
     QExplicitlySharedDataPointer<TransferData> onTransferEvent(mega::MegaTransfer* transfer, mega::MegaError *e);
     QList<QExplicitlySharedDataPointer<TransferData>> extractFromCache(QMap<int, QExplicitlySharedDataPointer<TransferData>>& dataMap, int spaceForTransfers);
@@ -179,21 +231,28 @@ private:
     cacheTransfers mTransfersToProcess;
     QMutex mCacheMutex;
     QMutex mCountersMutex;
+    QMutex mTrackTransferMutex;
     TransfersCount mTransfersCount;
     LastTransfersCount mLastTransfersCount;
-    std::atomic<int16_t> mMaxTransfersToProcess;
+    std::atomic<int> mMaxTransfersToProcess;
 
     QList<int> mRetriedFolder;
     QList<int> mIgnoredFiles;
+
+    std::unique_ptr<mega::QTMegaTransferListener> mDelegateListener;
+
+    QHash<QString, std::shared_ptr<TransferTrack>> mTrackToTransfers;
 };
 
 struct DownloadTransferInfo
 {
+    TransferData::TransferStates state;
     mega::MegaHandle nodeHandle;
 };
 
 struct UploadTransferInfo
 {
+    TransferData::TransferStates state;
     QString localPath;
     QString filename;
     mega::MegaHandle parentHandle;
@@ -204,7 +263,7 @@ class TransfersModel : public QAbstractItemModel
     Q_OBJECT
 
 public:
-    explicit TransfersModel(QObject* parent = 0);
+    explicit TransfersModel();
     ~TransfersModel();
 
     virtual Qt::ItemFlags flags(const QModelIndex& index) const override;
@@ -223,9 +282,15 @@ public:
                   const QModelIndex &destinationParent, int destinationChild) override;
 
     void ignoreMoveRowsSignal(bool state);
-    void inverseMoveRowsSignal(bool state);
-    bool moveTransferPriority(const QModelIndex& sourceParent, const QList<int>& rows,
-                  const QModelIndex& destinationParent, int destinationChild);
+
+    bool moveTransferPriorityByDrag(const QModelIndex& sourceParent,
+                                    const QList<int>& rows,
+                                    const QModelIndex& destinationParent,
+                                    int destinationChild);
+    void moveTransferPriorityUp(const QModelIndexList& sourceIndexes);
+    void moveTransferPriorityDown(const QModelIndexList& sourceIndexes);
+    void moveTransferPriorityToTop(const QModelIndexList& sourceIndexes);
+    void moveTransferPriorityToBottom(const QModelIndexList& sourceIndexes);
 
     void resetModel();
 
@@ -240,7 +305,7 @@ public:
 
     void retryTransferByIndex(const QModelIndex& index);
     void retryTransfers(QModelIndexList indexes, unsigned long long suggestedUploadAppData = 0, unsigned long long suggestedDownloadAppData = 0);
-    void retryTransfersByAppDataId(const std::shared_ptr<TransferMetaData> &data);
+    void retryTransfersByAppDataId(const std::shared_ptr<TransferMetaData>& data);
 
     void cancelAndClearTransfers(const QModelIndexList& indexes, QWidget *canceledFrom);
     void cancelAllTransfers(QWidget *canceledFrom);
@@ -265,11 +330,11 @@ public:
 
     void lockModelMutex(bool lock);
 
-    long long  getNumberOfTransfersForFileType(Utilities::FileType fileType) const;
-    long long  getNumberOfFinishedForFileType(Utilities::FileType fileType) const;
+    uint getNumberOfTransfersForFileType(Utilities::FileType fileType) const;
+    uint  getNumberOfFinishedForFileType(Utilities::FileType fileType) const;
     TransfersCount getTransfersCount();
     TransfersCount getLastTransfersCount();
-    long long failedTransfers();
+    uint failedTransfers();
 
     void startTransfer(QExplicitlySharedDataPointer<TransferData> transfer);
     void updateTransfer(QExplicitlySharedDataPointer<TransferData> transfer, int row);
@@ -278,12 +343,20 @@ public:
 
     bool areAllPaused() const;
 
+    //////////////
+    // Track transfers
+    std::shared_ptr<TransferTrack> addTrackToTransfer(const QString& id,
+                                                      TransferData::TransferType type);
+    std::shared_ptr<TransferTrack> getTrackToTransfer(const QString& id);
+    void removeTrackToTransfer(const QString& id);
 
-    const QExplicitlySharedDataPointer<const TransferData> activeDownloadTransferFound(DownloadTransferInfo *info) const;
+    const QExplicitlySharedDataPointer<const TransferData>
+        downloadTransferFound(DownloadTransferInfo* info) const;
     const QExplicitlySharedDataPointer<const TransferData> activeUploadTransferFound(UploadTransferInfo* info) const;
 
     const QExplicitlySharedDataPointer<const TransferData> getTransferByTag(int tag) const;
     QExplicitlySharedDataPointer<TransferData> getTransferByTag(int tag);
+    //////////////
 
     int getRowByTransferTag(int tag) const;
     void sendDataChangedByTag(int tag);
@@ -303,7 +376,6 @@ public:
     void showSyncCancelledWarning();
 
     QList<int> getDragAndDropRows(const QMimeData* data);
-
 signals:
     void pauseStateChanged(bool pauseState);
     void transferPauseStateChanged();
@@ -335,11 +407,11 @@ private slots:
     void cacheCancelTransfersTags();
     void processFailedTransfers();
     void onProcessTransfers();
-    void updateTransfersCount();
     void onClearTransfersFinished();
     void onUpdateTransfersFinished();
     void onAskForMostPriorityTransfersFinished();
     void onKeepPCAwake();
+    void updateTransfersCount();
 
 private:
     void removeRows(QModelIndexList &indexesToRemove);
@@ -350,12 +422,17 @@ private:
     void restoreTagsByRow();
     QList<QExplicitlySharedDataPointer<TransferData>> getTransfersToIterate() const;
 
+    void moveTransferPriority(const QModelIndexList& sourceIndexes,
+                              bool up,
+                              const std::function<void(QExplicitlySharedDataPointer<TransferData>,
+                                                       mega::MegaRequestListener*)>& func);
+
     void retryTransfers(const QMultiMap<unsigned long long, QExplicitlySharedDataPointer<TransferData>> &transfersToRetry);
 
     bool isUiBlockedModeActive() const ;
     void setUiBlockedMode(bool state);
 
-    void setUiBlockedModeByCounter(uint32_t transferCount);
+    void setUiBlockedModeByCounter(int transferCount);
     void updateUiBlockedByCounter(int updates);
     bool isUiBlockedByCounter() const;
     void setUiBlockedByCounterMode(bool state);
@@ -365,10 +442,15 @@ private:
     void mostPriorityTransferMayChanged(bool state);
 
     int performPauseResumeAllTransfers(int activeTransfers, bool useEventUpdater);
-    int performPauseResumeVisibleTransfers(const QModelIndexList& indexes, bool pauseState, bool useEventUpdater);
+    void performPauseResumeVisibleTransfers(const QModelIndexList& indexes,
+                                            bool pauseState,
+                                            bool useEventUpdater);
 
     void openFolder(const QFileInfo& info);
 
+    const QExplicitlySharedDataPointer<const TransferData>
+        downloadTransferFound(TransferData::TransferStates state, DownloadTransferInfo* info) const;
+
     void updateMetaDataBeforeRetryingTransfers(std::shared_ptr<mega::MegaTransfer> transfer);
 
 private:
@@ -376,19 +458,18 @@ private:
     std::shared_ptr<Preferences> mPreferences;
     QThread* mTransferEventThread;
     TransferThread* mTransferEventWorker;
-    mega::QTMegaTransferListener *mDelegateListener;
     QTimer mProcessTransfersTimer;
     TransfersCount mTransfersCount;
     LastTransfersCount mLastTransfersCount;
 
     QList<QExplicitlySharedDataPointer<TransferData>> mTransfers;
-    QHash<int,QExplicitlySharedDataPointer<TransferData>> mFailedFoldersByTag;
-    QHash<mega::MegaHandle,QPersistentModelIndex> mCompletedTransfersByTag;
+    QHash<int, QExplicitlySharedDataPointer<TransferData>> mFailedFoldersByTag;
 
     TransferThread::TransfersToProcess mTransfersToProcess;
     QFutureWatcher<void> mUpdateTransferWatcher;
     QFutureWatcher<void> mClearTransferWatcher;
     QFutureWatcher<QPair<int, int>> mAskForMostPriorityTransfersWatcher;
+    QFutureWatcher<void> mRetryTransferWatcher;
 
     uint8_t mTransfersProcessChanged;
     uint8_t mUpdateMostPriorityTransfer;
@@ -412,7 +493,6 @@ private:
     QSet<TransferTag> mActiveTransfers;
 
     bool mIgnoreMoveSignal;
-    bool mInverseMoveSignal;
 
     QSet<int> mRetriedFolderTags;
 };
